//*****************************************************************************
//
//pmap.c - Driver for the PMAP Module.
//
//*****************************************************************************
#include "inc/hw_types.h"
#include "debug.h"
#include "pmap.h"
#ifdef  __IAR_SYSTEMS_ICC__
#include "deprecated/IAR/msp430xgeneric.h"
#else
#include "deprecated/CCS/msp430xgeneric.h"
#endif

//*****************************************************************************
//
//! This function configures the MSP430 Port Mapper
//!
//! \param baseAddress is the base address of the Port Mapper module.
//! \param portMapping is the pointer to init Data
//! \param PxMAPy is the pointer start of first Port Mapper to initialize
//! \param numberOfPorts is the number of Ports to initialize
//! \param portMapReconfigure is used to enable/disable reconfiguration
//!             Valid values are
//!             \b PMAP_ENABLE_RECONFIGURATION
//!             \b PMAP_DISABLE_RECONFIGURATION [Default value]
//! Modified registers are \b PMAPKEYID, \b PMAPCTL
//!
//! \return None
//
//*****************************************************************************
void PMAP_configurePorts ( unsigned int baseAddress,
    const unsigned char *portMapping,
    unsigned char *PxMAPy,
    unsigned char numberOfPorts,
    unsigned char portMapReconfigure
    )
{
    ASSERT((portMapReconfigure == PMAP_ENABLE_RECONFIGURATION) ||
        (portMapReconfigure == PMAP_DISABLE_RECONFIGURATION)
        );

    //Store current interrupt state, then disable all interrupts
    unsigned int globalInterruptState = __get_SR_register() & GIE;
    __disable_interrupt();

    //Get write-access to port mapping registers:
    HWREG(baseAddress + OFS_PMAPKEYID) = PMAPPW;

    //Enable/Disable reconfiguration during runtime
    HWREGB(baseAddress + OFS_PMAPCTL) &= ~PMAPRECFG;
    HWREGB(baseAddress + OFS_PMAPCTL) |= portMapReconfigure;

    //Configure Port Mapping:
    unsigned int i;
    for (i = 0; i < numberOfPorts * 8; i++)
    {
        PxMAPy[i] = portMapping[i];
    }

    //Disable write-access to port mapping registers:
    HWREGB(baseAddress + OFS_PMAPKEYID) = 0;

    //Restore previous interrupt state
    __bis_SR_register(globalInterruptState);
}

//*****************************************************************************
//
//Close the Doxygen group.
//! @}
//
//*****************************************************************************
