//*****************************************************************************
//
//ldoPwr.c - Driver for the LDO-PWR Module.
//
//*****************************************************************************
#include "inc/hw_types.h"
#include "debug.h"
#include "ldopwr.h"
#ifdef  __IAR_SYSTEMS_ICC__
#include "deprecated/IAR/msp430xgeneric.h"
#else
#include "deprecated/CCS/msp430xgeneric.h"
#endif

//*****************************************************************************

//*****************************************************************************
//
//! Unlocks the configuration registers and enables write access
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b LDOKEYPID
//!
//! \returns NONE
//
//*****************************************************************************
void LDOPWR_unLockConfiguration ( unsigned int baseAddress )
{
    HWREG(baseAddress + OFS_LDOKEYPID) = 0x9628;
}

//*****************************************************************************
//
//! Locks the configuration registers and disables write access
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b LDOKEYPID
//!
//! \returns NONE
//
//*****************************************************************************
void LDOPWR_lockConfiguration ( unsigned int baseAddress )
{
    HWREG(baseAddress + OFS_LDOKEYPID) = 0x0000;
}

//*****************************************************************************
//
//! Enables Port U inputs
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b PUCTL
//!
//! \returns NONE
//
//*****************************************************************************
void LDOPWR_enablePort_U_inputs (unsigned int baseAddress )
{
    HWREGB(baseAddress + OFS_PUCTL_H) |= PUIPE_H;
}

//*****************************************************************************
//
//! Disables Port U inputs
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b PUCTL
//!
//! \returns NONE
//
//*****************************************************************************
void LDOPWR_disablePort_U_inputs (unsigned int baseAddress )
{
    HWREGB(baseAddress + OFS_PUCTL_H) &= ~PUIPE_H;
}

//*****************************************************************************
//
//! Enables Port U outputs
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b PUCTL
//!
//! \returns NONE
//
//*****************************************************************************
void LDOPWR_enablePort_U_outputs (unsigned int baseAddress )
{
    HWREGB(baseAddress + OFS_PUCTL_L) |= PUOPE;
}

//*****************************************************************************
//
//! Disables Port U inputs
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b PUCTL
//!
//! \returns NONE
//
//*****************************************************************************
void LDOPWR_disablePort_U_outputs (unsigned int baseAddress )
{
    HWREGB(baseAddress + OFS_PUCTL_L) &= ~PUOPE;
}

//*****************************************************************************
//
//! Returns PU.1 input data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! \returns LDOPWR_PORTU_PIN_HIGH or LDOPWR_PORTU_PIN_LOW - PU.1 input data
//
//*****************************************************************************
unsigned char LDOPWR_getPort_U1_inputData (unsigned int baseAddress )
{
    return ((HWREGB(baseAddress + OFS_PUCTL_L) & PUIN1) >> 3 );
}

//*****************************************************************************
//
//! Returns PU.0 input data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! \returns LDOPWR_PORTU_PIN_HIGH or LDOPWR_PORTU_PIN_LOW - PU.0 input data
//
//*****************************************************************************
unsigned char LDOPWR_getPort_U0_inputData (unsigned int baseAddress )
{
    return ((HWREGB(baseAddress + OFS_PUCTL_L) & PUIN0) >> 2 );
}

//*****************************************************************************
//
//! Returns PU.1 output data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! \returns LDOPWR_PORTU_PIN_HIGH or LDOPWR_PORTU_PIN_LOW - PU.1 output data
//
//*****************************************************************************
unsigned char LDOPWR_getPort_U1_outputData (unsigned int baseAddress )
{
    return ((HWREGB(baseAddress + OFS_PUCTL_L) & PUOUT1) >> 1 );
}

//*****************************************************************************
//
//! Returns PU.0 output data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! \returns LDOPWR_PORTU_PIN_HIGH or LDOPWR_PORTU_PIN_LOW - PU.0 output data
//
//*****************************************************************************
unsigned char LDOPWR_getPort_U0_outputData (unsigned int baseAddress )
{
    return (HWREGB(baseAddress + OFS_PUCTL_L) & PUOUT0);
}

//*****************************************************************************
//
//! Sets PU.1 output data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//! \param value.
//!         Valid values are
//!  \b LDOPWR_PORTU_PIN_HIGH
//!  \b LDOPWR_PORTU_PIN_LOW
//!
//! Modified register is \b PUCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_setPort_U1_outputData (unsigned int baseAddress,
    unsigned char value
    )
{
    if (LDOPWR_PORTU_PIN_HIGH == value){
        HWREGB(baseAddress + OFS_PUCTL_L) |= PUOUT1;
    } else   {
        HWREGB(baseAddress + OFS_PUCTL_L) &= ~PUOUT1;
    }
}

//*****************************************************************************
//
//! Sets PU.0 output data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//! \param value
//! Valid values are
//!     \b LDOPWR_PORTU_PIN_HIGH
//!     \b LDOPWR_PORTU_PIN_LOW
//!
//! Modified register is \b PUCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_setPort_U0_outputData (unsigned int baseAddress,
    unsigned char value
    )
{
    if (LDOPWR_PORTU_PIN_HIGH == value){
        HWREGB(baseAddress + OFS_PUCTL_L) |= PUOUT0;
    } else   {
        HWREGB(baseAddress + OFS_PUCTL_L) &= ~PUOUT0;
    }
}

//*****************************************************************************
//
//! Toggles PU.1 output data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is PUCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_togglePort_U1_outputData (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PUCTL_L) ^= PUOUT1;
}

//*****************************************************************************
//
//! Toggles PU.0 output data
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b PUCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_togglePort_U0_outputData (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PUCTL_L) ^= PUOUT0;
}

//*****************************************************************************
//
//! Enables LDO-PWR module interrupts
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//! \param mask is the logical OR of
//!        \b LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT,
//!        \b LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT or
//!        \b LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
//!
//! Modified register is \b LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_enableInterrupt (unsigned int baseAddress,
    unsigned int mask
    )
{
    ASSERT(0x00 == (mask & ~(LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT +
                             LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT +
                             LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
                             )
                    )
        );
    HWREGB(baseAddress + OFS_LDOPWRCTL_L) &=  ~mask;
    HWREGB(baseAddress + OFS_LDOPWRCTL_H) |= mask;
}

//*****************************************************************************
//
//! Disables LDO-PWR module interrupts
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//! \param mask is the logical OR of
//!     \b LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT,
//!     \b LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT,
//!        LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
//!
//! Modified register is LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_disableInterrupt (unsigned int baseAddress,
    unsigned int mask
    )
{
    ASSERT(0x00 == (mask & ~(LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT +
                             LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT +
                             LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
                             )
                    )
        );
    HWREGB(baseAddress + OFS_LDOPWRCTL_H) &= ~mask;
}

//*****************************************************************************
//
//! Enables LDO-PWR module
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_enable (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_LDOPWRCTL_H) |= LDOOEN_H;
}

//*****************************************************************************
//
//! Disables LDO-PWR module
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_disable (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_LDOPWRCTL_H) &= ~LDOOEN_H;
}

//*****************************************************************************
//
//! Returns the interrupt status of LDO-PWR module interrupts
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//! \param mask is the logical OR of
//!        \b LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT,
//!        \b LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT or
//!        \b LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
//!
//! \returns unsigned char - masked interrupt flags
//
//*****************************************************************************
unsigned char LDOPWR_getInterruptStatus (unsigned int baseAddress,
    unsigned int mask
    )
{
    ASSERT(0x00 == (mask & ~(LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT +
                             LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT +
                             LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
                             )
                    )
        );
    return (HWREGB(baseAddress + OFS_LDOPWRCTL_L) & mask);
}

//*****************************************************************************
//
//! Clears the interrupt status of LDO-PWR module interrupts
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//! \param mask is the logical OR of
//!        \b LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT,
//!        \b LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT,
//!        \b LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
//!
//! Modified register is \b LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_clearInterruptStatus (unsigned int baseAddress,
    unsigned int mask
    )
{
    ASSERT(0x00 == (mask & ~(LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT +
                             LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT +
                             LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
                             )
                    )
        );
    HWREGB(baseAddress + OFS_LDOPWRCTL_L) &=  ~mask;
}

//*****************************************************************************
//
//! Returns if the the LDOI is valid and within bounds
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! \returns unsigned char - Valid values are
//!         \b LDOPWR_LDO_INPUT_VALID
//!         \b LDOPWR_LDO_INPUT_INVALID
//
//*****************************************************************************
unsigned char LDOPWR_isLDOInputValid (unsigned int baseAddress)
{
    return (HWREGB(baseAddress + OFS_LDOPWRCTL_L) & LDOBGVBV);
}

//*****************************************************************************
//
//! Enables the LDO overload auto-off
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_enableOverloadAutoOff (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_LDOPWRCTL_L) |= OVLAOFF_L;
}

//*****************************************************************************
//
//! Disables the LDO overload auto-off
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! Modified register is \b LDOPWRCTL
//!
//! \returns None
//
//*****************************************************************************
void LDOPWR_disableOverloadAutoOff (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_LDOPWRCTL_L) &= ~OVLAOFF_L;
}

//*****************************************************************************
//
//! Returns if the LDOI overload auto-off is enabled or disabled
//!
//! \param baseAddress is the base address of the LDO-PWR module.
//!
//! \returns unsigned char - Valid values are
//!         \b LDOPWR_AUTOOFF_ENABLED
//!         \b LDOPWR_AUTOOFF_DISABLED
//
//*****************************************************************************
unsigned char LDOPWR_getOverloadAutoOffStatus (unsigned int baseAddress)
{
    return (HWREGB(baseAddress + OFS_LDOPWRCTL_L) & OVLAOFF_L);
}

//*****************************************************************************
//
//Close the Doxygen group.
//! @}
//
//*****************************************************************************
