//*****************************************************************************
//
//pmm.c - Driver for the PMM Module.
//
//*****************************************************************************
#include "inc/hw_types.h"
#include "debug.h"
#include "pmm.h"
#ifdef  __IAR_SYSTEMS_ICC__
#include "deprecated/IAR/msp430xgeneric.h"
#else
#include "deprecated/CCS/msp430xgeneric.h"
#endif

//*****************************************************************************
//
//! Enables the low-side SVS circuitry
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0, \b SVSMLCTL.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_enableSVSL (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0) |= PMM_SVSLE;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Disables the low-side SVS circuitry
//!
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_disableSVSL (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0) &= ~PMM_SVSLE;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Enables the high-side SVS circuitry
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_enableSVSH (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0_L) |= PMM_SVSHE;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Disables the high-side SVS circuitry
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_disableSVSH (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0_L) &= ~PMM_SVSHE;//~PMM_SVSHE;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Makes the low-dropout voltage regulator (LDO) remain ON when going into LPM 3/4. [Default]
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_regOn (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0) &= ~PMM_PMMREGOFF;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Turns OFF the low-dropout voltage regulator (LDO) when going into LPM3/4,
//!   thus the system will enter LPM3.5 or LPM4.5 respectively
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_regOff (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0) |= PMM_PMMREGOFF;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Calling this function will trigger a software Power On Reset (POR).
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_trigPOR (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0) |= PMM_PMMSWPOR;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Calling this function will trigger a software Brown Out Rest (BOR).
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! Modified registers are \b PMMCTL0.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_trigBOR (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREGB(baseAddress + OFS_PMMCTL0) |= PMM_PMMSWBOR;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}


//*****************************************************************************
//
//! Clear all interrupt flags for the PMM
//!
//! \param baseAddress is the base address of the PMM module.
//! \param mask is the mask for specifying the required flag
//!        Valid values are
//!            \b  PMM_PMMBORIFG
//!            \b  PMM_PMMRSTIFG,
//!            \b  PMM_PMMPORIFG,
//!            \b  PMM_SVSLIFG,
//!            \b  PMM_SVSHIFG,
//!            \b  PMM_PMMLPM5IFG,
//!            \b  PMM_ALL
//!
//! Modified registers are \b PMMCTL0, \b PMMIFG.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_clearInterrupt (unsigned int baseAddress,
		unsigned int mask)
{
    HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
    HWREG(baseAddress + OFS_PMMIFG) &= ~mask;
    HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}

//*****************************************************************************
//
//! Returns interrupt status
//!
//! \param baseAddress is the base address of the PMM module.
//! \param mask is the mask for specifying the required flag
//!        Valid values are
//!            \b  PMM_PMMBORIFG
//!            \b  PMM_PMMRSTIFG,
//!            \b  PMM_PMMPORIFG,
//!            \b  PMM_SVSLIFG,
//!            \b  PMM_SVSHIFG,
//!            \b  PMM_PMMLPM5IFG,
//!
//! \return STATUS_SUCCESS (0x01) or STATUS_FAIL (0x00)
//
//*****************************************************************************
unsigned int PMM_getInterruptStatus (unsigned int baseAddress,
    unsigned int mask)
{
    return ( (HWREG(baseAddress + OFS_PMMIFG)) & mask );
}


//*****************************************************************************
//
//! Lock I/O pin and other LPMx.5 relevant (e.g. RTC) configurations
//!  upon entry/exit to/from LPMx.5. Once power is applied to the device,
//!  this bit, once set, can only be cleared by the user or
//!  via another power cycle. LPMx.5 configuration remains locked.
//!  Pin state is held during LPMx.5 entry and exit.
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_lockLPM5 (unsigned int baseAddress)
{
	HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
	HWREGB(baseAddress + OFS_PM5CTL0) |= PMM_LOCKLPM5;
	HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;

}

//*****************************************************************************
//
//! LPMx.5 configuration is not locked and defaults to its reset condition.
//!
//! \param baseAddress is the base address of the PMM module.
//!
//! \return NONE
//
//*****************************************************************************
void PMM_unlockLPM5 (unsigned int baseAddress)
{
	HWREGB(baseAddress + OFS_PMMCTL0_H) = PMM_PW_H;
	HWREGB(baseAddress + OFS_PM5CTL0) &= ~PMM_LOCKLPM5;
	HWREGB(baseAddress + OFS_PMMCTL0_H) = 0x00;
}


//*****************************************************************************
//
//Close the Doxygen group.
//! @}
//
//*****************************************************************************
