//*****************************************************************************
//
//ref.c - Driver for the REF Module.
//
//*****************************************************************************
#include "inc/hw_types.h"
#include "debug.h"
#include "ref.h"
#ifdef  __IAR_SYSTEMS_ICC__
#include "deprecated/IAR/msp430xgeneric.h"
#else
#include "deprecated/CCS/msp430xgeneric.h"
#endif

//*****************************************************************************
//
//! Sets the reference voltage for the voltage generator.
//!
//! \param baseAddress is the base address of the REF module.
//! \param referenceVoltageSelect is the desired voltage to generate for a
//!       reference voltage.
//!        Valid values are
//!        \b REF_VREF1_5V [Default]
//!        \b REF_VREF2_0V
//!        \b REF_VREF2_5V
//!        Modified bits are \b REFVSEL of \b REFCTL0 register.
//!
//! This function sets the reference voltage generated by the voltage generator
//! to be used by other peripherals. This reference voltage will only be valid
//! while the REF module is in control.
//! Please note, if the REF_isRefGenBusy() returns REF_ BUSY, this function
//! will have no effect.
//!
//! \return NONE
//
//*****************************************************************************
void REF_setReferenceVoltage (unsigned int baseAddress,
    unsigned char referenceVoltageSelect)
{
    ASSERT(referenceVoltageSelect <= REF_VREF2_5V);

    HWREGB(baseAddress + OFS_REFCTL0_L) &= ~(REFVSEL_3);
    HWREGB(baseAddress + OFS_REFCTL0_L) |= referenceVoltageSelect;
}

//*****************************************************************************
//
//! Disables the internal temperature sensor to save power consumption.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to turn off the internal temperature sensor to save
//! on power consumption. The temperature sensor is enabled by default. Please
//! note, that giving ADC12 module control over the REF module, the state of the
//! temperature sensor is dependent on the controls of the ADC12 module.
//! Please note, if the REF_isRefGenBusy() returns REF_ BUSY, this function
//! will have no effect.
//!
//! Modified bits are \b REFTCOFF of \b REFCTL0 register.
//! \return NONE
//
//*****************************************************************************
void REF_disableTempSensor (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_REFCTL0_L) |= REFTCOFF;
}

//*****************************************************************************
//
//! Enables the internal temperature sensor.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to turn on the internal temperature sensor to use by
//! other peripherals. The temperature sensor is enabled by default.
//! Please note, if the REF_isRefGenBusy() returns REF_ BUSY, this function
//! will have no effect.
//!
//! Modified bits are \b REFTCOFF of \b REFCTL0 register.
//! \return NONE
//
//*****************************************************************************
void REF_enableTempSensor (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_REFCTL0_L) &= ~(REFTCOFF);
}

//*****************************************************************************
//
//! Enables the reference voltage to be used by peripherals.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to enable the generated reference voltage to be used
//! other peripherals or by an output pin, if enabled. Please note, that giving
//! ADC12 module control over the REF module, the state of the reference voltage
//! is dependent on the controls of the ADC12 module.
//! Please note, if the REF_isRefGenBusy() returns REF_ BUSY, this function
//! will have no effect.
//!
//! Modified bits are \b REFON of \b REFCTL0 register.
//! \return NONE
//
//*****************************************************************************
void REF_enableReferenceVoltage (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_REFCTL0_L) |= REFON;
}

//*****************************************************************************
//
//! Disables the reference voltage.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to disable the generated reference voltage.
//! Please note, if the REF_isRefGenBusy() returns REF_ BUSY, this function
//! will have no effect.
//!
//! Modified bits are \b REFON of \b REFCTL0 register.
//! \return NONE
//
//*****************************************************************************
void REF_disableReferenceVoltage (unsigned int baseAddress)
{
    HWREGB(baseAddress + OFS_REFCTL0_L) &= ~(REFON);
}

//*****************************************************************************
//
//! Returns the bandgap mode of the REF module.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to return the bandgap mode of the REF module,
//! requested by the peripherals using the bandgap. If a peripheral requests
//! static mode, then the bandgap mode will be static for all modules, whereas
//! if all of the peripherals using the bandgap request sample mode, then that
//! will be the mode returned. Sample mode allows the bandgap to be active only
//! when necessary to save on power consumption, static mode requires the
//! bandgap to be active until no peripherals are using it anymore.
//!
//! \return The bandgap mode of the REF module:
//!        REF_STATICMODE if the bandgap is operating in static mode
//!        REF_SAMPLEMODE if the bandgap is operating in sample mode
//
//*****************************************************************************
unsigned short REF_getBandgapMode (unsigned int baseAddress)
{
    if (HWREGB((baseAddress) + OFS_REFCTL0_H) & BGMODE){
        return ( REF_SAMPLEMODE) ;
    } else   {
        return ( REF_STATICMODE) ;
    }
}

//*****************************************************************************
//
//! Returns the active status of the bandgap in the REF module.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to return the active status of the bandgap in the REF
//! module. If the bandgap is in use by a peripheral, then the status will be
//! seen as active.
//!
//! \return The bandgap active status of the REF module:
//!        REF_INACTIVE if the bandgap is not being used at the time of query
//!        REF_ACTIVE if the bandgap is being used at the time of query
//
//*****************************************************************************
unsigned short  REF_isBandgapActive (unsigned int baseAddress)
{
    if (HWREGB((baseAddress) + OFS_REFCTL0_H) & REFBGACT){
        return ( REF_ACTIVE) ;
    } else   {
        return ( REF_INACTIVE) ;
    }
}

//*****************************************************************************
//
//! Returns the busy status of the reference generator in the REF module.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to return the busy status of the reference generator
//! in the REF module. If the ref. generator is in use by a peripheral, then the
//! status will be seen as busy.
//!
//! \return The reference generator busy status of the REF module:
//!        REF_NOTBUSY if the reference generator is not being used
//!        REF_BUSY if the reference generator is being used, disallowing any
//!                  changes to be made to the REF module controls
//
//*****************************************************************************
unsigned short REF_isRefGenBusy (unsigned int baseAddress)
{
    if (HWREGB((baseAddress) + OFS_REFCTL0_H) & REFGENBUSY){
        return ( REF_BUSY) ;
    } else   {
        return ( REF_NOTBUSY) ;
    }
}

//*****************************************************************************
//
//! Returns the active status of the reference generator in the REF module.
//!
//! \param baseAddress is the base address of the REF module.
//!
//! This function is used to return the active status of the reference generator
//! in the REF module. If the ref. generator is on and ready to use, then the
//! status will be seen as active.
//!
//! \return The reference generator active status of the REF module:
//!        REF_INACTIVE if the ref. generator is off and not operating
//!        REF_ACTIVE if the ref. generator is on and ready to be used
//
//*****************************************************************************
unsigned short REF_isRefGenActive (unsigned int baseAddress)
{
    if (HWREGB((baseAddress) + OFS_REFCTL0_H) & REFGENACT){
        return ( REF_ACTIVE) ;
    } else   {
        return ( REF_INACTIVE) ;
    }
}

//*****************************************************************************
//
//Close the Doxygen group.
//! @}
//
//*****************************************************************************
