//*****************************************************************************
//
//sfr.c - Driver for the SFR Module.
//
//*****************************************************************************
#include "sfr.h"
#include "inc/hw_types.h"
#ifdef  __IAR_SYSTEMS_ICC__
#include "deprecated/IAR/msp430xgeneric.h"
#else
#include "deprecated/CCS/msp430xgeneric.h"
#endif
#include "debug.h"

//*****************************************************************************
//
//! Enables selected SFR interrupt sources.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param interruptMask is the bit mask of interrupts that will be enabled.
//!        Mask Value is the logical OR of any of the following
//!        \b SFR_JTAG_OUTBOX_INTERRUPT - JTAG outbox interrupt enable
//!        \b SFR_JTAG_INBOX_INTERRUPT- JTAG inbox interrupt enable
//!        \b SFR_NMI_PIN_INTERRUPT - NMI pin interrupt enable, 
//!               if NMI function is chosen
//!        \b SFR_VACANT_MEMORY_ACCESS_INTERRUPT - Vacant memory access 
//!               interrupt enable
//!        \b SFR_OSCILLATOR_FAULT_INTERRUPT - Oscillator fault interrupt 
//!               enable
//!        \b SFR_WATCHDOG_INTERVAL_TIMER_INTERRUPT - Watchdog interval 
//!               timer interrupt enable
//!
//! This function enables the selected SFR interrupt sources.  Only the sources
//! that are enabled can be reflected to the processor interrupt; disabled
//! sources have no effect on the processor.
//!
//! \return None
//
//*****************************************************************************
void SFR_enableInterrupt (unsigned int baseAddress,
    unsigned char interruptMask)
{
    HWREGB(baseAddress + OFS_SFRIFG1_L) &= ~(interruptMask);
    HWREGB(baseAddress + OFS_SFRIE1_L) |= interruptMask;
}

//*****************************************************************************
//
//! Disables selected SFR interrupt sources.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param interruptMask is the bit mask of interrupts that will be disabled.
//!        Mask Value is the logical OR of any of the following
//!        \b SFR_JTAG_OUTBOX_INTERRUPT - JTAG outbox interrupt enable
//!        \b SFR_JTAG_INBOX_INTERRUPT- JTAG inbox interrupt enable
//!        \b SFR_NMI_PIN_INTERRUPT - NMI pin interrupt enable, 
//!               if NMI function is chosen
//!        \b SFR_VACANT_MEMORY_ACCESS_INTERRUPT - Vacant memory access 
//!               interrupt enable
//!        \b SFR_OSCILLATOR_FAULT_INTERRUPT - Oscillator fault interrupt 
//!               enable
//!        \b SFR_WATCHDOG_INTERVAL_TIMER_INTERRUPT - Watchdog interval 
//!               timer interrupt enable
//!
//! This function disables the selected SFR interrupt sources.  Only the sources
//! that are enabled can be reflected to the processor interrupt; disabled
//! sources have no effect on the processor.
//!
//! \return None
//
//*****************************************************************************
void SFR_disableInterrupt (unsigned int baseAddress,
    unsigned char interruptMask)
{
    HWREGB(baseAddress + OFS_SFRIE1_L) &= ~(interruptMask);
}

//*****************************************************************************
//
//! Returns the status of the selected SFR interrupt flags.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param interruptFlagMask is the bit mask of interrupt flags that the status
//!       of should be returned.
//!        Mask Value is the logical OR of any of the following
//!        \b SFR_JTAG_OUTBOX_INTERRUPT - JTAG outbox interrupt flag
//!        \b SFR_JTAG_INBOX_INTERRUPT- JTAG inbox interrupt flag
//!        \b SFR_NMI_PIN_INTERRUPT - NMI pin interrupt flag
//!        \b SFR_VACANT_MEMORY_ACCESS_INTERRUPT - Vacant memory access 
//!               interrupt flag
//!        \b SFR_OSCILLATOR_FAULT_INTERRUPT - Oscillator fault interrupt 
//!               flag
//!        \b SFR_WATCHDOG_INTERVAL_TIMER_INTERRUPT - Watchdog interval 
//!               timer interrupt flag
//!
//! This function returns the status of the selected SFR interrupt flags in a
//! bit mask format matching that passed into the interruptFlagMask parameter.
//!
//! \return A bit mask of the status of the selected interrupt flags.
//
//*****************************************************************************
unsigned char SFR_getInterruptStatus (unsigned int baseAddress,
    unsigned char interruptFlagMask)
{
    return ( HWREGB(baseAddress + OFS_SFRIFG1_L) & interruptFlagMask );
}

//*****************************************************************************
//
//! Clears the selected SFR interrupt flags.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param interruptFlagMask is the bit mask of interrupt flags that will be
//!       cleared.
//!        Mask Value is the logical OR of any of the following
//!        \b SFR_JTAG_OUTBOX_INTERRUPT - JTAG outbox interrupt flag
//!        \b SFR_JTAG_INBOX_INTERRUPT- JTAG inbox interrupt flag
//!        \b SFR_NMI_PIN_INTERRUPT - NMI pin interrupt flag
//!        \b SFR_VACANT_MEMORY_ACCESS_INTERRUPT - Vacant memory access 
//!               interrupt flag
//!        \b SFR_OSCILLATOR_FAULT_INTERRUPT - Oscillator fault interrupt 
//!               flag
//!        \b SFR_WATCHDOG_INTERVAL_TIMER_INTERRUPT - Watchdog interval 
//!               timer interrupt flag
//!
//!
//! This function clears the status of the selected SFR interrupt flags.
//!
//! \return None
//
//*****************************************************************************
void SFR_clearInterrupt (unsigned int baseAddress,
    unsigned char interruptFlagMask)
{
    HWREGB(baseAddress + OFS_SFRIFG1_L) &= ~(interruptFlagMask);
}

//*****************************************************************************
//
//! Sets the pull-up/down resistor on the ~RST/NMI pin.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param pullResistorSetup is the selection of how the pull-up/down resistor
//!       on the ~RST/NMI pin should be setup or disabled.
//!        Valid values are
//!        \b SFR_RESISTORDISABLE
//!        \b SFR_RESISTORENABLE_PULLUP [Default]
//!        \b SFR_RESISTORENABLE_PULLDOWN
//!        Modified bits are \b SYSRSTRE and \b SYSRSTUP of \b SFRRPCR register.
//!
//! This function sets the pull-up/down resistors on the ~RST/NMI pin to the
//! settings from the pullResistorSetup parameter.
//!
//! \return None
//
//*****************************************************************************
void SFR_setResetPinPullResistor (unsigned int baseAddress,
    unsigned int pullResistorSetup)
{
    HWREGB(baseAddress + OFS_SFRRPCR_L) &= ~(SYSRSTRE + SYSRSTUP);
    HWREGB(baseAddress + OFS_SFRRPCR_L) |= pullResistorSetup;
}

//*****************************************************************************
//
//! Sets the edge direction that will assert an NMI from a signal on the
//! ~RST/NMI pin if NMI function is active.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param edgeDirection is the direction that the signal on the ~RST/NMI pin
//!       should go to signal an interrupt, if enabled.
//!        Valid values are
//!        \b SFR_NMI_RISINGEDGE [Default]
//!        \b SFR_NMI_FALLINGEDGE
//!        Modified bits are \b SYSNMIIES of \b SFRRPCR register.
//!
//! This function sets the edge direction that will assert an NMI from a signal
//! on the ~RST/NMI pin if the NMI function is active. To activate the NMI
//! function of the ~RST/NMI use the SFR_setResetNMIPinFunction() passing
//! SFR_RESETPINFUNC_NMI into the resetPinFunction parameter.
//!
//! \return None
//
//*****************************************************************************
void SFR_setNMIEdge (unsigned int baseAddress,
    unsigned int edgeDirection)
{
    HWREGB(baseAddress + OFS_SFRRPCR_L) &= ~(SYSNMIIES);
    HWREGB(baseAddress + OFS_SFRRPCR_L) |= edgeDirection;
}

//*****************************************************************************
//
//! Sets the function of the ~RST/NMI pin.
//!
//! \param baseAddress is the Base Address of the SFR Module.
//! \param resetPinFunction is the function that the ~RST/NMI pin should take on.
//!        Valid values are
//!        \b SFR_RESETPINFUNC_RESET [Default]
//!        \b SFR_RESETPINFUNC_NMI
//!        Modified bits are \b SYSNMI of \b SFRRPCR register.
//!
//! This function sets the functionality of the ~RST/NMI pin, whether in reset
//! mode which will assert a reset if a low signal is observed on that pin, or
//! an NMI which will assert an interrupt from an edge of the signal dependent
//! on the setting of the edgeDirection parameter in SFR_setNMIEdge().
//!
//! \return None
//
//*****************************************************************************
void SFR_setResetNMIPinFunction (unsigned int baseAddress,
    unsigned char resetPinFunction)
{
    HWREGB(baseAddress + OFS_SFRRPCR_L) &= ~(SYSNMI);
    HWREGB(baseAddress + OFS_SFRRPCR_L) |= resetPinFunction;
}
//*****************************************************************************
//
//Close the Doxygen group.
//! @}
//
//*****************************************************************************

