// *************************************************************************************************
//
//
// *************************************************************************************************
// Battery voltage measurement functions.
// *************************************************************************************************

// *************************************************************************************************
// Include section

// system
#include "project.h"

// driver
#include "display.h"
#include "ports.h"
#include "adc12.h"

// logic
#include "menu.h"
#include "battery.h"

// *************************************************************************************************
// Prototypes section
void reset_batt_measurement(void);
void battery_measurement(void);

// *************************************************************************************************
// Defines section

// *************************************************************************************************
// Global Variable section
struct batt sBatt;

// *************************************************************************************************
// Extern section

// *************************************************************************************************
// @fn          reset_batt_measurement
// @brief       Reset battery measurement module.
// @param       none
// @return      none
// *************************************************************************************************
void reset_batt_measurement(void)
{
    // Reset lobatt display counter
    sBatt.lobatt_display = BATTERY_LOW_MESSAGE_CYCLE;

    // Start with battery voltage of 3.00V
    sBatt.voltage = 300;
}

// *************************************************************************************************
// @fn          battery_measurement
// @brief       Init ADC12. Do single conversion of AVCC voltage. Turn off ADC12.
// @param       none
// @return      none
// *************************************************************************************************
void battery_measurement(void)
{
    u16 voltage;

    // Convert external battery voltage (ADC12INCH_11=AVCC-AVSS/2)
    voltage = adc12_single_conversion(REFVSEL_1, ADC12SHT0_10, ADC12INCH_11);

    // Convert ADC value to "x.xx V"
    // Ideally we have A11=0->AVCC=0V ... A11=4095(2^12-1)->AVCC=4V
    // --> (A11/4095)*4V=AVCC --> AVCC=(A11*4)/4095
    voltage = (voltage * 2 * 2) / 41;

    // Correct measured voltage with calibration value
    voltage += sBatt.offset;

    // Discard values that are clearly outside the measurement range
    if (voltage > BATTERY_HIGH_THRESHOLD)
    {
        voltage = sBatt.voltage;
    }

    // Filter battery voltage
    sBatt.voltage = ((voltage * 2) + (sBatt.voltage * 8)) / 10;

    // If battery voltage falls below low battery threshold, set system flag and modify LINE2
    // display function pointer
    if (sBatt.voltage < BATTERY_LOW_THRESHOLD)
    {
        sys.flag.low_battery = 1;

        // Set sticky battery icon
        display_symbol(LCD_SYMB_BATTERY, SEG_ON);
    }
    else
    {
        sys.flag.low_battery = 0;

        // Clear sticky battery icon
        display_symbol(LCD_SYMB_BATTERY, SEG_OFF);
    }
    // Update LINE2
    display.flag.line2_full_update = 1;

    // Indicate to display function that new value is available
    display.flag.update_battery_voltage = 1;
}

