// *************************************************************************************************
//
//
// *************************************************************************************************
// Time functions.
// *************************************************************************************************

// *************************************************************************************************
// Include section

// system
#include "project.h"

// driver
#include "ports.h"
#include "display.h"
#include "timer.h"

// logic
#include "menu.h"
#include "clock.h"
#include "user.h"
#include "bluerobin.h"
#include "date.h"

// *************************************************************************************************
// Prototypes section
void reset_clock(void);
void clock_tick(void);
void mx_time(u8 line);
void sx_time(u8 line);

void calc_24H_to_12H(u8 * hours, u8 * timeAM);
void conv_24H_to_12H(u8 * hours24, u8 * hours12, u8 * timeAMorPM);

// *************************************************************************************************
// Defines section

// *************************************************************************************************
// Global Variable section
struct time sTime;

// *************************************************************************************************
// Extern section

// *************************************************************************************************
// @fn          reset_clock
// @brief       Resets clock time to 00:00:00, 24H time format.
// @param       none
// @return      none
// *************************************************************************************************
void reset_clock(void)
{
    // Set global system time to 0
    sTime.system_time = 0;

    // Set main 24H time to start value
    sTime.hour = 4;
    sTime.minute = 30;
    sTime.second = 0;

    // Display style of both lines is default (HH:MM)
    sTime.line1ViewStyle = DISPLAY_DEFAULT_VIEW;

    // Reset timeout detection
    sTime.last_activity = 0;
}

// *************************************************************************************************
// @fn          clock_tick
// @brief       Add 1 second to system time and to display time
// @param       none
// @return      none
// *************************************************************************************************
void clock_tick(void)
{
    // Use sTime.drawFlag to minimize display updates
    // sTime.drawFlag = 1: second
    // sTime.drawFlag = 2: minute, second
    // sTime.drawFlag = 3: hour, minute
    sTime.drawFlag = 1;

    // Increase global system time
    sTime.system_time++;

    // Add 1 second
    sTime.second++;

    // Add 1 minute
    if (sTime.second == 60)
    {
        sTime.second = 0;
        sTime.minute++;
        sTime.drawFlag++;

        // Add 1 hour
        if (sTime.minute == 60)
        {
            sTime.minute = 0;
            sTime.hour++;
            sTime.drawFlag++;

            // Add 1 day
            if (sTime.hour == 24)
            {
                sTime.hour = 0;
                add_day();
            }
        }
    }
}

// *************************************************************************************************
// @fn          convert_hour_to_12H_format
// @brief       Convert internal 24H time to 12H time.
// @param       u8 hour         Hour in 24H format
// @return      u8                      Hour in 12H format
// *************************************************************************************************
u8 convert_hour_to_12H_format(u8 hour)
{
    // 00:00 .. 11:59 --> AM 12:00 .. 11:59
    if (hour == 0)
        return (hour + 12);
    else if (hour <= 12)
        return (hour);
    // 13:00 .. 23:59 --> PM 01:00 .. 11:59
    else
        return (hour - 12);
}

// *************************************************************************************************
// @fn          is_hour_am
// @brief       Checks if internal 24H time is AM or PM
// @param       u8 hour         Hour in 24H format
// @return      u8                      1 = AM, 0 = PM
// *************************************************************************************************
u8 is_hour_am(u8 hour)
{
    // 00:00 .. 11:59 --> AM 12:00 .. 11:59
    if (hour < 12)
        return (1);
    // 12:00 .. 23:59 --> PM 12:00 .. 11:59
    else
        return (0);
}

// *************************************************************************************************
// @fn          sx_time
// @brief       Time user routine. Toggles view style between HH:MM and SS.
// @param       line            LINE1
// @return      none
// *************************************************************************************************
void sx_time(u8 line)
{
    // Toggle display view style
    if (sTime.line1ViewStyle == DISPLAY_DEFAULT_VIEW)
        sTime.line1ViewStyle = DISPLAY_ALTERNATIVE_VIEW;
    else
        sTime.line1ViewStyle = DISPLAY_DEFAULT_VIEW;
}

// *************************************************************************************************
// @fn          display_time
// @brief       Clock display routine. Supports 24H and 12H time format.
// @param       u8 line                 LINE1
//                              u8 update               DISPLAY_LINE_UPDATE_FULL,
// DISPLAY_LINE_UPDATE_PARTIAL
// @return      none
// *************************************************************************************************
void display_time(u8 line, u8 update)
{
    u8 hour12;

    // Partial update
    if (update == DISPLAY_LINE_UPDATE_PARTIAL)
    {
        if (sTime.drawFlag != 0)
        {
            if (sTime.line1ViewStyle == DISPLAY_DEFAULT_VIEW)
            {
                switch (sTime.drawFlag)
                {
                    case 3:
                        if (sys.flag.use_metric_units)
                        {
                            // Display 24H time "HH"
                            display_chars(switch_seg(line, LCD_SEG_L1_3_2,
                                                     LCD_SEG_L2_3_2), int_to_array(sTime.hour, 2,
                                                                                   0), SEG_ON);
                        }
                        else
                        {
                            // Display 12H time "HH" + AM/PM
                            hour12 = convert_hour_to_12H_format(sTime.hour);
                            display_chars(switch_seg(line, LCD_SEG_L1_3_2,
                                                     LCD_SEG_L2_3_2), int_to_array(hour12, 2,
                                                                                   0), SEG_ON);
                            display_am_pm_symbol(sTime.hour);
                        }

                    case 2:
                        display_chars(switch_seg(line, LCD_SEG_L1_1_0,
                                                 LCD_SEG_L2_1_0), int_to_array(sTime.minute, 2,
                                                                               0), SEG_ON);
                }
            }
            else
            {
                // Seconds are always updated
                display_chars(switch_seg(line, LCD_SEG_L1_1_0,
                                         LCD_SEG_L2_1_0), int_to_array(sTime.second, 2, 0), SEG_ON);
            }
        }
    }
    else if (update == DISPLAY_LINE_UPDATE_FULL)
    {
        // Full update
        if (sTime.line1ViewStyle == DISPLAY_DEFAULT_VIEW)
        {
            // Display 24H/12H time
            if (sys.flag.use_metric_units)
            {
                // Display 24H time "HH"
                display_chars(switch_seg(line, LCD_SEG_L1_3_2,
                                         LCD_SEG_L2_3_2), int_to_array(sTime.hour, 2, 0), SEG_ON);
            }
            else
            {
                // Display 12H time "HH" + AM/PM information
                hour12 = convert_hour_to_12H_format(sTime.hour);
                display_chars(switch_seg(line, LCD_SEG_L1_3_2,
                                         LCD_SEG_L2_3_2), int_to_array(hour12, 2, 0), SEG_ON);
                // Display AM/PM information
                if (line == LINE1)
                {
                    display_am_pm_symbol(sTime.hour);
                }
            }

            // Display minute
            display_chars(switch_seg(line, LCD_SEG_L1_1_0,
                                     LCD_SEG_L2_1_0), int_to_array(sTime.minute, 2, 0), SEG_ON);
            display_symbol(switch_seg(line, LCD_SEG_L1_COL, LCD_SEG_L2_COL0), SEG_ON_BLINK_ON);
        }
        else
        {
            // Display seconds
            display_chars(switch_seg(line, LCD_SEG_L1_1_0,
                                     LCD_SEG_L2_1_0), int_to_array(sTime.second, 2, 0), SEG_ON);
            display_symbol(switch_seg(line, LCD_SEG_L1_DP1, LCD_SEG_L2_DP), SEG_ON);
        }
    }
    else if (update == DISPLAY_LINE_CLEAR)
    {
        display_symbol(switch_seg(line, LCD_SEG_L1_COL, LCD_SEG_L2_COL0), SEG_OFF_BLINK_OFF);
        // Change display style to default (HH:MM)
        sTime.line1ViewStyle = DISPLAY_DEFAULT_VIEW;
        // Clean up AM/PM icon
        display_symbol(LCD_SYMB_AM, SEG_OFF);
    }
}

