//******************************************************************************
//  MSP430G2230 Demo - ADC10, Sample A10 Temp, Set P1.2 if Temp ++ ~2C
//
//  Description: Set ADC10 and the integrated temperature sensor to detect
//  temperature gradients. The temperature sensor output voltage is sampled
//  ~ every 120ms and compared with the defined delta values using an ISR.
//  (ADC10OSC/4)/64 determines sample time which needs to be greater than
//  30us for temperature sensor.
//  ADC10 is operated in repeat-single channel mode with the sample and
//  convert trigger sourced from Timer_A CCR1. The ADC10IFG at the end
//  of each converstion will trigger an ISR.
//  ACLK = VLO, MCLK = SMCLK = default DCO ~ 1.2MHz, ADC10CLK = ADC10OSC
//
//                MSP430G2230
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |A10          P1.2|-->LED
//
//  B. Nisarga
//  Texas Instruments, Inc
//  December 2011
//  Built with CCE Version: 5.1 and IAR Embedded Workbench Version: 5.4
//******************************************************************************

#include <msp430.h>

static unsigned int FirstADCVal;            // holds 1st ADC result
#define ADCDeltaOn       3                  // ~ 2 Deg C delta for LED on

int main(void)
{
  WDTCTL = WDTPW | WDTHOLD;                 // Stop watchdog
  
  /* Initialization Code */
  P1REN = 0x1B;                             // Terminate unavailable Port1 pins (P1.0/1/3/4) properly
                                            // Config as Input with pull-down enabled
  BCSCTL3 |= LFXT1S_2;                      // Select VLO as low freq clock     
  /* End Initialization Code */   
  
  ADC10CTL1 = ADC10DIV_3 | INCH_10 | SHS_1 | CONSEQ_2;  // TA trig., rpt, A10
  ADC10CTL0 = SREF_1 | ADC10SHT_3 | REF2_5V | ADC10IE | REFON | ADC10ON;
  __enable_interrupt();                     // Enable interrupts.
  TA0CCR0 = 30;                             // Delay to allow Ref to settle
  TA0CCTL0 |= CCIE;                         // Compare-mode interrupt.
  TA0CTL = TASSEL_2 | MC_1;                 // TACLK = SMCLK, Up mode.
  LPM0;                                     // Wait for delay.
  TA0CCTL0 &= ~CCIE;                        // Disable timer Interrupt
  __disable_interrupt();
  ADC10CTL0 |= ENC;
  TA0CCTL1 = OUTMOD_4;                      // Toggle on EQU1 (TAR = 0)
  TA0CTL = TASSEL_2 | MC_2;                 // SMCLK, cont-mode
  while (!(ADC10IFG & ADC10CTL0));          // First conversion?
  FirstADCVal = ADC10MEM;                   // Read out 1st ADC value
  P1OUT = 0x00;                             // Clear P1
  P1DIR = BIT2;                             // P1.2 as output
  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0 w/ interrupt
}

#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
  if (ADC10MEM >= FirstADCVal + ADCDeltaOn)
    P1OUT |= BIT2;                          // LED on
  else
    P1OUT &= ~BIT2;                         // LED off
}

#pragma vector=TIMERA0_VECTOR
__interrupt void ta0_isr(void)
{
  TA0CTL = 0;
  LPM0_EXIT;                                // Exit LPM0 on return
}
