//******************************************************************************
//  MSP430G2230 Demo - ADC10, DTC Sample A5 32x, 1.5V, Repeat Single, DCO
//
//  Description: Use DTC to sample A5 32 times with reference to internal 1.5v.
//  Vref Software writes to ADC10SC to trigger sample burst. In Mainloop MSP430
//  waits in LPM0 to save power until ADC10 conversion complete, ADC10_ISR(DTC)
//  will force exit from any LPMx in Mainloop on return. ADC10 internal
//  oscillator times sample period (16x) and conversion (13x). DTC transfers
//  conversion code to RAM 200h - 240h. P1.2 set at start of conversion burst,
//  reset on completion.
//
//                MSP430G2230
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//        >---|P1.1/A5      P1.2|-->LED
//
//  B. Nisarga
//  Texas Instruments, Inc
//  December 2011
//  Built with CCE Version: 5.1 and IAR Embedded Workbench Version: 5.4
//******************************************************************************

#include <msp430.h>

int main(void)
{
  WDTCTL = WDTPW | WDTHOLD;                 // Stop WDT
  
  /* Initialization Code */
  P1REN = 0x1B;                             // Terminate unavailable Port1 pins (P1.0/1/3/4) properly
                                            // Config as Input with pull-down enabled
  BCSCTL3 |= LFXT1S_2;                      // Select VLO as low freq clock     
  /* End Initialization Code */ 
  
  ADC10CTL1 = CONSEQ_2+INCH_5;              // Repeat single channel
  ADC10CTL0 = SREF_1 | ADC10SHT_2 | MSC | REFON | ADC10ON | ADC10IE;
  __enable_interrupt();                     // Enable interrupts.
  TA0CCR0 = 30;                             // Delay to allow Ref to settle
  TA0CCTL0 |= CCIE;                         // Compare-mode interrupt.
  TA0CTL = TASSEL_2 | MC_1;                 // TACLK = SMCLK, Up mode.
  LPM0;                                     // Wait for delay.
  TA0CCTL0 &= ~CCIE;                        // Disable timer Interrupt
  __disable_interrupt();
  ADC10DTC1 = 0x20;                         // 32 conversions
  ADC10AE0 |= BIT5;                         // P1.5 ADC option select
  P1DIR |= BIT2;                            // Set P1.2 output

  for (;;)
  {
    ADC10CTL0 &= ~ENC;
    while (ADC10CTL1 & BUSY);               // Wait if ADC10 core is active
    ADC10SA = 0x200;                        // Data buffer start
    P1OUT |= BIT2;                          // Set P1.2 LED on
    ADC10CTL0 |= ENC | ADC10SC;             // Sampling and conversion start
    __bis_SR_register(CPUOFF + GIE);        // LPM0, ADC10_ISR will force exit
    P1OUT &= ~BIT2;                         // Clear P1.2 LED off
  }
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR(void)
{
  __bic_SR_register_on_exit(CPUOFF);        // Clear CPUOFF bit from 0(SR)
}

#pragma vector=TIMERA0_VECTOR
__interrupt void ta0_isr(void)
{
  TA0CTL = 0;
  LPM0_EXIT;                                // Exit LPM0 on return
}
