//******************************************************************************
//  MSP430G2230 Demo - ADC10, Sample A5, 1.5V, TA1 Trig, Set P1.2 if > 0.5V
//
//  Description: A5 is sampled every ACLK/1024 with reference to 1.5V.
//  Timer_A is run in upmode and TA1 is used to automatically trigger
//  ADC10 conversion, TA0 defines the period. Internal oscillator times sample
//  (16x) and conversion (13x). Inside ADC10_ISR if A5 > 0.5Vcc, P1.2 is set,
//  else reset. Normal mode is LPM3.
//  ACLK = VLO (~10kHz), MCLK = SMCLK = default DCO ~ 1.2MHz, ADC10CLK = ADC10OSC
//
//               MSP430G2230
//            -----------------
//        /|\|              XIN|-
//         | |                 | 
//         --|RST          XOUT|-
//           |                 |
//       >---|P1.5/A5     P1.2 |--> LED
//
//  B. Nisarga
//  Texas Instruments, Inc
//  December 2011
//  Built with CCE Version: 5.1 and IAR Embedded Workbench Version: 5.4
//******************************************************************************

#include <msp430.h>

int main(void)
{
  WDTCTL = WDTPW | WDTHOLD;                 // Stop WDT
  
  /* Initialization Code */
  P1REN = 0x1B;                             // Terminate unavailable Port1 pins (P1.0/1/3/4) properly
                                            // Config as Input with pull-down enabled
  BCSCTL3 |= LFXT1S_2;                      // Select VLO as low freq clock     
  /* End Initialization Code */
  
  ADC10CTL1 = SHS_1 | CONSEQ_2 | INCH_5;    // TA1 trigger sample start
  ADC10CTL0 = SREF_1 | ADC10SHT_2 | REFON | ADC10ON | ADC10IE;
  
  // Reference Settling Delay
  __enable_interrupt();                     // Enable interrupts.
  TA0CCR0 = 30;                             // Delay to allow Ref to settle
  TA0CCTL0 |= CCIE;                         // Compare-mode interrupt.
  TA0CTL = TASSEL_2 | MC_1;                 // TACLK = SMCLK, Up mode.
  LPM0;                                     // Wait for delay.
  TA0CCTL0 &= ~CCIE;                        // Disable timer Interrupt
  __disable_interrupt();
  
  ADC10CTL0 |= ENC;                         // ADC10 Enable
  ADC10AE0 |= BIT5;                         // P1.5 ADC10 option select
  P1DIR |= BIT2;                            // Set P1.2 output
  TA0CCR0 = 1024-1;                         // PWM Period
  TA0CCTL1 = OUTMOD_3;                      // TA0CCR1 set/reset
  TA0CCR1 = 1023;                           // TA0CCR1 PWM Duty Cycle
  TA0CTL = TASSEL_1 | MC_1;                 // ACLK, up mode

  __bis_SR_register(LPM3_bits + GIE);       // Enter LPM3 w/ interrupts
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR(void)
{
  if (ADC10MEM < 0x155)                     // ADC10MEM = A5 > 0.5V?
    P1OUT &= ~BIT2;                         // Clear P1.2 LED off
  else
    P1OUT |= BIT2;                          // Set P1.2 LED on
}

#pragma vector=TIMERA0_VECTOR
__interrupt void ta0_isr(void)
{
  TA0CTL = 0;
  LPM0_EXIT;                                // Exit LPM0 on return
}
