//******************************************************************************
//  MSP430G22x0 Demo - Timer_A, Toggle P1.5-6, Cont. Mode ISR, DCO SMCLK
//
//  Description: Use Timer_A TA0CCRx units and overflow to generate four
//  independent timing intervals. For demonstration, TA0CCR0 and TA0CCR1 output
//  units are optionally selected with port pins P1.5 and P1.6 in toggle
//  mode. As such, these pins will toggle when respective TA0CCRx registers match
//  the TAR counter. Interrupts are also enabled with all TA0CCRx units,
//  software loads offset to next interval only - as long as the interval offset
//  is aded to CCRx, toggle rate is generated in hardware. Timer_A overflow ISR
//  is used to toggle P1.2 with software. Proper use of the TAIV interrupt
//  vector generator is demonstrated.
//  ACLK = VLO, MCLK = SMCLK = TACLK = default DCO ~1MHz
//  As coded and assuming ~1MHz DCO, toggle rates are:
//  P1.5 = TA0CCR0 ~ 1MHz/(200) ~5000Hz
//  P1.6 = TA0CCR1 ~ 1MHz/(1000) ~1000Hz
//  P1.2 = overflow ~ 1MHz/(65536) ~15Hz
//
//               MSP430G22x0
//            -----------------
//        /|\|              XIN|-
//         | |                 |
//         --|RST          XOUT|-
//           |                 |
//           |         P1.5/TA0|--> TA0CCR0
//           |         P1.6/TA1|--> TA0CCR1
//           |             P1.2|--> Overflow/software
//
//  B. Nisarga
//  Texas Instruments, Inc
//  December 2011
//  Built with CCE Version: 5.1 and IAR Embedded Workbench Version: 5.4
//******************************************************************************

#include <msp430.h>

int main(void)
{
  WDTCTL = WDTPW | WDTHOLD;                 // Stop watchdog timer
  
  /* Initialization Code */
  P1REN = 0x1B;                             // Terminate unavailable Port1 pins (P1.0/1/3/4) properly
                                            // Config as Input with pull-down enabled
  BCSCTL3 |= LFXT1S_2;                      // Select VLO as low freq clock     
  /* End Initialization Code */ 
  
  P1SEL |= BIT5 | BIT6;                     // P1.5 - P1.6 option select
  P1DIR |= BIT2 | BIT5 | BIT6;              // P1.2, P1.5, P1.6 outputs
  TA0CCTL0 = OUTMOD_4 | CCIE;               // TA0CCR0 toggle, interrupt enabled
  TA0CCTL1 = OUTMOD_4 | CCIE;               // TA0CCR1 toggle, interrupt enabled
  TA0CTL = TASSEL_2 | MC_2 | TAIE;          // SMCLK, Contmode, int enabled

  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0 w/ interrupt
}

// Timer A0 interrupt service routine
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A0 (void)
{
  TA0CCR0 += 200;                              // Add Offset to TA0CCR0
}

// Timer_A2 Interrupt Vector (TAIV) handler
#pragma vector=TIMERA1_VECTOR
__interrupt void Timer_A1(void)
{
  switch( TAIV )
  {
  case  2: TA0CCR1 += 1000;                   // Add Offset to TA0CCR1
           break;
  case 10: P1OUT ^= BIT2;                     // Timer_A3 overflow
           break;
 }
}

