//******************************************************************************
//  MSP430F543xA Demo - USCI_B0 I2C Master TX multiple bytes to MSP430 Slave
//						using a repeated restart.
//
//  Description: This demo connects two MSP430's via the I2C bus. The master
//  transmits to the slave, issues a repeated restart, and then receives data
//  from the slave. This is the MASTER CODE. It continuously transmits an array
//  of data and demonstrates how to implement an I2C master transmitter sending
//  multiple bytes using the USCI_B0 TX interrupt followed by a master receiver 
//  receiving multiple bytes using the USCI_B0 RX Interrupt.
//  ACLK = n/a, MCLK = SMCLK = BRCLK = default DCO = ~1.045MHz
//
//                                /|\  /|\
//                MSP430F5438A     10k  10k      MSP430F5438A
//                   slave         |    |         master
//             -----------------   |    |   -----------------
//           -|XIN  P3.1/UCB0SDA|<-|----+->|P3.1/UCB0SDA  XIN|-
//            |                 |  |       |                 |
//           -|XOUT             |  |       |             XOUT|-
//            |     P3.2/UCB0SCL|<-+------>|P3.2/UCB0SCL     |
//            |                 |          |                 |
//
//   P. Liska
//   Texas Instruments Inc.
//   June 2012
//   Built with CCS Version: 5.1.0
//******************************************************************************

#include <msp430.h>

unsigned char *PTxData;                     // Pointer to TX data
unsigned char TXByteCtr;

const unsigned char TxData[] =              // Table of data to transmit
{
  0x11,
  0x22,
  0x33,
  0x44,
  0x55
};

unsigned char *PRxData;                     // Pointer to RX data
unsigned char RXByteCtr;
volatile unsigned char RxBuffer[128];       // Allocate 128 byte of RAM

int main(void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  P3SEL |= 0x06;                            // Assign I2C pins to USCI_B0
  UCB0CTL1 |= UCSWRST;                      // Enable SW reset
  UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
  UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
  UCB0BR0 = 12;                             // fSCL = SMCLK/12 = ~100kHz
  UCB0BR1 = 0;
  UCB0I2CSA = 0x48;                         // Slave Address is 048h
  UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
  UCB0IE |= UCTXIE + UCRXIE;                // Enable TX and RX interrupt

  PTxData = (unsigned char *)TxData;      // TX array start address
                                          // Place breakpoint here to see each
                                          // transmit operation.
  TXByteCtr = sizeof TxData;              // Load TX byte counter

  PRxData = (unsigned char *)RxBuffer;    // Start of RX buffer
  RXByteCtr = 5;                          // Load RX byte counter

  UCB0CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition

  while (1)
  {
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0, enable interrupts
  }
}

//------------------------------------------------------------------------------
// The USCIAB0TX_ISR is structured such that it can be used to transmit any
// number of bytes by pre-loading TXByteCtr with the byte count. Also, TXData
// points to the next byte to transmit.
//------------------------------------------------------------------------------
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
{
  switch(__even_in_range(UCB0IV,12))
  {
  case  0: break;                           // Vector  0: No interrupts
  case  2: break;                           // Vector  2: ALIFG
  case  4: break;                           // Vector  4: NACKIFG
  case  6: break;   		                // Vector  6: STTIFG
  case  8: break;                           // Vector  8: STPIFG
  case 10:                                  // Vector 10: RXIFG
    RXByteCtr--;                            // Decrement RX byte counter
    if (RXByteCtr)
    {
      *PRxData++ = UCB0RXBUF;               // Move RX data to address PRxData
    }
    else
    {
      *PRxData = UCB0RXBUF;                 // Move final RX data to PRxData
      UCB0CTL1 |= UCTR + UCTXSTT;           // I2C TX, start condition
      PRxData = (unsigned char *)RxBuffer;    // Start of RX buffer
      RXByteCtr = 5;                          // Load RX byte counter
    }
    break;
  case 12:                                  // Vector 12: TXIFG
    if (TXByteCtr)                          // Check TX byte counter
    {
      UCB0TXBUF = *PTxData++;               // Load TX buffer
      TXByteCtr--;                          // Decrement TX byte counter
    }
    else
    {
        PTxData = (unsigned char *)TxData;  // TX array start address
                                            // Place breakpoint here to see each
                                            // transmit operation.
        TXByteCtr = sizeof TxData;          // Load TX byte counter

        UCB0CTL1 &= ~UCTR;         			// I2C RX
        UCB0CTL1 |= UCTXSTT;         		// I2C start condition

        UCB0TXBUF = *PTxData++;             // Load TX buffer
        TXByteCtr--;                        // Decrement TX byte counter
    }
  default: break;
  }
}
