//******************************************************************************
//   MSP430F66x Demo - CRC16, Compare CRC output with software-based algorithm
//
//   Description: An array of 16 random 16-bit values are moved through the CRC
//   module, as well as a software-based CRC-CCIT-BR algorithm. Due to the fact 
//   that the software-based algorithm handles 8-bit inputs only, the 16-bit 
//   words are broken into 2 8-bit words before being run through (lower byte 
//   first). The outputs of both methods are then compared to ensure that the 
//   operation of the CRC module is consistent with the expected outcome. If 
//   the values of each output are equal, set P1.0, else reset.
//
//   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//
//                MSP430F66xx
//             -----------------
//         /|\|                 |
//          | |                 |
//          --|RST              |
//            |                 |
//            |             P1.0|--> LED
//
//   Tyler Witt
//   Texas Instruments Inc.
//   October 2011
//   Built with IAR Embedded Workbench V6.0 & Code Composer Studio V4.2
//******************************************************************************

#include <msp430.h>
unsigned int CRC_Init = 0xFFFF;
unsigned int i;
unsigned int CRC_Input[]= {0x0fc0, 0x1096, 0x5042, 0x0010,  // 16 random 16-bit numbers
	                       0x7ff7, 0xf86a, 0xb58e, 0x7651,  // these numbers can be 
	                       0x8b88, 0x0679, 0x0123, 0x9599,  // modified if desired
	                       0xc58c, 0xd1e2, 0xe144, 0xb691 };
unsigned int CRC_Results;
unsigned int SW_Results;
unsigned int CRC_New;

// Software Algorithm Function Definition
unsigned int CCITT_Update(unsigned int, unsigned int);

int main (void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  P1DIR |= BIT0;						    // P1.0 Output
  P1OUT &= ~BIT0;							// Clear LED to start

  // Init CRC
  CRCINIRES = CRC_Init;                     // Init CRC with 0xFFFF

  for(i=0;i<16;i++)
  {
    // Input random values into CRC Hardware
    CRCDIRB = CRC_Input[i];                  // Input data in CRC
    __no_operation();
  }
  CRC_Results = CRCINIRES;				    // Save results (per CRC-CCITT standard)
  
  for(i=0;i<16;i++)
  {
    // Input values into Software algorithm (requires 8-bit inputs)
    unsigned int LowByte = (CRC_Input[i] & 0x00FF); // Clear upper 8 bits to get lower byte
    unsigned int UpByte = (CRC_Input[i] >> 8); // Shift right 8 bits to get upper byte
    // First input lower byte
    if(i==0)
      CRC_New = CCITT_Update(CRC_Init,LowByte);
    else
      CRC_New = CCITT_Update(CRC_New,LowByte);
    // Then input upper byte
    CRC_New = CCITT_Update(CRC_New,UpByte);
  }
  SW_Results = CRC_New;
  
  // Compare data output results
  if(CRC_Results==SW_Results)               // if data is equal
    P1OUT |= BIT0;							// set the LED
  else
    P1OUT &= ~BIT0;							// if not, clear LED
  
  while(1);                                 // infinite loop
}

// Software algorithm - CCITT CRC16 code
unsigned int CCITT_Update(unsigned int init, unsigned int input)
{
  unsigned int CCITT;
  CCITT =(unsigned char)(init >> 8)|(init << 8);
  CCITT ^= input;
  CCITT ^= (unsigned char)(CCITT & 0xFF) >> 4;
  CCITT ^= (CCITT << 8) << 4;
  CCITT ^= ((CCITT & 0xFF) << 4) << 1;
  return CCITT;
}
