//******************************************************************************
//   MSP430F66x Demo - CRC16, fed by DMA, compare w/ software algorithm
//
//   Description: DMA0 is used to transfer a block of 16 16-bit values word-
//   by-word as a repeating block to CRCDIRB. The CRC16 output is then saved,
//   and a similar operation is done to feed an array, that will then be run 
//   through the software loop. The outputs of both methods are then compared 
//   to ensure that the operation of the CRC module is consistent with the 
//   expected outcome. If the values of each output are equal, set P1.0, else 
//   reset.
//
//  ** RAM location 0x3400 - 0x341F used - make sure no compiler conflict **
//   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//  Use large memory model
//
//                MSP430F66xx
//             -----------------
//         /|\|                 |
//          | |                 |
//          --|RST              |
//            |                 |
//            |             P1.0|--> LED
//
//   Tyler Witt
//   Texas Instruments Inc.
//   October 2011
//   Built with IAR Embedded Workbench V6.0 & Code Composer Studio V4.2
//******************************************************************************

#include <msp430.h>
unsigned int CRC_Init = 0xFFFF;
unsigned int CRC_Input[16];
unsigned int i;
unsigned int CRC_Results;
unsigned int SW_Results;
unsigned int CRC_New;

// Software Algorithm Function Definition
unsigned int CCITT_Update(unsigned int, unsigned int);

int main (void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  P1DIR |= BIT0;						    // P1.0 Output
  P1OUT &= ~BIT0;							// Clear LED to start

  // Init CRC
  CRCINIRES = CRC_Init;                     // Init CRC with 0xFFFF

  // Use DMA to feed CRC
  __data16_write_addr((unsigned short) &DMA0SA,(unsigned long) 0x3400);
                                            // Source block address
  __data16_write_addr((unsigned short) &DMA0DA,(unsigned long) &CRCDIRB);
                                            // Destination single address --> CRCDIRB
  DMA0SZ = 16;                              // Block size
  DMA0CTL = DMADT_5+DMASRCINCR_3+DMADSTINCR_0; // rpt, inc source only
  DMA0CTL |= DMAEN;                         // Enable DMA0
  DMA0CTL |= DMAREQ;						// Trigger block transfer
  __no_operation();
  
  CRC_Results = CRCINIRES;				    // Save results (per CRC-CCITT standard)

  // Use DMA to fill an array to run through the software algorithm
  __data16_write_addr((unsigned short) &DMA0SA,(unsigned long) 0x3400);
                                            // Source block address
  __data16_write_addr((unsigned short) &DMA0DA,(unsigned long) &CRC_Input);
                                            // Destination single address --> CRCDIRB
  DMA0SZ = 16;                              // Block size
  DMA0CTL = DMADT_5+DMASRCINCR_3+DMADSTINCR_3; // rpt, inc
  DMA0CTL |= DMAEN;                         // Enable DMA0
  DMA0CTL |= DMAREQ;						// Trigger block transfer
  __no_operation();
  
  
  for(i=0;i<16;i++)
  {
    // Input values into Software algorithm (requires 8-bit inputs)
    unsigned int LowByte = (CRC_Input[i] & 0x00FF); // Clear upper 8 bits to get lower byte
    unsigned int UpByte = (CRC_Input[i] >> 8); // Shift right 8 bits to get upper byte
    // First input lower byte
    if(i==0)
      CRC_New = CCITT_Update(CRC_Init,LowByte);
    else
      CRC_New = CCITT_Update(CRC_New,LowByte);
    // Then input upper byte
    CRC_New = CCITT_Update(CRC_New,UpByte);
  }
  SW_Results = CRC_New;
  
  // Compare data output results
  if(CRC_Results==SW_Results)               // if data is equal
    P1OUT |= BIT0;							// set the LED
  else
    P1OUT &= ~BIT0;							// if not, clear LED
  
  while(1);                                 // infinite loop
}

// Software algorithm - CCITT CRC16 code
unsigned int CCITT_Update(unsigned int init, unsigned int input)
{
  unsigned int CCITT;
  CCITT =(unsigned char)(init >> 8)|(init << 8);
  CCITT ^= input;
  CCITT ^= (unsigned char)(CCITT & 0xFF) >> 4;
  CCITT ^= (CCITT << 8) << 4;
  CCITT ^= ((CCITT & 0xFF) << 4) << 1;
  return CCITT;
}
