//******************************************************************************
//   MSP430F66x Demo - RTC_B in real time clock mode
//
//   Description: The RTC_B module is used to set the time, start RTC operation,
//   and read the time from the respective RTC registers. Software will set the
//   original time to 11:59:45 am on Friday October 7, 2011. Then the RTC will
//   be activated through software, and the time will be read out once every 
//   second. The proper handling of the RTC interrupt is shown as well.
//
//  //* An external watch crystal on XIN XOUT is required for ACLK *//	
//   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//
//                MSP430F66xx
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|--> LED
//
//   Tyler Witt
//   Texas Instruments Inc.
//   October 2011
//   Built with IAR Embedded Workbench V6.0 & Code Composer Studio V4.2
//******************************************************************************

#include <msp430.h>

unsigned int Seconds;
unsigned int Minutes;
unsigned int Hours;

int main (void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  
  while(BAKCTL & LOCKIO)                    // Unlock XT1 pins for operation
     BAKCTL &= ~(LOCKIO);
  
  UCSCTL6 &= ~(XT1OFF);                     // XT1 On
  UCSCTL6 |= XCAP_3;                        // Internal load cap
  
  // Loop until XT1,XT2 & DCO stabilizes - In this case loop until XT1 and DCo settle
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

  P1DIR |= BIT0;						    // P1.0 Output
  P1OUT &= ~BIT0;                           // Clear LED to start

  // Configure RTC_B
  RTCCTL01 |= RTCRDYIE + RTCBCD + RTCHOLD;  // BCD mode, RTC hold, enable RTC read ready interrupt

  RTCYEAR = 0x2011;                         // Year = 0x2011
  RTCMON = 0x10;                            // Month = 0x10 = October
  RTCDAY = 0x07;                            // Day = 0x07 = 7th
  RTCDOW = 0x05;                            // Day of week = 0x05 = Friday
  RTCHOUR = 0x11;                           // Hour = 0x11
  RTCMIN = 0x59;                            // Minute = 0x59
  RTCSEC = 0x45;                            // Seconds = 0x45

  RTCCTL01 &= ~(RTCHOLD);                   // Start RTC calendar mode

  __bis_SR_register(LPM3_bits + GIE);       // Enter LPM3 mode with interrupts
                                            // enabled
  __no_operation();

}

#pragma vector=RTC_VECTOR
__interrupt void RTCISR (void)
{
  switch(__even_in_range(RTCIV,14))
  {
  case  0: break;                           // Vector  0:  No interrupt
  case  2:                                  // Vector  2:  RTCRDYIFG
    P1OUT ^= BIT0;                          // Toggle LED every second
    Seconds = RTCSEC;                       // Read all associated time registers
    Minutes = RTCMIN;
    Hours = RTCHOUR;
    break;
  case  4: break;                           // Vector  4:  RTCEVIFG
  case  6: break;                           // Vector  6:  RTCAIFG
  case  8: break;                           // Vector  8:  RT0PSIFG
  case 10: break;                           // Vector 10:  RT1PSIFG
  case 12: break;                           // Vector 12:  RTCOFIFG
  case 14: break;                           // Vector 14:  Reserved
  default: break;
  }
}
