//******************************************************************************
//   MSP430F66x Demo - RTC_B, LPM3.5, & alarm
//
//   Description: The RTC_B module is used to set the time, start RTC operation,
//   and read the time from the respective RTC registers. Software will set the
//   original time to 11:59:45 am on Friday October 7, 2011. Then the RTC will
//   be activated through software, and an alarm will be created for the next 
//   minute (12:00:00 pm). The device will then enter LPM3.5 awaiting
//   the event interrupt. Upon being woken up by the event, the LED on the board
//   will be set. 
//
//   NOTE: This code example was created and tested using the MSP-TS430PZ100USB
//   Rev1.2 board. To ensure that LPM3.5 is entered properly, upon opening the 
//   debug window either hit the "Run Free" button on the dropdown menu next to 
//   the run button if you are using CCS, or make sure to check that the 
//   "Release JTAG on GO" option is selected if you are using IAR. 
//   ~1.13uA should be measured on JP1. 
//
//  //* An external watch crystal on XIN XOUT is required for ACLK *//	
//   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//
//                MSP430F66xx
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|--> LED
//
//   Tyler Witt
//   Texas Instruments Inc.
//   October 2011
//   Built with IAR Embedded Workbench V6.0 & Code Composer Studio V4.2
//******************************************************************************

#include <msp430.h>

void Board_Init(void);
void RTC_Init(void);
void EnterLPM35(void);
void WakeUpLPM35(void);

int main (void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
   
  if(SYSRSTIV == 0x08) 
  {
    // When woken up from LPM3.5, reinit
    WakeUpLPM35();
    // If woken up at noon, set LED
    if((RTCHOUR == 0x12)&(RTCMIN == 0x00)&(RTCSEC == 0x00))
    {
      while(1)
        P1OUT |= BIT0;
    }
    // If values do not match, blink LED fast
    else
    {
      while(1)                                  // continuous loop
      {
        P1OUT ^= BIT0;                          // XOR P1.0
        __delay_cycles(20000);                  // Delay
      }
    }
  }
  
  // Init board & RTC, then enter LPM3.5
  Board_Init();
  RTC_Init();
  EnterLPM35();
  
  // Code should not get here
  // Blink LED slowly if LPM3.5 not entered properly
  WakeUpLPM35();
  while(1)
  {
    P1OUT ^= BIT0;                          // XOR P1.0
    __delay_cycles(200000);                 // Delay
  }
}

#pragma vector=RTC_VECTOR
__interrupt void RTCISR (void)
{
  PMMCTL0_H = PMMPW_H;                      // open PMM
  PM5CTL0 &= ~LOCKIO;                       // Clear LOCKBAK and enable ports
  PMMCTL0_H = 0x00;                         // close PMM

  RTCCTL0 &= ~RTCTEVIFG;
  __bic_SR_register_on_exit(LPM4_bits);     // Exit LPM4.5
  __no_operation();
}

void Board_Init(void)
{
  // Port Configuration
  P1OUT = 0x00;P2OUT = 0x00;P3OUT = 0x00;P4OUT = 0x00;P5OUT = 0x00;P6OUT = 0x00;
  P7OUT = 0x00;P8OUT = 0x00;P9OUT = 0x00;PJOUT = 0x00;
  P1DIR = 0xFF;P2DIR = 0xFF;P3DIR = 0xFF;P4DIR = 0xFF;P5DIR = 0xFF;P6DIR = 0xFF;
  P7DIR = 0xFF;P8DIR = 0xFF;P9DIR = 0xFF;PJDIR = 0xFF;

  // Disable VUSB LDO and SLDO
  USBKEYPID   =     0x9628;                 // set USB KEYandPID to 0x9628 
                                            // enable access to USB config reg
  USBPWRCTL &= ~(SLDOEN+VUSBEN);            // Disable the VUSB LDO and the SLDO
  USBKEYPID   =    0x9600;                  // disable access to USB config reg
  __delay_cycles(10000);                    // settle FLL
}

void RTC_Init(void)
{
  // Setup Clock
  UCSCTL6 &= ~(XT1OFF);                     // XT1 On
  UCSCTL6 |= XCAP_3;                        // Internal load cap
  while(BAKCTL & LOCKBAK)                   // Unlock XT1 pins for operation
     BAKCTL &= ~(LOCKBAK);  
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

  // Configure RTC_B
  RTCCTL01 |= RTCTEVIE + RTCBCD + RTCHOLD;  // BCD mode, RTC hold, enable RTC 
                                            // event interrupt
  RTCYEAR = 0x2011;                         // Year = 0x2011
  RTCMON = 0x10;                            // Month = 0x10 = October
  RTCDAY = 0x07;                            // Day = 0x07 = 7th
  RTCDOW = 0x05;                            // Day of week = 0x05 = Friday
  RTCHOUR = 0x11;                           // Hour = 0x11
  RTCMIN = 0x59;                            // Minute = 0x59
  RTCSEC = 0x45;                            // Seconds = 0x45
  
  RTCCTL1 |= RTCTEV_0;                      // Set RTCTEV for 1 minute alarm
  RTCCTL01 &= ~(RTCHOLD);                   // Start RTC calendar mode
  
  // Turn off Clock for LPM4.5 operation
  UCSCTL6 |= XT1OFF;                        // XT1 Off
}

void EnterLPM35(void)
{
  __enable_interrupt();
  PMMCTL0_H = PMMPW_H;                      // Open PMM Registers for write
  PMMCTL0_L |= PMMREGOFF;                   // and set PMMREGOFF
  __bis_SR_register(LPM4_bits);             // Enter LPM3.5 mode with interrupts
  __no_operation();                         // enabled
}

void WakeUpLPM35(void)
{
  PMMCTL0_H = PMMPW_H;                       // open PMM
  PM5CTL0 &= ~LOCKIO;                        // Clear LOCKBAK and enable ports
  PMMCTL0_H = 0x00;                          // close PMM

  // Restore Port settings
  P1OUT = 0x00;P2OUT = 0x00;P3OUT = 0x00;P4OUT = 0x00;P5OUT = 0x00;P6OUT = 0x00;
  P7OUT = 0x00;P8OUT = 0x00;P9OUT = 0x00;PJOUT = 0x00;
  P1DIR = 0xFF;P2DIR = 0xFF;P3DIR = 0xFF;P4DIR = 0xFF;P5DIR = 0xFF;P6DIR = 0xFF;
  P7DIR = 0xFF;P8DIR = 0xFF;P9DIR = 0xFF;PJDIR = 0xFF;

  // Restore Clock so that RTC will be read
  UCSCTL6 &= ~(XT1OFF);                     // XT1 On
  UCSCTL6 |= XCAP_3;                        // Internal load cap
  while(BAKCTL & LOCKBAK)                   // Unlock XT1 pins for operation
     BAKCTL &= ~(LOCKBAK);  
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag  
  
  // Reconfig/start RTC
  RTCCTL01 |= RTCBCD + RTCHOLD;
  RTCCTL01 &= ~RTCHOLD;
}
