//******************************************************************************
//  MSP430F673x Demo - ADC10, Sample A0, AVcc Ref, Set P1.0 if A0 > 0.5*AVcc
//
//  Description: A single sample is made on A0 with reference to AVcc.
//  Software sets ADC10SC to start sample and conversion - ADC10SC
//  automatically cleared at EOC. ADC10 internal oscillator times sample (16x)
//  and conversion. In main loop MSP430 waits in LPM0 to save power until ADC10
//  conversion complete, ADC10_ISR will force exit from LPM0 in main loop on
//  the return from the interrupt. If A0 > 0.5*AVcc, P1.0 set, else reset.
//
//                MSP430F673x
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//        >---|P1.2/A0      P1.0|-->LED
//
//   M. Swanson
//   Texas Instruments Inc.
//   December 2011
//   Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//******************************************************************************
#include <msp430.h>

unsigned int ADC_Result;

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;                     // Stop WDT

    // Setup P1.2 for A0, P1.0 output
    P1SEL |= BIT2;                                // Set P1.2 to non-IO
    __disable_interrupt();                        // Disable interrupts; Port Map cnfg
    PMAPKEYID = PMAPKEY;                          // Enable access Port Mapping regs
    P1MAP2 = PM_ANALOG;                           // Enable A0
    PMAPKEYID = 0;                                // Disable access Port Mapping regs
    __enable_interrupt();                         // Re-enable all interrupts
    P1DIR |= BIT0;                                // Set P1.0 to output direction
    P1OUT &= ~BIT0;                               // Clear P1.0

    // Setup ADC10
    ADC10CTL0 |= ADC10SHT_2 | ADC10ON;            // S&H=16 ADC clks, Enable ADC10
    ADC10CTL1 |= ADC10SHP;                        // ADCCLK = MODOSC; sampling timer
    ADC10CTL2 |= ADC10RES;                        // 10-bit conversion results
    ADC10MCTL0 |= ADC10INCH_0;                    // A0 ADC input select; Vref=AVCC
    ADC10IE |= ADC10IE0;                          // Enable ADC conv cmplete interrupt

    while (1)
    {
        __delay_cycles(5000);

        ADC10CTL0 |= ADC10ENC | ADC10SC;          // Sampling and conversion start

        __bis_SR_register(LPM0_bits | GIE);       // Enter LPM0 w/ interrupt
        __no_operation();                         // For debug only

        if (ADC_Result < 0x1FF)
        {
            P1OUT &= ~BIT0;                       // Clear P1.0 LED off
        }
        else
        {
            P1OUT |= BIT0;                        // Set P1.0 LED on
        }
    }
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR(void)
{
    switch (__even_in_range(ADC10IV, 12))
    {
        case  ADC10IV_NONE: break;                // No interrupt
        case  ADC10IV_ADC10OVIFG: break;          // conversion result overflow
        case  ADC10IV_ADC10TOVIFG: break;         // conversion time overflow
        case  ADC10IV_ADC10HIIFG: break;          // ADC10HI
        case  ADC10IV_ADC10LOIFG: break;          // ADC10LO
        case  ADC10IV_ADC10INIFG: break;          // ADC10IN
        case  ADC10IV_ADC10IFG:                   // ADC10
            ADC_Result = ADC10MEM0;               // Store ADC10 channel 0 result
            __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0 on return
            break;
        default: break;
    }
}

