//******************************************************************************
//  MSP430F673x Demo - ADC10, DMA Sample A1 64x, AVcc, Repeat Single, DCO
//
//  Description: Sample A1 64 times with AVcc ref. Software writes to ADC10SC
//  to trigger sample burst. In main loop MSP430 waits in LPM0 to save power
//  until ADC10 conversion burst complete, DMA_IFG will force exit from LPM0
//  in main loop on return from interrupt. ADC10 internal oscillator times
//  sample period (16x) and conversion (13x). DMA transfers conversion result in
//  ADC10MEM0 to array ADC_Result. P1.0 set at start of conversion burst, reset
//  on completion.
//
//                MSP430F673x
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//        >---|P1.1/A1          |
//
//   M. Swanson
//   Texas Instruments Inc.
//   December 2011
//   Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//******************************************************************************
#include <msp430.h>

unsigned int ADC_Result[64];

int main(void)
{
    unsigned char i;
    volatile unsigned int ADC_Result_Sum;
    volatile unsigned int ADC_Result_Average;

    WDTCTL = WDTPW | WDTHOLD;                     // Stop WDT

    // Setup P1.1 A1, P1.0 output
    P1SEL |= BIT1;                                // Set P1.1 to non-IO
    __disable_interrupt();                        // Disable interrupts; Port Map cnfg
    PMAPKEYID = PMAPKEY;                          // Enable access Port Mapping regs
    P1MAP1 = PM_ANALOG;                           // Enable A1
    PMAPKEYID = 0;                                // Disable access Port Mapping regs
    __enable_interrupt();                         // Re-enable all interrupts
    P1DIR |= BIT0;                                // Set P1.0 to output direction
    P1OUT &= ~BIT0;                               // Clear P1.0

    // Configure ADC10 - pulse sample mode; software trigger;
    ADC10CTL0 = ADC10SHT_2 | ADC10ON | ADC10MSC;  // 16ADCclks, ADC on
    ADC10CTL1 = ADC10SHP | ADC10CONSEQ_2;         // pulse sample mode, rpt single ch
    ADC10CTL2 = ADC10RES;                         // 10-bits of resolution
    ADC10MCTL0 = ADC10INCH_1;                     // A1, AVCC ref

    // Configure DMA (ADC10IFG trigger)
    DMACTL0 = DMA0TSEL_24;                        // ADC10IFG trigger
    DMA0SZ = 64;                                  // 64 conversions
    __data16_write_addr((unsigned short) &DMA0SA, (unsigned long) &ADC10MEM0);
    // Source single address
    __data16_write_addr((unsigned short) &DMA0DA, (unsigned long) &ADC_Result[0]);
    // Destination array address
    DMA0CTL = DMADT_4 | DMADSTINCR_3 | DMAEN | DMAIE;
    // Repeated single transfer
    // Increment destination
    // Word access
    // Enable int after 64 conversions
    while (1)
    {
        ADC10CTL0 |= ADC10ENC | ADC10SC;          // Sampling and conversion start

        __bis_SR_register(LPM0_bits | GIE);       // Enter LMP0 w/ interrupt
        __no_operation();                         // For debug only

        ADC_Result_Sum = 0x0;                     // clear accumulate register
        for (i = 0; i < 64; i++)                  // BREAKPOINT HERE; View ADC_Result
        {
            ADC_Result_Sum += ADC_Result[i];
        }
        ADC_Result_Average = ADC_Result_Sum >> 6; // Average of 64 conversions results

        __delay_cycles(50000);                    // delay before next 64 conversions
    }
}

#pragma vector=DMA_VECTOR
__interrupt void DMA0_ISR(void)
{
    switch (__even_in_range(DMAIV, 16))
    {
        case DMAIV_NONE: break;                   // No interrupts
        case DMAIV_DMA0IFG:                       // DMA0IFG = DMA Channel 0
            ADC10CTL0 &= ~ADC10ENC;               // 64 conversions complete
            __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0 on return
            break;
        case DMAIV_DMA1IFG: break;                // DMA1IFG = DMA Channel 1
        case DMAIV_DMA2IFG: break;                // DMA2IFG = DMA Channel 2
        case  8: break;                           // Reserved
        case 10: break;                           // Reserved
        case 12: break;                           // Reserved
        case 14: break;                           // Reserved
        case 16: break;                           // Reserved
        default: break;
    }
}

