//******************************************************************************
//   MSP430F673x Demo - LCD_C, Alternates Between Two Strings
//
//   Description: The LCD blinks on a WDT interrupt, showing two different
//                character strings on the LCD.
//                ACLK = REF0 = 32Khz, MCLK = SMCLK = DCO = Default
//
//                MSP430F673x
//             -----------------
//         /|\|                 |
//          | |                 |
//          --|RST              |
//            |                 |    TI LCD
//            |              S0 |     -----------------
//            |               - |--> | + 7 6 5 4 3 2 1 |
//            |             S11 |     -----------------
//            |             COM0|-----||||
//            |             COM1|------|||
//            |             COM2|-------||
//            |             COM3|--------|
//            |                 |
//
//
//  W. Goh
//  Texas Instruments Inc.
//  January 2011
//  Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//******************************************************************************
#include <msp430.h>

// LCD Segment Mapping
const unsigned int LCD_Char_Map[] =
{
    BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7,                // '0' or 'O'
    BIT5 | BIT6,                                            // '1' or 'I'
    BIT0 | BIT1 | BIT3 | BIT4 | BIT6 | BIT7,                // '2' or 'Z'
    BIT0 | BIT1 | BIT4 | BIT5 | BIT6 | BIT7,                // '3'
    BIT0 | BIT1 | BIT2 | BIT5 | BIT6,                       // '4' or 'y'
    BIT0 | BIT1 | BIT2 | BIT4 | BIT5 | BIT7,                // '5' or 'S'
    BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT7,         // '6' or 'b'
    BIT5 | BIT6 | BIT7,                                     // '7'
    BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7,  // '8' or 'B'
    BIT0 | BIT1 | BIT2 | BIT4 | BIT5 | BIT6 | BIT7,         // '9' or 'g'
    BIT2 | BIT3 | BIT5 | BIT6 | BITD | BITF,                // 'M'        10
    BIT0 | BIT1 | BIT2 | BIT4 | BIT5 | BIT7,                // 'S'        11
    BIT0 | BIT1 | BIT2 | BIT3 | BIT6 |BIT7                  // 'P'        12
};

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;               // Stop WDT

    // Setup LCD_C
    // LCD_FREQ = ACLK/32/2, LCD Mux 4, turn on LCD
    // LCD_BLK_FREQ = ACLK/512
    LCDCCTL0 =  LCDDIV_31 | LCDPRE_1 | LCD4MUX | LCDON;

    //Charge pump generated internally at 2.96V, external bias (V2-V4) generation
    //Internal reference for charge pump
    LCDCVCTL = LCDCPEN | VLCD_2_96;
    REFCTL0 &= ~REFMSTR;

    LCDCPCTL0 = 0x0FFF;                     //Select LCD Segments 0-11
    LCDCPCTL1 = 0x0000;                     //

    //LCD Memory
    LCDM1 = (unsigned char) (LCD_Char_Map[10] & 0xFF);          // M - Lower byte
    LCDM2 = (unsigned char) ((LCD_Char_Map[10] >> 8) & 0xFF);   // M - Upper byte
    LCDM3 = (unsigned char) (LCD_Char_Map[11] & 0xFF);          // S - Lower byte
    LCDM4 = (unsigned char) ((LCD_Char_Map[11] >> 8) & 0xFF);   // S - Upper byte
    LCDM5 = (unsigned char) (LCD_Char_Map[12] & 0xFF);          // P - Lower byte
    LCDM6 = (unsigned char) ((LCD_Char_Map[12] >> 8) & 0xFF);   // P - Upper byte

    // Blink Memory
    LCDBM1 = (unsigned char) (LCD_Char_Map[4] & 0xFF);          // 4 - Lower byte
    LCDBM2 = (unsigned char) ((LCD_Char_Map[4] >> 8) & 0xFF);   // 4 - Upper byte
    LCDBM3 = (unsigned char) (LCD_Char_Map[3] & 0xFF);          // 3 - Lower byte
    LCDBM4 = (unsigned char) ((LCD_Char_Map[3] >> 8) & 0xFF);   // 3 - Upper byte
    LCDBM5 = (unsigned char) (LCD_Char_Map[0] & 0xFF);          // 0 - Lower byte
    LCDBM6 = (unsigned char) ((LCD_Char_Map[0] >> 8) & 0xFF);   // 0 - Upper byte

    // Setup WDT
    WDTCTL = WDT_ADLY_1000;                 // WDT 1000ms, ACLK, interval timer
    SFRIE1 |= WDTIE;                        // Enable WDT interrupt

    __bis_SR_register(LPM0_bits | GIE);     // Enter LPM3 w/ interrupt
}

// Watchdog Timer interrupt service routine
#pragma vector = WDT_VECTOR
__interrupt void watchdog_timer(void)
{
    LCDCMEMCTL ^= LCDDISP;                  // Change LCD character string
}

