//*****************************************************************************
//  MSP430F673x Demo - SD24_B, Single Conversion on a Single Channel Using ISR
//                     with 24 bit Result
//
//  Description: This program uses the SD24_B module to perform a single
//  conversion on a single channel. An SD24_B interrupt occurs when a
//  conversion has completed. Test by applying a voltage to channel 2
//  (SD2P0, SD2N0) and setting a breakpoint at the line indicated below.
//  Run program until it reaches the breakpoint, then use the debugger's
//  watch window to view the conversion result. Results (entire 24 bits)
//  for channel 2 are stored in variable "results".
//  ACLK = n/a, MCLK = SMCLK = DCO =  ~ 1.1MHz
//  //* For minimum Vcc required for SD24_B module - see datasheet          *//
//  //* 100nF cap between Vref and AVss is recommended when using 1.5V REF  *//
//
//  NOTE: Example intended to display method to retrieve the full 24 bit result
//        value, not how to attain 24-bit accuracy from SD24_B module.
//
//                MSP430F673x
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//    Vin+ -->|SD2P0            |
//    Vin- -->|SD2N0            |
//            |                 |
//            |            VREF |---+
//            |                 |   |
//            |                 |  -+- 100nF
//            |                 |  -+-
//            |                 |   |
//            |            AVss |---+
//            |                 |
//
//  M. Swanson
//  Texas Instruments, Inc
//  December 2011
//  Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//*****************************************************************************
#include <msp430.h>

/* Unsigned integer to store SD24_B conversion result */
unsigned long results;

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;                       // Stop WDT

    SD24BCTL0 = SD24REFS | SD24SSEL_1;              // Select internal REF
                                                    // Select SMCLK as SD24_B clock source

    SD24BCCTL2 |= SD24SNGL;                         // Single conversion

    SD24BINCTL2 |= SD24INTDLY0;                     // Interrupt on 3rd sample
    SD24BIE |= SD24IE2;                             // Enable channel 2 interrupt

    __delay_cycles(0x3600);                         // Delay for 1.5V REF startup

    while (1)
    {
        SD24BCCTL2 |= SD24SC;                       // Set bit to start conversion
        __bis_SR_register(LPM0_bits | GIE);         // Enter LPM0 w/ interrupts
        __no_operation();
        __no_operation();                           // SET BREAKPOINT HERE
    }
}

#pragma vector=SD24B_VECTOR
__interrupt void SD24BISR(void)
{
    switch (SD24BIV)
    {
        case SD24BIV_SD24OVIFG:                     // SD24MEM Overflow
            break;
        case SD24BIV_SD24TRGIFG:                    // SD24 Trigger IFG
            break;
        case SD24BIV_SD24IFG0:                      // SD24MEM0 IFG
            break;
        case SD24BIV_SD24IFG1:                      // SD24MEM1 IFG
            break;
        case SD24BIV_SD24IFG2:                      // SD24MEM2 IFG
            results = SD24BMEMH2;                   // Save CH2 results (clears IFG)
            results = (results << 16) | SD24BMEML2; // Concatenate lower and upper words
            break;
    }

    __bic_SR_register_on_exit(LPM0_bits);           // Exit LPM0
}

