//******************************************************************************
//  MSP430F673x Demo - Timer2_A2, Toggle P1.0;P8.6-7, Cont. Mode ISR, 32kHz ACLK
//
//  Description: Use Timer2_A CCRx units and overflow to generate three
//  independent timing intervals. For demonstration, CCR0 and CCR1 output units
//  are optionally selected with port pins P8.6 and P8.7
//  in toggle mode. These pins toggle when respective CCRx
//  registers match the TAR counter. Interrupts are also enabled with all
//  CCRx units, software loads offset to next interval only - as long as
//  the interval offset is added to CCRx, toggle rate is generated in
//  hardware. Timer1_A overflow ISR is used to toggle P1.0 with software.
//  Proper use of the TAIV interrupt vector generator is demonstrated.
//  ACLK = TACLK = 32kHz, MCLK = SMCLK = default DCO ~1.045MHz
//
//  As coded and with TACLK = 32768Hz, toggle rates are:
//  P8.6= CCR0 = 32768/(2*4) = 4096Hz
//  P8.7= CCR1 = 32768/(2*16) = 1024Hz
//  P1.0= overflow = 32768/(2*65536) = 0.25Hz
//
//               MSP430F673x
//            -------------------
//        /|\|                   |
//         | |                   |
//         --|RST                |
//           |                   |
//           |         P8.6/TA2.0|--> CCR0
//           |         P8.7/TA2.1|--> CCR1
//           |               P1.0|--> Overflow/software
//
//  M. Swanson
//  Texas Instruments Inc.
//  December 2011
//  Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//******************************************************************************
#include <msp430.h>

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;                // Stop WDT

    // Setup P1.0 output, P8.6 TA2.0, P8.7 TA2.1
    P1DIR |= BIT0;                           // Set P1.0 to output direction
    P1OUT &= ~BIT0;                          // Clear P1.0
    P8DIR |= BIT6 | BIT7;                    // P8.6/TA2.0, P8.7/TA2.1 output
    P8SEL |= BIT6 | BIT7;                    // Output TA0, TA1

    // Setup LFXT1
    UCSCTL6 &= ~(XT1OFF);                    // XT1 On
    UCSCTL6 |= XCAP_3;                       // Internal load cap
    // Loop until XT1 fault flag is cleared
    do
    {
        UCSCTL7 &= ~(XT2OFFG | XT1LFOFFG | DCOFFG);
        // Clear XT2,XT1,DCO fault flags
        SFRIFG1 &= ~OFIFG;                   // Clear fault flags
    } while (SFRIFG1 & OFIFG);               // Test oscillator fault flag

    // Setup TA2
    TA2CCTL0 = OUTMOD_4 | CCIE;              // CCR0 toggle, interrupt enabled
    TA2CCTL1 = OUTMOD_4 | CCIE;              // CCR1 toggle, interrupt enabled
    TA2CTL = TASSEL_1 | MC_2 | TACLR | TAIE; // ACLK, contmode, clear TAR,
                                             // interrupt enabled

    __bis_SR_register(LPM3_bits | GIE);      // Enter LPM3, interrupts enabled
    __no_operation();                        // For debugger
}

// Timer2 A0 interrupt service routine
#pragma vector=TIMER2_A0_VECTOR
__interrupt void Timer2_A0(void)
{
    TA2CCR0 += 4;                            // Add Offset to CCR0
}

// Timer2_A3 Interrupt Vector (TAIV) handler
#pragma vector=TIMER2_A1_VECTOR
__interrupt void TIMER2_A1_ISR(void)
{
    switch (__even_in_range(TA2IV, 14))
    {
        case  TA0IV_NONE: break;             // No interrupt
        case  TA0IV_TA0CCR1:                 // TA0CCR1_CCIFG
            TA2CCR1 += 16;                   // Add Offset to CCR1
            break;
        case  4: break;                      // Reserved
        case  6: break;                      // Reserved
        case  8: break;                      // Reserved
        case 10: break;                      // Reserved
        case 12: break;                      // Reserved
        case TA0IV_TA0IFG:                   // TA0IFG
            P1OUT ^= BIT0;                   // Toggle P1.0
            break;
        default: break;
    }
}

