//******************************************************************************
//   MSP430F673x Demo - USCI_A0, SPI 3-Wire Slave Data Echo
//
//   Description: SPI slave talks to SPI master using 3-wire mode. Data received
//   from master is echoed back.  USCI RX ISR is used to handle communication,
//   CPU normally in LPM0.
//
//   ACLK = ~32.768kHz, MCLK = SMCLK = DCO ~ 1048kHz
//
//   Use with SPI Master Incremented Data code example.
//
//
//                       MSP430F673x
//                    -----------------
//               /|\ |                 |
//                |  |                 |
//                +->|RST              |
//                   |                 |
//                   |             P1.3|-> Data Out (UCA0SIMO)
//                   |                 |
//                   |             P1.2|<- Data In (UCA0SOMI)
//                   |                 |
//                   |             P1.6|-> Serial Clock Out (UCA0CLK)
//
//  M. Swanson
//  Texas Instruments Inc.
//  December 2011
//  Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//******************************************************************************
#include <msp430.h>

int main(void)
{
    WDTCTL = WDTPW | WDTHOLD;               // Stop WDT

    // Setup P1.2 UCA0SOMI, P1.3 UCA0SIMO, P1.6 UCA0CLK
    P1SEL |= BIT2 | BIT3 | BIT6;            // Set P1.2, P1.3, P1.6 to non-IO

    // Setup eUSCI_A0
    UCA0CTLW0 |= UCSWRST;                   // **Put state machine in reset**
    UCA0CTLW0 |= UCSYNC | UCCKPL | UCMSB;   // 3-pin, 8-bit SPI slave,
                                            // Clock polarity high, MSB
    UCA0CTLW0 |= UCSSEL_2;                  // SMCLK
    UCA0BRW_L = 0x02;                       // /2
    UCA0BRW_H = 0;                          //
    UCA0MCTLW = 0;                          // No modulation
    UCA0CTLW0 &= ~UCSWRST;                  // **Initialize USCI state machine**
    UCA0IE |= UCRXIE;                       // Enable RX interrupt

    __bis_SR_register(LPM0_bits | GIE);     // Enter LPM0, enable interrupts
}

// USCI_A0 interrupt service routine
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
    switch (__even_in_range(UCA0IV, 4))
    {
        case USCI_NONE: break;              // No interrupt
        case USCI_SPI_UCRXIFG:              // RXIFG
            while (!(UCA0IFG & UCTXIFG)) ;  // USCI_A0 TX buffer ready?
            UCA0TXBUF = UCA0RXBUF;
            break;
        case USCI_SPI_UCTXIFG: break;       // TXIFG
        default: break;
    }
}

