//******************************************************************************
//   CC430x514x Demo - RTC_D, LPM3.5, & minute toggle
//
//   Description: The RTC_D module is used to set the time, start RTC operation,
//   and read the time from the respective RTC registers. The device will then 
//   enter LPM3.5 awaiting the event interrupt. Upon being woken up by the event
//   (each minute), the LED on the board will toggle.
//
//   NOTE: This code example was created and tested using the EM430F5147RF900
//   Rev3.4 board. To ensure that LPM3.5 is entered and exited properly:
//
//   1. In debug mode:  enable  LPM3.5 debug option for the project and do not 
//      select "Release JTAG on Go option in IAR and Free Run option in CCS.
// 
//   2. In stand-alone mode (JTAG disconnected, for accurate current measurements): 
//      remove JTAG after programming device and externally power board. A reset 
//      may be required after removing the JTAG. When device is in LPM3.5 ~1uA 
//      should be measured on JP2. 
//
//  //* An external watch crystal on XIN XOUT is required for ACLK *//	
//   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//
//                CC430x514x
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|--> LED
//
//   G. Larmore
//   Texas Instruments Inc.
//   June 2012
//   Built with IAR Embedded Workbench V5.40.1 & Code Composer Studio V5.2
//******************************************************************************

#include <msp430.h>

void Board_Init(void);
void Clock_Init(void);
void RTC_Init(void);
void EnterLPM35(void);

int main (void)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  
  // Init board and clock
  Board_Init();
  Clock_Init();
  
  // If waking up from LPM3.5
  if(SYSRSTIV == SYSRSTIV_LPM5WU)
  {
    // If valid minute interrupt toggle LED
    if(RTCSEC == 0)
    {
      P1OUT ^= BIT0;                          // XOR P1.0
    } else
    {
      while(1)                                  // continuous loop
      {
        P1OUT ^= BIT0;                          // XOR P1.0
        __delay_cycles(20000);                  // Delay
      }
    }
  }
  
  // Init RTC and enter LPM3.5
  RTC_Init();
  EnterLPM35();

  // Code should not get here
  // Blink LED slowly if LPM3.5 not entered properly
  Board_Init();
  Clock_Init();
  while(1)
  {
    P1OUT ^= BIT0;                          // XOR P1.0
    __delay_cycles(200000);                 // Delay
  }
}

#pragma vector=RTC_VECTOR
__interrupt void RTCISR (void)
{
  RTCCTL01&= ~(RTCTEVIFG);                //Clear event interrupt
  RTCCTL01 &= ~(RTCTEVIE);                //Clear interrupt enable
  __bic_SR_register_on_exit(LPM4_bits);     // Exit LPM3.5
  __no_operation();
}

void Clock_Init(void)
{
  // Setup Clock
  P5SEL |= BIT0 + BIT1;                     // Port select XT1
  UCSCTL6 |= XCAP_3 + XT1OFF + XT2OFF;      // Internal load cap
                                            // XT1 and XT2 off
  // Loop until XT1 & DCO stabilizes - In this case loop until XT1 and DCO settle
  do
  {
	UCSCTL7 &= ~(XT1LFOFFG + DCOFFG);
											// Clear XT1,DCO fault flags
	SFRIFG1 &= ~OFIFG;                        // Clear fault flags
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag
  
  UCSCTL6 &= ~(XT1DRIVE_3);                 // Xtal is now stable, reduce drive
                                            // strength
}

void Board_Init(void)
{
  PMMCTL0_H = PMMPW_H;                      // open PMM
  PM5CTL0 &= ~LOCKIO;                       // Clear LOCKBAK and enable ports
  PMMCTL0_H = 0x00;                         // close PMM

  // Keep last output for LED
  if (P1IN & BIT0) {
    P1OUT |= BIT0;
  } else {
    P1OUT &= ~(BIT0);
  }

  // Port Configuration (clear all except LED)
  P1OUT &= BIT0;
  P2OUT = 0x00;P3OUT = 0x00;P4OUT = 0x00;P5OUT = 0x00;PJOUT = 0x00;
  P1DIR = 0xFF;P2DIR = 0xFF;P3DIR = 0xFF;P4DIR = 0xFF;P5DIR = 0xFF;PJDIR = 0xFF;
}

void RTC_Init(void)
{
  // Configure RTC_D
  RTCCTL01&= ~(RTCTEVIFG);                //Clear event interrupt
  RTCCTL01 |= RTCTEVIE + RTCHOLD;         //RTC hold, enable RTC
                                          // event interrupt
  RTCCTL01|= RTCTEV_0;                      // Set RTCTEV for 1 minute alarm
  RTCCTL01 &= ~(RTCHOLD);                   // Start RTC calendar mode
}

void EnterLPM35(void)
{
  __enable_interrupt();
  PMMCTL0_H = PMMPW_H;                      // Open PMM Registers for write
  PMMCTL0_L |= PMMREGOFF;                   // and set PMMREGOFF
  __bis_SR_register(LPM4_bits);             // Enter LPM3.5 mode with interrupts
  __no_operation();                         // enabled
}
