//******************************************************************************
//  MSP430FR59xx Demo - ADC12, Window Comparator, 2.5V ref, 2's Complement
//
//  Description; A1 is sampled in single ch/ single conversion mode.
//  Window comparator is used to generate interrupts to
//  indicate when the input voltage goes above the High_Threshold or below the
//  Low_Threshold or is in between the high and low thresholds. TimerB0 is used
//  as an interval timer used to control the LED at P1.0 to toggle slow/fast
//  or turn off according to the ADC12 Hi/Lo/IN interupts.
//  NOTE: This example shows how to write to the ADC threshold registers when
//  2's complement mode is used.
//
//               MSP430FR5969
//            -----------------
//        /|\|              XIN|-
//         | |                 | 32kHz
//         --|RST          XOUT|-
//           |                 |
//       >---|P1.1/A1     P1.0 |--> LED
//
//   Priya Thanigai
//   Texas Instruments Inc.
//   August 2012
//   Built with IAR Embedded Workbench V5.40 & Code Composer Studio V5.2
//******************************************************************************
#include "msp430.h"

#define High_Threshold 0x4CC0//~2V expressed in 2's complement
#define Low_Threshold 0xE670//~1V expressed in 2's complement

// To calculate the thresholds for the window comparator in 2's C mode
// Use (Vin/Vref* 4096) - 2048
// Append zeros to get a 16 bit result
// If the result is a negative number, take the 2's C before appending zeros

unsigned int SlowToggle_Period = 20000-1;
unsigned int FastToggle_Period = 1000-1;

int main(void)
{
  WDTCTL = WDTPW | WDTHOLD;                  // Stop WDT

  // Configure ADC P1.1/A1
  P1SEL1 |= BIT1;
  P1SEL0 |= BIT1;

  // Clock System Setup
  PJSEL0 |= BIT4 | BIT5;
  CSCTL0_H = 0xA5;
  CSCTL1 = DCOFSEL0 | DCOFSEL1;             // Set DCO setting = 8MHz
  CSCTL2 = SELA__LFXTCLK | SELS__DCOCLK | SELM__DCOCLK;
  CSCTL3 = DIVA__1 | DIVS__8 | DIVM__8;     // MCLK = SMCLK = 1MHz
  CSCTL4 |= LFXTDRIVE_0;
  CSCTL4 &= ~LFXTOFF;

  do
  {
    CSCTL5 &= ~LFXTOFFG;
                                            // Clear XT1 fault flag
    SFRIFG1 &= ~OFIFG;
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

  // Configure ADC12
  // tsample = 16ADC12CLK cycles, tconvert = 14 ADC12CLK cycles
  // software trigger for SOC, MODOSC, single ch-single conversion,
  // 2's complement mode
  // tsample controlled by SHT0x settings
  // Channel 1, reference = internal, enable window comparator
  // Set thresholds for ADC12 interrupts
  // Enable Interrupts
  ADC12CTL0 = ADC12SHT0_2 | ADC12ON;
  ADC12CTL1 = ADC12SHS_0 | ADC12SSEL_0 | ADC12CONSEQ_0 | ADC12SHP;
  ADC12CTL2 |= ADC12DF;
  ADC12MCTL0 = ADC12INCH_1 | ADC12VRSEL_1 | ADC12WINC;
  ADC12HI = High_Threshold;
  ADC12LO = Low_Threshold;
  ADC12IER2 = ADC12HIIE | ADC12LOIE | ADC12INIE;
 // Configure internal reference
  while(REFCTL0 & REFGENBUSY);              // If ref generator busy, WAIT
  REFCTL0 |= REFVSEL_2|REFON;               // Select internal ref = 2.5V
                                            // Internal Reference ON
  __delay_cycles(75);                       // Delay (~75us) for Ref to settle

  P1OUT &= ~BIT0;                           // Clear P1.0
  P1DIR |= BIT0;                            // Set P1.0 output direction

  // Configure TB0 period-timer for LED toggle
  TB0CCTL0 = CCIE;                          // CCR0 interrupt enabled
  TB0CTL = TBSSEL_1 | TBCLR;                // ACLK, clear TBR

  while(1)
  {
  ADC12CTL0 |= ADC12ENC | ADC12SC;          // Enable & start conversion
  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0 w/ interrupts
  __no_operation();                         // for debugger
  }
}


#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
  switch(__even_in_range(ADC12IV,12))
  {
    case  0: break;                         // Vector  0:  No interrupt
    case  2: break;                         // Vector  2:  ADC12MEMx Overflow
    case  4: break;                         // Vector  4:  Conversion time overflow
    case  6:                                // Is A1 > 2V?: High Interrupt
        ADC12IFGR2 &= ~ADC12HIIFG;          // Clear interrupt flag
        TB0CTL &= ~MC_1;                    // Turn off Timer
        TB0CCR0 = FastToggle_Period;        // Set Timer Period for fast LED toggle
        TB0CTL |= MC_1;
        __bic_SR_register_on_exit(LPM0_bits); // Exit active CPU
    	break;
    case  8:                                // Is A1 < 1V?: Low Interrupt
        ADC12IFGR2 &= ~ADC12LOIFG;          // Clear interrupt flag
        TB0CTL &= ~MC_1;                    // Turn off Timer
        P1OUT &= ~BIT0;    	                // Turn off LED
        __bic_SR_register_on_exit(LPM0_bits); // Exit active CPU
        break;
    case 10:
        ADC12IFGR2 &= ~ADC12INIFG;          // Clear interrupt flag
        TB0CTL &= ~MC_1;                    // Turn off Timer
        TB0CCR0 = SlowToggle_Period;        // Set Timer Period for slow LED toggle
        TB0CTL |= MC_1;
        __bic_SR_register_on_exit(LPM0_bits); // Exit active CPU
    	break;                              // Vector 10:  ADC12IN
    case 12: break;                         // Vector 12:  ADC12MEM0
    case 14: break;                         // Vector 14:  ADC12MEM1
    case 16: break;                         // Vector 16:  ADC12MEM2
    case 18: break;                         // Vector 18:  ADC12MEM3
    case 20: break;                         // Vector 20:  ADC12MEM4
    case 22: break;                         // Vector 22:  ADC12MEM5
    case 24: break;                         // Vector 24:  ADC12MEM6
    case 26: break;                         // Vector 26:  ADC12MEM7
    case 28: break;                         // Vector 28:  ADC12MEM8
    case 30: break;                         // Vector 30:  ADC12MEM9
    case 32: break;                         // Vector 32:  ADC12MEM10
    case 34: break;                         // Vector 34:  ADC12MEM11
    case 36: break;                         // Vector 36:  ADC12MEM12
    case 38: break;                         // Vector 38:  ADC12MEM13
    case 40: break;                         // Vector 40:  ADC12MEM14
    case 42: break;                         // Vector 42:  ADC12MEM15
    case 44: break;                         // Vector 44:  ADC12MEM16
    case 46: break;                         // Vector 46:  ADC12MEM17
    case 48: break;                         // Vector 48:  ADC12MEM18
    case 50: break;                         // Vector 50:  ADC12MEM19
    case 52: break;                         // Vector 52:  ADC12MEM20
    case 54: break;                         // Vector 54:  ADC12MEM21
    case 56: break;                         // Vector 56:  ADC12MEM22
    case 58: break;                         // Vector 58:  ADC12MEM23
    case 60: break;                         // Vector 60:  ADC12MEM24
    case 62: break;                         // Vector 62:  ADC12MEM25
    case 64: break;                         // Vector 64:  ADC12MEM26
    case 66: break;                         // Vector 66:  ADC12MEM27
    case 68: break;                         // Vector 68:  ADC12MEM28
    case 70: break;                         // Vector 70:  ADC12MEM29
    case 72: break;                         // Vector 72:  ADC12MEM30
    case 74: break;                         // Vector 74:  ADC12MEM31
    case 76: break;                         // Vector 76:  ADC12RDY
    default: break;
  }
}

// Timer1 A0 interrupt service routine
#pragma vector=TIMER0_B0_VECTOR
__interrupt void TIMER0_B0_ISR(void)
{
  P1OUT ^= BIT0;                            // Toggle LED P1.0
}

