//******************************************************************************
//   MSP430FR57xx Demo - RTC_B, LPM3.5, & alarm
//
//   Description: The RTC_B module is used to set the time, start RTC operation,
//   and read the time from the respective RTC registers. Software will set the
//   original time to 11:59:45 am on Friday October 7, 2011. Then the RTC will
//   be activated through software, and an alarm will be created for the next
//   minute (12:00:00 pm). The device will then enter LPM3.5 awaiting
//   the event interrupt. Upon being woken up by the event, the LED on the board
//   will be set.
//
//   NOTE: This example was created and tested using an MSP430FR5739 on an
//   MSP-TS430RHA40A board. To ensure that LPM3.5 is entered properly, upon opening
//   the debug window, run the program then immediately detach the FET from the
//   board. Then use an external power source to run the example code correctly.
//
//   If the device is successfully in LPM3.5, the current through JP2 should be
//   around 1.89uA.
//
//  //* An external watch crystal on XIN XOUT is required for ACLK *//
//   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//
//               MSP430FR57xx
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|--> LED
//
//   T. Witt
//   Texas Instruments Inc.
//   January 2012
//   Built with Code Composer Studio V5.1
//******************************************************************************

#include <msp430.h>

void Board_Init(void);
void RTC_Init(void);
void EnterLPM35(void);
void WakeUpLPM35(void);

void debugger_safety(void);

int main(void)
{
  WDTCTL = WDTPW | WDTHOLD;                 // Stop WDT

  debugger_safety();

  if(SYSRSTIV == 0x08)
  {
    // When woken up from LPM3.5, reinit
    WakeUpLPM35();
    P1DIR |= BIT0;
    // If woken up at noon, set LED
    if((RTCHOUR == 0x12)&(RTCMIN == 0x00)&(RTCSEC == 0x00))
    {
      while(1)
        P1OUT |= BIT0;
    }
    // If values do not match, blink LED fast
    else
    {
      while(1)                                  // continuous loop
      {
        P1OUT ^= BIT0;                          // XOR P1.0
        __delay_cycles(20000);                  // Delay
      }
    }
  }

  // Init board & RTC, then enter LPM3.5
  Board_Init();
  RTC_Init();
  EnterLPM35();

  // Code should not get here
  // Blink LED slowly if LPM3.5 not entered properly
  WakeUpLPM35();
  while(1)
  {
    P1OUT ^= BIT0;                          // XOR P1.0
    __delay_cycles(200000);                 // Delay
  }
}

#pragma vector=RTC_VECTOR
__interrupt void RTCISR (void)
{
  PMMCTL0_H = PMMPW_H;                      // open PMM
  PM5CTL0 &= ~LOCKLPM5;                     // Clear LOCKBAK and enable ports
  PMMCTL0_H = 0x00;                         // close PMM

  RTCCTL0 &= ~RTCTEVIFG;
  __bic_SR_register_on_exit(LPM4_bits);     // Exit LPM4.5
  __no_operation();
}

void Board_Init(void)
{
  // Setup Clocks
  CSCTL0_H = 0xA5;
  PJSEL0 |= BIT4 + BIT5;
  CSCTL2 = SELA_0 + SELS_3 + SELM_3;        // set ACLK = XT1; MCLK = DCO
  CSCTL3 = DIVA_0 + DIVS_0 + DIVM_0;        // set all dividers
  CSCTL4 |= LFXTDRIVE_0;
  CSCTL4 &= ~LFXTOFF;

  do
  {
    CSCTL5 &= ~LFXTOFFG;
                                            // Clear XT1 fault flag
    SFRIFG1 &= ~OFIFG;
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

  // Port Configuration
  P1OUT = 0x00;P2OUT = 0x00;P3OUT = 0x00;P4OUT = 0x00;PJOUT = 0x00;
  P1DIR = 0xFF;P2DIR = 0xFF;P3DIR = 0xFF;P4DIR = 0xFF;PJDIR = 0xFF;

  REFCTL0 |= REFTCOFF;                      // Turn of Temp Sensor
  REFCTL0 &= ~REFON;
}

void RTC_Init(void)
{
  // Configure RTC_B
  RTCCTL01 |= RTCTEVIE + RTCBCD + RTCHOLD;  // BCD mode, RTC hold, enable RTC
                                            // event interrupt
  RTCYEAR = 0x2011;                         // Year = 0x2011
  RTCMON = 0x10;                            // Month = 0x10 = October
  RTCDAY = 0x07;                            // Day = 0x07 = 7th
  RTCDOW = 0x05;                            // Day of week = 0x05 = Friday
  RTCHOUR = 0x11;                           // Hour = 0x11
  RTCMIN = 0x59;                            // Minute = 0x59
  RTCSEC = 0x45;                            // Seconds = 0x45

  RTCCTL1 |= RTCTEV_0;                      // Set RTCTEV for 1 minute alarm
  RTCCTL01 &= ~(RTCHOLD);                   // Start RTC calendar mode

  // Turn off Clock for LPM4.5 operation
  CSCTL4 |= LFXTOFF;
  //CSCTL4 |= XT1OFF;                        // XT1 Off
}

void EnterLPM35(void)
{
  __enable_interrupt();
  PMMCTL0_H = PMMPW_H;                      // Open PMM Registers for write
  PMMCTL0_L |= PMMREGOFF;                   // and set PMMREGOFF
  __bis_SR_register(LPM4_bits);             // Enter LPM3.5 mode with interrupts
  __no_operation();                         // enabled
}

void WakeUpLPM35(void)
{
  PMMCTL0_H = PMMPW_H;                       // open PMM
  PM5CTL0 &= ~LOCKLPM5;                      // Clear LOCKBAK and enable ports
  PMMCTL0_H = 0x00;                          // close PMM

  // Restore Port settings
  P1OUT = 0x00;P2OUT = 0x00;P3OUT = 0x00;P4OUT = 0x00;PJOUT = 0x00;
  P1DIR = 0xFF;P2DIR = 0xFF;P3DIR = 0xFF;P4DIR = 0xFF;PJDIR = 0xFF;

  // Restore Clocks so that RTC will be read
  CSCTL0_H = 0xA5;
  PJSEL0 |= BIT4 + BIT5;
  CSCTL2 = SELA_0 + SELS_3 + SELM_3;        // set ACLK = XT1; MCLK = DCO
  CSCTL3 = DIVA_0 + DIVS_0 + DIVM_0;        // set all dividers
  CSCTL4 |= LFXTDRIVE_0;
  CSCTL4 &= ~LFXTOFF;

  do
  {
    CSCTL5 &= ~LFXTOFFG;
                                            // Clear XT1 fault flag
    SFRIFG1 &= ~OFIFG;
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

  // Reconfig/start RTC
  RTCCTL01 |= RTCBCD + RTCHOLD;
  RTCCTL01 &= ~RTCHOLD;
}


void debugger_safety(void)
{
  //P1.2 = Stop Button
  P1DIR &= ~BIT2;                           // P1.2 Input
  P1REN |= BIT2;
  if(P1IN & BIT2)
  {
	while(1);
  }
}
