//***************************************************************************************
//  MSP430FR59x Demo - Timer0_A3 Capture of VLO Period using DCO SMCLK
//
//  Description; Capture a number of periods of the VLO clock and store them in an array.
//  When the set number of periods is captured the program is trapped and the LED on
//  P1.0 is toggled. At this point halt the program execution read out the values using
//  the debugger.
//  ACLK = VLOCLK = 9.4kHz (typ.), MCLK = SMCLK = default DCO / default divider = 1MHz
//
//            MSP430FR58xxFR59xx
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|-->LED
//
//  Andreas Dannenberg/ P. Thanigai
//  Texas Instruments, Inc
//  August 2012
//  Built with IAR Embedded Workbench V5.30 & Code Composer Studio V5.2
//***************************************************************************************
#include "msp430.h"

#define NUMBER_TIMER_CAPTURES		20

volatile unsigned int timerAcaptureValues[NUMBER_TIMER_CAPTURES];
unsigned int timerAcapturePointer = 0;

int main(void)
{
	WDTCTL = WDTPW | WDTHOLD;		        // Stop watchdog timer
	// Port Setup
	P1OUT &= ~0x01;					        // Clear P1.0 output
	P1DIR |= 0x01;					        // Set P1.0 to output direction
	// Clock System Setup
	CSCTL0 = CSKEY;					        // Unlock CS module
	CSCTL2 &= ~SELA_7;
	CSCTL2 |= SELA__VLOCLK;		            // Select ACLK=VLOCLK
	CSCTL0_H = 0x00;				        // Lock CS module (use byte mode to upper byte)

	__delay_cycles(1000);			        // Allow clock system to settle

	// Timer0_A3 Setup
	TA0CCTL2 = CM_1 | CCIS_1 | SCS | CAP | CCIE;
									        // Capture rising edge,
									        // Use CCI2B=ACLK,
									        // Synchronous capture,
									        // Enable capture mode,
									        // Enable capture interrupt

	TA0CTL = TASSEL_2 | MC_2;		        // Use SMCLK as clock source,
									        // Start timer in continuous mode

	__bis_SR_register(LPM0_bits | GIE);
	__no_operation();
}

// Timer0_A3 CC1-4, TA Interrupt Handler
#pragma vector = TIMER0_A1_VECTOR
__interrupt void Timer0_A1_isrHandler(void)
{
	switch (__even_in_range(TA0IV, TA0IV_TA0IFG)) {
		case TA0IV_TA0CCR1:
			break;
		case TA0IV_TA0CCR2:
			timerAcaptureValues[timerAcapturePointer++] = TA0CCR2;

			if (timerAcapturePointer >= 20) {
				while (1) {
					P1OUT ^= 0x01;			// Toggle P1.0 (LED)
					__delay_cycles(100000);
				}
			}
			break;
		case TA0IV_TA0IFG:
			break;
		default:
			break;
	}
}
