#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/adc10.h"
#include "driverlib/5xx_6xx/ref.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"

//******************************************************************************
//!
//! ADC10 - Sample A1 Input, Toggle Internal Reference
//!
//!  MSP430F550x Demo
//!  Sample and convert analog voltage at ADC i/p ch A1 in single channel single
//!  conversion mode. Internal reference is toggled between 1.5V and 2.5V and
//!  the ADC measurements are made. With 1V input at A1, Vref+ of 1.5V and 2.5V
//!  gives ADC conv result of ~682 and ~409 respectively. LED at P1.0 is turned
//!  on when ADC conv result >500 (that is with Vref=1.5V) and turned off
//!  otherwise (when Vref=2.5V) Basically with an input between 0.8V and 1.3V
//!  the LED will toggle consistently, with an input below 0.8V the LED will
//!  stay off, and an input above 1.3V the LED will remain on.
//!  ACLK = n/a, MCLK = SMCLK = default DCO ~1.2MHz, ADC10CLK = ADC10OSC
//!
//!               MSP430F550x
//!            -----------------
//!        /|\|              XIN|-
//!         | |                 |
//!         --|RST          XOUT|-
//!           |                 |
//!           |          P6.1/A1|<- 1V
//!           |             P1.0|-> LED
//!           |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - ADC10 peripheral
//! - GPIO Port peripheral
//! - A1
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

//Software delay function definition
void delay (void);

void main (void)
{
    unsigned short vRefStatus;

    //Stop Watchdog Timer
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

    //Initialize the ADC10 Module
    /*
     * Base Address for the ADC10 Module
     * Use internal ADC10 bit as sample/hold signal to start conversion
     * USE MODOSC 5MHZ Digital Oscillator as clock source
     * Use default clock divider of 1
     */
    ADC10_init(__MSP430_BASEADDRESS_ADC10_A__,
        ADC10_SAMPLEHOLDSOURCE_SC,
        ADC10_CLOCKSOURCE_ADC10OSC,
        ADC10_CLOCKDIVIDER_1);
    
    ADC10_enable(__MSP430_BASEADDRESS_ADC10_A__);
    
    /*
     * Base Address for the ADC10 Module
     * Sample/hold for 16 clock cycles
     * Do not enable Multiple Sampling
     */
    ADC10_setupSamplingTimer(__MSP430_BASEADDRESS_ADC10_A__,
        ADC10_CYCLEHOLD_16_CYCLES,
        ADC10_MULTIPLESAMPLESDISABLE);

    //Configure Memory Buffer
    /*
     * Base Address for the ADC10 Module
     * Use input A1
     * Use positive reference of Internally generated Vref
     * Use negative reference of AVss
     */
    ADC10_memoryConfigure(__MSP430_BASEADDRESS_ADC10_A__,
        ADC10_INPUT_A1,
        ADC10_VREFPOS_INT,
        ADC10_VREFNEG_AVSS);

    //Configure internal reference
    //If ref generator busy, WAIT
    while ( REF_BUSY == REF_isRefGenBusy(__MSP430_BASEADDRESS_REF__) ) ;
    //Select internal ref = 1.5V
    REF_setReferenceVoltage(__MSP430_BASEADDRESS_REF__,
        REF_VREF1_5V);
    //Internal Reference ON
    REF_enableReferenceVoltage(__MSP430_BASEADDRESS_REF__);
    //vRefStatus == 1 for Vref == 1.5V
    vRefStatus = 1;

    //Delay (~75us) for Ref to settle
    __delay_cycles(75);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Clear P1.0 LED off
    GPIO_setOutputLowOnPin(
        __MSP430_BASEADDRESS_PORT1_R__,
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    while (1)
    {
        //Enable and Start the conversion
        //in Single-Channel, Single-Conversion Mode
        ADC10_startConversion(__MSP430_BASEADDRESS_ADC10_A__,
            ADC10_SINGLECHANNEL);

        //Toggle internal reference voltage
        //If ref generator busy, WAIT
        while ( REF_BUSY == REF_isRefGenBusy(__MSP430_BASEADDRESS_REF__) ) ;

        //Toggle Internal Ref between 1.5V/2.5V
        if (vRefStatus){  //Vref == 1.5V
            //Select internal ref = 2.5V
            REF_setReferenceVoltage(__MSP430_BASEADDRESS_REF__,
                REF_VREF2_5V);
            //vRefStatus == 0 for Vref == 2.5V
            vRefStatus = 0;
        } else   { //Vref == 2.5V
            //Select internal ref = 1.5V
            REF_setReferenceVoltage(__MSP430_BASEADDRESS_REF__,
                REF_VREF1_5V);
            //vRefStatus == 1 for Vref == 1.5V
            vRefStatus = 1;
        }

        //Conversion complete?
        while (!(ADC10_getInterruptStatus(__MSP430_BASEADDRESS_ADC10_A__,
                     ADC10IFG0))) ;

        //Is ADC10MEM0 = A1 > 500
        if (ADC10_getResults(__MSP430_BASEADDRESS_ADC10_A__) > 500){
            //Set P1.0 LED on
            GPIO_setOutputHighOnPin(
                __MSP430_BASEADDRESS_PORT1_R__,
                GPIO_PORT_P1,
                GPIO_PIN0
                );
        } else   {
            //Clear P1.0; LED off; Vref=2.5V used
            GPIO_setOutputLowOnPin(
                __MSP430_BASEADDRESS_PORT1_R__,
                GPIO_PORT_P1,
                GPIO_PIN0
                );
        }

        //Delay between conversions
        delay();
    }
}

void delay (void)
{
    volatile unsigned long i;

    for (i = 0x7FFF; i > 0; i--){}
}

