//******************************************************************************
// MSP430FR57xx Demo - COMPD interrupt capability; Vcompare is compared against 
//                    internal 1.5V reference
//
// Description: Use CompD and internal reference to determine if input'Vcompare'
// is high of low.  For the first time, when Vcompare exceeds the 1.5V internal
// reference, CDIFG is set and device enters the CompD ISR. In the ISR, CDIES is
// toggled such that when Vcompare is less than 1.5V internal reference;
// CDIFG is set.
// LED is toggled inside the CompD ISR
//                                                   
//                 MSP430FR5739
//             ------------------                        
//         /|\|                  |                       
//          | |                  |                       
//          --|RST      P3.1/CD13|<--Vcompare
//            |                  |                                         
//            |              P3.4|----> 'high'(Vcompare>2.0V); 'low'(Vcompare<2.5V)
//            |                  |  |
//            |                  |  | LED 'ON'(Vcompare>2.0V); 'OFF'(Vcompare<2.5V)
//            |                  | 
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/comp_e.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

void main (void)
{
    //Stop WDT
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

    //Set P3.4 as an output pin.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 4 as output
     */
  	FRGPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN4
  	);
    //Set P3.4 as Output Low.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 4 to output Low.
     */
    FRGPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN4
  	);

    //Initialize the Comparator B module
    /*
     * Base Address of Comparator B,
     * Pin CD13 to Positive(+) Terminal
     * Reference Voltage to Negative(-) Terminal
     * Normal Power Mode
     * Output Filter On with minimal delay
     * Non-Inverted Output Polarity
     */
    COMPE_init(__MSP430_BASEADDRESS_COMPE__,
        COMPE_INPUT13,
        COMPE_VREF,
        COMPE_FILTEROUTPUT_OFF,
        COMPE_NORMALOUTPUTPOLARITY
        );

    //Set the reference voltage that is being supplied to the (-) terminal
    /*
     * Base Address of Comparator D,
     * Reference Voltage of 1.5 V,
	 * Lower Limit of 1.5*(32/32) = 1.5V,
     * Upper Limit of 1.5*(32/32) = 1.5V
     */
    COMPE_setReferenceVoltage(__MSP430_BASEADDRESS_COMPE__,
        COMPE_VREFBASE1_5V,
        32,
        32
        );

    //Enable CompB Interrupt on default rising edge for CDIFG
    COMPE_interruptSetEdgeDirection(__MSP430_BASEADDRESS_COMPE__, COMPE_RISINGEDGE);

    // Clear any erroneous interrupts
    COMPE_clearInterrupt(__MSP430_BASEADDRESS_COMPE__, (COMPE_INTERRUPT_FLAG + COMPE_INTERRUPT_FLAG_INVERTED_POLARITY));

    //Enable Interrupts
    /*
     * Base Address of Comparator B,
     * Enable CompB Interrupt on default rising edge for CDIFG
     */
    COMPE_enableInterrupt(__MSP430_BASEADDRESS_COMPE__,
        COMPE_OUTPUT_INTERRUPT
        );

    //Allow power to Comparator module
    COMPE_enable(__MSP430_BASEADDRESS_COMPE__);

    //Enter LPM4 with interrupts enabled
    __bis_SR_register(LPM4_bits + GIE);

    //For debug
    __no_operation();
}

//******************************************************************************
//
//This is the COMP_B_VECTOR interrupt vector service routine.
//
//******************************************************************************
#pragma vector=COMP_B_VECTOR
__interrupt void COMPE_E_ISR (void)
{
    //Toggle the edge at which an interrupt is generated
    COMPE_interruptToggleEdgeDirection(__MSP430_BASEADDRESS_COMPE__);

    //Clear Interrupt flag
    COMPE_clearInterrupt(__MSP430_BASEADDRESS_COMPE__, COMPE_INTERRUPT_FLAG);

	//Toggle P3.4 output pin.
	/*
	* Base Address for Port 3
	* Select Port 3
	* Set Pin 4 as output
	*/
	FRGPIO_toggleOutputOnPin(__MSP430_BASEADDRESS_PORT3_R__,
		FRGPIO_PORT_P3,
		FRGPIO_PIN4
	);
}
