//******************************************************************************
// MSP430FR59xx Demo - COMPE Toggle from LPM4; CompE in LPM4
// 		Vcompare is compared against the Vcc*1/2
//
// Description: Use CompE and shared reference to determine if input 'Vcompare'
//    is high of low.  When Vcompare exceeds Vcc*1/2 COUT goes high and when
//    Vcompare is less than Vcc*1/2 then CDOUT goes low. 
//    P3.5/CDOUT is connected internally to an LED; it will toggle accordingly.
//                                                   
//                 MSP430FR5969
//             ------------------                        
//         /|\|                  |                       
//          | |                  |                       
//          --|RST      P1.2/CD2 |<--Vcompare
//            |                  |                                         
//            |        P3.5/CDOUT|----> 'high'(Vcompare>Vcc*1/2); 'low'(Vcompare<Vcc*1/2)
//            |                  |  |
//            |                  |  | LED 'ON'(Vcompare>Vcc*1/2); 'OFF'(Vcompare<Vcc*1/2)
//            |                  | 
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/comp_e.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

void main(void)
{
  //Stop WDT
  WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

	//Set P3.5 as Ternary Module Function Output.
	/*
	* Base Address for Port 3
	* Select Port 3
	* Set Pin 5 to output Ternary Module Function, (CBOUT).
	*/
	FRGPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT3_R__,
		FRGPIO_PORT_P3,
		FRGPIO_PIN5,
		FRGPIO_TERNARY_MODULE_FUNCTION
	);

  //Initialize the Comparator E module
    /* Base Address of Comparator E,
     * Pin C2 to Positive(+) Terminal,
     * Reference Voltage to Negative(-) Terminal,
     * Normal Power Mode,
     * Output Filter On with minimal delay,
     * Non-Inverted Output Polarity
     */
    COMPE_init(__MSP430_BASEADDRESS_COMPE__,
        COMPE_INPUT2,
        COMPE_VREF,
        COMPE_FILTEROUTPUT_OFF,
        COMPE_NORMALOUTPUTPOLARITY
        );

    //Set the reference voltage that is being supplied to the (-) terminal
    /* Base Address of Comparator E,
     * Reference Voltage of Vcc V,
	 * Lower Limit of Vcc*(16/32) = Vcc*1/2,
     * Upper Limit of Vcc*(16/32) = Vcc*1/2
     */
    COMPE_setReferenceVoltage(__MSP430_BASEADDRESS_COMPE__,
    	COMPE_VREFBASE_VCC,
        16,
        16
        );

    //Disable Input Buffer on P1.2/CD2
	/* Base Address of Comparator E,
	 * Input Buffer port
	 * Selecting the CDx input pin to the comparator
	 * multiplexer with the CDx bits automatically
	 * disables output driver and input buffer for
	 * that pin, regardless of the state of the
	 * associated CDPE.x bit
	 */
	  COMPE_disableInputBuffer(__MSP430_BASEADDRESS_COMPE__,
	  	  COMPE_INPUT2);

    //Allow power to Comparator module
    COMPE_enable(__MSP430_BASEADDRESS_COMPE__);

  __delay_cycles(400);           // delay for the reference to settle

  __bis_SR_register(LPM4_bits);             // Enter LPM4
  __no_operation();                         // For debug 
}
