#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/ei2c.h"
#include "driverlib/5xx_6xx/cs.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

//*****************************************************************************
//! This example shows how to configure the I2C module as a master for
//! single byte reception in interrupt driven mode. The address of the slave
//! module that the master is communicating with also set in this example.
//!
//!  Description: This demo connects two MSP430's via the I2C bus. The master
//!  reads from the slave. This is the MASTER CODE. The data from the slave
//!  transmitter begins at 0 and increments with each transfer.
//!  The USCI_B0 RX interrupt is used to know when new data has been received.
//!  ACLK = n/a, MCLK = SMCLK = BRCLK =  DCO = 1MHz
//!
//!                                /|\  /|\
//!               MSP430FR5739      10k  10k     MSP430F5739
//!                   slave         |    |        master
//!             -----------------   |    |   -----------------
//!           -|XIN  P1.6/UCB0SDA|<-|----+->|P1.6/UCB0SDA  XIN|-
//!            |                 |  |       |                 | 32kHz
//!           -|XOUT             |  |       |             XOUT|-
//!            |     P1.7/UCB0SCL|<-+------>|P1.7/UCB0SCL     |
//!            |                 |          |             P1.0|--> LED
//!

//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//!
//
//*****************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************

#define SLAVE_ADDRESS 0x48

unsigned char RXData;
void main (void)
{

	//Stop WDT
	WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

	//Set DCO frequency to 8MHz
	CS_setDCOFreq(__MSP430_BASEADDRESS_CS__,CS_DCORSEL_0,CS_DCOFSEL_3);
	//Set ACLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_ACLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
	//Set SMCLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
	//Set MCLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);

    // Configure Pins for I2C
	//Set P1.6 and P1.7 as Secondary Module Function Input.
	/*
	* Base Address for Port 1
	* Select Port 1
	* Set Pin 6, 7 to input Secondary Module Function, (UCB0SIMO/UCB0SDA, UCB0SOMI/UCB0SCL).
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT1_R__,
		FRGPIO_PORT_P1,
		FRGPIO_PIN6 + FRGPIO_PIN7,
		FRGPIO_SECONDARY_MODULE_FUNCTION
	);

    eI2C_masterInit(__MSP430_BASEADDRESS_EUSCI_B0__,
            eI2C_CLOCKSOURCE_SMCLK,
            CS_getSMCLK(__MSP430_BASEADDRESS_CS__),
            eI2C_SET_DATA_RATE_400KBPS,
            1,
            eI2C_SEND_STOP_AUTOMATICALLY_ON_BYTECOUNT_THRESHOLD
            );

    //Specify slave address
    eI2C_setSlaveAddress(__MSP430_BASEADDRESS_EUSCI_B0__,
            SLAVE_ADDRESS
            );

    //Set Master in receive mode
    eI2C_setMode(__MSP430_BASEADDRESS_EUSCI_B0__,
            eI2C_RECEIVE_MODE
            );

    //Enable I2C Module to start operations
    eI2C_enable(__MSP430_BASEADDRESS_EUSCI_B0__);

    //Enable master Receive interrupt
    eI2C_enableInterrupt(__MSP430_BASEADDRESS_EUSCI_B0__,
    		eI2C_RECEIVE_INTERRUPT0 +
          eI2C_BYTE_COUNTER_INTERRUPT
          );


    //Set P1.0 as an output pin.
	/*
	 * Base Address for Port 1
	 * Select Port 1
	 * Set Pin 0 as output
	 */
	FRGPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
			FRGPIO_PORT_P1,
			FRGPIO_PIN0
	);


    while (1)
    {
      __delay_cycles(2000);
      while (UCB0CTL1 & UCTXSTP);           // Ensure stop condition got sent


      // I2C start condition
      eI2C_masterReceiveStart(__MSP430_BASEADDRESS_EUSCI_B0__);

      __bis_SR_register(CPUOFF+GIE);        // Enter LPM0 w/ interrupt
    }
}





#pragma vector = USCI_B0_VECTOR
__interrupt void USCIB0_ISR(void)
{

  switch(__even_in_range(UCB0IV,0x1E))

  {
      case 0x00: break; // Vector 0: No interrupts break;
      case 0x02: break; // Vector 2: ALIFG break;
      case 0x04: break; // Vector 4: NACKIFG break;
      case 0x06: break; // Vector 6: STTIFG break;
      case 0x08: break; // Vector 8: STPIFG break;
      case 0x0a: break; // Vector 10: RXIFG3 break;
      case 0x0c: break; // Vector 14: TXIFG3 break;
      case 0x0e: break; // Vector 16: RXIFG2 break;
      case 0x10: break; // Vector 18: TXIFG2 break;
      case 0x12: break; // Vector 20: RXIFG1 break;
      case 0x14: break; // Vector 22: TXIFG1 break;
      case 0x16:
                  RXData = eI2C_masterSingleReceive(
                          __MSP430_BASEADDRESS_EUSCI_B0__
                          );   // Get RX data
                  __bic_SR_register_on_exit(CPUOFF); // Exit LPM0
                  break; // Vector 24: RXIFG0 break;
      case 0x18: break; // Vector 26: TXIFG0 break;
      case 0x1a:  	FRGPIO_toggleOutputOnPin(__MSP430_BASEADDRESS_PORT1_R__,
							FRGPIO_PORT_P1,
							FRGPIO_PIN0
					);		
                 break; // Vector 28: BCNTIFG break;
      case 0x1c: break; // Vector 30: clock low timeout break;
      case 0x1e: break; // Vector 32: 9th bit break;
      default: break;

  }

}

