#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/ei2c.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/cs.h"
#include "driverlib/5xx_6xx/frgpio.h"

//******************************************************************************
//! This example shows how to configure the I2C module as a master for
//! multi byte transmission in interrupt driven mode. The address of the slave
//! module is set in this example.
//!
//!  MSP430FR57xx Demo - USCI_B0 I2C Master TX multiple bytes to MSP430 Slave
//!
//!  Description: This demo connects two MSP430's via the I2C bus. The master
//!  transmits to the slave. This is the MASTER CODE. It cntinuously
//!  transmits an array of data and demonstrates how to implement an I2C
//!  master transmitter sending multiple bytes using the USCI_B0 TX interrupt.
//!  ACLK = n/a, MCLK = SMCLK = BRCLK = default DCO = ~1.045MHz
//!
//!                                /|\  /|\
//!                MSP430FR5739    10k  10k      MSP430FR5739
//!                   slave         |    |         master
//!             -----------------   |    |   -----------------
//!           -|XIN  P1.6/UCB0SDA|<-|----+->|P1.6/UCB0SDA  XIN|-
//!            |                 |  |       |                 |
//!           -|XOUT             |  |       |             XOUT|-
//!            |     P1.7/UCB0SCL|<-+------>|P1.7/UCB0SCL     |
//!            |                 |          |                 |
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//

//******************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************
#define SLAVE_ADDRESS 0x48

unsigned char TXData =0;                    // Pointer to TX data
unsigned char TXByteCtr;

void main(void)

{
	WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

	//Set DCO frequency to 8MHz
	CS_setDCOFreq(__MSP430_BASEADDRESS_CS__,CS_DCORSEL_0,CS_DCOFSEL_3);
	//Set ACLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_ACLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
	//Set SMCLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_SMCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
	//Set MCLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);

    // Configure Pins for I2C
	//Set P1.6 and P1.7 as Secondary Module Function Input.
	/*
	* Base Address for Port 1
	* Select Port 1
	* Set Pin 6, 7 to input Secondary Module Function, (UCB0SIMO/UCB0SDA, UCB0SOMI/UCB0SCL).
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT1_R__,
		FRGPIO_PORT_P1,
		FRGPIO_PIN6 + FRGPIO_PIN7,
		FRGPIO_SECONDARY_MODULE_FUNCTION
	);

     eI2C_masterInit(__MSP430_BASEADDRESS_EUSCI_B0__,
                    eI2C_CLOCKSOURCE_SMCLK,
                    //UCS_getSMCLK(__MSP430_BASEADDRESS_UCS__),
                    1000000,
                    eI2C_SET_DATA_RATE_400KBPS,
                    0,
                    eI2C_NO_AUTO_STOP
                    );

	//Specify slave address
	eI2C_setSlaveAddress(__MSP430_BASEADDRESS_EUSCI_B0__,
				SLAVE_ADDRESS
				);

	//Set Master in receive mode
	eI2C_setMode(__MSP430_BASEADDRESS_EUSCI_B0__,
				eI2C_TRANSMIT_MODE
				);

	//Enable I2C Module to start operations
	eI2C_enable(__MSP430_BASEADDRESS_EUSCI_B0__);

	//Enable master Receive interrupt
	eI2C_enableInterrupt(__MSP430_BASEADDRESS_EUSCI_B0__,
				eI2C_TRANSMIT_INTERRUPT0 +
				eI2C_NAK_INTERRUPT
			  );
	while(1)
	{
	__delay_cycles(1000);                   // Delay between transmissions
	TXByteCtr = 4;                          // Load TX byte counter
	TXData = 0;

	while (eI2C_SENDING_STOP == eI2C_masterIsSTOPSent
			(__MSP430_BASEADDRESS_EUSCI_B0__));


	eI2C_masterMultiByteSendStart(__MSP430_BASEADDRESS_EUSCI_B0__,
			TXData++);

	__bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
											// Remain in LPM0 until all data
											// is TX'd
											// Increment data byte
	}

}


//------------------------------------------------------------------------------
// The USCIAB0TX_ISR is structured such that it can be used to transmit any
// number of bytes by pre-loading TXByteCtr with the byte count. Also, TXData
// points to the next byte to transmit.
//------------------------------------------------------------------------------
#pragma vector = USCI_B0_VECTOR
__interrupt void USCIB0_ISR(void)
{
  switch(__even_in_range(UCB0IV,0x1E))
  {
		case 0x00: break;                    // Vector 0: No interrupts break;
		case 0x02: break;
		case 0x04:
			//resend start if NACK
			eI2C_masterSendStart(__MSP430_BASEADDRESS_EUSCI_B0__);
		  break;                            // Vector 4: NACKIFG break;
		case 0x18:
	  if (TXByteCtr)                    // Check TX byte counter
		 {

		  eI2C_masterMultiByteSendNext(__MSP430_BASEADDRESS_EUSCI_B0__,
				  TXData++);
			TXByteCtr--;                    // Decrement TX byte counter

		 }
		  else
		  {
			  eI2C_masterMultiByteSendStop(__MSP430_BASEADDRESS_EUSCI_B0__);

			__bic_SR_register_on_exit(CPUOFF);// Exit LPM0
		  }
		  break;                            // Vector 26: TXIFG0 break;
		default: break;
  }
}
