#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/espi.h"
#include "driverlib/5xx_6xx/cs.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

//*****************************************************************************
//!   SPI slave talks to SPI master using 3-wire mode. Data is received
//!   from master and data from slave is then transmitted back to master.
//!   USCI RX ISR is used to handle communication, CPU normally in LPM4.
//!   Prior to initial data exchange, master pulses slaves RST for complete
//!   reset.
//!   ACLK = ~32.768kHz, MCLK = SMCLK = DCO ~ 1048kHz
//!
//!   Use with SPI Master Incremented Data code example.  If the slave is in
//!   debug mode, the reset signal from the master will conflict with slave's
//!   JTAG; to work around, use IAR's "Release JTAG on Go" on slave device.  If
//!   breakpoints are set in slave RX ISR, master must stopped also to avoid
//!   overrunning slave RXBUF.
//!
//!                   MSP430FR5739
//!                 -----------------
//!            /|\ |                 |
//!             |  |                 |
//!    Master---+->|RST              |
//!                |                 |
//!                |             P2.0|-> Data Out (UCA0SIMO)
//!                |                 |
//!                |             P2.1|<- Data In (UCA0SOMI)
//!                |                 |
//!                |             P1.5|<- Serial Clock Out (UCA0CLK)
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - SPI peripheral
//! - GPIO Port peripheral (for SPI pins)
//! - UCA0SIMO
//! - UCA0SOMI
//! - UCA0CLK
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR
//!
//
//*****************************************************************************
unsigned char transmitData = 0x01, receiveData = 0x00;

void main(void)
{
  unsigned short returnValue = STATUS_FAIL;

  //Stop watchdog timer
  WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

  // Configure Pins for LFXIN
	//Set PJ.4 and PJ.5 as Primary Module Function Input.
	/*
	* Base Address for Port J
	* Select Port J
	* Set Pin 4, 5 to input Primary Module Function, (LFXIN).
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORTJ_R__,
		FRGPIO_PORT_PJ,
		FRGPIO_PIN4 + FRGPIO_PIN5,
		FRGPIO_PRIMARY_MODULE_FUNCTION
	);

  //Set external frequency for XT1
  CS_setExternalClockSource(__MSP430_BASEADDRESS_CS__,32768,0);
  //Set DCO frequency to max DCO setting
  CS_setDCOFreq(__MSP430_BASEADDRESS_CS__,CS_DCORSEL_0,CS_DCOFSEL_3);
  //Select XT1 as the clock source for ACLK with no frequency divider
  CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_ACLK,CS_XT1CLK_SELECT,CS_CLOCK_DIVIDER_1);
  //Start XT1 with no time out
  CS_XT1Start(__MSP430_BASEADDRESS_CS__,CS_XT1DRIVE_0);

	// Configure SPI pins
	// Configure Pins for UCA0CLK
	//Set P1.5 as Secondary Module Function Input.
	/*
	* Base Address for Port 1
	* Select Port 1
	* Set Pin 5 to input Secondary Module Function, (UCA0CLK).
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT1_R__,
		FRGPIO_PORT_P1,
		FRGPIO_PIN5,
		FRGPIO_SECONDARY_MODULE_FUNCTION
	);
	// Configure Pins for UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI 
	//Set P2.0, P2.1 as Secondary Module Function Input.
	/*
	* Base Address for Port 2
	* Select Port 2
	* Set Pin 0, 1 to input Secondary Module Function, (UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI).
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT2_R__,
		FRGPIO_PORT_P2,
		FRGPIO_PIN0 + FRGPIO_PIN1,
		FRGPIO_SECONDARY_MODULE_FUNCTION
	);

  //Initialize slave to MSB first, inactive high clock polarity and 3 wire SPI
  returnValue = eSPI_slaveInit(__MSP430_BASEADDRESS_EUSCI_A0__,
          eSPI_MSB_FIRST,
          eSPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
          eSPI_CLOCKPOLARITY_INACTIVITY_HIGH
          );

  if (STATUS_FAIL == returnValue){
          return;
      }

  //Enable SPI Module
  eSPI_enable(__MSP430_BASEADDRESS_EUSCI_A0__);

  //Enable Receive interrupt
  eSPI_enableInterrupt(__MSP430_BASEADDRESS_EUSCI_A0__,
		  eSPI_RECEIVE_INTERRUPT
          );

  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0, enable interrupts



}

//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:
            //USCI_A0 TX buffer ready?
            while (!eSPI_getInterruptStatus(__MSP430_BASEADDRESS_EUSCI_A0__,
                       eSPI_TRANSMIT_INTERRUPT
                       )) ;

            //Transmit data to master
            eSPI_transmitData(__MSP430_BASEADDRESS_EUSCI_A0__,
            transmitData
            );

            //Receive data from master
            receiveData = eSPI_receiveData(__MSP430_BASEADDRESS_EUSCI_A0__);

            //Increment data to be transmitted
            transmitData++;

            break;

        default: break;
    }
}

