//******************************************************************************
//!  EUSCI_A0 External Loopback test using eUART_initAdvanced API
//!
//!  Description: This demo connects TX to RX of the MSP430 UART
//!  The example code shows proper initialization of registers
//!  and interrupts to receive and transmit data.
//!
//!  ACLK = BRCLK = 32.768kHz, MCLK = SMCLK = DCO/8 = ~1MHz
//!
//!
//!                MSP430FR5739
//!             -----------------
//!       RST -|     P2.0/UCA0TXD|----|
//!            |                 |    |
//!            |                 |    |
//!            |     P2.1/UCA0RXD|----|
//!            |                 |
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UART peripheral
//! - GPIO Port peripheral (for UART pins)
//! - UCA0TXD
//! - UCA0RXD
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR.
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/euart.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/cs.h"
#include "driverlib/5xx_6xx/frgpio.h"

unsigned int i;
unsigned char RXData = 0, TXData = 0;
unsigned char check = 0;

void main(void)
{
    // stop watchdog
	WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

  	// LFXT Setup
	//Set PJ.4 and PJ.5 as Primary Module Function Input.
	/*
	* Base Address for Port J
	* Select Port J
	* Set Pin 4, 5 to input Primary Module Function, LFXT.
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORTJ_R__,
		FRGPIO_PORT_PJ,
		FRGPIO_PIN4 + FRGPIO_PIN5,
		FRGPIO_PRIMARY_MODULE_FUNCTION
	);

	//Set DCO frequency to 8 MHz
	CS_setDCOFreq(__MSP430_BASEADDRESS_CS__,CS_DCORSEL_1,CS_DCOFSEL_3);
	//Set external clock frequency to 32.768 KHz
	CS_setExternalClockSource(__MSP430_BASEADDRESS_CS__,32768,0);
	//Set ACLK=LFXT
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_ACLK,CS_XT1CLK_SELECT,CS_CLOCK_DIVIDER_1);
	//Start XT1 with no time out
	CS_XT1Start(__MSP430_BASEADDRESS_CS__,CS_XT1DRIVE_0);
	//Set SMCLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_ACLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);
	//Set MCLK = DCO with frequency divider of 8
	CS_clockSignalInit(__MSP430_BASEADDRESS_CS__,CS_ACLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_8);

    // Configure UART pins
	//Set P2.0 and P2.1 as Secondary Module Function Input.
	/*
	* Base Address for Port 2
	* Select Port 2d
	* Set Pin 0, 1 to input Secondary Module Function, (UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI).
	*/
	FRGPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT2_R__,
		FRGPIO_PORT_P2,
		FRGPIO_PIN0 + FRGPIO_PIN1,
		FRGPIO_SECONDARY_MODULE_FUNCTION
	);

  // Configure UART
  if ( STATUS_FAIL == eUART_initAdvance(__MSP430_BASEADDRESS_EUSCI_A0__,
              eUART_CLOCKSOURCE_ACLK,
              15,
              0,
              68,
              eUART_NO_PARITY,
              eUART_LSB_FIRST,
              eUART_ONE_STOP_BIT,
              eUART_MODE,
              eUART_LOW_FREQUENCY_BAUDRATE_GENERATION )){
         return;
     }

  eUART_enable(__MSP430_BASEADDRESS_EUSCI_A0__);

  // Enable USCI_A0 RX interrupt
  eUART_enableInterrupt(__MSP430_BASEADDRESS_EUSCI_A0__,
		  eUART_RECEIVE_INTERRUPT);                     // Enable interrupt

  __enable_interrupt();
  while (1)
  {
    TXData = TXData+1;                      // Increment TX data
    // Load data onto buffer
    eUART_transmitData(__MSP430_BASEADDRESS_EUSCI_A0__,
    				   TXData);
    while(check != 1);
    check = 0;
  }
}
//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
  switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
  {
    case USCI_NONE: break;
    case USCI_UART_UCRXIFG:
       RXData = eUART_receiveData(__MSP430_BASEADDRESS_EUSCI_A0__);
      if(!(RXData == TXData))                   // Check value
      {
        while(1);
      }
      check =1;
      break;
    case USCI_UART_UCTXIFG: break;
    case USCI_UART_UCSTTIFG: break;
    case USCI_UART_UCTXCPTIFG: break;
  }
}
