//******************************************************************************
//  MSP430FR57xx PMM Test
// This example shows transitions between a user Reset via the reset button, a
// software triggered BOR, a previous LPMx.5 state indicator flag, and entrance into LPM4.5
//
// Upon device power on you will see the following sequence of LED's
// X = off
// O = on
//
// _7 = On indicates BOR
// _6 = On indicates Reset (from user button)
// _5 = on indicates device was previously in LPMx.5 state before the reset
// _4 = On indicates device is NOT sleeping.  Off means device is in LPM4.5
//*******************************************************************************
//
// 	1) Power on device:
// 		X7 X6 X5 O4  			LED4 is on, device is running and was not in LPM4.5 mode previously.
//
// 	2) Press User Reset button:
//
// 		X7 O6 X5 O4				Device is running and has reset due to RST pin
// 		O7 X6 X5 O4				Device is running and has reset due to a software BOR
// 		X7 X6 X5 X4				Device is now in LPM4.5
//
//  3) Press User Reset button:
//  	X7 X6 O5 O4				Device is running and was previously in LPM4.5
//
//	4) Press User Reset button:
//		Device will restart the sequence beginning at step 2).
//
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//********************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/frpmm.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

void main(void)
{
    //Stop WDT
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);
    //Unlock the GPIO pins.
      /*
       * Base Address of PMM,
       * By default, the pins are unlocked unless waking
       * up from an LPMx.5 state in which case all GPIO
       * are previously locked.
       *
       */
    FRPMM_unlockLPM5(__MSP430_BASEADDRESS_PMM_FR5xx__);
    //Set P3.4, 3.5, 3.6, 3.7 as output pins.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 4, 5, 6, 7 as output
     */
  	FRGPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN4 + FRGPIO_PIN5 + FRGPIO_PIN6 + FRGPIO_PIN7
  	);
    //Set P3.5, 3.6, 3.7 Low.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 5, 6, 7 as Low
     */
  	FRGPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN5 + FRGPIO_PIN6 + FRGPIO_PIN7
  	);
    //Set P3.4 as High.
    /*
     * Base Address for Port 3
     * Select Port 3
     * Set Pin 4 as High
     */
  	FRGPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT3_R__,
  			FRGPIO_PORT_P3,
  			FRGPIO_PIN4
  	);

	  //Get Interrupt Status from the PMMIFG register.
	    /* Base Address of PMM,
	     * mask:
	     * 	FRPMM_PMMBORIFG
	     * 	FRPMM_PMMRSTIFG,
	     * 	FRPMM_PMMPORIFG,
	     * 	FRPMM_SVSLIFG,
	     * 	FRPMM_SVSHIFG
	     * 	FRPMM_PMMLPM5IFG,
	     * return STATUS_SUCCESS (0x01) or STATUS_FAIL (0x00)
	     */
	if (FRPMM_getInterruptStatus(__MSP430_BASEADDRESS_PMM_FR5xx__, FRPMM_PMMLPM5IFG))	// Was this device in LPMx.5 mode before the reset was triggered?
	{
	  //Clear Interrupt Flag from the PMMIFG register.
		/* Base Address of PMM,
		 * mask:
		 * 	FRPMM_PMMBORIFG
		 * 	FRPMM_PMMRSTIFG,
		 * 	FRPMM_PMMPORIFG,
		 * 	FRPMM_SVSLIFG,
		 * 	FRPMM_SVSHIFG
		 * 	FRPMM_PMMLPM5IFG,
		 * 	FRPMM_ALL
		 */
		FRPMM_clearInterrupt(__MSP430_BASEADDRESS_PMM_FR5xx__, FRPMM_PMMLPM5IFG);			// Clear the LPMx.5 flag
      	FRGPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT3_R__,
      			FRGPIO_PORT_P3,
      			FRGPIO_PIN5
      	);
    }

	if (FRPMM_getInterruptStatus(__MSP430_BASEADDRESS_PMM_FR5xx__, FRPMM_PMMRSTIFG))	// Was this reset triggered by the Reset flag?
	{
		FRPMM_clearInterrupt(__MSP430_BASEADDRESS_PMM_FR5xx__, FRPMM_PMMRSTIFG);			// Clear reset flag
	  	FRGPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT3_R__,
	  			FRGPIO_PORT_P3,
	  			FRGPIO_PIN6
	  	);
        __delay_cycles(1000000); 													// to include a visual delay.
  	  //Lock GPIO output states (before triggering a BOR)
  		/*
  		 * Base Address of PMM,
  		 * Forces all GPIO to retain their output
  		 * states during a reset.
  		 */
        FRPMM_lockLPM5(__MSP430_BASEADDRESS_PMM_FR5xx__);
	  //Trigger a software Brown Out Reset (BOR)
		/*
		 * Base Address of PMM,
		 * Forces the devices to perform a BOR.
		 */
        FRPMM_trigBOR(__MSP430_BASEADDRESS_PMM_FR5xx__);								// Software trigger a BOR.
    }

	if (FRPMM_getInterruptStatus(__MSP430_BASEADDRESS_PMM_FR5xx__, FRPMM_PMMBORIFG))	// Was this reset triggered by the BOR flag?
	{
		FRPMM_clearInterrupt(__MSP430_BASEADDRESS_PMM_FR5xx__, FRPMM_PMMBORIFG);			// Clear BOR flag
	  	FRGPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT3_R__,
	  			FRGPIO_PORT_P3,
	  			FRGPIO_PIN7
	  	);
        __delay_cycles(1000000); 													// to include a visual delay.
      	FRGPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT3_R__,
      			FRGPIO_PORT_P3,
      			FRGPIO_PIN4 + FRGPIO_PIN7
      	);

        FRPMM_lockLPM5(__MSP430_BASEADDRESS_PMM_FR5xx__);

	  //Disable SVSH
		/*
		 * Base Address of PMM,
		 * High-side SVS (SVSH) is disabled in LPM4.5. SVSH is
		 * always enabled in active mode and LPM0/1/2/3/4 and LPM3.5.
		 */
        FRPMM_disableSVSH(__MSP430_BASEADDRESS_PMM_FR5xx__);
  	  //Disable SVSL
  		/*
  		 * Base Address of PMM,
  		 * Low-side SVS (SVSL) is disabled in low power modes.
  		 * SVSL is always enabled in active mode and LPM0.
  		 */
        FRPMM_disableSVSL(__MSP430_BASEADDRESS_PMM_FR5xx__);
	  //Disable Regulator
		/*
		 * Base Address of PMM,
		 * Regulator is turned off when going to LPM3/4.
		 * System enters LPM3.5 or LPM4.5, respectively.
		 */
        FRPMM_regOff(__MSP430_BASEADDRESS_PMM_FR5xx__);
        __bis_SR_register(LPM4_bits); // Enter LPM4.5, This automatically locks
        							  //(if not locked already) all GPIO pins.
        							  //  and will set the LPM5 flag and set the LOCKLPM5 bit
        							  //  in the PM5CTL0 register upon wake up.
    }
//------------------
    while (1)
    {
    	__no_operation();             // Don't sleep
    }
}
