#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/i2c.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"

//*****************************************************************************
//!
//! This example shows how to configure the I2C module as a slave for
//! single byte transmission in interrupt driven mode. The address of the slave
//! module is set in this example.
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//!
//*****************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************
#define SLAVE_ADDRESS 0x48

unsigned char transmitData;

void main (void)
{
    //Stop WDT
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

    //Assign I2C pins to USCI_B0
    GPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT3_R__,
        GPIO_PORT_P3,
        GPIO_PIN1 + GPIO_PIN2
        );

    //Initialize I2C as a slave device
    I2C_slaveInit(__MSP430_BASEADDRESS_USCI_B0__,
        SLAVE_ADDRESS
        );

    //Set in transmit mode
    I2C_setMode(__MSP430_BASEADDRESS_USCI_B0__,
        I2C_TRANSMIT_MODE
        );

    //Enable I2C Module to start operations
    I2C_enable(__MSP430_BASEADDRESS_USCI_B0__);

    //Enable Start, Stop & Transmit interrupt
    I2C_enableInterrupt(__MSP430_BASEADDRESS_USCI_B0__,
        I2C_STOP_INTERRUPT +
        I2C_TRANSMIT_INTERRUPT
        );

    //Initialize transmit data buffer
    transmitData = 0xAB;

    //Enter low power mode 0 with interrupts enabled.
    __bis_SR_register(LPM0_bits + GIE);

    __no_operation();
}

//******************************************************************************
//
//This is the USCI_B0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR (void)
{
    switch (__even_in_range(UCB0IV,12)){
        //Vector  8: Stop condition received - STPIFG
        case USCI_I2C_UCSTPIFG:
        {
            //Increment TXData
            transmitData++;
            break;
        }

        //Vector 12: Transmit buffer empty - TXIFG
        case USCI_I2C_UCTXIFG:
        {
            //Transmit data
            I2C_slaveDataPut(__MSP430_BASEADDRESS_USCI_B0__,
                transmitData
                );
            break;
        }

        default:    break;
    }
}

