//******************************************************************************
// 
//   REFA - ADC12B, Using REFA Module to control Internal Reference
// 
//   This example shows how to configure the internal reference of REFA module
//   using the REFACTL0 control register. The ADC12B uses the internal 2.5V
//   reference and performs a single conversion on channel A0. The conversion
//   results are stored in ADC12BMEM0. Test by applying a voltage to channel A0,
//   then setting and running to a break point at the "__no_operation()"
//   instruction. To view the conversion results, open an ADC12B register window
//   in debugger and view the contents of ADC12BMEM0.
// 
//   NOTE: REFAMSTR bit in REFACTL0 regsiter is by default set to 1. This allows
//   the REFACTL0 control regsiters to configure the reference setting. The
//   legacy ADC12B reference control bits ADC12REFAON, ADC12REFA2_5, ADC12BTCOFF
//   and ADC12BREFAOUT are dont care in this case.
// 
//                 MSP430FR5969
//              -----------------
//          /|\|                 |
//           | |                 |
//           --|RST              |
//             |                 |
//             |          P1.0/A0|<- Vin
//             |                 |
// 
// 
//   This example uses the following peripherals and I/O signals.  You must
//   review these and change as needed for your own board:
//   - REFA peripheral
//   - ADC12B peripheral
//   - GPIO Port peripheral
//   - A0
//   
//   This example uses the following interrupt handlers.  To use this example
//   in your own application you must add these interrupt handlers to your
//   vector table.
//   - None.
// 
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/refa.h"
#include "driverlib/5xx_6xx/adc12b.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/frgpio.h"

void main (void)
{
    //Stop watchdog timer
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

	//Set P1.0 as Ternary Module Function Output.
	/*
	* Base Address for Port 1
	* Select Port 1
	* Set Pin 0 to output Ternary Module Function, (A0, C0, VREFA-, VeREFA-).
	*/
	FRGPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
		FRGPIO_PORT_P1,
		FRGPIO_PIN0,
		FRGPIO_TERNARY_MODULE_FUNCTION
	);

    //If ref generator busy, WAIT
    while (REFA_isRefGenBusy(__MSP430_BASEADDRESS_REF_A__)) ;
    //Select internal ref = 2.5V
    REFA_setReferenceVoltage(__MSP430_BASEADDRESS_REF_A__,
        REFA_VREF2_5V);
    //Internal Reference ON
    REFA_enableReferenceVoltage(__MSP430_BASEADDRESS_REF_A__);

    //Delay (~75us) for Ref to settle
    __delay_cycles(75);

	//Initialize the ADC12 Module
	/*
	* Base address of ADC12 Module
	* Use internal ADC12 bit as sample/hold signal to start conversion
	* USE MODOSC 5MHZ Digital Oscillator as clock source
	* Use default clock divider/pre-divider of 1
	 * Map to internal channel 0
	*/
	ADC12B_init(__MSP430_BASEADDRESS_ADC12_B__,
		ADC12B_SAMPLEHOLDSOURCE_SC,
		ADC12B_CLOCKSOURCE_ADC12OSC,
		ADC12B_CLOCKDIVIDER_1,
		ADC12B_CLOCKPREDIVIDER__1,
		ADC12B_MAPINTCH0);
    
	//Enable the ADC12B module
    ADC12B_enable(__MSP430_BASEADDRESS_ADC12_B__);
    
    /*
     * Base address of ADC12B Module
     * For memory buffers 0-7 sample/hold for 64 clock cycles
     * For memory buffers 8-15 sample/hold for 4 clock cycles (default)
     * Disable Multiple Sampling
     */
    ADC12B_setupSamplingTimer(__MSP430_BASEADDRESS_ADC12_B__,
        ADC12B_CYCLEHOLD_64_CYCLES,
        ADC12B_CYCLEHOLD_4_CYCLES,
        ADC12B_MULTIPLESAMPLESDISABLE);

    //Configure Memory Buffer
    /*
     * Base address of the ADC12B Module
     * Configure memory buffer 0
     * Map input A0 to memory buffer 0
     * Vref+ = Vref+ (INT)
     * Vref- = AVss
     */
    ADC12B_memoryConfigure(__MSP430_BASEADDRESS_ADC12_B__,
        ADC12B_MEMORY_0,
        ADC12B_INPUT_A0,
        ADC12B_VREFPOS_INTBUF_VREFNEG_VSS,
        ADC12B_NOTENDOFSEQUENCE,
        ADC12B_WINDOW_COMPARATOR_DISABLE,
        ADC12B_DIFFERENTIAL_MODE_DISABLE);

    while (1)
    {
        //Enable/Start sampling and conversion
        /*
         * Base address of ADC12B Module
         * Start the conversion into memory buffer 0
         * Use the single-channel, single-conversion mode
         */
        ADC12B_startConversion(__MSP430_BASEADDRESS_ADC12_B__,
            ADC12B_MEMORY_0,
            ADC12B_SINGLECHANNEL);

        //Poll for interrupt on memory buffer 0
        while (!ADC12B_getInterruptStatus(__MSP430_BASEADDRESS_ADC12_B__,
                   0,
                   ADC12IFG0));

        //SET BREAKPOINT HERE
        __no_operation();
    }
}

