#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/sd24_b.h"

//*****************************************************************************
//  MSP430F673x Demo - SD24_B, Single Conversion on a Group of 3 Channels
//
//  Description: This program uses the SD24_B module to perform a single
//  conversion on a group of 3 channels (0, 1 and 2). An SD24_B interrupt
//  occurs when the conversions have completed. Test by applying voltages
//  to the 3 input channels and setting a breakpoint at the line indicated
//  below. Run program until it reaches the breakpoint, then use the
//  debugger's watch window to view the conversion results. Results (upper 16
//  bits only) for all 3 channels are stored in the array "results".
//  ACLK = n/a, MCLK = SMCLK = DCO =  ~ 1.1MHz
//  //* For minimum Vcc required for SD24_B module - see datasheet          *//
//  //* 100nF cap between Vref and AVss is recommended when using 1.5V REF  *//
//
//                MSP430F673x
//            -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//   Vin1+ -->|SD0P0            |
//   Vin1- -->|SD0N0            |
//   Vin2+ -->|SD1P0            |
//   Vin2- -->|SD1N0            |
//   Vin3+ -->|SD2P0            |
//   Vin3- -->|SD2N0            |
//            |            VREF |---+
//            |                 |   |
//            |                 |  -+- 100nF
//            |                 |  -+-
//            |                 |   |
//            |            AVss |---+
//            |                 |
//
//  M. Swanson
//  Texas Instruments, Inc
//  December 2011
//  Built with CCS Version: 5.1.0 and IAR Embedded Workbench Version: 5.40.1
//*****************************************************************************

/* Array to store SD24_B conversion results */
unsigned long results[3];

void main(void)
{
	 WDT_hold(__MSP430_BASEADDRESS_WDT_A__); // Stop WDT

	 SD24B_init(__MSP430_BASEADDRESS_SD24_B__,
	 		SD24B_CLOCKSOURCE_SMCLK,
	 		SD24B_PRECLOCKDIVIDER_1,
	 		SD24B_CLOCKDIVIDER_1,
	 	 	SD24B_REF_INTERNAL);		    // Select internal REF
                                            // Select SMCLK as SD24_B clock source

	 SD24B_configureConverter(__MSP430_BASEADDRESS_SD24_B__,
	 	 	SD24B_CONVERTER_0,
	 	 	SD24B_ALIGN_RIGHT,
	 	 	SD24B_CONVERSION_SELECT_GROUP1,
	 	 	SD24B_CONTINUOUS_MODE);

	 SD24B_configureConverter(__MSP430_BASEADDRESS_SD24_B__,
	 	 	SD24B_CONVERTER_1,
	 	 	SD24B_ALIGN_RIGHT,
	 	 	SD24B_CONVERSION_SELECT_GROUP1,
	 	 	SD24B_CONTINUOUS_MODE);

	 SD24B_configureConverter(__MSP430_BASEADDRESS_SD24_B__,
	 	 	SD24B_CONVERTER_2,
	 	 	SD24B_ALIGN_RIGHT,
	 	 	SD24B_CONVERSION_SELECT_GROUP1,
	 	 	SD24B_CONTINUOUS_MODE);


	 SD24B_enableInterrupt(__MSP430_BASEADDRESS_SD24_B__,
	 		SD24B_CONVERTER_2,
	 		SD24_CONVERTER_INTERRUPT	);                   // Enable channel 2 interrupt

    __delay_cycles(0x3600);                 // Delay for 1.5V REF startup

    while (1)
    {

    	SD24B_startGroupConversion(__MSP430_BASEADDRESS_SD24_B__,
    	    SD24B_CONVERTER_1);								// Set bit to start conversion

    	__bis_SR_register(LPM0_bits | GIE); // Enter LPM0 w/ interrupts

    	SD24B_stopGroupConversion(__MSP430_BASEADDRESS_SD24_B__,
    		SD24B_CONVERTER_1);	  			      			// Clear bit for next conversion

        __no_operation();                   // SET BREAKPOINT HERE
    }
}

#pragma vector=SD24B_VECTOR
__interrupt void SD24BISR(void)
{
    switch (SD24BIV)
    {
        case SD24BIV_SD24OVIFG:             // SD24MEM Overflow
            break;
        case SD24BIV_SD24TRGIFG:            // SD24 Trigger IFG
            break;
        case SD24BIV_SD24IFG0:              // SD24MEM0 IFG
            break;
        case SD24BIV_SD24IFG1:              // SD24MEM1 IFG
            break;
        case SD24BIV_SD24IFG2:              // SD24MEM2 IFG
            results[0] = SD24B_getResults(__MSP430_BASEADDRESS_SD24_B__,
            	SD24B_CONVERTER_0);							// Save CH0 results (clears IFG)
            results[1] = SD24B_getResults(__MSP430_BASEADDRESS_SD24_B__,
                SD24B_CONVERTER_1);							// Save CH1 results (clears IFG)
            results[2] = SD24B_getResults(__MSP430_BASEADDRESS_SD24_B__,
            	SD24B_CONVERTER_2);							// Save CH2 results (clears IFG)
            break;
    }

    __bic_SR_register_on_exit(LPM0_bits);   // Exit LPM0
}

