//******************************************************************************
//! MSP430F543xA Demo - Timer_A, Toggle P1.0, Overflow ISR, 32kHz ACLK
//! 
//! Description: Toggle P1.0 using software and the Timer_B overflow ISR.
//! In this example an ISR triggers when TB overflows. Inside the ISR P1.0
//! is toggled. Toggle rate is exactly 0.25Hz = [32kHz/FFFFh]/2. Proper use of the
//! TBIV interrupt vector generator is demonstrated.
//! ACLK = TBCLK = 32kHz, MCLK = SMCLK = default DCO ~ 1.045MHz
//! 
//! 		MSP430F5438A
//! 		---------------
//! 	/|\|               |
//! 	 | |               |
//! 	 --|RST            |
//!        |               |
//!        |           P1.0|-->LED
//! 
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - Timer peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - Timer_B7
//!
//
//*****************************************************************************

#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/timera.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"

void main (void)
{
    //Stop WDT
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Start timer in continuous mode sourced by ACLK
    TimerA_configureContinuousMode( __MSP430_BASEADDRESS_T0B7__,
        TIMERA_CLOCKSOURCE_ACLK,
        TIMERA_CLOCKSOURCE_DIVIDER_1,
        TIMERA_TAIE_INTERRUPT_ENABLE,
        TIMERA_DO_CLEAR
        );

    TimerA_startCounter( __MSP430_BASEADDRESS_T0B7__,
            TIMERA_CONTINUOUS_MODE
            );

    //Enter LPM3, enable interrupts
    __bis_SR_register(LPM3_bits + GIE);

    //For debugger
    __no_operation();
}

//******************************************************************************
//
//This is the Timer_B7 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=TIMERB1_VECTOR
__interrupt void TIMERB1_ISR (void)
{
    //Any access, read or write, of the TBIV register automatically resets the
    //highest "pending" interrupt flag
    switch ( __even_in_range(TBIV,14) ){
        case  0: break;                          //No interrupt
        case  2: break;                          //CCR1 not used
        case  4: break;                          //CCR2 not used
        case  6: break;                          //CCR3 not used
        case  8: break;                          //CCR4 not used
        case 10: break;                          //CCR5 not used
        case 12: break;                          //CCR6 not used
        case 14:
            //Toggle P1.0					 // overflow
            GPIO_toggleOutputOnPin(
            __MSP430_BASEADDRESS_PORT1_R__,
            GPIO_PORT_P1,
            GPIO_PIN0
            );
            break;
        default: break;
    }
}

