//******************************************************************************
//  MSP430F51x2 Demo - TimerD0, Dual Input capture mode, Normal timer mode, 
//                                Period measurement
//  Description: This code example implements input capture in dual capture 
//  mode using TimerD in normal timer mode. Dual capture mode of TimerD is 
//  demonstrated here. This example captures two consequent rising edges and 
//  computes the Period of the input signal. The TD0.0 input signal is generated 
//  by the Timer A0 module (TA0.0) and the freq of this PWM input signal can be 
//  changed by modifying the TA0CCR0 register. 
//  
//  ACLK = LFXT1 = 32kHz; SMCLK = MCLK = 12MHz 
//
//                 MSP430F51x2
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |       P1.6/TD0.0|<-- CCI0A <-| 
//            |       P3.7/TA0.0|--> CCR0 -->| 
//            |                 |
//            |             P3.0|--> MCLK = 12MHz DCO
//            |             P3.1|--> SMCLK = 12MHz DCO
//            |             P3.2|--> ACLK = 32kHz LFXT1
//
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/timerd.h"
#include "driverlib/5xx_6xx/timera.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"
#include "driverlib/5xx_6xx/ucs.h"
#include "driverlib/5xx_6xx/pmm.h"
#include "driverlib/5xx_6xx/pmap.h"

unsigned int REdge1, REdge2, Period;

const unsigned char port_mapping[] = {
    //Port P3:
		PM_TD0CLKMCLK,
		PM_TD0_0SMCLK,
		PM_TD1OUTH,
		PM_NONE,
		PM_NONE,
		PM_NONE,
		PM_NONE,
		PM_TA0_0,
	};


void main(void)
{
  // Stop WDT
  WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

  // Configure XT1
  GPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORTJ_R__,
	            		  GPIO_PORT_PJ,
	            		  GPIO_PIN4 + GPIO_PIN5
	            		  );
  
  UCS_LFXT1Start ( __MSP430_BASEADDRESS_UCS__,
  		  UCS_XT1_DRIVE3,
  		  UCS_XCAP_3
        );


  // Increase Vcore setting to level1 to support fsystem=12MHz
  // NOTE: Change core voltage one level at a time..
  PMM_setVCore (__MSP430_BASEADDRESS_PMM__,
		  PMM_CORE_LEVEL_1);
  
  // Initialize DCO to 12MHz
  UCS_initFLLSettle( __MSP430_BASEADDRESS_UCS__,
    		  12000,
    		  374);


  // Setup Port Pins 
  // P3.0 - P3.2 output
  // Port map P3.0 - P3.2
  GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT3_R__,
                		  GPIO_PORT_P3,
                		  GPIO_PIN0 + GPIO_PIN1 + GPIO_PIN2
                		  );


 PMAP_configurePorts(__MSP430_BASEADDRESS_PORT_MAPPING__,
                (const unsigned char *)port_mapping,
                (unsigned char *)&P3MAP0,
                1,
                PMAP_DISABLE_RECONFIGURATION
                );

  // Configure ports TD0.0 input and TA0.0 output
  // TD0.0 input
  // TD0.0 option select
  // TA0.0 output
  // TA0.0 option select
  GPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT1_R__,
            		  GPIO_PORT_P1,
            		  GPIO_PIN6
            		  );

  GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT3_R__,
              		  GPIO_PORT_P3,
              		  GPIO_PIN7
              		  );
  
  // Configure LED on P1.0, output and driving low
  GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT1_R__,
                 		  GPIO_PORT_P1,
                 		  GPIO_PIN0
                 		  );
  GPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
               		  GPIO_PORT_P1,
               		  GPIO_PIN0
               		  );
    
  // Configure TA0.0 compare output, 1kHz freq, 50% dutycycle 
  TimerA_initCompare(__MSP430_BASEADDRESS_T0A3__,
                      TIMERA_CAPTURECOMPARE_REGISTER_0,
                      TIMERA_CAPTURECOMPARE_INTERRUPT_DISABLE,
                      TIMERA_OUTPUTMODE_TOGGLE,
                      32
                      );

  TimerA_configureUpMode(__MSP430_BASEADDRESS_T0A3__,
     		  TIMERA_CLOCKSOURCE_ACLK,
               TIMERA_CLOCKSOURCE_DIVIDER_1,
               32,
               TIMERA_TAIE_INTERRUPT_DISABLE,
               TIMERA_CAPTURECOMPARE_INTERRUPT_DISABLE,
               TIMERA_DO_CLEAR
               );

  TimerA_startCounter(__MSP430_BASEADDRESS_T0A3__,
		  TIMERA_UP_MODE
		  );

  TimerD_configureContinuousMode(__MSP430_BASEADDRESS_T0D3__,
  		  	  	  	  	  	 TIMERD_CLOCKSOURCE_SMCLK,
  		  	  	  	  	  	 TIMERD_CLOCKSOURCE_DIVIDER_1,
  		  	  	  	  	  	 TIMERD_CLOCKINGMODE_EXTERNAL_CLOCK,
  		  	  	  	  	  	 TIMERD_TDIE_INTERRUPT_DISABLE,
  		  	  	  	  	  	 TIMERD_DO_CLEAR
  		  	  	  	  		);

  TimerD_startCounter(__MSP430_BASEADDRESS_T0D3__,
  		  TIMERD_CONTINUOUS_MODE
  		  );

  TimerD_initCapture(__MSP430_BASEADDRESS_T0D3__,
  		  TIMERD_CAPTURECOMPARE_REGISTER_0,
  		  TIMERD_CAPTUREMODE_RISING_EDGE,
  		  TIMERD_CAPTURE_INPUTSELECT_CCIxA,
  		  TIMERD_CAPTURE_SYNCHRONOUS,
  		  TIMERD_CAPTURECOMPARE_INTERRUPT_ENABLE,
  		  TIMERD_OUTPUTMODE_OUTBITVALUE,
  		  TIMERD_DUAL_CAPTURE_MODE
  		  );

  
  while(1)
  {
      __bis_SR_register(LPM0_bits + GIE);   // Enter LPM0
      __no_operation();                     // For debugger  
      // On exiting LPM0
      if (TIMERD_CAPTURE_OVERFLOW == TimerD_getCaptureCompareInterruptStatus
          		  	  (__MSP430_BASEADDRESS_T0D3__,
            	  	  	  TIMERD_CAPTURECOMPARE_REGISTER_1,
            	  	  	  TIMERD_CAPTURE_OVERFLOW
            	  	  	  )              )     // Check for Capture Overflow
          while(1);                         // Loop Forever
     
      Period = REdge2-REdge1;               // Measured Period 
      __no_operation();                     // BREAKPOINT HERE - measured period ~ 2ms 
                                            // Period ~ 2m*TD0 clock counts    
  }
}
  
// Timer0_D0 Interrupt Vector
#pragma vector=TIMER0_D0_VECTOR
__interrupt void TIMER0_D0_ISR (void)
{
  REdge1 = TimerD_getCaptureCompareLatchCount(__MSP430_BASEADDRESS_T0D3__,
  	          				TIMERD_CAPTURECOMPARE_REGISTER_0);

  REdge2 = TimerD_getCaptureCompareCount(__MSP430_BASEADDRESS_T0D3__,
          				TIMERD_CAPTURECOMPARE_REGISTER_0);
  __bic_SR_register_on_exit(LPM0_bits);      // Exit LPM0 
}


