//******************************************************************************
//  MSP430F51x2 Demo - TimerD0, Hi-Res Free Running mode, PWM TD0.0-2, TDCLK = 64Mhz
//
//  Description: This code example shows how to configure TimerD in Hi-Res 
//  Free Running mode. The TD0HCTL1 register with the TDHCLKTRIMx, TDHCLKSRx and 
//  TDHCLKRx bit settings are configured with the calibration data in the TLV 
//  table. 
//
//  SMCLK = MCLK = DCOclock = default DCO; TDCLK(TLV Cal Data)= 64Mhz
//  Note: 1. TDHMx=00(8x) should be used to use the 64Mhz TimerD TLV calibration data
//        2. Use code example with hal_tlv.c and hal_tlv.h files to read the TimerD TLV data 
//
//                 MSP430F51x2
//             -----------------
//         /|\|              XIN|-
//          | |                 | 
//          --|RST          XOUT|-
//            |                 |
//            |       P1.6/TD0.0|--> CCR0 - 50% dutycycle; PWM period = (2 x 128)/64MHz~ 4us = 250kHz
//            |       P1.7/TD0.1|--> CCR1 - 25% dutycycle; ON period ~ 32/64MHz~ 500ns
//            |       P2.0/TD0.2|--> CCR2 - 75% dutycycle; ON period ~ 96/64MHz~ 1500ns
//
//******************************************************************************

#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/timerd.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"
#include "driverlib/5xx_6xx/tlv.h"

void main(void)
{
  struct s_TLV_Timer_D_Cal_Data * pTD0CAL;  // Structure initialized in tlv.h
  unsigned char bTD0CAL_bytes;                  
  
  // Stop WDT
  WDT_hold(__MSP430_BASEADDRESS_WDT_A__);
  
  //Get TimerD0 Cal Values (instance 0)
  TLV_getInfo(TLV_TAG_TIMER_D_CAL, 0,
		  &bTD0CAL_bytes,
		  (unsigned int **) &pTD0CAL
		  );

  if(bTD0CAL_bytes == 0x0)
  {
      // No TimerD free running cal data found
      while(1);                             // Loop here
  }  

  // Configure TimerD in Hi-Res Free Running Mode
  TimerD_configureHighResGeneratorInFreeRunningMode(__MSP430_BASEADDRESS_T0D3__,
		  TIMERD_HIGHRES_64MHZ
		  );

  // Configure TD0.x GPIO pins
  GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
    		  GPIO_PORT_P1,
    		  GPIO_PIN6 + GPIO_PIN7
    		  );

  GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT2_R__,
    		  GPIO_PORT_P2,
    		  GPIO_PIN0
    		  );
  // Configure the CCRx blocks
 TimerD_initCompare(__MSP430_BASEADDRESS_T0D3__,
                TIMERD_CAPTURECOMPARE_REGISTER_1,
                TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
                TIMERD_OUTPUTMODE_TOGGLE_SET,
                32
                );

  TimerD_initCompare(__MSP430_BASEADDRESS_T0D3__,
                TIMERD_CAPTURECOMPARE_REGISTER_2,
                TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
                TIMERD_OUTPUTMODE_TOGGLE_SET,
                96
                );

  TimerD_initCompare(__MSP430_BASEADDRESS_T0D3__,
                  TIMERD_CAPTURECOMPARE_REGISTER_0,
                  TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
                  TIMERD_OUTPUTMODE_TOGGLE,
                  128-1
                  );

 TimerD_configureUpMode(__MSP430_BASEADDRESS_T0D3__,
		  TIMERD_CLOCKSOURCE_EXTERNAL_TDCLK,
          TIMERD_CLOCKSOURCE_DIVIDER_1,
          TIMERD_CLOCKINGMODE_EXTERNAL_CLOCK,
          128-1,
          TIMERD_TDIE_INTERRUPT_DISABLE,
          TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
          TIMERD_DO_CLEAR
          );

 TimerD_startCounter(__MSP430_BASEADDRESS_T0D3__,
		 TIMERD_UP_MODE
		 );

  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0
  __no_operation();                         // For debugger  
}
  
