//******************************************************************************
//  MSP430F51x2 Demo - TimerD0, Hi-Res Regulated Mode, PWM TD0.0-2, DCO SMCLK
//
//  Description: This code example shows how to configure TimerD in Hi-Res
//  regulated mode. The reference input clock freq (in this e.g. SMCLK=12MHz)
//  is multiplied by a factor of 16 to generate a hi-res freq of InputClk x 16
//  (in this e.g. 12MHz x 16 = 192Mhz. With TD0CCR0=128-1, TD0.0 outputs a
//  square wave of freq 746kHz and TD0.1 and TD0.2 output PWM waveforms with
//  dutycycles of 25% and 75% respectively.
//
//  NOTE: For using multiply factor 16x in hi-res calibration mode, make sure
//  input ref clock is in the following freq range: 8Mhz to 16MHz.
//
//  ACLK=LFXT1=32kHz; SMCLK=MCLK=12MHz; TDCLK(Hi-Res)= SMCLK x 16 = 192MHz
//
//                 MSP430F51x2
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |       P1.6/TD0.0|--> CCR0 - 50% dutycycle; PWM period = (2 x 128)/(12MHz x 16)~ 1.34us = 746kHz
//            |       P1.7/TD0.1|--> CCR1 - 25% dutycycle; ON period ~ 32/(12MHz x 16)~ 167ns
//            |       P2.0/TD0.2|--> CCR2 - 75% dutycycle; ON period ~ 96/(12MHz x 16)~ 500ns
//
//******************************************************************************
#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/timerd.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"
#include "driverlib/5xx_6xx/pmm.h"
#include "driverlib/5xx_6xx/ucs.h"

void main(void)
{

  WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

  // Configure XT1
  // Port select XT1
  GPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORTJ_R__,
	      		  GPIO_PORT_PJ,
	      		  GPIO_PIN4 + GPIO_PIN5
	      		  );
  
  UCS_LFXT1Start ( __MSP430_BASEADDRESS_UCS__,
			  UCS_XT1_DRIVE3,
			  UCS_XCAP_3
	      );

  // Increase Vcore setting to level1 to support fsystem=12MHz
  PMM_setVCore (__MSP430_BASEADDRESS_PMM__,
                  PMMCOREV_1);
  
  // Initialize DCO to 12MHz
  UCS_initFLLSettle( __MSP430_BASEADDRESS_UCS__,
                  12000,
                  374);
  
  // Configure TimerD in Hi-Res Regulated Mode
  TimerD_configureHighResGeneratorInRegulatedMode(
                  __MSP430_BASEADDRESS_T0D3__,
                  TIMERD_CLOCKSOURCE_SMCLK,
                  TIMERD_CLOCKSOURCE_DIVIDER_1,
                  TIMERD_CLOCKINGMODE_HIRES_LOCAL_CLOCK,
                  TIMERD_CLOCKSOURCE_DIVIDER_1,
                  TIMERD_HIGHRES_CLK_MULTIPLY_FACTOR_16x
                  );
  
  TimerD_enableHighResInterrupt(__MSP430_BASEADDRESS_T0D3__,
                  TIMERD_HIGH_RES_FREQUENCY_LOCK
                  );


  // Configure TimerD in Hi-Res Regulated Mode
  // TDCLK=SMCLK=12MHz=Hi-Res input clk select
  // Configure the CCRx blocks
  // Configure the CCRx blocks
   TimerD_initCompare(__MSP430_BASEADDRESS_T0D3__,
                     TIMERD_CAPTURECOMPARE_REGISTER_0,
                     TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
                     TIMERD_OUTPUTMODE_TOGGLE,
                     128-1
                     );

  
    TimerD_initCompare(__MSP430_BASEADDRESS_T0D3__,
                    TIMERD_CAPTURECOMPARE_REGISTER_1,
                    TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
                    TIMERD_OUTPUTMODE_TOGGLE_SET,
                    32
                    );

    TimerD_initCompare(__MSP430_BASEADDRESS_T0D3__,
                    TIMERD_CAPTURECOMPARE_REGISTER_2,
                    TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
                    TIMERD_OUTPUTMODE_TOGGLE_SET,
                    96
                    );

    TimerD_configureUpMode(__MSP430_BASEADDRESS_T0D3__,
    		 TIMERD_CLOCKSOURCE_SMCLK,
              TIMERD_CLOCKSOURCE_DIVIDER_1,
              TIMERD_CLOCKINGMODE_HIRES_LOCAL_CLOCK,
              128-1,
              TIMERD_TDIE_INTERRUPT_DISABLE,
              TIMERD_CAPTURECOMPARE_INTERRUPT_DISABLE,
              TIMERD_DO_CLEAR
              );
    TimerD_startCounter(__MSP430_BASEADDRESS_T0D3__,
      		 TIMERD_UP_MODE
      		 );

  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0
  __no_operation();                         // For debugger
}

// Timer0_D1 Interrupt Vector (TDIV) handler
#pragma vector=TIMER0_D1_VECTOR
__interrupt void TIMER0_D1_ISR(void)
{
  switch(__even_in_range(TD0IV,30))
  {
    case  0: break;                          // No interrupt
    case  2: break;                          // CCR1 not used
    case  4: break;                          // CCR2 not used
    case  6: break;                          // reserved
    case  8: break;                          // reserved
    case 10: break;                          // reserved
    case 12: break;                          // reserved
    case 14: break;
    case 16: break;
    case 18:                                 // Clock fail low
      while(1);                              // Input ref clock freq too low; trap here
    case 20:                                 // Clock fail high
      while(1);                              // Input ref clock freq too high; trap here
    case 22:                                 // Hi-res freq locked
      // Hi-Res freq locked; now configure ports to output PWMs at TD0.0/1/2
      // P1.6,7 option select
      // P1.6,7 output
      // P2.0 options select
      // P2.0 output
      GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT1_R__,
	      		  GPIO_PORT_P1,
	      		  GPIO_PIN6 + GPIO_PIN7
	      		  );
      
      GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT2_R__,
	      		  GPIO_PORT_P2,
	      		  GPIO_PIN0
	      		  );
      break;
    case 24: break;                          // Hi-res freq unlocked
    case 26: break;                          // reserved
    case 28: break;                          // reserved
    case 30: break;                          // reserved
    default: break;
  }
}


