#include "inc/hw_memmap.h"
#include "driverlib/5xx_6xx/uart.h"
#include "driverlib/5xx_6xx/ucs.h"
#include "driverlib/5xx_6xx/wdt.h"
#include "driverlib/5xx_6xx/gpio.h"

//******************************************************************************
//! This example shows how to configure the UART module to receive data in
//! idle line multiprocessor mode. To echo a received character, RX ISR is used.
//!
//!                MSP430F5438A
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |                 |
//!            |     P3.4/UCA0TXD|------------>
//!            |                 | 9600
//!            |     P3.5/UCA0RXD|<------------
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UART peripheral
//! - GPIO Port peripheral (for UART pins)
//! - UCA0TXD
//! - UCA0RXD
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR.
//******************************************************************************
#define BAUD_RATE                               9600
#define RECEIVE_DATA_COUNT                      0x02
#define UART_MULTIPROCESSOR_MODE_ADDRESS        0xAA

unsigned char receivedData = 0x00;
unsigned char transmitAddress = UART_MULTIPROCESSOR_MODE_ADDRESS,
              receiveDataCount = 0x00;

void main (void)
{
    //Stop WDT
    WDT_hold(__MSP430_BASEADDRESS_WDT_A__);

    //P3.4,5 = USCI_A0 TXD/RXD
    GPIO_setAsPeripheralModuleFunctionInputPin(__MSP430_BASEADDRESS_PORT3_R__,
        GPIO_PORT_P3,
        GPIO_PIN4 + GPIO_PIN5
        );

    //Initialize USCI UART module
    if ( STATUS_FAIL == UART_init(__MSP430_BASEADDRESS_USCI_A0__,
             UART_CLOCKSOURCE_SMCLK,
             UCS_getSMCLK(__MSP430_BASEADDRESS_UCS__),
             BAUD_RATE,
             UART_NO_PARITY,
             UART_MSB_FIRST,
             UART_ONE_STOP_BIT,
             UART_IDLE_LINE_MULTI_PROCESSOR_MODE,
             UART_OVERSAMPLING_BAUDRATE_GENERATION
             )){
        return;
    }

    //Enable UART module for operation
    UART_enable(__MSP430_BASEADDRESS_USCI_A0__);

    //Set USCI UART in dormant/sleep mode
    UART_setDormant(__MSP430_BASEADDRESS_USCI_A0__);

    //Enable Receive Interrupt
    UART_enableInterrupt(__MSP430_BASEADDRESS_USCI_A0__,
        UART_RECEIVE_INTERRUPT
        );

    //Enter LPM3, interrupts enabled
    __bis_SR_register(LPM3_bits + GIE);
    __no_operation();
}

//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:

            //USCI_A0 TX buffer ready?
            while (!UART_getInterruptStatus(__MSP430_BASEADDRESS_USCI_A0__,
                       UART_TRANSMIT_INTERRUPT_FLAG)
                   ) ;

            //Receive the data
            receivedData = UART_receiveData(__MSP430_BASEADDRESS_USCI_A0__);

            if (receiveDataCount){
                receiveDataCount--;
                if (0x00 == receiveDataCount){
                    //Set USCI module in dormant mode
                    UART_setDormant(__MSP430_BASEADDRESS_USCI_A0__);
                }
            } else   {
                if (UART_MULTIPROCESSOR_MODE_ADDRESS == receivedData){
                    //Wake up USCI module from dormant mode
                    UART_resetDormant(__MSP430_BASEADDRESS_USCI_A0__);
                    receiveDataCount = RECEIVE_DATA_COUNT;
                }
            }

            //Send data back "echo"
            UART_transmitData(__MSP430_BASEADDRESS_USCI_A0__,
            receivedData
            );

            break;
        default: break;
    }
}

