//******************************************************************************
//!
//! ADC10_A - Sample A1 Input, Signed Data, LED ON if A1 > 0.5AVcc
//!
//!  MSP430F550x Demo
//!  A single sample is made on A0 with SIGNED reference to AVcc. Software sets
//!  ADC10SC to start sample and conversion - ADC10SC automatically cleared at
//!  EOC. ADC10_A internal oscillator times sample (16x) and conversion. In
//!  Mainloop MSP430 waits in LPM0 to save power until ADC10_A conversion complete,
//!  ADC10_A_ISR will force exit from LPM0 in Mainloop on reti. If A1 > 0.5*AVcc,
//!  P1.0 set, else reset. To see the full extent of the negative conversion,
//!  add a watch to ADC_Result, and anything < 0.5&AVcc will be negative. Note,
//!  that a small change like +-1 voltage change ill result in large changes in
//!  this scenario because the least significant bits are 0.
//!
//!                MSP430F550x
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |          P6.1/A1|<- Sample Signal
//!            |             P1.0|-> LED
//!            |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - ADC10_A peripheral
//! - GPIO Port peripheral
//! - A1
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - ADC10_A_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "adc10_a.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Stop Watchdog Timer
    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Initialize the ADC10_A Module
    /*
     * Base Address for the ADC10_A Module
     * Use internal ADC10_A bit as sample/hold signal to start conversion
     * USE MODOSC 5MHZ Digital Oscillator as clock source
     * Use default clock divider of 1
     */
    ADC10_A_init(ADC10_A_BASE,
        ADC10_A_SAMPLEHOLDSOURCE_SC,
        ADC10_A_CLOCKSOURCE_ADC10OSC,
        ADC10_A_CLOCKDIVIDER_1);
    
    ADC10_A_enable(ADC10_A_BASE);
    
    /*
     * Base Address for the ADC10_A Module
     * Sample/hold for 16 clock cycles
     * Do not enable Multiple Sampling
     */
    ADC10_A_setupSamplingTimer(ADC10_A_BASE,
        ADC10_A_CYCLEHOLD_16_CYCLES,
        ADC10_A_MULTIPLESAMPLESDISABLE);
    //Convert data into signed, 2's complement format
    ADC10_A_setDataReadBackFormat(ADC10_A_BASE,
        ADC10_A_SIGNED_2SCOMPLEMENT);

    //Configure Memory Buffer
    /*
     * Base Address for the ADC10_A Module
     * Use input A1
     * Use positive reference of AVcc
     * Use negative reference of AVss
     */
    ADC10_A_memoryConfigure(ADC10_A_BASE,
        ADC10_A_INPUT_A1,
        ADC10_A_VREFPOS_AVCC,
        ADC10_A_VREFNEG_AVSS);

    //Enable Memory Buffer Interrupt
    ADC10_A_enableInterrupt(ADC10_A_BASE,
        ADC10IE0);

    while (1)
    {
        __delay_cycles(5000);

        //Enable and Start the conversion
        //in Single-Channel, Single-Conversion Mode
        ADC10_A_startConversion(ADC10_A_BASE,
            ADC10_A_SINGLECHANNEL);

        //LPM0, ADC10_A_ISR will force exit
        __bis_SR_register(CPUOFF + GIE);
        //For debug only
        __no_operation();

        
    }
}

//ADC10_A interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_A_ISR (void)
{
	switch (__even_in_range(ADC10IV,12)){
        case  0: break; //No interrupt
        case  2: break; //conversion result overflow
        case  4: break; //conversion time overflow
        case  6: break; //ADC10HI
        case  8: break; //ADC10LO
        case 10: break; //ADC10IN
        case 12:        //ADC10IFG0
            //Move Results from Memory Buffer
            //(Automatically clears ADC10IFG0 by reading memory buffer)
            if (ADC10_A_getResults(ADC10_A_BASE)  < 0){
	            //Clear P1.0 LED off
	            GPIO_setOutputLowOnPin(GPIO_PORT_P1,
	                GPIO_PIN0
	                );
	        } else   {
	            //Set P1.0 LED on
	            GPIO_setOutputHighOnPin(
	                GPIO_PORT_P1,
	                GPIO_PIN0
	                );
	        }
            //Clear CPUOFF bit from 0(SR)
            __bic_SR_register_on_exit(CPUOFF);
            break;
        default: break;
    }
}
