//******************************************************************************
//! COMPB - CB Interrupt, Input compared against 1.5V Internal Reference
//!
//!  Use CompB and internal reference to determine if input 'Vcompare' is high
//!  of low.  For the first time, when Vcompare exceeds the 1.5V internal
//!  reference, CBIFG is set and device enters the CompB ISR. In the ISR CBIES
//!  is toggled such that when Vcompare is less than 1.5V internal reference,
//!  CBIFG is set. LED is toggled inside the CompB ISR
//!
//!                 MSP430x552x
//!             ------------------
//!         /|\|                  |
//!          | |                  |
//!          --|RST       P6.0/CB0|<- Vcompare
//!            |              P1.0|-> LED 'ON'(Vcompare>1.5V); 'OFF'(Vcompare<1.5V)
//!            |                  |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - COMPB peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - COMP_B_B_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "comp_b.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Initialize the Comparator B module
    /*
     * Base Address of Comparator B,
     * Pin CB0 to Positive(+) Terminal
     * Reference Voltage to Negative(-) Terminal
     * Normal Power Mode
     * Output Filter On with minimal delay
     * Non-Inverted Output Polarity
     */
    COMP_B_init(COMP_B_BASE,
        COMP_B_INPUT0,
        COMP_B_VREF,
        COMP_B_POWERMODE_NORMALMODE,
        COMP_B_FILTEROUTPUT_DLYLVL1,
        COMP_B_NORMALOUTPUTPOLARITY
        );

    //Set the reference voltage that is being supplied to the (-) terminal
    /*
     * Base Address of Comparator B,
     * Reference Voltage of 1.5 V,
     * Lower Limit of 1.5*(32/32) = 1.5V,
     * Upper Limit of 1.5*(32/32) = 1.5V,
     * Static Mode Accuracy
     */
    COMP_B_setReferenceVoltage(COMP_B_BASE,
        COMP_B_VREFBASE1_5V,
        32,
        32,
        COMP_B_ACCURACY_STATIC
        );
    //Enable Interrupts
    /*
     * Base Address of Comparator B,
     * Enable CompB Interrupt on default rising edge for CBIFG
     */

    COMP_B_enableInterrupt(COMP_B_BASE,
        CBIE
        );

    //Allow power to Comparator module
    COMP_B_enable(COMP_B_BASE);

    //Enter LPM4 with inetrrupts enabled
    __bis_SR_register(LPM4_bits + GIE);

    //For debug
    __no_operation();
}

//******************************************************************************
//
//This is the COMP_B_VECTOR interrupt vector service routine.
//
//******************************************************************************
#pragma vector=COMP_B_VECTOR
__interrupt void COMP_B_B_ISR (void)
{
    //Toggle the edge at which an interrupt is generated
    COMP_B_interruptToggleEdgeDirection(COMP_B_BASE);

    //Clear Interrupt flag
    COMP_B_clearInterrupt(COMP_B_BASE, CBIFG);

    //Toggle P1.0
    GPIO_toggleOutputOnPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );
}
