//******************************************************************************
//! COMPB - LED Toggle in LPM4, Ultra Low Power Mode, Input compared against 0.5*Vcc
//!
//!  Use CompB and shared reference to determine if input 'Vcompare' is high of
//!  low.  When Vcompare exceeds Vcc*1/2 CBOUT goes high and when Vcompare is
//!  less than Vcc*1/2 then CBOUT goes low. Connect P1.6/CBOUT to P1.0 externally
//!  to see the LED toggle accordingly.
//!
//!             MSP430x552x
//!         ------------------
//!     /|\|                  |
//!      | |                  |
//!      --|RST       P6.0/CB0|<- Vcompare
//!        |        P1.6/CBOUT|--> 'high'(Vcompare>Vcc*1/2); 'low'(Vcompare<Vcc*1/2)
//!        |                  | |
//!        |              P1.0|_| LED 'ON'(Vcompare>Vcc*1/2); 'OFF'(Vcompare<Vcc*1/2)
//!        |                  |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - COMPB peripheral
//! - GPIO Port peripheral
//! - CB0
//! - CBOUT
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "comp_b.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);


    //Select CBOUT function on P1.6/CBOUT and set P1.6 output direction
    GPIO_setAsPeripheralModuleFunctionOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN6
        );

    //Initialize the Comparator B module
    /*
     * Base Address of Comparator B,
     * Pin CB0 to Positive(+) Terminal,
     * Reference Voltage to Negative(-) Terminal,
     * Ultra-Low Power Mode,
     * Output Filter On with minimal delay,
     * Non-Inverted Output Polarity
     */
    COMP_B_init(COMP_B_BASE,
        COMP_B_INPUT0,
        COMP_B_VREF,
        COMP_B_POWERMODE_ULTRALOWPOWER,
        COMP_B_FILTEROUTPUT_DLYLVL1,
        COMP_B_NORMALOUTPUTPOLARITY
        );
    //Set the reference voltage that is being supplied to the (-) terminal
    /*
     * Base Address of Comparator B,
     * Reference Voltage of Vcc,
     * Lower Limit of Vcc*(16/32) = (1/2)*Vcc,
     * Upper Limit of Vcc*(16/32) = (1/2)*Vcc,
     * Static Mode Accuracy
     */
    COMP_B_setReferenceVoltage(COMP_B_BASE,
        COMP_B_VREFBASE_VCC,
        16,
        16,
        COMP_B_ACCURACY_STATIC
        );

    //Allow power to Comparator module
    COMP_B_enable(COMP_B_BASE);

    //delay for the reference to settle
    __delay_cycles(75);

    //Enter LPM4
    __bis_SR_register(LPM4_bits);

    //For debug
    __no_operation();
}
