//******************************************************************************
//! COMPB - LED Toggle in LPM4, Vcomp_0 is compared against Vcomp_1 voltage
//!
//!  Use CompB and internal reference to determine if input 'Vcompare' is high
//!  of low.  When Vcomp_0 exceeds Vcomp_1 CBOUT goes high and when Vcomp_0 is
//!  less than Vcomp_1 then CBOUT goes low. Connect P1.6/CBOUT to P1.0
//!  externally to see the LED toggle accordingly.
//!
//!                 MSP430x552x
//!             ------------------
//!         /|\|                  |
//!          | |          P6.0/CB0|<- Vcomp_0
//!          --|RST       P6.1/CB1|<- Vcomp_1
//!            |        P1.6/CBOUT|--> 'high'(Vcomp_0>Vcomp_1);
//!            |                  | |  'low'(Vcomp_0<Vcomp_1)
//!            |                  | |
//!            |              P1.0|_| LED 'ON'(Vcomp_0>Vcomp_1);
//!            |                  |       'OFF'(Vcomp_0<Vcomp_1)
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - COMPB peripheral
//! - GPIO Port peripheral
//! - CB0
//! - CB1
//! - CBOUT
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "comp_b.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Select CBOUT function on P1.6/CBOUT and set P1.6 output direction
    GPIO_setAsPeripheralModuleFunctionOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN6
        );

    //Initialize the Comparator B module
    /*
     * Base Address of Comparator B,
     * Pin CB0 to Positive(+) Terminal,
     * Reference Voltage to Negative(-) Terminal,
     * Normal Power Mode,
     * Output Filter On with minimal delay,
     * Non-Inverted Output Polarity
     */
    COMP_B_init(COMP_B_BASE,
        COMP_B_INPUT0,
        COMP_B_INPUT1,
        COMP_B_POWERMODE_NORMALMODE,
        COMP_B_FILTEROUTPUT_DLYLVL1,
        COMP_B_NORMALOUTPUTPOLARITY
        );
    //Allow power to Comparator module
    COMP_B_enable(COMP_B_BASE);

    //delay for the reference to settle
    __delay_cycles(75);

    //Enter LPM4
    __bis_SR_register(LPM4_bits);

    //For debug
    __no_operation();
}
