//******************************************************************************
//! DAC12_A - Positive Ramp on DAC12_A_0
//!
//!  Using DAC12_A_0 and AVcc reference with a gain of 1, output positive ramp on
//!  DAC0. Normal mode is LPM0 with CPU off. Timer B used to provide a rising
//!  edge to TIMER_B OUT2 signal to latch data from DAC0 data buffer. TIMER_B
//!  OUT2 signal rises after the first ~31us, and then toggles for every ~31us
//!  afterwards, giving a period of ~62us for the DAC output to rise.
//!  ACLK = n/a, MCLK = SMCLK = default DCO
//!
//!                MSP430F66x
//!            -----------------
//!        /|\|              XIN|-
//!         | |                 |
//!         --|RST          XOUT|-
//!           |                 |
//!           |        P7.6/DAC0|-> Ramp_positive [Period ~400ms]
//!           |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - DAC12_A perpheral
//! - GPIO Port peripheral
//! - DAC0
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - DAC12_A_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "dac12_a.h"
#include "timer_b.h"
#include "wdt_a.h"
#include "gpio.h"

static unsigned int dataToConvert = 0;

void main (void)
{
    //Stop Watchdog Timer
    WDT_A_hold(WDT_A_BASE);

    /*
     * // Initialize DAC12_A_0,
     * // Choose P7.6 as output
     * // Use AVcc as Vref+
     * // Multiply Vout by 1
     * // Use medium settling speed/current
     * // Use TIMER_B OUT2 as data-latch trigger
     */
    DAC12_A_init(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0,
        DAC12_A_OUTPUT_1,
        DAC12_A_VREF_AVCC,
        DAC12_A_VREFx1,
        DAC12_A_AMP_MEDIN_MEDOUT,
        DAC12_A_TRIGGER_TB
        );

    //Enable the DAC12_A_0 interrupt
    DAC12_A_enableInterrupt(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0);

    //Calibrate output buffer for DAC12_A_0
    DAC12_A_calibrateOutput(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0);

    //Enable conversions for DAC12_A_0
    DAC12_A_enableConversions(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0);

    //// Set the variable dataToConvert into data buffer for DAC12_A_0
    DAC12_A_setData(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0,
        dataToConvert
        );

    //Start Timer B0
    TIMER_B_startUpMode(
        TIMER_B0_BASE,
        TIMER_B_CLOCKSOURCE_SMCLK,
        TIMER_B_CLOCKSOURCE_DIVIDER_1,
        0x33,
        TIMER_B_TBIE_INTERRUPT_DISABLE,
        TIMER_B_CCIE_CCR0_INTERRUPT_DISABLE,
        TIMER_B_DO_CLEAR
        );

    TIMER_B_initCompare(TIMER_B0_BASE,
        TIMER_B_CAPTURECOMPARE_REGISTER_2,
        TIMER_B_CAPTURECOMPARE_INTERRUPT_DISABLE,
        TIMER_B_OUTPUTMODE_TOGGLE,
        0x32
        );

    //Enter LPM0
    __bis_SR_register(CPUOFF + GIE);
}

//******************************************************************************
//
//This is the DAC12_A_VECTOR interrupt vector service routine.
//
//******************************************************************************
#pragma vector=DAC12_VECTOR
__interrupt void DAC12_A_ISR (void)
{
    //Clear interrupt flag to stop asserts
    DAC12_A_clearInterrupt(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0);

    //Increment ramp
    dataToConvert++;

    //Set the variable dataToConvert into data buffer for DAC12_A_0
    DAC12_A_setData(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0,
        dataToConvert
        );
    __no_operation();
}
