//******************************************************************************
//MSP430F66x Demo - DAC12_A_0 & DAC12_A_1 grouped, positive ramp on DAC12_A_0 and
//negative ramp on DAC12_A_1, high settling speed/current.
//!
//! \addtogroup dac12_examples_list
//! DAC12_A - Grouped, Positive Ramp on DAC12_A_0, Negative Ramp on DAC12_A_1,
//!       High Settling Speed/Current
//!
//!  Using DAC12_A_0 grouped with DAC12_A_1 and AVcc reference with a gain of 1,
//!  output positive ramp on DAC0, negative ramp on DAC1. Data is not latched
//!  from either data buffers until both data buffers have been set.
//!  ACLK = n/a, MCLK = SMCLK = default DCO
//!
//!                MSP430F66x
//!            -----------------
//!        /|\|              XIN|-
//!         | |                 |
//!         --|RST          XOUT|-
//!           |                 |
//!           |        P7.6/DAC0|-> Ramp_positive
//!           |        P7.7/DAC1|-> Ramp_negative
//!           |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - DAC12_A peripheral
//! - GPIO Port peripheral
//! - DAC0
//! - DAC1
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "dac12_a.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    unsigned int dataToConvert = 0;

    //Stop Watchdog Timer
    WDT_A_hold(WDT_A_BASE);

    /*
     * Initialize DAC12_A_0
     * Multiply Vout by 1
     * Use high settling speed/current,
     * Auto trigger when data is set as,
     * long as conversions are enabled
     */
    DAC12_A_init(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0,
        DAC12_A_OUTPUT_1,
        DAC12_A_VREF_AVCC,
        DAC12_A_VREFx1,
        DAC12_A_AMP_HIGHIN_HIGHOUT,
        DAC12_A_TRIGGER_ENC
        );

    /*Initialize DAC12_A_1
     * Choose P6.7 as output
     * Use AVcc as Vref+
     * Multiply Vout by 1
     * Use high settling speed/current
     * Auto trigger when data is set as long as conversions are enabled
     */
    DAC12_A_init(DAC12_A_BASE,
        DAC12_A_SUBMODULE_1,
        DAC12_A_OUTPUT_1,
        DAC12_A_VREF_AVCC,
        DAC12_A_VREFx1,
        DAC12_A_AMP_HIGHIN_HIGHOUT,
        DAC12_A_TRIGGER_ENC
        );

    //Group both DAC12_A's to sync output
    DAC12_A_enableGrouping(DAC12_A_BASE);

    //Calibrate output buffer for DAC12_A_0
    DAC12_A_calibrateOutput(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0);

    //Calibrate output buffer for DAC12_A_0
    DAC12_A_calibrateOutput(DAC12_A_BASE,
        DAC12_A_SUBMODULE_1);

    //Enable Conversions for both DAC12_A_0 and DAC12_A_1
    DAC12_A_enableConversions(DAC12_A_BASE,
        DAC12_A_SUBMODULE_0);
    DAC12_A_enableConversions(DAC12_A_BASE,
        DAC12_A_SUBMODULE_1);

    for (;;)
    {
        dataToConvert++;

        //Set data for DAC12_A_0, which will not latch because it is grouped
        DAC12_A_setData(DAC12_A_BASE,
            DAC12_A_SUBMODULE_0,
            dataToConvert);

        //Set data for DAC12_A_1, which will latch both DAC's data buffers
        DAC12_A_setData(DAC12_A_BASE,
            DAC12_A_SUBMODULE_1,
            ~(dataToConvert));
        __no_operation();
    }
}

