//******************************************************************************
//!
//! DMA - Single Transfer using SPI, TX on DMA Channel 0, RX on DMA Channel 1
//!
//!  This is the SPI Master.
//!  This code has to be used with USCI_SPI_slave.c (slave SPI). DMA channel 0 is
//!  used to transfer a single byte while DMA channel 1 is used to RX from slave
//!  SPI at the same time. This code will set P1.0 if RX character is correct
//!  and clears P1.0 if received character is wrong. Watchdog in interval mode
//!  triggers block transfer every 1000ms.
//!  ACLK = REFO = 32kHz, MCLK = SMCLK = default DCO 1048576Hz
//!
//!                MSP430F552x
//!             -----------------
//!         /|\|              XIN|-
//!          | |                 | 32kHz
//!          --|RST          XOUT|-
//!            |                 |
//!            |             P1.0|-> LED
//!            |    P3.3/UCA0SIMO|-> Data Out
//!            |    P3.4/UCA0SOMI|<- Data In
//!            |     P2.7/UCA0CLK|-> Serial Clock Out
//!            |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - DMA peripheral
//! - SPI peripheral
//! - GPIO Port peripheral
//! - UCA0SIMO
//! - UCA0SOMI
//! - UCA0CLK
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - WDT_A_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "dma.h"
#include "sfr.h"
#include "usci_spi.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"

char TxString;
char RxString;

void main (void)
{
    //Initialize Watchdog in Interval Timer Mode
    /*
     * Base Address of WDT module
     * Use ACLK as clock source to WDT module
     * Divide ACLK by 32k to assert interval every 1 second
     */
    WDT_A_intervalTimerInit(WDT_A_BASE,
        WDT_A_CLOCKSOURCE_ACLK,
        WDT_A_CLOCKDIVIDER_32K);

    //Enable Watchdog Interupt
    SFR_enableInterrupt(SFR_BASE,
        WDTIE);

    //Clear P1.0 LED off
    GPIO_setOutputLowOnPin(
        
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //P3.4,3 & P2.7 option select
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P3,
        GPIO_PIN4 + GPIO_PIN3
        );
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P2,
        GPIO_PIN7
        );

    //Initialize Master
    USCI_SPI_masterInit(USCI_A0_BASE,
        USCI_SPI_CLOCKSOURCE_ACLK,
        UCS_getACLK(UCS_BASE),
        500000, //Desired SPI CLOCK
        USCI_SPI_MSB_FIRST,
        USCI_SPI_PHASE_DATA_CAPTURED_ONFIRST_CHANGED_ON_NEXT,
        USCI_SPI_CLOCKPOLARITY_INACTIVITY_HIGH
        );

    //Enable SPI module
    USCI_SPI_enable(USCI_A0_BASE);

    //Wait for slave to initialize
    __delay_cycles(100);

    //USCI_A0 TX buffer ready?
    while (!USCI_SPI_getInterruptStatus(USCI_A0_BASE, UCTXIFG)) ;


    //Initialize and Setup DMA Channel 0
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 0
     * Configure channel for single transfer
     * DMA transfers will be disabled and interrupt flag will be set after every
     *   1 transfer
     * Use DMA Trigger Source 17 (UCA0TXIFG)
     * Transfer Byte-to-byte
     * Trigger transfer on signal held high
     */
    DMA_init(DMA_BASE,
        DMA_CHANNEL_0,
        DMA_TRANSFER_SINGLE,
        1,
        DMA_TRIGGERSOURCE_17,
        DMA_SIZE_SRCBYTE_DSTBYTE,
        DMA_TRIGGER_HIGH);
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 0
     * Use TxString as source
     * Increment source address after every transfer
     */
    DMA_setSrcAddress(DMA_BASE,
        DMA_CHANNEL_0,
        (unsigned long)&TxString,
        DMA_DIRECTION_INCREMENT);
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 0
     * Use SPI TX Buffer as destination
     * Don't move the destination address after every transfer
     */
    DMA_setDstAddress(DMA_BASE,
        DMA_CHANNEL_0,
        USCI_SPI_getTransmitBufferAddressForDMA(USCI_A0_BASE),
        DMA_DIRECTION_UNCHANGED);

    //Initialize and Setup DMA Channel 1
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 1
     * Configure channel for single transfer
     * DMA transfers will be disabled and interrupt flag will be set after every
     *   1 transfer
     * Use DMA Trigger Source 16 (UCA0RXIFG)
     * Transfer Byte-to-byte
     * Trigger transfer on signal held high
     */
    DMA_init(DMA_BASE,
        DMA_CHANNEL_1,
        DMA_TRANSFER_SINGLE,
        1,
        DMA_TRIGGERSOURCE_16,
        DMA_SIZE_SRCBYTE_DSTBYTE,
        DMA_TRIGGER_HIGH);
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 1
     * Use SPI RX Buffer as source
     * Don't move the source address after every transfer
     */
    DMA_setSrcAddress(DMA_BASE,
        DMA_CHANNEL_1,
        USCI_SPI_getReceiveBufferAddressForDMA(USCI_A0_BASE),
        DMA_DIRECTION_UNCHANGED);
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 1
     * Use RxString as destination
     * Increment destination address after every transfer
     */
    DMA_setDstAddress(DMA_BASE,
        DMA_CHANNEL_1,
        (unsigned long)&RxString,
        DMA_DIRECTION_INCREMENT);

    //Clear TxString && RxString
    TxString = RxString = 0;

    //Enter LPM3 w/ interrupts
    __bis_SR_register(LPM3_bits + GIE);
    //For Debugger
    __no_operation();
}

//------------------------------------------------------------------------------
//Trigger DMA0 & DMA1 block transfer.
//------------------------------------------------------------------------------
#pragma vector=WDT_VECTOR
__interrupt void WDT_A_ISR (void)
{
    if (TxString - 1 == RxString){
        //set P1.0
        GPIO_setOutputHighOnPin(
            
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    } else   {
        //Clear P1.0 LED off
        GPIO_setOutputLowOnPin(
            
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    }

    //Increment TxString counter
    TxString++;

    //Enable DMA transfers on channel 1
    DMA_enableTransfers(DMA_BASE,
        DMA_CHANNEL_1);

    //Enable DMA transfers on channel 0
    DMA_enableTransfers(DMA_BASE,
        DMA_CHANNEL_0);
}

