//*****************************************************************************
//!   Dummy calibration data is written to infoB.
//!   16 locations are written with char data, 8 locations with int data and
//!   4 locations with word data.  LED toggles upon completion.
//!
//!   Check infoB location @ 0x1900 on execution of program.
//!
//!   Tested On: MSP430F5529
//!             -----------------
//!         /|\|                 |
//!          | |             P1.0|-LED
//!          --|RST              |
//!            |                 |
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - Flash module
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//********************************************************************************

#include "inc/hw_memmap.h"
#include "flash.h"
#include "wdt_a.h"
#include "gpio.h"

//*****************************************************************************
//
//INFOB address in flash into which calibration data would be written
//
//*****************************************************************************
#define INFOB_START   (0x1900)

//*****************************************************************************
//
//Dummy char array of 'calibration data' - to be stored in flash
//
//*****************************************************************************
unsigned char calibration_data_char[16] = {0x00,
                                           0x01,
                                           0x02,
                                           0x03,
                                           0x04,
                                           0x05,
                                           0x06,
                                           0x07,
                                           0x08,
                                           0x09,
                                           0x10,
                                           0x11,
                                           0x12,
                                           0x13,
                                           0x14,
                                           0x15};
//*****************************************************************************
//
//Dummy int array of 'calibration data' - to be stored in flash
//
//*****************************************************************************
unsigned int calibration_data_int[8] = {0x1617,
                                        0x1819,
                                        0x2021,
                                        0x2223,
                                        0x2425,
                                        0x2627,
                                        0x2829,
                                        0x3031};
//*****************************************************************************
//
//Dummy long array of 'calibration data' - to be stored in flash
//
//*****************************************************************************
unsigned long calibration_data_long[4] = {0x35343332,
                                          0x39383736,
                                          0x43424140,
                                          0x47464544};

void main (void)
{
    unsigned int status;

    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Write calibration data to INFOB

    do
    {
        //Erase INFOB
        FLASH_segmentErase(FLASH_BASE,
            (unsigned char *)INFOB_START
            );
        status = FLASH_eraseCheck(FLASH_BASE,
            (unsigned char *)INFOB_START,
            128
            );
    } while (status == STATUS_FAIL);

    //Write calibration data to INFOB
    FLASH_write8(FLASH_BASE,
        calibration_data_char,
        (unsigned char *)INFOB_START,
        16
        );
    FLASH_write16(FLASH_BASE,
        calibration_data_int,
        (unsigned int *)(INFOB_START + 16),
        8
        );
    FLASH_write32(FLASH_BASE,
        calibration_data_long,
        (unsigned long *)(INFOB_START + 32),
        4
        );
    FLASH_memoryFill32(FLASH_BASE,
        0xBEEFBEEF,
        (unsigned long *)(INFOB_START + 48),
        (128 - 48) >> 2
        );

    while (1)
    {
        //Toggle P1.0
        GPIO_toggleOutputOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );

        //Delay
        __delay_cycles(1000000);
    }
}
