//******************************************************************************
//!   MSP430F530x Demo - Internal LDO enabled after device power-up
//!
//!  Description: This demo code shows example case where device is powered from
//!  an external supply and the LDO input coming on is detected to enable
//!  external system circuitry. When sufficient voltage at LDOI is present, P1.0
//!  is set, else reset; thus reflecting the status of the LDO being enabled or not
//!
//!   Note:
//!    1.External supply is used to power the device VCC. If using TI target board
//!      (MSP-TS430RGC64B), remove jumper on JP4
//!
//!   ACLK = 32.768kHz, MCLK = SMCLK = default DCO~1MHz
//!
//!                MSP430F530x
//!             -----------------
//!        /|\|              VCC|<- External Supply
//!         | |                 |
//!         --|RST              |
//!           |             LDOO|--> 3.3V LDO Output
//!           |             LDOI|<-- LDO Input Voltage (3.76V to 5.5V - refer to the d/s
//specs for actual range)
//!           |                 |
//!           |             P1.0|--> LED - Reflects LDO status
//!           |                 |
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - LDO-PWR module
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "ldopwr.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Detect if sufficient voltage at LDOI is present
    if (LDOPWR_LDO_INPUT_VALID == LDOPWR_isLDOInputValid(
            LDOPWR_BASE
            ) ){
        //Valid LDOI, LED on
        GPIO_setOutputHighOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    } else   {
        //Clear P1.0 LED off
        GPIO_setOutputLowOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    }


    //Enable access to config registers
    LDOPWR_unLockConfiguration(LDOPWR_BASE);

    //Clear all LDO-PWR IFGs before enabled IE bits
    LDOPWR_clearInterruptStatus(LDOPWR_BASE,
        LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT +
        LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT +
        LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT

        );

    //Enable LDO-PWR interrupts
    LDOPWR_enableInterrupt(LDOPWR_BASE,
        LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT +
        LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT +
        LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
        );


    //Disbale access to config registers
    LDOPWR_lockConfiguration(LDOPWR_BASE);

    //Enter LPM3 w/ interrupts
    __bis_SR_register(LPM3_bits + GIE);

    //For debugger
    __no_operation();
}

//******************************************************************************
//
//This is the LDO_PWR_VECTOR interrupt vector service routine.
//
//******************************************************************************
#pragma vector=LDO_PWR_VECTOR
__interrupt void LDOInterruptHandler (void)
{
    if (LDOPWR_getInterruptStatus(LDOPWR_BASE,
            LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT
            )){
        //Enable access to config registers
        LDOPWR_unLockConfiguration(LDOPWR_BASE);

        //Software clear IFG
        //Its possible that current peaks due to charging cap
        //at LDOO might cause the LDOOVLIFG to trigger
        LDOPWR_clearInterruptStatus(LDOPWR_BASE,
            LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT +
            LDOPWR_LDOI_VOLTAGE_COMING_ON_INTERRUPT
            );

        //Disbale access to config registers
        LDOPWR_lockConfiguration(LDOPWR_BASE);

        //Valid LDOI, LED on
        GPIO_setOutputHighOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    } else if (LDOPWR_getInterruptStatus(LDOPWR_BASE,
                   LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
                   )){
        //Enable access to config registers
        LDOPWR_unLockConfiguration(LDOPWR_BASE);

        //Software clear IFG
        LDOPWR_clearInterruptStatus(LDOPWR_BASE,
            LDOPWR_LDO_OVERLOAD_INDICATION_INTERRUPT
            );


        //Disbale access to config registers
        LDOPWR_lockConfiguration(LDOPWR_BASE);

        //Over load indication; take necessary steps in application firmware
        while (1) ;
    } else if (LDOPWR_getInterruptStatus(LDOPWR_BASE,
                   LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT
                   )){
        //Enable access to config registers
        LDOPWR_unLockConfiguration(LDOPWR_BASE);


        //Software clear IFG
        LDOPWR_clearInterruptStatus(LDOPWR_BASE,
            LDOPWR_LDOI_VOLTAGE_GOING_OFF_INTERRUPT
            );

        //Disbale access to config registers
        LDOPWR_lockConfiguration(LDOPWR_BASE);

        //Voltage LDOI < launch voltage, LED off
        GPIO_setOutputLowOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    }
}

