//******************************************************************************
//! MSP430F5529 Demo - Port Mapping single function to multiple pins; Single
//!                    runtime configuration.
//! Description:  Port 4 is port mapped to output TIMER_B digital signals.
//! TBCCR1-3 generate different DutyCycles and have been output on multiple pins.
//! ACLK = REFO~32kHz; MCLK = SMCLK = default DCO;
//!
//!                 MSP430F5529
//!             ------------------
//!         /|\|                  |
//!          | |                  |
//!          --|RST               |
//!            |     P4.0(TB0CCR1)|--> 25%
//!            |     P4.1(TB0CCR1)|--> 25%
//!            |     P4.2(TB0CCR2)|--> 50%
//!            |     P4.3(TB0CCR2)|--> 50%
//!            |     P4.4(TB0CCR3)|--> 75%
//!            |     P4.5(TB0CCR3)|--> 75%
//!            |     P4.6(default)|--> DVSS
//!            |     P4.7(default)|--> DVSS
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - PMAP module
//! - GPIO Port peripheral
//! - UCS module
//! - Timer module
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************
#include "inc/hw_memmap.h"
#include "pmap.h"
#include "ucs.h"
#include "wdt_a.h"
#include "timer_b.h"
#include "gpio.h"

//*****************************************************************************
//
//Refer Datasheet for Port4 Port Mapping definitions
//
//*****************************************************************************
const unsigned char port_mapping[] = {
    //Port P4:
    PM_TB0CCR1A,
    PM_TB0CCR1A,
    PM_TB0CCR2A,
    PM_TB0CCR2A,
    PM_TB0CCR3A,
    PM_TB0CCR3A,
    PM_NONE,
    PM_NONE,
};

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set DCO FLL reference = REFO
    UCS_clockSignalInit(
        UCS_BASE,
        UCS_FLLREF,
        UCS_REFOCLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );

    //VLO Clock Sources ACLK
    UCS_clockSignalInit(
        UCS_BASE,
        UCS_ACLK,
        UCS_REFOCLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );

    //Set Ratio and Desired MCLK Frequency  and initialize DCO
    UCS_initFLLSettle(
        UCS_BASE,
        1048,
        32
        );


    //Setup Port Pins
    //P4.0 - P4.7 output
    //P4.0 - P4.6 Port Map functions
    GPIO_setAsPeripheralModuleFunctionOutputPin(
		GPIO_PORT_P4,
        GPIO_PIN0 + GPIO_PIN1 + GPIO_PIN2 +
        GPIO_PIN3 + GPIO_PIN4 + GPIO_PIN5 +
        GPIO_PIN6 + GPIO_PIN7 + GPIO_PIN8 +
        GPIO_PIN9 + GPIO_PIN10 + GPIO_PIN11 +
        GPIO_PIN12 + GPIO_PIN13 + GPIO_PIN14 +
        GPIO_PIN15
        );

    //CONFIGURE PORTS- pass the port_mapping array, start @ P4MAP01, initialize
    //a single port, do not allow run-time reconfiguration of port mapping
    PMAP_configurePorts(PMAP_CTRL_BASE,
        (const unsigned char *)port_mapping,
        (unsigned char  *)&P4MAP01,
        1,
        PMAP_DISABLE_RECONFIGURATION
        );

    //Start Up Down Timer
    TIMER_B_startUpDownMode( TIMER_B0_BASE,
        TIMER_B_CLOCKSOURCE_ACLK,
        TIMER_B_CLOCKSOURCE_DIVIDER_2,
        128,
        TIMER_B_TBIE_INTERRUPT_DISABLE,
        TIMER_B_CCIE_CCR0_INTERRUPT_DISABLE,
        TIMER_B_SKIP_CLEAR
        );

    //Generate PWM 1
    TIMER_B_initCompare(TIMER_B0_BASE,
        TIMER_B_CAPTURECOMPARE_REGISTER_1,
        TIMER_B_CAPTURECOMPARE_INTERRUPT_DISABLE,
        TIMER_B_OUTPUTMODE_TOGGLE_SET,
        96
        );

    //Generate PWM 2
    TIMER_B_initCompare(TIMER_B0_BASE,
        TIMER_B_CAPTURECOMPARE_REGISTER_2,
        TIMER_B_CAPTURECOMPARE_INTERRUPT_DISABLE,
        TIMER_B_OUTPUTMODE_TOGGLE_SET,
        64
        );

    //Generate PWM 3
    TIMER_B_initCompare(TIMER_B0_BASE,
        TIMER_B_CAPTURECOMPARE_REGISTER_3,
        TIMER_B_CAPTURECOMPARE_INTERRUPT_DISABLE,
        TIMER_B_OUTPUTMODE_TOGGLE_SET,
        32
        );
    //Enter LPM3
    __bis_SR_register(LPM3_bits);

    //For debugger
    __no_operation();
}

