//*****************************************************************************
//! PMM @ Core 0, SVSL/SVSH optimized for fast wake up
//!
//!  Description: The PMM is at default core level 0.
//!  The SVS high side (Vcc) is configured for Fast Performance mode.
//!  The SVS low side (Vcore) is disabled.
//!  Specs:
//!  Wake up from interrupt: 5us* (as SVSL  disabled)
//!  Response to SVS event(propogation delay): 2us* (as SVSH enabled in LPM)
//!  Current consumption by SVS: 2uA* (as SVSH enabled in Fast Performance Mode)
//! !Note: Refer to the device D/s for actual values
//!
//!  The timer outputs a PWM with a frequency ~800Hz. The LED at P1.0 is toggled
//!  on entry into the ISR. The time difference between PWM event and LED
//!  toggle =SVS wakeup time + interrupt latency + LED turn on = ~20+us.
//!
//!  NOTE: TERMINATE THE DEBUG SESSION BEFORE TESTING THIS EXAMPLE
//!  Refer section 2.2.8.1 for debug impact on SVS
//!
//!  ACLK = REFO = TACLK = ~32KHz, MCLK = SMCLK = default DCO ~1.045MHz//!
//!
//!           MSP430F54xA
//!         ---------------
//!     /|\|               |
//!      | |               |
//!      --|RST            |
//!        |          TA0.4|--> output from timer
//!        |           P1.0|--> LED
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - PMM peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - Timer0_A5.
//*****************************************************************************

#include "inc/hw_memmap.h"
#include "pmm.h"
#include "ucs.h"
#include "timer_a.h"
#include "wdt_a.h"
#include "gpio.h"

#define DUTY_CYCLE1     20
#define TIMER_PERIOD    40

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set DCO FLL reference = REFO
    UCS_clockSignalInit(UCS_BASE,
        UCS_FLLREF,
        UCS_REFOCLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );

    //Set ACLK = REFO
    UCS_clockSignalInit(UCS_BASE,
        UCS_ACLK,
        UCS_REFOCLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );

    //~ 32*32768 = 1048576
    UCS_initFLLSettle(UCS_BASE,
        1048,
        32
        );

    //SVS Low side is turned off
    PMM_disableSvsL(PMM_BASE);

    //Monitor low side is turned off
    PMM_disableSvmL(PMM_BASE);

    //Monitor high side is turned off
    PMM_disableSvmH(PMM_BASE);

    //SVS High side is turned on
    PMM_enableSvsH(PMM_BASE);

    //Enable POR on SVS Event
    PMM_enableSvsHReset(PMM_BASE);

    //SVS high side Full perf mode,
    PMM_SvsHEnabledInLPMFullPerf(PMM_BASE);

    //stays on in LPM3,enhanced protect
    //Wait until high side, low side settled
    while ( 0 == PMM_getInterruptStatus(PMM_BASE,
                SVSMLDLYIFG + SVSMHDLYIFG )
            ) ;

    PMM_clearPMMIFGS(PMM_BASE);

    //Clear P1.0 LED off
    GPIO_setOutputLowOnPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //P1.5 TA0.4 output 50% duty cycle
    GPIO_setAsPeripheralModuleFunctionOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN5
        );

    //Start timer in upMode sourced by ACLK
    TIMER_A_startUpMode(
        TIMER_A0_BASE,
        TIMER_A_CLOCKSOURCE_ACLK,
        TIMER_A_CLOCKSOURCE_DIVIDER_1,
        TIMER_PERIOD,
        TIMER_A_TAIE_INTERRUPT_ENABLE,
        TIMER_A_CCIE_CCR0_INTERRUPT_DISABLE,
        TIMER_A_DO_CLEAR
        );

    //Generate PWM using capture
    TIMER_A_initCompare(TIMER_A0_BASE,
        TIMER_A_CAPTURECOMPARE_REGISTER_4,
        TIMER_A_CAPTURECOMPARE_INTERRUPT_ENABLE,
        TIMER_A_OUTPUTMODE_SET_RESET,
        DUTY_CYCLE1
        );

    //Enter LPM3, enable interrupts
    __bis_SR_register(LPM3_bits + GIE);

    //For debugger
    __no_operation();
}

//******************************************************************************
//
//This is the Timer0_A5 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=TIMER0_A1_VECTOR
__interrupt void TIMER0_A1_ISR (void)
{
    switch (__even_in_range(TA0IV,14)){
        case  0: break;                          //No interrupt
        case  2: break;                          //CCR1 not used
        case  4: break;                          //CCR2 not used
        case  6: break;                          //reserved
        case  8: break;                          //reserved
        case 10: break;                          //reserved
        case 12: break;                          //reserved
        case 14:                                 //overflow
            //Toggle P1.0
            GPIO_toggleOutputOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );
            break;
        default: break;
    }
}

