#include "inc/hw_memmap.h"
#include "ref.h"
#include "adc12_a.h"
#include "wdt_a.h"
#include "gpio.h"

//******************************************************************************
//!
//! \addtogroup ref_examples_list
//! REF - ADC12, Using REF Module to control Internal Reference
//!
//!  This example shows how to configure the internal reference of REF module
//!  using the REFCTL0 control register. The ADC12 uses the internal 2.5V
//!  reference and performs a single conversion on channel A0. The conversion
//!  results are stored in ADC12MEM0. Test by applying a voltage to channel A0,
//!  then setting and running to a break point at the "__no_operation()"
//!  instruction. To view the conversion results, open an ADC12 register window
//!  in debugger and view the contents of ADC12MEM0.
//!
//!  NOTE: REFMSTR bit in REFCTL0 regsiter is by default set to 1. This allows
//!  the REFCTL0 control regsiters to configure the reference setting. The
//!  legacy ADC12 reference control bits ADC12REFON, ADC12REF2_5, ADC12TCOFF
//!  and ADC12REFOUT are dont care in this case.
//!
//!      	  Tested on MSP430F5529
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |                 |
//!            |          P6.0/A0|<- Vin
//!            |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - REF peripheral
//! - ADC12 peripheral
//! - GPIO Port peripheral
//! - A0
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

void main (void)
{
    volatile unsigned int i;

    //Stop watchdog timer
    WDT_A_hold(WDT_A_BASE);

    //Enable A/D channel A0
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P6,
        GPIO_PIN0
        );


    //By default, REFMSTR=1 => REFCTL is used to configure the internal reference

    //If ref generator busy, WAIT
    while (REF_isRefGenBusy(REF_BASE)) ;
    //Select internal ref = 2.5V
    REF_setReferenceVoltage(REF_BASE,
        REF_VREF2_5V);
    //Internal Reference ON
    REF_enableReferenceVoltage(REF_BASE);

    //Delay (~75us) for Ref to settle
    __delay_cycles(75);

    //Initialize the ADC12 Module
    /*
     * Base address of ADC12 Module
     * Use internal ADC12 bit as sample/hold signal to start conversion
     * USE MODOSC 5MHZ Digital Oscillator as clock source
     * Use default clock divider of 1
     */
    ADC12_A_init(ADC12_A_BASE,
        ADC12_A_SAMPLEHOLDSOURCE_SC,
        ADC12_A_CLOCKSOURCE_ADC12OSC,
        ADC12_A_CLOCKDIVIDER_1);
    
    ADC12_A_enable(ADC12_A_BASE);
    
    /*
     * Base address of ADC12 Module
     * For memory buffers 0-7 sample/hold for 64 clock cycles
     * For memory buffers 8-15 sample/hold for 4 clock cycles (default)
     * Disable Multiple Sampling
     */
    ADC12_A_setupSamplingTimer(ADC12_A_BASE,
        ADC12_A_CYCLEHOLD_64_CYCLES,
        ADC12_A_CYCLEHOLD_4_CYCLES,
        ADC12_A_MULTIPLESAMPLESENABLE);

    //Configure Memory Buffer
    /*
     * Base address of the ADC12 Module
     * Configure memory buffer 0
     * Map input A0 to memory buffer 0
     * Vref+ = Vref+ (INT)
     * Vref- = AVss
     */
    ADC12_A_memoryConfigure(ADC12_A_BASE,
        ADC12_A_MEMORY_0,
        ADC12_A_INPUT_A0,
        ADC12_A_VREFPOS_INT,
        ADC12_A_VREFNEG_AVSS,
        ADC12_A_NOTENDOFSEQUENCE);

    while (1)
    {
        //Enable/Start sampling and conversion
        /*
         * Base address of ADC12 Module
         * Start the conversion into memory buffer 0
         * Use the single-channel, single-conversion mode
         */
        ADC12_A_startConversion(ADC12_A_BASE,
            ADC12_A_MEMORY_0,
            ADC12_A_SINGLECHANNEL);

        //Poll for interrupt on memory buffer 0
        while (ADC12IFG0 != ADC12_A_getInterruptStatus(ADC12_A_BASE,
                   ADC12IFG0)) ;

        //SET BREAKPOINT HERE
        __no_operation();
    }
}

