//******************************************************************************
//!
//! RTC_A in Counter Mode toggles P1.0 every 1s
//!
//!  This program demonstrates RTC in counter mode configured to source from ACLK
//!  to toggle P1.0 LED every 1s.
//!
//!                MSP430F5438A
//!             -----------------
//!        /|\ |                 |
//!         |  |                 |
//!         ---|RST              |
//!            |                 |
//!            |             P1.0|-->LED
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - RTC peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - RTC_A_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "rtc_a.h"
#include "wdt_a.h"
#include "gpio.h"


void main (void)
{
    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 LED on
    GPIO_setOutputHighOnPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Initialize Counter Mode of RTC_A_A
    /*
     * Base Address of the RTC_A_A
     * Use Prescaler 1 as source for counter
     * Specify counter as 8 bits, which asserts an interrupt for an overflow
     */
    RTC_A_counterInit(RTC_A_BASE,
        RTC_A_CLOCKSELECT_RT1PS,
        RTC_A_COUNTERSIZE_8BIT);

    //Initialize Prescalers
    /*
     * Base Address of the RTC_A_A
     * Specify Initialization of Prescaler 0
     * Use ACLK as source to prescaler
     * Divide source by 8 for this prescaler
     */
    RTC_A_counterPrescaleInit(RTC_A_BASE,
        RTC_A_PRESCALE_0,
        RTC_A_PSCLOCKSELECT_ACLK,
        RTC_A_PSDIVIDER_8);

    /*
     * Base Address of the RTC_A_A
     * Specify Initialization of Prescaler 1
     * Use Prescaler 0 as source to prescaler
     * Divide source by 16 for this prescaler
     */
    RTC_A_counterPrescaleInit(RTC_A_BASE,
        RTC_A_PRESCALE_1,
        RTC_A_PSCLOCKSELECT_RT0PS,
        RTC_A_PSDIVIDER_16);

    //Enable interrupt for counter overflow
    RTC_A_enableInterrupt(RTC_A_BASE,
        RTCTEVIE);

    //Start RTC Clock
    RTC_A_startClock(RTC_A_BASE);

    __bis_SR_register(LPM3_bits + GIE);
}

#pragma vector=RTC_VECTOR
__interrupt void RTC_A_ISR (void)
{
    switch (__even_in_range(RTCIV,16)){
        case 0: break;  //No interrupts
        case 2: break;  //RTCRDYIFG
        case 4:         //RTCEVIFG
            //Toggle P1.0
            GPIO_toggleOutputOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0);
            break;
        case 6: break;  //RTCAIFG
        case 8: break;  //RT0PSIFG
        case 10: break; //RT1PSIFG
        default: break;
    }
}

