//******************************************************************************
//  MSP430F51x2 Demo - TimerD0, Single Input Capture mode, Normal Timer Mode
//                                Input Period and Dutycycle measurement
//  Description: This code example implements input capture in single capture 
//  mode using TimerD in normal timer mode. TD1.1 is configured to output PWM
//  of 25% dutycycle, that is used as capture input at TD0.1. TD0.1 is 
//  configured as timer input capture that is triggered by both the rising and 
//  the falling edges. An external connection between TD1.1 and TD0.1 is 
//  required. Rising and Falling edges are captured and the dutycycle is 
//  computed. If the measured dutycycle is != 25%, then LED on P1.0 is set. 
//
//  ACLK = LFXT1 = 32kHz; SMCLK = MCLK = 2.45MHz; 
//
//                 MSP430F51x2
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |       P1.7/TD0.1|<-- CCI1A <-| 
//            |       P2.2/TD1.1|--> CCR1 -->| 
//            |                 |
//            |             P1.0|--> LED "ON" when dutycycle != 25%
//            |                 |
//
//******************************************************************************
#include "inc/hw_memmap.h"
#include "timer_d.h"
#include "wdt_a.h"
#include "gpio.h"
#include "ucs.h"

unsigned char Count, First_Time;
unsigned int REdge1, REdge2, FEdge;
unsigned int Period, ON_Period;
unsigned char DutyCycle;
void main(void)
{
  // Stop watchdog timer
  WDT_A_hold(WDT_A_BASE);

  GPIO_setAsOutputPin(
        		  GPIO_PORT_P1,
        		  GPIO_PIN0
        		  );
  // LED off
  GPIO_setOutputLowOnPin(
          		  GPIO_PORT_P1,
          		  GPIO_PIN0
          		  );
  
  // Configure XT1
  // Port select XT1
  GPIO_setAsPeripheralModuleFunctionInputPin(
          		  GPIO_PORT_PJ,
          		  GPIO_PIN4 + GPIO_PIN5
          		  );
  UCS_LFXT1Start ( UCS_BASE,
		  UCS_XT1_DRIVE3,
		  UCS_XCAP_3
      );

  // Initialize DCO to 2.45MHz
  UCS_initFLLSettle( UCS_BASE,
  		  2450,
  		  74);

  // Loop until XT1 & DCO fault flag is cleared
  // Configure Port Pins
  // P2.2/TD1.1 Output  TD1.1 Option select
  GPIO_setAsPeripheralModuleFunctionOutputPin(
            		  GPIO_PORT_P2,
            		  GPIO_PIN2
            		  );
  // P1.7/TD0.1 Input Capture TD0.1 option select
  GPIO_setAsPeripheralModuleFunctionInputPin(
            		  GPIO_PORT_P1,
            		  GPIO_PIN7
            		  );
  
  // Configure TD1.1 to output PWM signal 
  // Period = 82/32khz = 2.5ms ~ 400Hz Freq
  TIMER_D_initCompare(TIMER_D1_BASE,
                    TIMER_D_CAPTURECOMPARE_REGISTER_1,
                    TIMER_D_CAPTURECOMPARE_INTERRUPT_DISABLE,
                    TIMER_D_OUTPUTMODE_RESET_SET,
                    21
                    );

  TIMER_D_configureUpMode(TIMER_D1_BASE,
    	      TIMER_D_CLOCKSOURCE_ACLK,
              TIMER_D_CLOCKSOURCE_DIVIDER_1,
              TIMER_D_CLOCKINGMODE_EXTERNAL_CLOCK,
              82-1,
              TIMER_D_TDIE_INTERRUPT_DISABLE,
              TIMER_D_CAPTURECOMPARE_INTERRUPT_DISABLE,
              TIMER_D_DO_CLEAR
              );
  
  TIMER_D_configureContinuousMode(TIMER_D0_BASE,
		  	  	  	  	  	 TIMER_D_CLOCKSOURCE_SMCLK,
		  	  	  	  	  	 TIMER_D_CLOCKSOURCE_DIVIDER_1,
		  	  	  	  	  	 TIMER_D_CLOCKINGMODE_EXTERNAL_CLOCK,
		  	  	  	  	  	 TIMER_D_TDIE_INTERRUPT_DISABLE,
		  	  	  	  	  	 TIMER_D_DO_CLEAR
		  	  	  	  		);

  TIMER_D_startCounter(TIMER_D0_BASE,
		  TIMER_D_CONTINUOUS_MODE
		  );

  TIMER_D_startCounter(TIMER_D1_BASE,
  		  TIMER_D_UP_MODE
  		  );



  TIMER_D_initCapture(TIMER_D0_BASE,
		  TIMER_D_CAPTURECOMPARE_REGISTER_1,
		  TIMER_D_CAPTUREMODE_RISING_AND_FALLING_EDGE,
		  TIMER_D_CAPTURE_INPUTSELECT_CCIxA,
		  TIMER_D_CAPTURE_SYNCHRONOUS,
		  TIMER_D_CAPTURECOMPARE_INTERRUPT_ENABLE,
		  TIMER_D_OUTPUTMODE_OUTBITVALUE,
		  TIMER_D_SINGLE_CAPTURE_MODE
		  );

  // Variable Initialization
  Count = 0x0;
  First_Time = 0x01;
  
  while(1)
  {
      __bis_SR_register(LPM0_bits + GIE);   // Enter LPM0
      __no_operation();                     // For debugger  
      // On exiting LPM0
      if (TIMER_D_CAPTURE_OVERFLOW == TIMER_D_getCaptureCompareInterruptStatus
    		  	  (TIMER_D0_BASE,
      	  	  	  TIMER_D_CAPTURECOMPARE_REGISTER_1,
      	  	  	  TIMER_D_CAPTURE_OVERFLOW
      	  	  	  )              )     // Check for Capture Overflow
            while(1);                         // Loop Forever
      
      Period = REdge2 - REdge1;             // Calculate Period
      ON_Period = FEdge-REdge1;             // On period
      DutyCycle = ((unsigned long)ON_Period*100/Period);
      if(DutyCycle!= 25)
      {
          GPIO_setOutputHighOnPin(
                    		  GPIO_PORT_P1,
                    		  GPIO_PIN0
                    		  );
      }
  }
}

// TD0_D1 Interrupt vector
#pragma vector = TIMER0_D1_VECTOR
__interrupt void TIMER0_D1_ISR (void)
{
  switch(__even_in_range(TD0IV,0x1E))
  {
      case  0x0: break;                     // Vector  0:  No interrupt
      case  0x2:                            // Vector  2:  TDCCR1 CCIFG
   // Capture Input Pin Status
    	  if(TIMER_D_CAPTURECOMPARE_INPUT == TIMER_D_getCaptureCompareInputSignal(TIMER_D0_BASE,
    	          						TIMER_D_CAPTURECOMPARE_REGISTER_1))
    	  {
            // Rising Edge was captured
            if (!Count)
            {
                REdge1 = TIMER_D_getCaptureCompareCount(TIMER_D0_BASE,
        				TIMER_D_CAPTURECOMPARE_REGISTER_1);
                Count++;
            }
            else
            {
                REdge2 = TIMER_D_getCaptureCompareCount(TIMER_D0_BASE,
        				TIMER_D_CAPTURECOMPARE_REGISTER_1);
                Count=0x0;
                __bic_SR_register_on_exit(LPM0_bits + GIE);  // Exit LPM0 on return to main
            }

            if (First_Time)
                First_Time = 0x0;
        }

        else
        {
            // Falling Edge was captured
            if(!First_Time)
            {
                FEdge = TIMER_D_getCaptureCompareCount(TIMER_D0_BASE,
        				TIMER_D_CAPTURECOMPARE_REGISTER_1);
            }
        }
        break;
      case 0x4: break;                      // Vector  4:  TDCCR2 CCIFG
      case 0x6: break;                      // Vector  6:  TDCCR3 CCIFG
      case 0x8: break;                      // Vector  8:  TDCCR4 CCIFG
      case 0xA: break;                      // Vector 10:  TDCCR5 CCIFG
      case 0xC: break;                      // Vector 12:  TDCCR5 CCIFG
      case 0xE: break;                      // Vector 14:  -
      case 0x10:break;                      // Vector 16:  TDIFG
      case 0x12:break;                      // Vector 18:  TDHINT TDHFLIFG
      case 0x14:break;                      // Vector 20:  TDHINT TDHFHIFG
      case 0x16:break;                      // Vector 22:  TDHINT TDHLKIFG
      case 0x18:break;                      // Vector 24:  TDHINT TDHUNLKIFG
      case 0x1A:break;                      // Vector 26:  -
      case 0x1C:break;                      // Vector 28:  -
      case 0x1E:break;                      // Vector 28:  -
      default: 	break;
  }
}
