//******************************************************************************
//  MSP430F51x2 Demo - TimerD0, Dual Input capture mode, Normal timer mode, 
//                                Period measurement
//  Description: This code example implements input capture in dual capture 
//  mode using TimerD in normal timer mode. Dual capture mode of TimerD is 
//  demonstrated here. This example captures two consequent rising edges and 
//  computes the Period of the input signal. The TD0.0 input signal is generated 
//  by the Timer A0 module (TA0.0) and the freq of this PWM input signal can be 
//  changed by modifying the TA0CCR0 register. 
//  
//  ACLK = LFXT1 = 32kHz; SMCLK = MCLK = 12MHz 
//
//                 MSP430F51x2
//             -----------------
//         /|\|              XIN|-
//          | |                 | 32kHz
//          --|RST          XOUT|-
//            |                 |
//            |       P1.6/TD0.0|<-- CCI0A <-| 
//            |       P3.7/TA0.0|--> CCR0 -->| 
//            |                 |
//            |             P3.0|--> MCLK = 12MHz DCO
//            |             P3.1|--> SMCLK = 12MHz DCO
//            |             P3.2|--> ACLK = 32kHz LFXT1
//
//******************************************************************************
#include "inc/hw_memmap.h"
#include "timer_d.h"
#include "timer_a.h"
#include "wdt_a.h"
#include "gpio.h"
#include "ucs.h"
#include "pmm.h"
#include "pmap.h"

unsigned int REdge1, REdge2, Period;

const unsigned char port_mapping[] = {
    //Port P3:
		PM_TD0CLKMCLK,
		PM_TD0_0SMCLK,
		PM_TD1OUTH,
		PM_NONE,
		PM_NONE,
		PM_NONE,
		PM_NONE,
		PM_TA0_0,
	};


void main(void)
{
  // Stop WDT
  WDT_A_hold(WDT_A_BASE);

  // Configure XT1
  GPIO_setAsPeripheralModuleFunctionInputPin(
	            		  GPIO_PORT_PJ,
	            		  GPIO_PIN4 + GPIO_PIN5
	            		  );
  
  UCS_LFXT1Start ( UCS_BASE,
  		  UCS_XT1_DRIVE3,
  		  UCS_XCAP_3
        );


  // Increase Vcore setting to level1 to support fsystem=12MHz
  // NOTE: Change core voltage one level at a time..
  PMM_setVCore (PMM_BASE,
		  PMM_CORE_LEVEL_1);
  
  // Initialize DCO to 12MHz
  UCS_initFLLSettle( UCS_BASE,
    		  12000,
    		  374);


  // Setup Port Pins 
  // P3.0 - P3.2 output
  // Port map P3.0 - P3.2
  GPIO_setAsPeripheralModuleFunctionOutputPin(
                		  GPIO_PORT_P3,
                		  GPIO_PIN0 + GPIO_PIN1 + GPIO_PIN2
                		  );


 PMAP_configurePorts(PMAP_CTRL_BASE,
                (const unsigned char *)port_mapping,
                (unsigned char *)&P3MAP0,
                1,
                PMAP_DISABLE_RECONFIGURATION
                );

  // Configure ports TD0.0 input and TA0.0 output
  // TD0.0 input
  // TD0.0 option select
  // TA0.0 output
  // TA0.0 option select
  GPIO_setAsPeripheralModuleFunctionInputPin(
            		  GPIO_PORT_P1,
            		  GPIO_PIN6
            		  );

  GPIO_setAsPeripheralModuleFunctionOutputPin(
              		  GPIO_PORT_P3,
              		  GPIO_PIN7
              		  );
  
  // Configure LED on P1.0, output and driving low
  GPIO_setOutputLowOnPin(
                 		  GPIO_PORT_P1,
                 		  GPIO_PIN0
                 		  );
  GPIO_setAsOutputPin(
               		  GPIO_PORT_P1,
               		  GPIO_PIN0
               		  );
    
  // Configure TA0.0 compare output, 1kHz freq, 50% dutycycle 
  TIMER_A_initCompare(TIMER_A0_BASE,
                      TIMER_A_CAPTURECOMPARE_REGISTER_0,
                      TIMER_A_CAPTURECOMPARE_INTERRUPT_DISABLE,
                      TIMER_A_OUTPUTMODE_TOGGLE,
                      32
                      );

  TIMER_A_configureUpMode(TIMER_A0_BASE,
     		  TIMER_A_CLOCKSOURCE_ACLK,
               TIMER_A_CLOCKSOURCE_DIVIDER_1,
               32,
               TIMER_A_TAIE_INTERRUPT_DISABLE,
               TIMER_A_CAPTURECOMPARE_INTERRUPT_DISABLE,
               TIMER_A_DO_CLEAR
               );

  TIMER_A_startCounter(TIMER_A0_BASE,
		  TIMER_A_UP_MODE
		  );

  TIMER_D_configureContinuousMode(TIMER_D0_BASE,
  		  	  	  	  	  	 TIMER_D_CLOCKSOURCE_SMCLK,
  		  	  	  	  	  	 TIMER_D_CLOCKSOURCE_DIVIDER_1,
  		  	  	  	  	  	 TIMER_D_CLOCKINGMODE_EXTERNAL_CLOCK,
  		  	  	  	  	  	 TIMER_D_TDIE_INTERRUPT_DISABLE,
  		  	  	  	  	  	 TIMER_D_DO_CLEAR
  		  	  	  	  		);

  TIMER_D_startCounter(TIMER_D0_BASE,
  		  TIMER_D_CONTINUOUS_MODE
  		  );

  TIMER_D_initCapture(TIMER_D0_BASE,
  		  TIMER_D_CAPTURECOMPARE_REGISTER_0,
  		  TIMER_D_CAPTUREMODE_RISING_EDGE,
  		  TIMER_D_CAPTURE_INPUTSELECT_CCIxA,
  		  TIMER_D_CAPTURE_SYNCHRONOUS,
  		  TIMER_D_CAPTURECOMPARE_INTERRUPT_ENABLE,
  		  TIMER_D_OUTPUTMODE_OUTBITVALUE,
  		  TIMER_D_DUAL_CAPTURE_MODE
  		  );

  
  while(1)
  {
      __bis_SR_register(LPM0_bits + GIE);   // Enter LPM0
      __no_operation();                     // For debugger  
      // On exiting LPM0
      if (TIMER_D_CAPTURE_OVERFLOW == TIMER_D_getCaptureCompareInterruptStatus
          		  	  (TIMER_D0_BASE,
            	  	  	  TIMER_D_CAPTURECOMPARE_REGISTER_1,
            	  	  	  TIMER_D_CAPTURE_OVERFLOW
            	  	  	  )              )     // Check for Capture Overflow
          while(1);                         // Loop Forever
     
      Period = REdge2-REdge1;               // Measured Period 
      __no_operation();                     // BREAKPOINT HERE - measured period ~ 2ms 
                                            // Period ~ 2m*TD0 clock counts    
  }
}
  
// Timer0_D0 Interrupt Vector
#pragma vector=TIMER0_D0_VECTOR
__interrupt void TIMER0_D0_ISR (void)
{
  REdge1 = TIMER_D_getCaptureCompareLatchCount(TIMER_D0_BASE,
  	          				TIMER_D_CAPTURECOMPARE_REGISTER_0);

  REdge2 = TIMER_D_getCaptureCompareCount(TIMER_D0_BASE,
          				TIMER_D_CAPTURECOMPARE_REGISTER_0);
  __bic_SR_register_on_exit(LPM0_bits);      // Exit LPM0 
}


