//******************************************************************************
//! Software Toggle P1.0 with ~12MHz DCO
//!
//!   Toggle P1.0 by xor'ing P1.0 inside of a software loop.
//!   ACLK is brought out on pin P11.0, SMCLK on P11.2, and MCLK on P11.1.
//!   PMMCOREV = 1 to support up to 12MHz clock
//!
//!   ACLK = REFO = ~32kHz, MCLK = SMCLK = 12MHz
//!
//!                 MSP430F5438A
//!             -----------------
//!         /|\|                 |
//!          | |            P11.0|-->ACLK
//!          --|RST         P11.1|-->MCLK
//!            |            P11.2|-->SMCLK
//!            |                 |
//!            |             P1.0|-->LED
//!   Note:
//!   In order to run the system at up to 12MHz, VCore must be set at 1.6V
//!   or higher. This is done by invoking function SetVCore()
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UCS module
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "ucs.h"
#include "pmm.h"
#include "wdt_a.h"
#include "gpio.h"
#include "sfr.h"

//*****************************************************************************
//
//Target frequency for MCLK in kHz
//
//*****************************************************************************
#define UCS_MCLK_DESIRED_FREQUENCY_IN_KHZ   12000

//*****************************************************************************
//
//MCLK/FLLRef Ratio
//
//*****************************************************************************
#define UCS_MCLK_FLLREF_RATIO   366

//*****************************************************************************
//
//Variable to store current Clock values
//
//*****************************************************************************
unsigned long clockValue = 0;

//*****************************************************************************
//
//Variable to store status of Oscillator fault flags
//
//*****************************************************************************
unsigned int status;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set VCore = 1 for 12MHz clock
    PMM_setVCore(PMM_BASE,
        PMM_CORE_LEVEL_1
        );

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //ACLK, MCLK, MCLK set out to pins
    GPIO_setAsPeripheralModuleFunctionOutputPin(
        GPIO_PORT_P11,
        GPIO_PIN0 + GPIO_PIN1 + GPIO_PIN2
        );

    //Set DCO FLL reference = REFO
    UCS_clockSignalInit(
        UCS_BASE,
        UCS_FLLREF,
        UCS_REFOCLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );
    //Set ACLK = REFO
    UCS_clockSignalInit(
        UCS_BASE,
        UCS_ACLK,
        UCS_REFOCLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );

    //Set Ratio and Desired MCLK Frequency  and initialize DCO
    UCS_initFLLSettle(
        UCS_BASE,
        UCS_MCLK_DESIRED_FREQUENCY_IN_KHZ,
        UCS_MCLK_FLLREF_RATIO
        );
    
    // Enable global oscillator fault flag
    SFR_enableInterrupt(SFR_BASE,
                        SFR_OSCILLATOR_FAULT_INTERRUPT
                        );
    
    // Enable global interrupt
    __bis_SR_register(GIE);   

    //Verify if the Clock settings are as expected
    clockValue = UCS_getSMCLK(UCS_BASE);
    clockValue = UCS_getMCLK(UCS_BASE);
    clockValue = UCS_getACLK(UCS_BASE);

    while (1)
    {
        //Toggle P1.0
        GPIO_toggleOutputOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );

        //Delay
        __delay_cycles(1000000);
    }
}


#pragma vector=UNMI_VECTOR
__interrupt void NMI_ISR(void)
{
  do {
    // If it still can't clear the oscillator fault flags after the timeout, 
    // trap and wait here.
    status = UCS_clearAllOscFlagsWithTimeout(UCS_BASE,
                                             1000
                                             );
  } while(status != 0);
}
