//******************************************************************************
//!  This program demonstrates using an external 32kHz crystal to supply ACLK,
//!  and using a high speed crystal or resonator to supply SMCLK.
//!  MLCK for the CPU is supplied by the internal DCO.  The 32kHz crystal
//!  connects between pins Xin and Xout. The high frequency crystal or
//!  resonator connects between pins XT2IN and XT2OUT.  The DCO clock is
//!  generated internally and calibrated from the 32kHz crystal.
//!  ACLK is brought out on pin P11.0, MCLK is brought out on P11.1,
//!  SMCLK is brought out on P11.2.
//!
//!  ACLK = LFXT1 = 32768Hz, MCLK = default DCO = 32 x ACLK = 1048576Hz
//!  SMCLK = HF XTAL
//!   An external watch crystal between XIN & XOUT is required for ACLK
//!
//!                MSP430F54xA
//!             -----------------
//!        /|\ |              XIN|-
//!         |  |                 | 32kHz
//!         ---|RST          XOUT|-
//!            |                 |
//!            |                 |
//!            |            XT2IN|-
//!            |                 | HF XTAL or Resonator (add capacitors)
//!            |           XT2OUT|-
//!            |                 |
//!            |            P11.0|-->ACLK = 32kHz Crystal Out
//!            |                 |
//!            |            P11.1|-->MCLK
//!            |                 |
//!            |            P11.2|-->SMCLK = High Freq Xtal or Resonator Out
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UCS module
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//*****************************************************************************
#include "inc/hw_memmap.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"
#include "sfr.h"

//*****************************************************************************
//
//Desired Timeout for XT1 initialization
//
//*****************************************************************************
#define UCS_XT1_TIMEOUT 50000

//*****************************************************************************
//
//Desired Timeout for XT2 initialization
//
//*****************************************************************************
#define UCS_XT2_TIMEOUT 50000

//*****************************************************************************
//
//XT1 Crystal Frequency being used
//
//*****************************************************************************
#define UCS_XT1_CRYSTAL_FREQUENCY    32768

//*****************************************************************************
//
//XT2 Crystal Frequency being used
//
//*****************************************************************************
#define UCS_XT2_CRYSTAL_FREQUENCY   24000000

//*****************************************************************************
//
//Variable to store returned STATUS_SUCCESS or STATUS_FAIL
//
//*****************************************************************************
unsigned short returnValue = 0;

//*****************************************************************************
//
//Variable to store current clock values
//
//*****************************************************************************
unsigned long clockValue;

//*****************************************************************************
//
//Variable to store status of Oscillator fault flags
//
//*****************************************************************************
unsigned int status;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //ACLK, MCLK, SMCLk set out to pins
    GPIO_setAsPeripheralModuleFunctionOutputPin(
        GPIO_PORT_P11,
        GPIO_PIN0 + GPIO_PIN1 + GPIO_PIN2
        );

    //Port select XT1
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P7,
        GPIO_PIN0 + GPIO_PIN1
        );

    //Initializes the XT1 and XT2 crystal frequencies being used
    UCS_setExternalClockSource(  UCS_BASE,
        UCS_XT1_CRYSTAL_FREQUENCY,
        UCS_XT2_CRYSTAL_FREQUENCY
        );

    //Initailize XT1. Returns STATUS_SUCCESS if initializes successfully
    returnValue = UCS_LFXT1StartWithTimeout(
        UCS_BASE,
        UCS_XT1_DRIVE0,
        UCS_XCAP_3,
        UCS_XT1_TIMEOUT
        );

    //Startup HF XT2 crystal Port select XT2
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P5,
        GPIO_PIN2 + GPIO_PIN3
        );

    //Initailize XT2. Returns STATUS_SUCCESS if initializes successfully
    returnValue = UCS_XT2StartWithTimeout(
        UCS_BASE,
        UCS_XT2DRIVE_4MHZ_8MHZ,
        UCS_XT2_TIMEOUT
        );

    //Select XT1 as ACLK source
    UCS_clockSignalInit(
        UCS_BASE,
        UCS_ACLK,
        UCS_XT1CLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );

    //Select XT2 as SMCLK source
    UCS_clockSignalInit(
        UCS_BASE,
        UCS_SMCLK,
        UCS_XT2CLK_SELECT,
        UCS_CLOCK_DIVIDER_1
        );
    
    // Enable global oscillator fault flag
    SFR_enableInterrupt(SFR_BASE,
                        SFR_OSCILLATOR_FAULT_INTERRUPT
                        );
    
    // Enable global interrupt
    __bis_SR_register(GIE);                   

    //Verify if the Clock settings are as expected
    clockValue = UCS_getMCLK(UCS_BASE);
    clockValue = UCS_getACLK(UCS_BASE);
    clockValue = UCS_getSMCLK(UCS_BASE);

    //Loop in place
    while (1) ;
}

#pragma vector=UNMI_VECTOR
__interrupt void NMI_ISR(void)
{
  do {
    // If it still can't clear the oscillator fault flags after the timeout, 
    // trap and wait here.
    status = UCS_clearAllOscFlagsWithTimeout(UCS_BASE,
                                             1000
                                             );
  } while(status != 0);
}
