#include "inc/hw_memmap.h"
#include "usci_i2c.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"

//*****************************************************************************
//! This example shows how to configure the I2C module as a master for
//! single byte reception in interrupt driven mode. The address of the slave
//! module that the master is communicating with also set in this example.
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//!
//
//*****************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************
#define SLAVE_ADDRESS 0x48

unsigned char receivedData;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Assign I2C pins to USCI_B0
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P3,
        GPIO_PIN1 + GPIO_PIN2
        );

    //Initialize Master
    USCI_I2C_masterInit(USCI_B0_BASE,
        USCI_I2C_CLOCKSOURCE_SMCLK,
        UCS_getSMCLK(UCS_BASE),
        USCI_I2C_SET_DATA_RATE_400KBPS
        );

    //Specify slave address
    USCI_I2C_setSlaveAddress(USCI_B0_BASE,
        SLAVE_ADDRESS
        );

    //Set Master in receive mode
    USCI_I2C_setMode(USCI_B0_BASE,
        USCI_I2C_RECEIVE_MODE
        );

    //Enable I2C Module to start operations
    USCI_I2C_enable(USCI_B0_BASE);

    //Enable master Receive interrupt
    USCI_I2C_enableInterrupt(USCI_B0_BASE,
        USCI_I2C_RECEIVE_INTERRUPT
        );

    while (1)
    {
        //Initiate command to receive a single character from Slave
        USCI_I2C_masterSingleReceiveStart(USCI_B0_BASE);

        //Enter low power mode 0 with interrupts enabled.
        //Wait until character is received.
        __bis_SR_register(LPM0_bits + GIE);
        __no_operation();
    }
}

//******************************************************************************
//
//This is the USCI_B0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR (void)
{
    switch (__even_in_range(UCB0IV,12)){
        //Vector 10: Data received - RXIFG
        case USCI_I2C_UCRXIFG:
        {
            //Grab data from data register
            receivedData = USCI_I2C_masterSingleReceive(
                USCI_B0_BASE);

            //Exit low power mode 0 and disable GIE on interrupt exit
            __bic_SR_register_on_exit(LPM0_bits + GIE);

            break;
        }

        default:    break;
    }
}

