#include "inc/hw_memmap.h"
#include "usci_i2c.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"

//*****************************************************************************
//! This example shows how to configure the I2C module as a master for
//! single byte transmission in interrupt mode. The address of the slave
//! module that the master is communicating with also set in this example.
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//!
//
//*****************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************
#define SLAVE_ADDRESS 0x48

unsigned char transmitData;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Assign I2C pins to USCI_B0
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P3,
        GPIO_PIN1 + GPIO_PIN2
        );

    //Initialize transmit data packet
    transmitData = 0x01;

    //Initialize Master
    USCI_I2C_masterInit(USCI_B0_BASE,
        USCI_I2C_CLOCKSOURCE_SMCLK,
        UCS_getSMCLK(UCS_BASE),
        USCI_I2C_SET_DATA_RATE_400KBPS
        );

    //Specify slave address
    USCI_I2C_setSlaveAddress(USCI_B0_BASE,
        SLAVE_ADDRESS
        );

    //Set in transmit mode
    USCI_I2C_setMode(USCI_B0_BASE,
        USCI_I2C_TRANSMIT_MODE
        );

    //Enable I2C Module to start operations
    USCI_I2C_enable(USCI_B0_BASE);

    //Enable TX interrupt
    USCI_I2C_enableInterrupt(USCI_B0_BASE,
        USCI_I2C_TRANSMIT_INTERRUPT
        );

    while (1)
    {
        //Send single byte data.
        USCI_I2C_masterSendSingleByte(USCI_B0_BASE,
            transmitData
            );

        //Delay until transmission completes
        while (USCI_I2C_isBusBusy(USCI_B0_BASE)) ;

        //Delay between each transaction
        __delay_cycles(50);

        //Increment transmit data counter
        transmitData++;
    }
}

//******************************************************************************
//
//This is the USCI_B0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR (void)
{
    switch (__even_in_range(UCB0IV,12)){
        //Vector 12: Transmit buffer empty - TXIF
        case USCI_I2C_UCTXIFG:
        {
            __no_operation();
            break;
        }
        default:  break;
    }
}

