#include "inc/hw_memmap.h"
#include "usci_uart.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"

//******************************************************************************
//! This example shows how to configure the UART module to echo a received
//! character. To echo a received character, RX ISR is used.
//!
//!                MSP430F5438A
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |                 |
//!            |     P3.4/UCA0TXD|------------>
//!            |                 | 9600
//!            |     P3.5/UCA0RXD|<------------
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UART peripheral
//! - GPIO Port peripheral (for UART pins)
//! - UCA0TXD
//! - UCA0RXD
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR.
//******************************************************************************

#define BAUD_RATE                               9600
#define RECEIVE_DATA_COUNT                      0x02
#define USCI_UART_MULTIPROCESSOR_MODE_ADDRESS        0xAA

unsigned char receivedData = 0x00;
unsigned char receiveDataCount = 0x00;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //P3.4,5 = USCI_A0 TXD/RXD
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P3,
        GPIO_PIN4 + GPIO_PIN5
        );

    //Initialize USCI UART
    if ( STATUS_FAIL == USCI_UART_init(USCI_A0_BASE,
             USCI_UART_CLOCKSOURCE_ACLK,
             UCS_getACLK(UCS_BASE),
             BAUD_RATE,
             USCI_UART_NO_PARITY,
             USCI_UART_MSB_FIRST,
             USCI_UART_ONE_STOP_BIT,
             USCI_UART_ADDRESS_BIT_MULTI_PROCESSOR_MODE,
             USCI_UART_LOW_FREQUENCY_BAUDRATE_GENERATION
             )){
        return;
    }

    //Enable UART module for operation
    USCI_UART_enable(USCI_A0_BASE);

    //Set USCI UART in sleep mode
    USCI_UART_setDormant(USCI_A0_BASE);

    //Enable Receive Interrupt
    USCI_UART_enableInterrupt(USCI_A0_BASE,
        USCI_UART_RECEIVE_INTERRUPT
        );

    //Enter LPM3, interrupts enabled
    __bis_SR_register(LPM3_bits + GIE);
    __no_operation();
}

//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:

            //Receive the data
            receivedData = USCI_UART_receiveData(USCI_A0_BASE);

            if (receiveDataCount){
                receiveDataCount--;
                //If RECEIVE_DATA_COUNT number of data received go into dormant mode
                if (0x00 == receiveDataCount){
                    USCI_UART_setDormant(USCI_A0_BASE);
                }
            } else   {
                //If address received, wake up from dormant mode for more data
                if (USCI_UART_MULTIPROCESSOR_MODE_ADDRESS == receivedData){
                    USCI_UART_resetDormant(USCI_A0_BASE);
                    receiveDataCount = RECEIVE_DATA_COUNT;
                }
            }
            //Send data back "echo"
            USCI_UART_transmitData(USCI_A0_BASE,
            receivedData
            );

            break;
        default: break;
    }
}

