#include "inc/hw_memmap.h"
#include "usci_uart.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"

//******************************************************************************
//!
//! This example shows how to configure the UART module to Transmit data and
//! read response
//!
//!                MSP430F5438A
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |                 |
//!            |     P3.4/UCA0TXD|------------>
//!            |                 | 9600
//!            |     P3.5/UCA0RXD|<------------
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UART peripheral
//! - GPIO Port peripheral (for UART pins)
//! - UCA0TXD
//! - UCA0RXD
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR.
//******************************************************************************
#define BAUD_RATE                               9600
#define TRANSMIT_DATA_COUNT                     0x02
#define USCI_UART_MULTIPROCESSOR_MODE_ADDRESS        0xAA

unsigned char receivedData = 0x00, transmitData = 0x00;
unsigned char transmitAddress = USCI_UART_MULTIPROCESSOR_MODE_ADDRESS,
              transmitDataCount = TRANSMIT_DATA_COUNT;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //P3.4,5 = USCI_A0 TXD/RXD
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P3,
        GPIO_PIN4 + GPIO_PIN5
        );

    if ( STATUS_FAIL == USCI_UART_init(USCI_A0_BASE,
             USCI_UART_CLOCKSOURCE_ACLK,
             UCS_getACLK(UCS_BASE),
             BAUD_RATE,
             USCI_UART_NO_PARITY,
             USCI_UART_MSB_FIRST,
             USCI_UART_ONE_STOP_BIT,
             USCI_UART_ADDRESS_BIT_MULTI_PROCESSOR_MODE,
             USCI_UART_LOW_FREQUENCY_BAUDRATE_GENERATION )){
        return;
    }

    //Enable UART module for operation
    USCI_UART_enable(USCI_A0_BASE);

    //Send Address Frame
    USCI_UART_transmitAddress(USCI_A0_BASE, transmitAddress);

    //Enable Receive Interrupt
    USCI_UART_enableInterrupt(USCI_A0_BASE,
        USCI_UART_RECEIVE_INTERRUPT
        );

    //Enter LPM3, interrupts enabled
    __bis_SR_register(LPM3_bits + GIE);
    __no_operation();
}

//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:

            //Receive the data
            receivedData = USCI_UART_receiveData(USCI_A0_BASE);

            if (transmitDataCount){
                //Transmit data
                USCI_UART_transmitData(USCI_A0_BASE,
                    transmitData++);
                transmitDataCount--;
            } else   {
                //Transmit Address
                USCI_UART_transmitAddress(USCI_A0_BASE,
                    transmitAddress);
                transmitDataCount = TRANSMIT_DATA_COUNT;
            }


            break;
        default: break;
    }
}

