#include "inc/hw_memmap.h"
#include "usci_uart.h"
#include "ucs.h"
#include "wdt_a.h"
#include "gpio.h"

//******************************************************************************
//!
//! This example shows how to configure the UART module to send data in
//! auto baudrate detection mode. To echo a received character, RX ISR is used.
//!
//!                MSP430F5438A
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |                 |
//!            |     P3.4/UCA0TXD|------------>
//!            |                 | 9600
//!            |     P3.5/UCA0RXD|<------------
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - UART peripheral
//! - GPIO Port peripheral (for UART pins)
//! - UCA0TXD
//! - UCA0RXD
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR.
//******************************************************************************
#define BAUD_RATE                               9600
#define TRANSMIT_DATA_COUNT                     0x03
#define USCI_UART_MULTIPROCESSOR_MODE_ADDRESS        0xAA

unsigned char receivedData = 0x00, transmitData = 0x00;
unsigned char transmitDataCount = TRANSMIT_DATA_COUNT;

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //P3.4,5 = USCI_A0 TXD/RXD
    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P3,
        GPIO_PIN4 + GPIO_PIN5
        );

    //Initialize USCI UART module
    if ( STATUS_FAIL == USCI_UART_init(USCI_A0_BASE,
             USCI_UART_CLOCKSOURCE_SMCLK,
             UCS_getSMCLK(UCS_BASE),
             BAUD_RATE,
             USCI_UART_NO_PARITY,
             USCI_UART_LSB_FIRST,
             USCI_UART_ONE_STOP_BIT,
             USCI_UART_AUTOMATIC_BAUDRATE_DETECTION_MODE,
             USCI_UART_OVERSAMPLING_BAUDRATE_GENERATION
             )){
        return;
    }

    //Enable UART module for operation
    USCI_UART_enable(USCI_A0_BASE);

    //Transmit Break
    USCI_UART_transmitBreak(USCI_A0_BASE);

    //Wait till ,odule is ready to transmit again
    while (!USCI_UART_getInterruptStatus(USCI_A0_BASE,
               USCI_UART_TRANSMIT_INTERRUPT_FLAG)) ;

    //Transmit data
    USCI_UART_transmitData(USCI_A0_BASE,
        transmitData++);
    transmitDataCount--;

    //Enable Receive Interrupt
    USCI_UART_enableInterrupt(USCI_A0_BASE,
        USCI_UART_RECEIVE_INTERRUPT);

    //Enter LPM3, interrupts enabled
    __bis_SR_register(LPM3_bits + GIE);
    __no_operation();
}

//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:

            //USCI_A0 TX buffer ready?
            while (!USCI_UART_getInterruptStatus(USCI_A0_BASE,
                       USCI_UART_TRANSMIT_INTERRUPT_FLAG)) ;

            //Receive the data
            receivedData = USCI_UART_receiveData(USCI_A0_BASE);

            if (transmitDataCount){
                //Send data
                USCI_UART_transmitData(USCI_A0_BASE,
                    transmitData++);
                transmitDataCount--;
            } else   {
                //Send Break
                USCI_UART_transmitBreak(USCI_A0_BASE);
                transmitDataCount = TRANSMIT_DATA_COUNT - 1;

                //Wait till module is ready to transmit next data
                while (!USCI_UART_getInterruptStatus(USCI_A0_BASE,
                           USCI_UART_TRANSMIT_INTERRUPT_FLAG)) ;

                //Transmit next data
                USCI_UART_transmitData(USCI_A0_BASE,
                    transmitData++);
            }

            break;
        default: break;
    }
}

