#include "inc/hw_memmap.h"
#include "adc10_b.h"
#include "gpio.h"
#include "wdt_a.h"

//******************************************************************************
//!
//! ADC10B - Sample A1 Input, AVcc Ref, P1.0 and P3.5 is set if A1 > 0.5AVcc
//!
//!  MSP430FR573x Demo
//!  A single sample is made on A1 with reference to AVcc. Software sets ADC10SC
//!  to start sample and conversion - ADC10SC automatically cleared at EOC.ADC10B
//!  internal oscillator times sample (16x) and conversion. In Mainloop MSP430
//!  waits in LPM0 to save power until ADC10B conversion complete, ADC10_ISR will
//!  force exit from LPM0 in Mainloop on reti. If A1 > 0.5*AVcc, P1.0 and and P3.5
//!  set, else reset.
//!                MSP430FR5739
//!             -----------------
//!         /|\|              XIN|-
//!          | |                 |
//!          --|RST          XOUT|-
//!            |                 |
//!        >---|P1.1/A1      P1.0|-->LED
//!			   |             P3.5|-->LED
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - ADC10 peripheral
//! - GPIO Port peripheral
//! - A1
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - ADC10_VECTOR
//!
//!   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************


void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set P1.1 as an input pin.
    /*
     * Select Port 1
     * Set Pin 1 as input
     * Set Ternary module function
     */
	GPIO_setAsPeripheralModuleFunctionInputPin(
			GPIO_PORT_P1,
			GPIO_PIN1,
			GPIO_TERNARY_MODULE_FUNCTION);

	//Set P1.0 as an output pin.
    /*
     
     * Select Port 1
     * Set Pin 0 as output
     */
	GPIO_setAsOutputPin(
			GPIO_PORT_P1,
			GPIO_PIN0);
			
	//Set P3.5 as an output pin.
    /*

     * Select Port 3
     * Set Pin 5 as output
     */
	GPIO_setAsOutputPin(
			GPIO_PORT_P3,
			GPIO_PIN5);

	//Initialize the ADC10B Module
    /*
     * Base Address for the ADC10B Module
     * Use internal ADC10B bit as sample/hold signal to start conversion
     * USE MODOSC 5MHZ Digital Oscillator as clock source
     * Use default clock divider of 1
     */
    ADC10_B_init(ADC10_B_BASE,
        ADC10_B_SAMPLEHOLDSOURCE_SC,
        ADC10_B_CLOCKSOURCE_ADC10OSC,
        ADC10_B_CLOCKDIVIDER_1);

    ADC10_B_enable(ADC10_B_BASE);

    /*
     * Base Address for the ADC10B Module
     * Sample/hold for 16 clock cycles
     * Do not enable Multiple Sampling
     */
    ADC10_B_setupSamplingTimer(ADC10_B_BASE,
        ADC10_B_CYCLEHOLD_16_CYCLES,
        ADC10_B_MULTIPLESAMPLESDISABLE);

    //Configure the Memory Buffer
    /*
     * Base Address for the ADC10B Module
     * Use input A1
     * Use positive reference of AVcc
     * Use negative reference of AVss
     */
    ADC10_B_memoryConfigure(ADC10_B_BASE,
        ADC10_B_INPUT_A1,
        ADC10_B_VREFPOS_AVCC,
        ADC10_B_VREFNEG_AVSS);

    //Enable the Memory Buffer Interrupt
    ADC10_B_enableInterrupt(ADC10_B_BASE,
        ADC10IE0);

    for (;;)
    {
        __delay_cycles(5000);

        //Enable and Start the conversion
        //in Single-Channel, Single Conversion Mode
        ADC10_B_startConversion(ADC10_B_BASE,
            ADC10_B_SINGLECHANNEL);

        //LPM0, ADC10_ISR will force exit
        __bis_SR_register(CPUOFF + GIE);
        __no_operation();


    }
}

//ADC10B interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
	switch (__even_in_range(ADC10IV,12)){
        case  0: break; //No interrupt
        case  2: break; //conversion result overflow
        case  4: break; //conversion time overflow
        case  6: break; //ADC10HI
        case  8: break; //ADC10LO
        case 10: break; //ADC10IN
        case 12:        //ADC10IFG0
            //(Automatically clears ADC10IFG0 by reading memory buffer)
           	if (ADC10_B_getResults(ADC10_B_BASE) < 0x1FF){

           		//Clear P1.0 LED off
           		/*
           		
           		* Select Port 1
           		* Set Pin 0 to output Low.
           		*/
           		GPIO_setOutputLowOnPin(
           			GPIO_PORT_P1,
           			GPIO_PIN0
           		);
				
				//Clear P3.5 LED off
           		/*
           		
           		* Select Port 3
           		* Set Pin 5 to output Low.
           		*/
           		GPIO_setOutputLowOnPin(
           			GPIO_PORT_P3,
           			GPIO_PIN5
           		);

	        } else   {
	            //Set P1.0 LED on
           		/*
           		
           		* Select Port 1
           		* Set Pin 0 to output high.
           		*/
           		GPIO_setOutputHighOnPin(
           			GPIO_PORT_P1,
           			GPIO_PIN0
           		);
				
	            //Set P3.5 LED on
           		/*
           		
           		* Select Port 3
           		* Set Pin 5 to output high.
           		*/
           		GPIO_setOutputHighOnPin(
           			GPIO_PORT_P3,
           			GPIO_PIN5
           		);
				
	        }

            //Clear CPUOFF bit from 0(SR)
            __bic_SR_register_on_exit(CPUOFF);
            break;
        default: break;
    }
}

