#include "inc/hw_memmap.h"
#include "dma.h"
#include "wdt_a.h"
#include "gpio.h"

//******************************************************************************
//!
//! DMA - Repeated Block Transfer to-and-from RAM, Software Trigger.
//!
//!  A 16 word block from 1C00-1C1Fh is transfered to 1C20h-1C3fh using DMA0 in
//!  a burst block using software DMAREQ trigger. After each transfer, source,
//!  destination and DMA size are reset to initial software setting because DMA
//!  transfer mode 5 is used. P1.0 is toggled during DMA transfer only for
//!  demonstration purposes.
//!  ** RAM location 0x1C00 - 0x1C3F used - make sure no compiler conflict **
//!  ACLK = REFO = 32kHz, MCLK = SMCLK = default DCO 1048576Hz
//!
//!   Tested on MSP430F5529, MSP430FR5739, MSP430FR5969
//!             -----------------
//!         /|\|              XIN|-
//!          | |                 | 32kHz
//!          --|RST          XOUT|-
//!            |                 |
//!            |             P1.0|-> LED
//!            |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - DMA peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - None.
//!
//******************************************************************************

void main (void)
{
    //Stop Watchdog Timer
    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Initialize and Setup DMA Channel 0
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 0
     * Configure channel for repeated block transfers
     * DMA interrupt flag will be set after every 16 transfers
     * Use DMA_startTransfer() function to trigger transfers
     * Transfer Word-to-Word
     * Trigger upon Rising Edge of Trigger Source Signal
     */
    DMA_init(DMA_BASE,
        DMA_CHANNEL_0,
        DMA_TRANSFER_REPEATED_BLOCK,
        16,
        DMA_TRIGGERSOURCE_0,
        DMA_SIZE_SRCWORD_DSTWORD,
        DMA_TRIGGER_RISINGEDGE);
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 0
     * Use 0x1C00 as source
     * Increment source address after every transfer
     */
    DMA_setSrcAddress(DMA_BASE,
        DMA_CHANNEL_0,
        0x1C00,
        DMA_DIRECTION_INCREMENT);
    /*
     * Base Address of the DMA Module
     * Configure DMA channel 0
     * Use 0x1C20 as destination
     * Increment destination address after every transfer
     */
    DMA_setDstAddress(DMA_BASE,
        DMA_CHANNEL_0,
        0x1C20,
        DMA_DIRECTION_INCREMENT);

    //Enable transfers on DMA channel 0
    DMA_enableTransfers(DMA_BASE,
        DMA_CHANNEL_0);

    while (1)
    {
        //set P1.0
        GPIO_setOutputHighOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );

        //Start block tranfer on DMA channel 0
        DMA_startTransfer(DMA_BASE,
            DMA_CHANNEL_0);

        //Clear P1.0 LED off
        GPIO_setOutputLowOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0
            );
    }
}

