#include "inc/hw_memmap.h"
#include "eusci_i2c.h"
#include "wdt_a.h"
#include "gpio.h"

//*****************************************************************************
//!
//!  Description: This demo connects two MSP430's via the I2C bus. The master
//!  reads from the slave. This is the SLAVE code. The TX data begins at 0
//!  and is incremented each time it is sent. A stop condition
//!  is used as a trigger to increment the outgoing data.
//!  The USCI_B0 TX interrupt is used to know
//!  when to TX.
//!	 Tested on MSP430FR5739, MSP430FR5969
//!
//!                                /|\  /|\
//!                                 10k  10k                 
//!                   slave         |    |        master
//!             -----------------   |    |   -----------------
//!           -|XIN  P1.6/UCB0SDA|<-|----+->|P1.6/UCB0SDA  XIN|-
//!            |                 |  |       |                 |
//!           -|XOUT             |  |       |             XOUT|-
//!            |     P1.7/UCB0SCL|<-+------>|P1.7/UCB0SCL     |
//!            |                 |          |                 |
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - I2C peripheral
//! - GPIO Port peripheral (for I2C pins)
//! - SCL2
//! - SDA
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_B0_VECTOR.
//!
//*****************************************************************************
//*****************************************************************************
//
//Set the address for slave module. This is a 7-bit address sent in the
//following format:
//[A6:A5:A4:A3:A2:A1:A0:RS]
//
//A zero in the "RS" position of the first byte means that the master
//transmits (sends) data to the selected slave, and a one in this position
//means that the master receives data from the slave.
//
//*****************************************************************************
#define SLAVE_ADDRESS 0x48

unsigned char TXData = 0x00;

void main(void)

{

	WDT_A_hold(WDT_A_BASE);

    // Configure Pins for I2C
	//Set P1.6 and P1.7 as Secondary Module Function Input.
	/*
	
	* Select Port 1
	* Set Pin 6, 7 to input Secondary Module Function, (UCB0SIMO/UCB0SDA, UCB0SOMI/UCB0SCL).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P1,
		GPIO_PIN6 + GPIO_PIN7,
		GPIO_SECONDARY_MODULE_FUNCTION
	);

    // eUSCI configuration
    EUSCI_I2C_slaveInit(EUSCI_B0_BASE,
       		SLAVE_ADDRESS,
       		EUSCI_I2C_OWN_ADDRESS_OFFSET0,
       		EUSCI_I2C_OWN_ADDRESS_ENABLE
       		);

    //Set in receive mode
    EUSCI_I2C_setMode(EUSCI_B0_BASE,
                EUSCI_I2C_TRANSMIT_MODE
                );

    //Enable I2C Module to start operations
    EUSCI_I2C_enable(EUSCI_B0_BASE);

    EUSCI_I2C_enableInterrupt(EUSCI_B0_BASE,
        		EUSCI_I2C_TRANSMIT_INTERRUPT0 +
        		EUSCI_I2C_STOP_INTERRUPT
              );

    __bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
    __no_operation();
}

//******************************************************************************
//
//This is the USCI_B0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector = USCI_B0_VECTOR
__interrupt void USCIB0_ISR(void)

{
switch(__even_in_range(UCB0IV,0x1E))
    {
        case 0x00: break; // Vector 0: No interrupts break;
        case 0x02: break; // Vector 2: ALIFG
        case 0x04: break; // Vector 4: NACKIFG
        case 0x06: break; // Vector 6: STTIFG
        case 0x08:       // Vector 8: STPIFG
          TXData++;
        break;
        case 0x0a: break; // Vector 10: RXIFG3 break;
        case 0x0c: break; // Vector 14: TXIFG3 break;
        case 0x0e: break; // Vector 16: RXIFG2 break;
        case 0x10: break; // Vector 18: TXIFG2 break;
        case 0x12: break; // Vector 20: RXIFG1 break;
        case 0x14: break; // Vector 22: TXIFG1 break;
        case 0x16: break; // Vector 24: RXIFG0 break;
        case 0x18:
             EUSCI_I2C_slaveDataPut(EUSCI_B0_BASE,
                TXData
                );
           break; // Vector 26: TXIFG0 break;
        case 0x1a: break; // Vector 28: BCNTIFG break;
        case 0x1c: break; // Vector 30: clock low timeout break;
        case 0x1e: break; // Vector 32: 9th bit break;
        default: break;

    }
}
