#include "inc/hw_memmap.h"
#include "eusci_spi.h"
#include "cs.h"
#include "wdt_a.h"
#include "gpio.h"
//*****************************************************************************
//! USCI_A0, SPI 3-Wire Master Incremented Data
//! This example shows how SPI master talks to SPI slave using 3-wire mode.
//! Incrementing data is sent by the master starting at 0x01. Received data is
//! expected to be same as the previous transmission.  eUSCI RX ISR is used to
//! handle communication with the CPU, normally in LPM0. If high, P1.0 indicates
//! valid data reception.  Because all execution after LPM0 is in ISRs,
//! initialization waits for DCO to stabilize against ACLK.
//! ACLK = ~32.768kHz, MCLK = SMCLK = DCO ~ 1048kHz.  BRCLK = SMCLK/2
//!
//! Use with eusci_spi_ex1_slave code example.  If slave is in debug mode, P1.1
//! slave reset signal conflicts with slave's JTAG; to work around, use IAR's
//! "Release JTAG on Go" on slave device.  If breakpoints are set in
//! slave RX ISR, master must stopped also to avoid overrunning slave
//! RXBUF.
//!
//                   MSP430FR5739
//!                 -----------------
//!            /|\ |                 |
//!             |  |                 |
//!    Master---+->|RST              |
//!                |                 |
//!                |             P2.0|-> Data Out (UCA0SIMO)
//!                |                 |
//!                |             P2.1|<- Data In (UCA0SOMI)
//!                |                 |
//!                |             P1.5|-> Serial Clock Out (UCA0CLK)
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - SPI peripheral
//! - GPIO Port peripheral (for SPI pins)
//! - UCA0SIMO
//! - UCA0SOMI
//! - UCA0CLK
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR
//!
//*****************************************************************************

unsigned char RXData =0;
unsigned char TXData = 0;

void main(void)
{
  volatile unsigned int i;
  unsigned short returnValue = 0x00;

  //Stop watchdog timer
  WDT_A_hold(WDT_A_BASE);

    //Set P1.0 as an output pin.
    /*
     
     * Select Port 1
     * Set Pin 0 as output
     */
	GPIO_setAsOutputPin(
		GPIO_PORT_P1,
		GPIO_PIN0
	);
	//Set P1.0 as Output Low.
	/*
	
	* Select Port 1
	* Set Pin 0 to output Low.
	*/
	GPIO_setOutputLowOnPin(
		GPIO_PORT_P1,
		GPIO_PIN0
	);

    // Configure Pins for LFXIN
	//Set PJ.4 and PJ.5 as Primary Module Function Input.
	/*
	
	* Select Port J
	* Set Pin 4, 5 to input Primary Module Function, (LFXIN).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_PJ,
		GPIO_PIN4 + GPIO_PIN5,
		GPIO_PRIMARY_MODULE_FUNCTION
	);

  //Set external frequency for XT1
  CS_setExternalClockSource(CS_BASE,32768,0);
  //Set DCO frequency to max DCO setting
  CS_setDCOFreq(CS_BASE,CS_DCORSEL_0,CS_DCOFSEL_3);
  //Select XT1 as the clock source for ACLK with no frequency divider
  CS_clockSignalInit(CS_BASE,CS_ACLK,CS_XT1CLK_SELECT,CS_CLOCK_DIVIDER_1);
  //Start XT1 with no time out
  CS_XT1Start(CS_BASE,CS_XT1DRIVE_0);

	// Configure SPI pins
    // Configure Pins for UCA0CLK
	//Set P1.5 as Secondary Module Function Input.
	/*
	
	* Select Port 1
	* Set Pin 5 to input Secondary Module Function, (UCA0CLK).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P1,
		GPIO_PIN5,
		GPIO_SECONDARY_MODULE_FUNCTION
	);
	// Configure Pins for UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI 
	//Set P2.0, P2.1 as Secondary Module Function Input.
	/*
	
	* Select Port 2
	* Set Pin 0, 1 to input Secondary Module Function, (UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P2,
		GPIO_PIN0 + GPIO_PIN1,
		GPIO_SECONDARY_MODULE_FUNCTION
	);

  //Initialize Master
  returnValue = EUSCI_SPI_masterInit(EUSCI_A0_BASE,
          EUSCI_SPI_CLOCKSOURCE_ACLK,
          CS_getACLK(CS_BASE),
          500000,
          EUSCI_SPI_MSB_FIRST,
          EUSCI_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
          EUSCI_SPI_CLOCKPOLARITY_INACTIVITY_HIGH,
          EUSCI_SPI_3PIN
          );

      if (STATUS_FAIL == returnValue){
          return;
      }

  //Enable SPI module
  EUSCI_SPI_enable(EUSCI_A0_BASE);

  // Enable USCI_A0 RX interrupt
  EUSCI_SPI_enableInterrupt(EUSCI_A0_BASE,
                         EUSCI_SPI_RECEIVE_INTERRUPT);

  //Wait for slave to initialize
  __delay_cycles(100);

  TXData = 0x1;                             // Holds TX data

  //USCI_A0 TX buffer ready?
  while (!EUSCI_SPI_getInterruptStatus(EUSCI_A0_BASE,
              EUSCI_SPI_TRANSMIT_INTERRUPT)) ;

  //Transmit Data to slave
  EUSCI_SPI_transmitData(EUSCI_A0_BASE, TXData);


  __bis_SR_register(LPM0_bits + GIE);      // CPU off, enable interrupts
  __no_operation();                       // Remain in LPM0


}

#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:
			//USCI_A0 TX buffer ready?
			while (!EUSCI_SPI_getInterruptStatus(EUSCI_A0_BASE,
					   EUSCI_SPI_TRANSMIT_INTERRUPT)) ;

			RXData = EUSCI_SPI_receiveData(EUSCI_A0_BASE);

			//Increment data
			TXData++;

			//Send next value
			EUSCI_SPI_transmitData(EUSCI_A0_BASE,
							TXData
							);

			//Delay between transmissions for slave to process information
			__delay_cycles(40);

            break;
        default: break;
    }
}
