#include "inc/hw_memmap.h"
#include "eusci_spi.h"
#include "wdt_a.h"
#include "gpio.h"

//*****************************************************************************
//!   SPI slave talks to SPI master using 3-wire mode. Data is received
//!   from master and data from slave is then transmitted back to master.
//!   USCI RX ISR is used to handle communication, CPU normally in LPM4.
//!   Prior to initial data exchange, master pulses slaves RST for complete
//!   reset.
//!
//!   Use with eusci_spi_ex1_master code example.  If the slave is in
//!   debug mode, the reset signal from the master will conflict with slave's
//!   JTAG; to work around, use IAR's "Release JTAG on Go" on slave device.  If
//!   breakpoints are set in slave RX ISR, master must stopped also to avoid
//!   overrunning slave RXBUF.
//!
//!   Tesed on MSP430FR5969 and MSP430FR5739
//!                 -----------------
//!            /|\ |                 |
//!             |  |                 |
//!    Master---+->|RST              |
//!                |                 |
//!                |             P2.0|-> Data Out (UCA0SIMO)
//!                |                 |
//!                |             P2.1|<- Data In (UCA0SOMI)
//!                |                 |
//!                |             P1.5|<- Serial Clock Out (UCA0CLK)
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - SPI peripheral
//! - GPIO Port peripheral (for SPI pins)
//! - UCA0SIMO
//! - UCA0SOMI
//! - UCA0CLK
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - USCI_A0_VECTOR
//!
//
//*****************************************************************************
unsigned char transmitData = 0x01, receiveData = 0x00;

void main(void)
{
  unsigned short returnValue = STATUS_FAIL;

  //Stop watchdog timer
  WDT_A_hold(WDT_A_BASE);

  // Configure Pins for LFXIN
	//Set PJ.4 and PJ.5 as Primary Module Function Input.
	/*
	
	* Select Port J
	* Set Pin 4, 5 to input Primary Module Function, (LFXIN).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_PJ,
		GPIO_PIN4 + GPIO_PIN5,
		GPIO_PRIMARY_MODULE_FUNCTION
	);

	// Configure SPI pins
	// Configure Pins for UCA0CLK
	//Set P1.5 as Secondary Module Function Input.
	/*
	
	* Select Port 1
	* Set Pin 5 to input Secondary Module Function, (UCA0CLK).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P1,
		GPIO_PIN5,
		GPIO_SECONDARY_MODULE_FUNCTION
	);
	// Configure Pins for UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI 
	//Set P2.0, P2.1 as Secondary Module Function Input.
	/*
	
	* Select Port 2
	* Set Pin 0, 1 to input Secondary Module Function, (UCA0TXD/UCA0SIMO, UCA0RXD/UCA0SOMI).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P2,
		GPIO_PIN0 + GPIO_PIN1,
		GPIO_SECONDARY_MODULE_FUNCTION
	);

  //Initialize slave to MSB first, inactive high clock polarity and 3 wire SPI
  returnValue = EUSCI_SPI_slaveInit(EUSCI_A0_BASE,
          EUSCI_SPI_MSB_FIRST,
          EUSCI_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
          EUSCI_SPI_CLOCKPOLARITY_INACTIVITY_HIGH,
		  EUSCI_SPI_3PIN
          );

  if (STATUS_FAIL == returnValue){
          return;
      }

  //Enable SPI Module
  EUSCI_SPI_enable(EUSCI_A0_BASE);

  //Enable Receive interrupt
  EUSCI_SPI_enableInterrupt(EUSCI_A0_BASE,
		  EUSCI_SPI_RECEIVE_INTERRUPT
          );

  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0, enable interrupts



}

//******************************************************************************
//
//This is the USCI_A0 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR (void)
{
    switch (__even_in_range(UCA0IV,4)){
        //Vector 2 - RXIFG
        case 2:
            //USCI_A0 TX buffer ready?
            while (!EUSCI_SPI_getInterruptStatus(EUSCI_A0_BASE,
                       EUSCI_SPI_TRANSMIT_INTERRUPT
                       )) ;

            //Transmit data to master
            EUSCI_SPI_transmitData(EUSCI_A0_BASE,
            transmitData
            );

            //Receive data from master
            receiveData = EUSCI_SPI_receiveData(EUSCI_A0_BASE);

            //Increment data to be transmitted
            transmitData++;

            break;

        default: break;
    }
}

