//******************************************************************************
//  Timer_B, Toggle P1.0, CCR0 Cont. Mode ISR, DCO SMCLK
//
//  Description: Toggle P1.0 using software and TB_0 ISR. Toggles every
//  50000 SMCLK cycles. SMCLK provides clock source for TBCLK.
//  During the TB_0 ISR, P1.0 is toggled and 50000 clock cycles are added to
//  CCR0. TB_0 ISR is triggered every 50000 cycles. CPU is normally off and
//  used only during TB_ISR.
//  ACLK = n/a, MCLK = SMCLK = TBCLK = default DCO ~1.045MHz
//
//  Tested On: MSP430FR5739
//         ---------------
//     /|\|               |
//      | |               |
//      --|RST            |
//        |               |
//        |           P1.0|-->LED
//
//******************************************************************************

#include "inc/hw_memmap.h"
#include "timer_b.h"
#include "wdt_a.h"
#include "gpio.h"

#define COMPARE_VALUE 50000

void main(void)
{
	//Stop WDT
	WDT_A_hold(WDT_A_BASE);

	//Set P1.0 to output direction
	GPIO_setAsOutputPin(
	        GPIO_PORT_P1,
	        GPIO_PIN0
	        );

	//Start timer in continuous mode sourced by SMCLK
	TIMER_B_configureContinuousMode( TIMER_B0_BASE,
	        TIMER_B_CLOCKSOURCE_SMCLK,
	        TIMER_B_CLOCKSOURCE_DIVIDER_1,
	        TIMER_B_TBIE_INTERRUPT_DISABLE,
	        TIMER_B_DO_CLEAR
	        );

	 //Initiaze compare mode
	TIMER_B_initCompare(TIMER_B0_BASE,
	        TIMER_B_CAPTURECOMPARE_REGISTER_0,
	        TIMER_B_CAPTURECOMPARE_INTERRUPT_ENABLE,
	        TIMER_B_OUTPUTMODE_OUTBITVALUE,
	       COMPARE_VALUE
	       );

	TIMER_B_startCounter( TIMER_B0_BASE,
		TIMER_B_CONTINUOUS_MODE
		);

  __bis_SR_register(LPM0_bits + GIE);       // Enter LPM0, enable interrupts
  __no_operation();                         // For debugger
}

// Timer B0 interrupt service routine
#pragma vector=TIMERB0_VECTOR
__interrupt void TIMERB0_ISR (void)
{
	GPIO_toggleOutputOnPin(
	        GPIO_PORT_P1,
	        GPIO_PIN0
	        );

	// Add Offset to CCR0 [Cont mode]
	TIMER_B_setCompareValue(TIMER_B0_BASE,
	        TIMER_B_CAPTURECOMPARE_REGISTER_0,
	        COMPARE_VALUE
	        );
}

