//******************************************************************************
//  MSP430FR59xx Demo - ADC12B, Sample A1, 1.2V Shared Ref, Set P1.0 if A1 > 0.5V
//
//  Description: A single sample is made on A1 with reference to internal
//  1.2V Vref. Software sets ADC12BSC to start sample and conversion - ADC10SC
//  automatically cleared at EOC. ADC12B internal oscillator times sample (16x)
//  and conversion. In Mainloop MSP430 waits in LPM0 to save power until ADC10
//  conversion complete, ADC12_B_ISR will force exit from LPM0 in Mainloop on
//  reti. If A1 > 0.5V, P1.0 set, else reset.
//
//                MSP430FR5969
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//        >---|P1.1/A1      P1.0|-->LED
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "adc12_b.h"
#include "wdt_a.h"
#include "gpio.h"
#include "ref_a.h"

void main(void)
{

	// Stop WDT
    WDT_A_hold(WDT_A_BASE);
	//Set P1.0 as an output pin.
	/*
	 
	 * Select Port 1
	 * Set Pin 0 as output
	 */
	GPIO_setAsOutputPin(
			GPIO_PORT_P1,
			GPIO_PIN0
	);

	//Set P1.0 as Output Low.
	/*
	
	* Select Port 1
	* Set Pin 0 to output Low.
	*/
	GPIO_setOutputLowOnPin(
		GPIO_PORT_P1,
		GPIO_PIN0
	);
	//Set P1.1 as Ternary Module Function Output.
	/*
	
	* Select Port 1
	* Set Pin 1 to output Ternary Module Function, (A1, C1, VREF+, VeREF+).
	*/
	GPIO_setAsPeripheralModuleFunctionOutputPin(
		GPIO_PORT_P1,
		GPIO_PIN1,
		GPIO_TERNARY_MODULE_FUNCTION
	);

    //If ref generator busy, WAIT
    while (REF_A_isRefGenBusy(REF_A_BASE)) ;

    //Select internal ref = 1.2V
    REF_A_setReferenceVoltage(REF_A_BASE,
        REF_A_VREF1_2V);

    //Turn on Reference Voltage
    REF_A_enableReferenceVoltage(REF_A_BASE);

  //Initialize the ADC12B Module
  /*
   * Base address of ADC12B Module
   * Use internal ADC12B bit as sample/hold signal to start conversion
   * USE MODOSC 5MHZ Digital Oscillator as clock source
   * Use default clock divider/pre-divider of 1
	 * Map to internal channel 0
   */
  ADC12_B_init(ADC12_B_BASE,
		ADC12_B_SAMPLEHOLDSOURCE_SC,
		ADC12_B_CLOCKSOURCE_ADC12OSC,
		ADC12_B_CLOCKDIVIDER_1,
		ADC12_B_CLOCKPREDIVIDER__1,
		ADC12_B_MAPINTCH0);

	//Enable the ADC12B module
  ADC12_B_enable(ADC12_B_BASE);

  /*
   * Base address of ADC12B Module
   * For memory buffers 0-7 sample/hold for 64 clock cycles
   * For memory buffers 8-15 sample/hold for 4 clock cycles (default)
   * Disable Multiple Sampling
   */
  ADC12_B_setupSamplingTimer(ADC12_B_BASE,
      ADC12_B_CYCLEHOLD_16_CYCLES,
      ADC12_B_CYCLEHOLD_4_CYCLES,
      ADC12_B_MULTIPLESAMPLESDISABLE);

  //Configure Memory Buffer
  /*
   * Base address of the ADC12B Module
   * Configure memory buffer 0
   * Map input A1 to memory buffer 0
   * Vref+ = IntBuffer
   * Vref- = AVss
   * Memory buffer 0 is not the end of a sequence
   */
  ADC12_B_memoryConfigure(ADC12_B_BASE,
      ADC12_B_MEMORY_0,
      ADC12_B_INPUT_A1,
      ADC12_B_VREFPOS_INTBUF_VREFNEG_VSS,
      ADC12_B_NOTENDOFSEQUENCE,
      ADC12_B_WINDOW_COMPARATOR_DISABLE,
      ADC12_B_DIFFERENTIAL_MODE_DISABLE);

  //Enable memory buffer 0 interrupt
  ADC12_B_enableInterrupt(ADC12_B_BASE,
	  ADC12_B_IE0,
	  0,
	  0);

  __delay_cycles(75);                       // reference settling ~75us

  while(1)
  {
    __delay_cycles(5000);                    // Delay between conversions

	//Enable/Start sampling and conversion
    /*
     * Base address of ADC12B Module
     * Start the conversion into memory buffer 0
     * Use the single-channel, single-conversion mode
     */
    ADC12_B_startConversion(ADC12_B_BASE,
        ADC12_B_MEMORY_0,
        ADC12_B_SINGLECHANNEL);

    __bis_SR_register(LPM0_bits + GIE);      // LPM0, ADC10_ISR will force exit
    __no_operation();                        // For debug only
  }
}

#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
  switch(__even_in_range(ADC12IV,12))
  {
    case  0: break;                         // Vector  0:  No interrupt
    case  2: break;                         // Vector  2:  ADC12BMEMx Overflow
    case  4: break;                         // Vector  4:  Conversion time overflow
    case  6: break;                         // Vector  6:  ADC12BHI
    case  8: break;                         // Vector  8:  ADC12BLO
    case 10: break;                         // Vector 10:  ADC12BIN
    case 12:                                // Vector 12:  ADC12BMEM0 Interrupt
	  if (ADC12_B_getResults(ADC12_B_BASE, ADC12_B_MEMORY_0) >= 0x6B4)
      {
    		//Set P1.0 LED on
    		/*
    		
    		* Select Port 1
    		* Set Pin 0 to output high.
    		*/
    		GPIO_setOutputHighOnPin(
    			GPIO_PORT_P1,
    			GPIO_PIN0
    		);
      }
      else
      {
  		//Set P1.0 LED off
  		/*
  		
  		* Select Port 1
  		* Set Pin 0 to output high.
  		*/
  		GPIO_setOutputLowOnPin(
  			GPIO_PORT_P1,
  			GPIO_PIN0
  		);
      }
		  __bic_SR_register_on_exit(LPM0_bits); // Exit active CPU
      break;                                // Clear CPUOFF bit from 0(SR)
    case 14: break;                         // Vector 14:  ADC12BMEM1
    case 16: break;                         // Vector 16:  ADC12BMEM2
    case 18: break;                         // Vector 18:  ADC12BMEM3
    case 20: break;                         // Vector 20:  ADC12BMEM4
    case 22: break;                         // Vector 22:  ADC12BMEM5
    case 24: break;                         // Vector 24:  ADC12BMEM6
    case 26: break;                         // Vector 26:  ADC12BMEM7
    case 28: break;                         // Vector 28:  ADC12BMEM8
    case 30: break;                         // Vector 30:  ADC12BMEM9
    case 32: break;                         // Vector 32:  ADC12BMEM10
    case 34: break;                         // Vector 34:  ADC12BMEM11
    case 36: break;                         // Vector 36:  ADC12BMEM12
    case 38: break;                         // Vector 38:  ADC12BMEM13
    case 40: break;                         // Vector 40:  ADC12BMEM14
    case 42: break;                         // Vector 42:  ADC12BMEM15
    case 44: break;                         // Vector 44:  ADC12BMEM16
    case 46: break;                         // Vector 46:  ADC12BMEM17
    case 48: break;                         // Vector 48:  ADC12BMEM18
    case 50: break;                         // Vector 50:  ADC12BMEM19
    case 52: break;                         // Vector 52:  ADC12BMEM20
    case 54: break;                         // Vector 54:  ADC12BMEM21
    case 56: break;                         // Vector 56:  ADC12BMEM22
    case 58: break;                         // Vector 58:  ADC12BMEM23
    case 60: break;                         // Vector 60:  ADC12BMEM24
    case 62: break;                         // Vector 62:  ADC12BMEM25
    case 64: break;                         // Vector 64:  ADC12BMEM26
    case 66: break;                         // Vector 66:  ADC12BMEM27
    case 68: break;                         // Vector 68:  ADC12BMEM28
    case 70: break;                         // Vector 70:  ADC12BMEM29
    case 72: break;                         // Vector 72:  ADC12BMEM30
    case 74: break;                         // Vector 74:  ADC12BMEM31
    case 76: break;                         // Vector 76:  ADC12BRDY
    default: break;
  }
}
