//******************************************************************************
//  MSP430FR59xx Demo - ADC12B, Using an External Reference
//
//  Description: This example shows how to use an external positive reference for
//  the ADC12B. The external reference is applied to the VeREF+ pin. AVss is used
//  for the negative reference. A single conversion is performed on channel A0.
//  The conversion results are stored in ADC12BMEM0 and Test by applying a voltage
//  to channel A0, then setting and running to a break point at the "_NOP()"
//  instruction. To view the conversion results, open an SFR window in debugger
//  and view the contents of ADC12BMEM0.
//  NOTE: VeREF+ range: 1.4V (min) to AVCC (max)
//        VeREF- range: 0V (min) to 1.2V (max)
//        Differential ref voltage range: 1.4V(min) to AVCC(max)
//			(see datasheet for device specific information)
//
//
//                MSP430FR5969
//             -------------------
//         /|\|                   |
//          | |                   |
//          --|RST                |
//            |                   |
//     Vin -->|P1.0/A0            |
//            |                   |
//     REF -->|P1.1/VREF+/VeREF+  |
//            |                   |
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "adc12_b.h"
#include "wdt_a.h"
#include "gpio.h"

void main(void)
{
	// Stop WDT
    WDT_A_hold(WDT_A_BASE);

	//Set P1.0 as Ternary Module Function Output.
	/*
	
	* Select Port 1
	* Set Pin 0 to output Ternary Module Function, (A0, C0, VREF-, VeREF-).
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_P1,
		GPIO_PIN0,
		GPIO_TERNARY_MODULE_FUNCTION
	);

  //Initialize the ADC12B Module
  /*
   * Base address of ADC12B Module
   * Use internal ADC12B bit as sample/hold signal to start conversion
   * USE MODOSC 5MHZ Digital Oscillator as clock source
   * Use default clock divider/pre-divider of 1
	 * Map to internal channel 0
   */
  ADC12_B_init(ADC12_B_BASE,
		ADC12_B_SAMPLEHOLDSOURCE_SC,
		ADC12_B_CLOCKSOURCE_ADC12OSC,
		ADC12_B_CLOCKDIVIDER_1,
		ADC12_B_CLOCKPREDIVIDER__1,
		ADC12_B_MAPINTCH0);

  //Enable the ADC12B module
  ADC12_B_enable(ADC12_B_BASE);

  /*
   * Base address of ADC12B Module
   * For memory buffers 0-7 sample/hold for 16 clock cycles
   * For memory buffers 8-15 sample/hold for 4 clock cycles (default)
   * Disable Multiple Sampling
   */
  ADC12_B_setupSamplingTimer(ADC12_B_BASE,
      ADC12_B_CYCLEHOLD_16_CYCLES,
      ADC12_B_CYCLEHOLD_4_CYCLES,
      ADC12_B_MULTIPLESAMPLESDISABLE);

  //Configure Memory Buffer
  /*
   * Base address of the ADC12B Module
   * Configure memory buffer 0
   * Map input A0 to memory buffer 0
   * Vref+ = AVcc
   * Vref- = EXT Positive
   * Memory buffer 0 is not the end of a sequence
   */
  ADC12_B_memoryConfigure(ADC12_B_BASE,
      ADC12_B_MEMORY_0,
      ADC12_B_INPUT_A0,
      ADC12_B_VREFPOS_EXTPOS_VREFNEG_VSS,
      ADC12_B_NOTENDOFSEQUENCE,
      ADC12_B_WINDOW_COMPARATOR_DISABLE,
      ADC12_B_DIFFERENTIAL_MODE_DISABLE);

  while (1)
  {
    //Enable/Start first sampling and conversion cycle
    /*
     * Base address of ADC12B Module
     * Start the conversion into memory buffer 0
     * Use the single-channel, single-conversion mode
     */
	ADC12_B_startConversion(ADC12_B_BASE,
		ADC12_B_MEMORY_0,
		ADC12_B_SINGLECHANNEL);

    //Poll for interrupt on memory buffer 0
    while (!ADC12_B_getInterruptStatus(ADC12_B_BASE,
               0,
               ADC12_B_IFG0));

    __no_operation();                       // SET BREAKPOINT HERE
  }
}
