//******************************************************************************
// MSP430FR5969 Demo - COMPE interrupt capability; Vcompare is compared against
//                    internal 1.2V reference
//
// Description: Use COMPE and internal reference to determine if input'Vcompare'
// is high of low.  For the first time, when Vcompare exceeds the 1.2V internal
// reference, CEIFG is set and device enters the COMPE ISR. In the ISR, CEIES is
// toggled such that when Vcompare is less than 1.2V internal reference;
// CEIFG is set.
// LED is toggled inside the COMPE ISR
//                                                   
//                 MSP430FR5969
//             ------------------                        
//         /|\|                  |                       
//          | |                  |                       
//          --|RST      P3.1/CE13|<--Vcompare
//            |                  |                                         
//            |              P3.4|----> 'high'(Vcompare>2.0V); 'low'(Vcompare<2.5V)
//            |                  |  |
//            |                  |  | LED 'ON'(Vcompare>2.0V); 'OFF'(Vcompare<2.5V)
//            |                  | 
//
//   Nick Angelou
//   Texas Instruments Inc.
//   February 2012
//   Built with IAR Embedded Workbench Version: 5.40 & Code Composer Studio V5.1
//******************************************************************************
#include "inc/hw_memmap.h"
#include "comp_e.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //Set P3.4 as an output pin.
    /*
     
     * Select Port 3
     * Set Pin 4 as output
     */
  	GPIO_setAsOutputPin(
  			GPIO_PORT_P3,
  			GPIO_PIN4
  	);
    //Set P3.4 as Output Low.
    /*
     
     * Select Port 3
     * Set Pin 4 to output Low.
     */
    GPIO_setOutputLowOnPin(
  			GPIO_PORT_P3,
  			GPIO_PIN4
  	);

    //Initialize the Comparator E module
    /*
     * Base Address of Comparator E,
     * Pin CE13 to Positive(+) Terminal
     * Reference Voltage to Negative(-) Terminal
     * Normal Power Mode
     * Output Filter On with minimal delay
     * Non-Inverted Output Polarity
     */
    COMP_E_init(COMP_E_BASE,
        COMP_E_INPUT13,
        COMP_E_VREF,
        COMP_E_FILTEROUTPUT_OFF,
        COMP_E_NORMALOUTPUTPOLARITY
        );

    //Set the reference voltage that is being supplied to the (-) terminal
    /*
     * Base Address of Comparator E,
     * Reference Voltage of 1.5 V,
	 * Lower Limit of 1.2*(32/32) = 1.5V,
     * Upper Limit of 1.2*(32/32) = 1.5V
     */
    COMP_E_setReferenceVoltage(COMP_E_BASE,
        COMP_E_VREFBASE1_2V,
        32,
        32
        );

    //Enable COMP_E Interrupt on default rising edge for CEIFG
    COMP_E_interruptSetEdgeDirection(COMP_E_BASE, COMP_E_RISINGEDGE);

    // Clear any erroneous interrupts
    COMP_E_clearInterrupt(COMP_E_BASE, (COMP_E_INTERRUPT_FLAG + COMP_E_INTERRUPT_FLAG_INVERTED_POLARITY));

    //Enable Interrupts
    /*
     * Base Address of Comparator E,
     * Enable COMPE Interrupt on default rising edge for CEIFG
     */
    COMP_E_enableInterrupt(COMP_E_BASE,
        COMP_E_OUTPUT_INTERRUPT
        );

    //Allow power to Comparator module
    COMP_E_enable(COMP_E_BASE);

    //Enter LPM4 with interrupts enabled
    __bis_SR_register(LPM4_bits + GIE);

    //For debug
    __no_operation();
}

//******************************************************************************
//
//This is the COMP_E_VECTOR interrupt vector service routine.
//
//******************************************************************************
#pragma vector=COMP_E_VECTOR
__interrupt void COMP_E_ISR (void)
{
    //Toggle the edge at which an interrupt is generated
    COMP_E_interruptToggleEdgeDirection(COMP_E_BASE);

    //Clear Interrupt flag
    COMP_E_clearInterrupt(COMP_E_BASE, COMP_E_INTERRUPT_FLAG);

	//Toggle P3.4 output pin.
	/*
	
	* Select Port 3
	* Set Pin 4 as output
	*/
	GPIO_toggleOutputOnPin(
		GPIO_PORT_P3,
		GPIO_PIN4
	);
}
