//******************************************************************************
//  MSP430FR59xx Demo - COMPE Hysteresis, COUT Toggle in LPM4; High speed mode
//
//  Description: Use CompE and shared reference to determine if input 'Vcompare'
//  is high of low.  Shared reference is configured to generate hysteresis.
//  When Vcompare exceeds Vcc*3/4 COUT goes high and when Vcompare is less
//  than Vcc*1/4 then CEOUT goes low.
//  P3.5/COUT is connected internally to an LED; it will toggle accordingly.
//
//                 MSP430FR5969
//             ------------------                        
//         /|\|                  |                       
//          | |                  |                       
//          --|RST      P1.2/CE2 |<--Vcompare
//            |                  |                       
//            |         P3.5/COUT|----> 'high'(Vcompare>Vcc*3/4); 'low'(Vcompare<Vcc*1/4)
//            |                  |  |   
//            |                  |  | LED 'ON'(Vcompare>Vcc*3/4); 'OFF'(Vcompare<Vcc*1/4)
//            |                  | 
//                                                       
//******************************************************************************
#include "inc/hw_memmap.h"
#include "comp_e.h"
#include "wdt_a.h"
#include "gpio.h"

void main(void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);
 
  	//Set P3.5 as Ternary Module Function Output.
  	/*
  	
  	* Select Port 3
  	* Set Pin 5 to output Ternary Module Function, (COUT).
  	*/
  	GPIO_setAsPeripheralModuleFunctionOutputPin(
  		GPIO_PORT_P3,
  		GPIO_PIN5,
  		GPIO_TERNARY_MODULE_FUNCTION
  	);

  //Initialize the Comparator E module
    /* Base Address of Comparator E,
     * Pin CE2 to Positive(+) Terminal,
     * Reference Voltage to Negative(-) Terminal,
     * Normal Power Mode,
     * Output Filter On with minimal delay,
     * Non-Inverted Output Polarity
     */
    COMP_E_init(COMP_E_BASE,
        COMP_E_INPUT2,
        COMP_E_VREF,
        COMP_E_FILTEROUTPUT_OFF,
        COMP_E_NORMALOUTPUTPOLARITY
        );

    //Set the reference voltage that is being supplied to the (-) terminal
    /* Base Address of Comparator E,
     * Reference Voltage of Vcc V,
	 * Lower Limit of Vcc*(8/32) = Vcc*1/4,
     * Upper Limit of Vcc*(24/32) = Vcc*3/4
     */
    COMP_E_setReferenceVoltage(COMP_E_BASE,
    	COMP_E_VREFBASE_VCC,
        8,
        24
        );

    //Disable Input Buffer on P1.2/CE2
	/* Base Address of Comparator E,
	 * Input Buffer port
	 * Selecting the CEx input pin to the comparator
	 * multiplexer with the CEx bits automatically
	 * disables output driver and input buffer for
	 * that pin, regardless of the state of the
	 * associated CEPD.x bit
	 */
	  COMP_E_disableInputBuffer(COMP_E_BASE,
	  	  COMP_E_INPUT2);

    //Allow power to Comparator module
    COMP_E_enable(COMP_E_BASE);

  __delay_cycles(400);           // delay for the reference to settle
  
  __bis_SR_register(LPM4_bits); // Enter LPM4
  __no_operation();             // For debug 
} 
