//******************************************************************************
// MSP430FR59xx PMM Test
// This example shows transitions between a user Reset via the reset button, a
// software triggered BOR, a previous LPMx.5 state indicator flag, and entrance into LPM4.5
//
// Upon device power on you will see the following sequence of LED's
// X = off
// O = on
//
// _0 = On indicates BOR
// _6 = On indicates Reset (from user button)
// _5 = on indicates device was previously in LPMx.5 state before the reset
// _4 = On indicates device is NOT sleeping.  Off means device is in LPM4.5
//*******************************************************************************
//
// 	1) Power on device:
// 		X0 X6 X5 O4  			LED4 is on, device is running and was not in LPM4.5 mode previously.
//
// 	2) Press User Reset button:
//
// 		X0 O6 X5 O4				Device is running and has reset due to RST pin
// 		O0 X6 X5 O4				Device is running and has reset due to a software BOR
// 		X0 X6 X5 X4				Device is now in LPM4.5
//
//	3) Press User Reset button:
//		Device will restart the sequence beginning at step 2).
//
//
//********************************************************************************
#include "inc/hw_memmap.h"
#include "pmm.h"
#include "wdt_a.h"
#include "gpio.h"

void main(void)
{
    //Stop WDT
    WDT_A_hold(WDT_A_BASE);

      /*
       * Base Address of PMM,
       * By default, the pins are unlocked unless waking
       * up from an LPMx.5 state in which case all GPIO
       * are previously locked.
       *
       */
    PMM_unlockLPM5(PMM_BASE);
    //Set P3.4, 3.5, 3.6, 3.0 as output pins.
    /*
     
     * Select Port 3
     * Set Pin 4, 5, 6, 0 as output
     */
  	GPIO_setAsOutputPin(
  			GPIO_PORT_P3,
  			GPIO_PIN4 + GPIO_PIN5 + GPIO_PIN6 + GPIO_PIN0
  	);
    //Set P3.5, 3.6, 3.0 Low.
    /*
     
     * Select Port 3
     * Set Pin 5, 6, 0 as Low
     */
  	GPIO_setOutputLowOnPin(
  			GPIO_PORT_P3,
  			GPIO_PIN5 + GPIO_PIN6 + GPIO_PIN0
  	);
    //Set P3.4 as High.
    /*
     
     * Select Port 3
     * Set Pin 4 as High
     */
  	GPIO_setOutputHighOnPin(
  			GPIO_PORT_P3,
  			GPIO_PIN4
  	);

	if (PMM_getInterruptStatus(PMM_BASE, PMM_PMMRSTIFG))	// Was this reset triggered by the Reset flag?
	{
		PMM_clearInterrupt(PMM_BASE, PMM_PMMRSTIFG);			// Clear reset flag
	  	GPIO_setOutputHighOnPin(
	  			GPIO_PORT_P3,
	  			GPIO_PIN6
	  	);
        __delay_cycles(1000000); 													// to include a visual delay.
  	  //Lock GPIO output states (before triggering a BOR)
  		/*
  		 * Base Address of PMM,
  		 * Forces all GPIO to retain their output
  		 * states during a reset.
  		 */
        PMM_lockLPM5(PMM_BASE);
	  //Trigger a software Brown Out Reset (BOR)
		/*
		 * Base Address of PMM,
		 * Forces the devices to perform a BOR.
		 */
        PMM_trigBOR(PMM_BASE);								// Software trigger a BOR.
    }

	if (PMM_getInterruptStatus(PMM_BASE, PMM_PMMBORIFG))	// Was this reset triggered by the BOR flag?
	{
		PMM_clearInterrupt(PMM_BASE, PMM_PMMBORIFG);			// Clear BOR flag
	  	GPIO_setOutputHighOnPin(
	  			GPIO_PORT_P3,
	  			GPIO_PIN0
	  	);
        __delay_cycles(1000000); 													// to include a visual delay.
      	GPIO_setOutputLowOnPin(
      			GPIO_PORT_P3,
      			GPIO_PIN4 + GPIO_PIN0
      	);

        PMM_lockLPM5(PMM_BASE);

	   //Disable Regulator
		/*
		 * Base Address of PMM,
		 * Regulator is turned off when going to LPM3/4.
		 * System enters LPM3.5 or LPM4.5, respectively.
		 */
        PMM_regOff(PMM_BASE);
        __bis_SR_register(LPM4_bits); // Enter LPM4.5, This automatically locks
        							  // (if not locked already) all GPIO pins.
        							  //  and will set the LPM5 flag and set the LOCKLPM5 bit
        							  //  in the PM5CTL0 register upon wake up.
    }

    while (1)
    {
    	__no_operation();             // Don't sleep
    }
}
