//******************************************************************************
//!
//! RTC_B in Calendar mode, Interruptions every 1s, 1m, and 5th day of
//!     week at 5:00pm
//!
//!  This program demonstrates the RTC mode by triggering an interrupt every
//!  second and minute. This code toggles P1.0 every second. This code
//!  recommends an external LFXT1 crystal for RTC accuracy.
//!  Note that if XT1 is not present the code loops in an infinite loop.
//!  ACLK = LFXT1 = 32768Hz, MCLK = SMCLK = default DCO = 32 x ACLK = 1048576Hz
//!
//!  Tested On: MSP430FR5969
//!             -----------------
//!        /|\ |              XIN|-
//!         |  |                 | 32kHz
//!         ---|RST          XOUT|-
//!            |                 |
//!            |            P1.0 |--> Toggles every second
//!            |                 |
//!
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - RTC peripheral
//! - UCS peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - RTC_B_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "rtc_b.h"
#include "sfr.h"
#include "wdt_a.h"
#include "gpio.h"
#include "cs.h"

volatile Calendar newTime;

void main (void)
{
    Calendar currentTime;

    WDT_A_hold(WDT_A_BASE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    /*
	* Select Port J
	* Set Pin 4, 5 to input Primary Module Function, LFXT.
	*/
	GPIO_setAsPeripheralModuleFunctionInputPin(
		GPIO_PORT_PJ,
		GPIO_PIN4 + GPIO_PIN5,
		GPIO_PRIMARY_MODULE_FUNCTION
	);


    //Initialize LFXT1
    CS_LFXTStart( CS_BASE,
    	CS_LFXT_DRIVE3
    	);

    //Setup Current Time for Calendar
    currentTime.Seconds    = 0x00;
    currentTime.Minutes    = 0x26;
    currentTime.Hours      = 0x13;
    currentTime.DayOfWeek  = 0x03;
    currentTime.DayOfMonth = 0x20;
    currentTime.Month      = 0x07;
    currentTime.Year       = 0x2011;

    //Initialize Calendar Mode of RTC
    /*
     * Base Address of the RTC_B
     * Pass in current time, intialized above
     * Use BCD as Calendar Register Format
     */
    RTC_B_calendarInit(RTC_B_BASE,
        currentTime,
        RTC_B_FORMAT_BCD);

    //Setup Calendar Alarm for 5:00pm on the 5th day of the week.
    //Note: Does not specify day of the week.
    RTC_B_setCalendarAlarm(RTC_B_BASE,
        0x00,
        0x17,
        RTC_B_ALARMCONDITION_OFF,
        0x05);

    //Specify an interrupt to assert every minute
    RTC_B_setCalendarEvent(RTC_B_BASE,
        RTC_B_CALENDAREVENT_MINUTECHANGE);

    //Enable interrupt for RTC Ready Status, which asserts when the RTC
    //Calendar registers are ready to read.
    //Also, enable interrupts for the Calendar alarm and Calendar event.
    RTC_B_enableInterrupt(RTC_B_BASE,
        RTCRDYIE + RTCTEVIE + RTCAIE);

    //Start RTC Clock
    RTC_B_startClock(RTC_B_BASE);

    //Enter LPM3 mode with interrupts enabled
    __bis_SR_register(LPM0_bits + GIE);
    __no_operation();
}

#pragma vector=RTC_VECTOR
__interrupt void RTC_B_ISR (void)
{
    switch (__even_in_range(RTCIV,16)){
        case 0: break;  //No interrupts
        case 2:         //RTCRDYIFG
            //Toggle P1.0 every second
            GPIO_toggleOutputOnPin(
            GPIO_PORT_P1,
            GPIO_PIN0);
            break;
        case 4:         //RTCEVIFG
            //Interrupts every minute
            __no_operation();

            //Read out New Time a Minute Later BREAKPOINT HERE
            newTime = RTC_B_getCalendarTime(RTC_B_BASE);
            break;
        case 6:         //RTCAIFG
            //Interrupts 5:00pm on 5th day of week
            __no_operation();
            break;
        case 8: break;  //RT0PSIFG
        case 10: break; //RT1PSIFG
        case 12: break; //Reserved
        case 14: break; //Reserved
        case 16: break; //Reserved
        default: break;
    }
}

