//******************************************************************************
//!
//! WDT - Toggle P1.0, Interval Overflow ISR, 32kHz ACLK
//!
//!  Toggle P1.0 using software timed by WDT ISR. Toggle rate is exactly 250ms
//!  based on 32kHz ACLK WDT clock source. In this example the WDT is configured
//!  to divide 32768 watch-crystal(2^15) by 2^13 with an ISR triggered
//!  @ 4Hz = [WDT CLK source/32768].
//!  ACLK = REFO , MCLK = SMCLK = default DCO ~1.045MHz
//!
//!  Tested On: MSP430F5529,MSP430FR5739,MSP430FR5969
//!             -----------------
//!         /|\|                 |
//!          | |                 |
//!          --|RST              |
//!            |             P1.0|-->LED
//!            |                 |
//!
//! This example uses the following peripherals and I/O signals.  You must
//! review these and change as needed for your own board:
//! - WDT peripheral
//! - GPIO Port peripheral
//!
//! This example uses the following interrupt handlers.  To use this example
//! in your own application you must add these interrupt handlers to your
//! vector table.
//! - WDT_A_VECTOR
//!
//******************************************************************************

#include "inc/hw_memmap.h"
#include "sfr.h"
#include "wdt_a.h"
#include "gpio.h"

void main (void)
{
    //Initialize WDT module in timer interval mode,
    //with ACLK as source at an interval of 250 ms.
    WDT_A_intervalTimerInit(WDT_A_BASE,
        WDT_A_CLOCKSOURCE_ACLK,
        WDT_A_CLOCKDIVIDER_8192);

    //Enable Watchdog Interupt
    SFR_enableInterrupt(SFR_BASE,
        WDTIE);

    //Set P1.0 to output direction
    GPIO_setAsOutputPin(
        GPIO_PORT_P1,
        GPIO_PIN0
        );

    //Enter LPM3, enable interrupts
    __bis_SR_register(LPM3_bits + GIE);
    //For debugger
    __no_operation();
}

//Watchdog Timer interrupt service routine
#pragma vector = WDT_VECTOR
__interrupt void WDT_A_ISR (void)
{
    //Toggle P1.0
    GPIO_toggleOutputOnPin(
        GPIO_PORT_P1,
        GPIO_PIN0);
}

