//*****************************************************************************
//
// Hitachi138x110_HD66753.c - Display driver for the Hitachi EA Hitachi102W-6 Dot Matrix
//                                     display with a HD66753 controller.
//
// Copyright (c) 2008-2011 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
//
//*****************************************************************************
//
//! \addtogroup display_api
//! @{
//
//*****************************************************************************

#include "inc\hw_memmap.h"
#include "grlib.h"
#include "driverlibHeaders.h"
#include "Hitachi138x110_HD66753.h"


unsigned int Hitachi138x110Memory[18*110] = {0};
//extern unsigned int FFT_Image[];

unsigned char ucBacklightLevel = 8;
unsigned char ucContrast = 0x66;

unsigned char LcdInitializationMacro[] = {
    0x74, 0x00, 0x00, 0x76, 0x00, 0x01,            // R00 start oscillation
    0x74, 0x00, 0x01, 0x76, 0x00, 0x0D,            // R01 driver output control
    0x74, 0x00, 0x02, 0x76, 0x00, 0x4C,            // R02 LCD - driving waveform control
    0x74, 0x00, 0x03, 0x76, 0x12, 0x14,            // R03 Power control
    0x74, 0x00, 0x04, 0x76, 0x04, 0x66,            // R04 Contrast control
    0x74, 0x00, 0x05, 0x76, 0x00, 0x10,            // R05 Entry mode
    0x74, 0x00, 0x06, 0x76, 0x00, 0x00,            // R06 RAM data write mask
    0x74, 0x00, 0x07, 0x76, 0x00, 0x15,            // R07 Display control
    0x74, 0x00, 0x08, 0x76, 0x00, 0x03,            // R08 Cursor Control
    0x74, 0x00, 0x09, 0x76, 0x00, 0x00,            // R09 RAM data write mask
    0x74, 0x00, 0x0A, 0x76, 0x00, 0x15,            // R0A
    0x74, 0x00, 0x0B, 0x76, 0x00, 0x03,            // R0B Horizontal Cursor Position
    0x74, 0x00, 0x0C, 0x76, 0x00, 0x03,            // R0C Vertical Cursor Position
    0x74, 0x00, 0x0D, 0x76, 0x00, 0x00,            // R0D
    0x74, 0x00, 0x0E, 0x76, 0x00, 0x15,            // R0E
    0x74, 0x00, 0x0F, 0x76, 0x00, 0x03,            // R0F
    0x74, 0x00, 0x10, 0x76, 0x00, 0x15,            // R0E
    0x74, 0x00, 0x11, 0x76, 0x00, 0x03,            // R0F
};

//unsigned char Read_Block_Macro[] = {0x74, 0x00, 0x12, 0x77, 0x00, 0x00};
unsigned char Set_Address_Macro[] = {0x74, 0x00, 0x11, 0x76, 0x00, 0x00};
unsigned char Draw_Block_Macro[] = {0x74, 0x00, 0x12, 0x76, 0xFF, 0xFF};

//*****************************************************************************
//
// Writes a command to the HD66753.  This function implements the basic SPI
// interface to the LCD display.
//
//*****************************************************************************
static void
WriteCommand(unsigned char * pucCommand)
{
    unsigned char i;
    // Store current GIE state
    unsigned int gie = __get_SR_register() & GIE;

    // Make this operation atomic
    __disable_interrupt();

    LCD_CS_RST_OUT &= ~LCD_CS_PIN;                 //CS = 0 --> Start Transfer
    for (i = 0; i < 6; i++)
    {
        // Wait for TXIFG
        while(!(SPI_getInterruptStatus(__MSP430_BASEADDRESS_USCI_B2__,
                                    SPI_TRANSMIT_INTERRUPT)));
        // Send Data
        SPI_transmitData(__MSP430_BASEADDRESS_USCI_B2__,
                         *pucCommand++);

        if (i == 2)                                //Pull CS up after 3 bytes
        {
            while (UCB2STAT & UCBUSY) ;
            LCD_CS_RST_OUT |= LCD_CS_PIN;          //CS = 1 --> Stop Transfer
            LCD_CS_RST_OUT &= ~LCD_CS_PIN;         //CS = 0 --> Start Transfer
        }
    }
    while (UCB2STAT & UCBUSY) ;
    LCD_CS_RST_OUT |= LCD_CS_PIN;                  //CS = 1 --> Stop Transfer

    // Restore original GIE state
    __bis_SR_register(gie);
}

//*****************************************************************************
//
// Sets the pixel address of the HD66753.  This function implements the basic SPI
// interface to the LCD display.
//
//*****************************************************************************
void SetAddress(int lX, int lY)
{
    unsigned int ulAddress;
    ulAddress = (lY << 5) + (lX >> 3);            //Narrow down to 8 possible pixels
    Set_Address_Macro[4] = ulAddress >> 8;
    Set_Address_Macro[5] = ulAddress & 0xFF;
    WriteCommand(Set_Address_Macro);
}

//*****************************************************************************
//
//! Enables the display driver.
//!
//! This function enables the HD66753 display controller on the panel,
//! and also clears display data.
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_enable(void)
{
    WriteCommand(LcdInitializationMacro);         // R00 start oscillation

    // Wait a minimum of 25ms after issuing "start oscillation"
    // command (to accomodate for MCLK up to 25MHz)
    __delay_cycles(250000);

    LcdInitializationMacro[3 * 6 + 5] |= BIT3;
    LcdInitializationMacro[3 * 6 + 5] &= ~BIT0;
    WriteCommand(&LcdInitializationMacro[3 * 6]); // R03 Power control
}

//*****************************************************************************
//
//! Disables the display driver.
//!
//! This function disables the HD66753 display controller on the panel,
//! and also clears display data.
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_disable(void)
{
    // Disable backlight
    LCD_BACKLT_DIR |= LCD_BACKLIGHT_PIN;
    LCD_BACKLT_OUT &= ~(LCD_BACKLIGHT_PIN);
    LCD_BACKLT_SEL &= ~LCD_BACKLIGHT_PIN;
  
    TA0CCTL3 = 0;
    TA0CTL = 0;
  
    ucBacklightLevel = 0;
  
    // LCD screen in standby
    LcdInitializationMacro[3 * 6 + 5] &= (~BIT3) & (~BIT2);
    LcdInitializationMacro[3 * 6 + 5] |= BIT0;
    WriteCommand(&LcdInitializationMacro[3 * 6]);
}

//*****************************************************************************
//
// Initializes the pins required for the GPIO-based LCD interface.
//
// This function configures the GPIO pins used to control the LCD display
// when the basic GPIO interface is in use. On exit, the LCD controller
// has been reset and is ready to receive command and data writes.
//
// \return None.
//
//*****************************************************************************
static void
InitGPIOLCDInterface(void)
{
    //
    // Configure the pins that connect to the LCD as outputs.
    //
    
    LCD_CS_RST_OUT |= LCD_CS_PIN | LCD_RESET_PIN;
    LCD_CS_RST_DIR |= LCD_CS_PIN | LCD_RESET_PIN;

    LCD_BACKLT_SEL |= LCD_BACKLIGHT_PIN;

    LCD_CS_RST_OUT &= ~LCD_RESET_PIN;              // Reset LCD
    __delay_cycles(0x47FF);                        //Reset Pulse
    LCD_CS_RST_OUT |= LCD_RESET_PIN;

    // UCLK,MOSI setup, SOMI cleared
    LCD_SPI_SEL |= LCD_MOSI_PIN + LCD_CLK_PIN;
    LCD_SPI_SEL &= ~LCD_MISO_PIN;
    LCD_SPI_DIR &= ~(LCD_MISO_PIN + LCD_MOSI_PIN); // Pin direction controlled by module,
                                                   // Set both pins to input as default

    // Initialize the USCI_B2 module for SPI operation
    SPI_masterInit(__MSP430_BASEADDRESS_USCI_B2__,
                   SPI_CLOCKSOURCE_SMCLK,
                   UCS_getSMCLK(__MSP430_BASEADDRESS_UCS__),
                   4000000,
                   SPI_MSB_FIRST,
                   SPI_PHASE_DATA_CAPTURED_ONFIRST_CHANGED_ON_NEXT,
                   SPI_CLOCKPOLARITY_INACTIVITY_LOW);
    
    SPI_enable(__MSP430_BASEADDRESS_USCI_B2__);

    UCB2IFG &= ~UCRXIFG;
}

//*****************************************************************************
//
//! Inverts the display color.
//!
//! This function inverts the colors displayed without changing
//! any data in the local buffer
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_InvertDisplay(void)
{
    LcdInitializationMacro[7 * 6 + 5] ^= BIT1;
    WriteCommand(&LcdInitializationMacro[7 * 6]);
}

//*****************************************************************************
//
//! Clear the screen
//!
//! This function "clears" by filling the entire screen with white pixels
//! and changes the corresponding pixels in the buffer as well
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_HD66753ClearDisplay(void *pvDisplayData, unsigned char ucValue)
{
  unsigned int i, j;
  
  //
  // Update Macro for write to LCD
  //
  Draw_Block_Macro[4] = 0xFF;
  Draw_Block_Macro[5] = 0xFF;
      
  //Row Address
  for(i = 0; i < 110; i++)
  {
    SetAddress(0,i);
    
    //Column Address
    for(j = 0; j < 18; j++)
    {
      //
      // Update Buffer
      //
      Hitachi138x110Memory[i * 18 + j] = 0xFFFF;
            
      //
      // Write new data to LCD
      //
      WriteCommand(Draw_Block_Macro); 
    }
  }
}


//*****************************************************************************
//
//! Initialize the backlight.
//!
//! This function initializes the backlight of the display
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_backlightInit(void)
{
    // Turn on Backlight
    // Uses PWM to control brightness
    LCD_BACKLT_DIR |= LCD_BACKLIGHT_PIN;
    LCD_BACKLT_OUT |= LCD_BACKLIGHT_PIN;
    LCD_BACKLT_SEL |= LCD_BACKLIGHT_PIN;

    TA0CCTL3 = OUTMOD_7;
    TA0CCR3 = TA0CCR0 >> 1;
    ucBacklightLevel = 8;

    TA0CCR0 = 400;
    TA0CTL = TASSEL_2 + MC_1;
}


//*****************************************************************************
//
//! Sets the backlight.
//!
//! This function sets the backlight level of the display,
//! from a level of 0-11 where 11 is the brightest setting.
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_setBacklight(unsigned char ucBrightness)
{
    unsigned int dutyCycle = 0, i, dummy;

    if (ucBrightness > 0)
    {
        TA0CCTL3 = OUTMOD_7;
        dummy = (TA0CCR0 >> 4);

        for (i = 0; i < ucBrightness; i++)
            dutyCycle += dummy;

        TA0CCR3 = dutyCycle;

        // If the backlight was previously turned off, turn it on.
        if (!ucBacklightLevel)
            TA0CTL |= MC0;
    }
    else
    {
        TA0CCTL3 = 0;
        TA0CTL &= ~MC0;
    }
    ucBacklightLevel = ucBrightness;
}

//*****************************************************************************
//
//! Shuts off the backlight.
//!
//! This function shuts off the backlight of the display
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_backlightOff(void)
{
    LCD_BACKLT_DIR |= LCD_BACKLIGHT_PIN;
    LCD_BACKLT_OUT &= ~(LCD_BACKLIGHT_PIN);
    LCD_BACKLT_SEL &= ~LCD_BACKLIGHT_PIN;

    TA0CCTL3 = 0;
    TA0CTL = 0;

    ucBacklightLevel = 0;
}

//*****************************************************************************
//
//! Sets the contrast.
//!
//! This function sets the contrast of the HD66753 display controller,
//! from a level of 0-31 where 31 is the darkest setting.
//!
//! \return None.
//
//*****************************************************************************
void Hitachi138x110_setContrast(unsigned char ucNewContrast)
{
    if (ucNewContrast > 127) ucNewContrast = 127;
    if (ucNewContrast < 70) ucNewContrast = 70;
    LcdInitializationMacro[0x04 * 6 + 5] = ucNewContrast;
    WriteCommand(&LcdInitializationMacro[0x04 * 6]);
    ucContrast = ucNewContrast;
}

//*****************************************************************************
//
//! Initializes the display driver.
//!
//! This function initializes the HD66753 display controller on the panel,
//! preparing it to display data.
//!
//! \return None.
//
//*****************************************************************************

void
Hitachi138x110_HD66753Init(void)
{
    InitGPIOLCDInterface();     

    // Wake-up the LCD as per datasheet specifications
    Hitachi138x110_enable();

    // LCD Initialization Routine Using Predefined Macros
    WriteCommand(&LcdInitializationMacro[1 * 6]);
    WriteCommand(&LcdInitializationMacro[2 * 6]);
    WriteCommand(&LcdInitializationMacro[4 * 6]);
    WriteCommand(&LcdInitializationMacro[5 * 6]);
    WriteCommand(&LcdInitializationMacro[6 * 6]);
    WriteCommand(&LcdInitializationMacro[7 * 6]);
    
    // Clear Display and turn on backlight
    Hitachi138x110_backlightInit();
    Hitachi138x110_setBacklight(ucBacklightLevel);
    Hitachi138x110_setContrast(ucContrast);
}

//*****************************************************************************
//
//! Draws a pixel on the screen.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX is the X coordinate of the pixel.
//! \param lY is the Y coordinate of the pixel.
//! \param ulValue is the color of the pixel.
//!
//! This function sets the given pixel to a particular color.  The coordinates
//! of the pixel are assumed to be within the extents of the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Hitachi138x110_HD66753PixelDraw(void *pvDisplayData, int lX, int lY,
                                   unsigned int ulValue)
{  
  unsigned int ulBufferLocation, ulBufferValue, ulPixelOffset;
  
  //
  // Pixel location in the global LCD buffer
  //
  ulBufferLocation = lY * 18 + (lX >> 3);
  
  //
  // Pixel offset into the global buffer word containing 8 2-bit pixels
  // Multiply the X%8 by 2 because each pixel requires 2 bits
  //
  ulPixelOffset = (lX & 0x07) << 1;
  
  //
  // Extract the current value in the buffer so that the new
  // pixel won't affect the other 7 pixels present
  //
  ulBufferValue = Hitachi138x110Memory[ulBufferLocation];
  
  //
  // Clear the current pixel value. 0x03 = 0b11 to clear
  // 2 bits that represent the single pixel
  //
  ulBufferValue &= ~(0x03 << ulPixelOffset);
  
  //
  // Add in the new pixel value
  //
  ulBufferValue |= ulValue << ulPixelOffset;
  
  //
  // Rewrite the new buffer element to the buffer
  //
  Hitachi138x110Memory[ulBufferLocation] = ulBufferValue;
  
  //
  // Set the address for the pixel write
  //
  SetAddress(lX, lY);
  
  //
  // Update Macro for write to LCD
  //
  Draw_Block_Macro[4] = ulBufferValue >> 8;
  Draw_Block_Macro[5] = ulBufferValue & 0xFF;
  
  //
  // Write new data to LCD
  //
  WriteCommand(Draw_Block_Macro);  
}

//*****************************************************************************
//
//! Draws a horizontal sequence of pixels on the screen.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX is the X coordinate of the first pixel.
//! \param lY is the Y coordinate of the first pixel.
//! \param lX0 is sub-pixel offset within the pixel data, which is valid for 1
//! or 4 bit per pixel formats.
//! \param lCount is the number of pixels to draw.
//! \param lBPP is the number of bits per pixel; must be 1, 4, or 8.
//! \param pucData is a pointer to the pixel data.  For 1 and 4 bit per pixel
//! formats, the most significant bit(s) represent the left-most pixel.
//! \param pucPalette is a pointer to the palette used to draw the pixels.
//!
//! This function draws a horizontal sequence of pixels on the screen, using
//! the supplied palette.  For 1 bit per pixel format, the palette contains
//! pre-translated colors; for 4 and 8 bit per pixel formats, the palette
//! contains 24-bit RGB values that must be translated before being written to
//! the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Hitachi138x110_HD66753PixelDrawMultiple(void *pvDisplayData, int lX,
                                           int lY, int lX0, int lCount,
                                           int lBPP,
                                           const unsigned char *pucData,
                                           const unsigned int *pucPalette)
{
  unsigned int ulBufferLocation, ulPixelOffset;
  unsigned int ulByte, ulValue; 
  
  SetAddress(lX, lY);
  
  ulBufferLocation = lY * 18 + (lX >> 3);
  ulPixelOffset = (lX & 0x07) << 1;       
  
  //
  // Determine how to interpret the pixel data based on the number of bits
  // per pixel. Only 1BPP and 2BPP is supported on this display
  //
  if(lBPP == 1)
  {
      // The pixel data is in 1 bit per pixel format
    
      // Loop while there are more pixels to draw
      while(lCount > 0)
      {
        // Get the next byte of image data
        ulByte = *pucData++;
          
        // Loop through the pixels in this byte of image data
        for(; (lX0 < 8) && lCount; lX0++, lCount--)
        {
          ulValue = ((unsigned int *)pucPalette)[(ulByte >> (7 - lX0)) & 1];
          
          if(ulValue)
          {
            // Set bits for White Pixel
            Hitachi138x110Memory[ulBufferLocation] |= (0x03 << ulPixelOffset);
          }
          else
          {
            // Clear bits for Black Pixel
            Hitachi138x110Memory[ulBufferLocation] &= ~(0x03 << ulPixelOffset);
          }          
          // End of integer, write to LCD
          // Update pixel offset and buffer location
          if(ulPixelOffset == 0x0E)
          {
            if(lCount != 0)
            {
              // Update data in macro
              Draw_Block_Macro[4] = Hitachi138x110Memory[ulBufferLocation] >> 8;
              Draw_Block_Macro[5] = Hitachi138x110Memory[ulBufferLocation] & 0xFF;
              // Write pixel data to LCD
              WriteCommand(Draw_Block_Macro);
              ulBufferLocation++;
              ulPixelOffset = 0;
            }
          }
          else
          {
            ulPixelOffset += 2;
          }
        }
        
        // Start at the beginning of the next byte of image data
        lX0 = 0;
      }
      // Update data in macro for final write
      Draw_Block_Macro[4] = Hitachi138x110Memory[ulBufferLocation] >> 8;
      Draw_Block_Macro[5] = Hitachi138x110Memory[ulBufferLocation] & 0xFF;
      // Write pixel data to LCD
      WriteCommand(Draw_Block_Macro);
      // The image data has been drawn
  }
  else if(lBPP == 2)
  {
    // Loop while there are more pixels to draw
    while(lCount > 0)
    {
      // Get the next byte of image data
      ulByte = *pucData++;
      
      // Loop through the pixels in this byte of image data
      for(; (lX0 < 4) && lCount; lX0++, lCount--)
      {
        ulValue = ((unsigned int *)pucPalette)[(ulByte >> (6 - (lX0 << 1))) & 3];
        
        // Clear pixel bits
        Hitachi138x110Memory[ulBufferLocation] &= ~(0x03 << ulPixelOffset);
        // Add in new pixel bits
        Hitachi138x110Memory[ulBufferLocation] |= ulValue << ulPixelOffset;
        // Update pixel offset and buffer location
        if(ulPixelOffset == 0x0E)
        {
          if(lCount != 0)
          {
            // Update Macro for write to LCD
            Draw_Block_Macro[4] = Hitachi138x110Memory[ulBufferLocation] >> 8;
            Draw_Block_Macro[5] = Hitachi138x110Memory[ulBufferLocation] & 0xFF;
            // Write pixel data to LCD
            WriteCommand(Draw_Block_Macro);
            ulBufferLocation++;
            ulPixelOffset = 0;
          }
        }
        else
        {
          ulPixelOffset += 2;
        }
      }
      // Start at the beginning of the next byte of image data
      lX0 = 0;
    }
    // Update data in macro for final write
    Draw_Block_Macro[4] = Hitachi138x110Memory[ulBufferLocation] >> 8;
    Draw_Block_Macro[5] = Hitachi138x110Memory[ulBufferLocation] & 0xFF;
    // Write pixel data to LCD
    WriteCommand(Draw_Block_Macro);
    // The image data has been drawn
  }
}

//*****************************************************************************
//
//! Draws a horizontal line.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX1 is the X coordinate of the start of the line.
//! \param lX2 is the X coordinate of the end of the line.
//! \param lY is the Y coordinate of the line.
//! \param ulValue is the color of the line.
//!
//! This function draws a horizontal line on the display.  The coordinates of
//! the line are assumed to be within the extents of the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Hitachi138x110_HD66753LineDrawH(void *pvDisplayData, int lX1, int lX2,
                                   int lY, unsigned int ulValue)
{
  unsigned int ulBufferLocation, ulPixelMask1, ulPixelMask2, ulBufferValue, ulByte;
  unsigned char ucWordAddress1, ucWordAddress2;
  
  SetAddress(lX1, lY);
  
  // Create pixel masks for the start/end of the line
  ulPixelMask1 = 0xFFFF << ((lX1 & 0x07) << 1);
  ulPixelMask2 = 0xFFFF >> (0x0E - ((lX2 & 0x07) << 1));
  
  // Find buffer locations of the start/end of the line
  ucWordAddress1 = lX1 >> 3;    //lX1/16 * 2BPP;
  ucWordAddress2 = lX2 >> 3;    //lX2/16 * 2BPP;
  
  // Get current buffer value
  ulBufferLocation = lY * 18 + ucWordAddress1;
  ulBufferValue = Hitachi138x110Memory[ulBufferLocation];
  
  // Horizontal Line spans more than 1 integer write to LCD
  if(ucWordAddress1 != ucWordAddress2)
  {
    switch (ulValue)
    {
      // Black
      case 0:
        ulBufferValue &= ~ulPixelMask1;
        break;
      // Dark Gray
      case 1: 
        ulBufferValue &= ~ulPixelMask1;
        ulByte = 0x5555 & ulPixelMask1;
        ulBufferValue |= ulByte;
        break;
      // Light Gray
      case 2: 
        ulBufferValue &= ~ulPixelMask1;
        ulByte = 0xAAAA & ulPixelMask1;
        ulBufferValue |= ulByte;
        break;
      // White
      case 3: 
        ulBufferValue |= ulPixelMask1;
        break;
      default: 
        ulBufferValue &= ~ulPixelMask1;
        break;
    }
    // Rewrite the new buffer element to the buffer
    Hitachi138x110Memory[ulBufferLocation++] = ulBufferValue;
    // Update Macro for write to LCD
    Draw_Block_Macro[4] = ulBufferValue >> 8;
    Draw_Block_Macro[5] = ulBufferValue & 0xFF;
    // Write new data to LCD
    WriteCommand(Draw_Block_Macro);
    ucWordAddress1++;
    
    while(ucWordAddress1 < ucWordAddress2)
    {
      switch (ulValue)
      {
        // Black
        case 0:
          Hitachi138x110Memory[ulBufferLocation++] = 0x0000;
          Draw_Block_Macro[4] = 0x00;
          Draw_Block_Macro[5] = 0x00;
          WriteCommand(Draw_Block_Macro);
          break;
        // Dark Gray
        case 1: 
          Hitachi138x110Memory[ulBufferLocation++] = 0x5555;
          Draw_Block_Macro[4] = 0x55;
          Draw_Block_Macro[5] = 0x55;
          WriteCommand(Draw_Block_Macro);
          break;
        // Light Gray
        case 2: 
          Hitachi138x110Memory[ulBufferLocation++] = 0xAAAA;
          Draw_Block_Macro[4] = 0xAA;
          Draw_Block_Macro[5] = 0xAA;
          WriteCommand(Draw_Block_Macro);
          break;
        // White
        case 3: 
          Hitachi138x110Memory[ulBufferLocation++] = 0xFFFF;
          Draw_Block_Macro[4] = 0xFF;
          Draw_Block_Macro[5] = 0xFF;
          WriteCommand(Draw_Block_Macro);
          break;
        default: 
          Hitachi138x110Memory[ulBufferLocation++] = 0x0000;
          Draw_Block_Macro[4] = 0x00;
          Draw_Block_Macro[5] = 0x00;
          WriteCommand(Draw_Block_Macro);
          break;
      }
      ucWordAddress1++;
    }
    
    // Get buffer value at end of the line
    ulBufferValue = Hitachi138x110Memory[ulBufferLocation];
    switch (ulValue)
    {
      // Black
      case 0:
        ulBufferValue &= ~ulPixelMask2;
        break;
      // Dark Gray
      case 1: 
        ulBufferValue &= ~ulPixelMask2;
        ulByte = 0x5555 & ulPixelMask2;
        ulBufferValue |= ulByte;
        break;
      // Light Gray
      case 2: 
        ulBufferValue &= ~ulPixelMask2;
        ulByte = 0xAAAA & ulPixelMask2;
        ulBufferValue |= ulByte;
        break;
      // White
      case 3: 
        ulBufferValue |= ulPixelMask2;
        break;
      default: 
        ulBufferValue &= ~ulPixelMask2;
        break;
    }
    // Rewrite the new buffer element to the buffer
    Hitachi138x110Memory[ulBufferLocation] = ulBufferValue;
    // Update Macro for write to LCD
    Draw_Block_Macro[4] = ulBufferValue >> 8;
    Draw_Block_Macro[5] = ulBufferValue & 0xFF;
    // Write new data to LCD
    WriteCommand(Draw_Block_Macro);
  }
  // Horizontal line can be written with 1 integer write
  else
  {
    // Combine start/end pixel masks
    ulPixelMask1 &= ulPixelMask2;
    switch (ulValue)
    {
      // Black
      case 0:
        ulBufferValue &= ~ulPixelMask1;
        break;
      // Dark Gray
      case 1: 
        ulBufferValue &= ~ulPixelMask1;
        ulByte = 0x5555 & ulPixelMask1;
        ulBufferValue |= ulByte;
        break;
      // Light Gray
      case 2: 
        ulBufferValue &= ~ulPixelMask1;
        ulByte = 0xAAAA & ulPixelMask1;
        ulBufferValue |= ulByte;
        break;
      // White
      case 3: 
        ulBufferValue |= ulPixelMask1;
        break;
      default: 
        ulBufferValue &= ~ulPixelMask1;
        break;
    }
    // Rewrite the new buffer element to the buffer
    Hitachi138x110Memory[ulBufferLocation] = ulBufferValue;
    // Update Macro for write to LCD
    Draw_Block_Macro[4] = ulBufferValue >> 8;
    Draw_Block_Macro[5] = ulBufferValue & 0xFF;
    // Write new data to LCD
    WriteCommand(Draw_Block_Macro);
  }
}

//*****************************************************************************
//
//! Draws a vertical line.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX is the X coordinate of the line.
//! \param lY1 is the Y coordinate of the start of the line.
//! \param lY2 is the Y coordinate of the end of the line.
//! \param ulValue is the color of the line.
//!
//! This function draws a vertical line on the display.  The coordinates of the
//! line are assumed to be within the extents of the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Hitachi138x110_HD66753LineDrawV(void *pvDisplayData, int lX, int lY1,
                                   int lY2, unsigned int ulValue)
{
  do
  {
    Hitachi138x110_HD66753PixelDraw(pvDisplayData, lX, lY1, ulValue);
  }
  while(lY1++ < lY2);
}

//*****************************************************************************
//
//! Fills a rectangle.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param pRect is a pointer to the structure describing the rectangle.
//! \param ulValue is the color of the rectangle.
//!
//! This function fills a rectangle on the display.  The coordinates of the
//! rectangle are assumed to be within the extents of the display, and the
//! rectangle specification is fully inclusive (in other words, both sXMin and
//! sXMax are drawn, along with sYMin and sYMax).
//!
//! \return None.
//
//*****************************************************************************
static void
Hitachi138x110_HD66753RectFill(void *pvDisplayData, const tRectangle *pRect,
                                  unsigned int ulValue)
{
  int x0 = pRect->sXMin;
  int x1 = pRect->sXMax;
  int y0 = pRect->sYMin;
  int y1 = pRect->sYMax;
  
  do
  {
    Hitachi138x110_HD66753LineDrawH(pvDisplayData, x0, x1, y0, ulValue);
  }
  while(y0++ < y1);
}

//*****************************************************************************
//
//! Translates a 24-bit RGB color to a display driver-specific color.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param ulValue is the 24-bit RGB color.  The least-significant byte is the
//! blue channel, the next byte is the green channel, and the third byte is the
//! red channel.
//!
//! This function translates a 24-bit RGB color into a value that can be
//! written into the display's frame buffer in order to reproduce that color,
//! or the closest possible approximation of that color.
//!
//! \return Returns the display-driver specific color.
//
//*****************************************************************************
static unsigned int
Hitachi138x110_HD66753ColorTranslate(void *pvDisplayData,
                                        unsigned long ulValue)
{
    //
    // Translate from a 24-bit RGB color to a Black or White 1BPP color.
    //
    return(DPYCOLORTRANSLATE(ulValue));
}

//*****************************************************************************
//
//! Flushes any cached drawing operations.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//!
//! This functions flushes any cached drawing operations to the display.  This
//! is useful when a local frame buffer is used for drawing operations, and the
//! flush would copy the local frame buffer to the display. This function isn't
//! implemented for the main buffer because each driver operation updates the 
//! buffer and the display. It is used however to draw the FFT Image. This image
//! is placed in a buffer called FFT_Image[] instead of Hitachi138x110Memory[].
//! This flush does not update Hitachi138x110Memory[]. 
//!
//! \return None.
//
//*****************************************************************************
static void
Hitachi138x110_HD66753Flush(void *pvDisplayData)
{
    // Draw 128x84 pixel image for FFT image
    // 128 pixels = 256 bits for 2BPP
    // 256 bits / 16 bits/write = 16 writes per line
    unsigned int * pulData = (unsigned int *)pvDisplayData;
    int i, j;
    
    for (i = 0; i < 84; i++)
    {
      SetAddress(1, 12 + i);
      for(j = 0; j < 16; j ++)
      {
        Draw_Block_Macro[4] = *pulData >> 8;
        Draw_Block_Macro[5] = *pulData++ & 0xFF;
        WriteCommand(Draw_Block_Macro);  
      }
    }
}

//*****************************************************************************
//
//! The display structure that describes the driver for the Hitachi
//! 138x110 LCD with an HD66753 controller.
//
//*****************************************************************************
const tDisplay g_sHitachi138x110_HD66753 =
{
    sizeof(tDisplay),
    0,
#if defined(PORTRAIT) || defined(PORTRAIT_FLIP)
    110,
    138,
#else
    138,
    110,
#endif
    Hitachi138x110_HD66753PixelDraw,
    Hitachi138x110_HD66753PixelDrawMultiple,
    Hitachi138x110_HD66753LineDrawH,
    Hitachi138x110_HD66753LineDrawV,
    Hitachi138x110_HD66753RectFill,
    Hitachi138x110_HD66753ColorTranslate,
    Hitachi138x110_HD66753Flush,
    Hitachi138x110_HD66753ClearDisplay
};

//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
