//*****************************************************************************
//
// Sharp400x240.c
//
//*****************************************************************************
//
//! \addtogroup display_api
//! @{
//
//*****************************************************************************

#include <msp430.h>
#include "grlib.h"
#include "sharp400x240.h"
#include "driverlib/5xx_6xx/gpio.h"
#include "driverlib/5xx_6xx/spi.h"
#include "driverlib/5xx_6xx/timer.h"
#include "driverlib/5xx_6xx/ucs.h"

#define SPICLK  1000000

#define LANDSCAPE
//#define LANDSCAPE_FLIP

unsigned char DisplayBuffer[LCD_VERTICAL_MAX][LCD_HORIZONTAL_MAX/8];
unsigned char VCOMbit= 0x40;
unsigned char flagSendToggleVCOMCommand = 0;

//*****************************************************************************
//
// Reverses the bit order
//
//*****************************************************************************

unsigned char reverse(unsigned char x)
{
	unsigned char y = 0;
    y |= (((x & 0x10) >> 1) | ((x & 0x08) << 1));
    y |= (((x & 0x20) >> 3) | ((x & 0x04) << 3));
    y |= (((x & 0x40) >> 5) | ((x & 0x02) << 5));
    y |= (((x & 0x80) >> 7) | ((x & 0x01) << 7));

    return y;
}

//*****************************************************************************
//
//! Initializes the display driver.
//!
//! This function initializes the Sharp400*240 display. This function
//! configures the GPIO pins used to control the LCD display when the basic
//! GPIO interface is in use. On exit, the LCD has been reset and is ready to
//! receive command and data writes.
//!
//! \return None.
//
//*****************************************************************************
void
Sharp400x240_LCDInit(void)
{

      /* Config Boost enable pin as output */
      GPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT9_R__,GPIO_PORT_P9,
                              GPIO_PIN0);
      GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT9_R__,GPIO_PORT_P9,
                              GPIO_PIN0);
      /* Config DISP output as low */
      GPIO_setAsOutputPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                              (GPIO_PIN7 + GPIO_PIN6));
      GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                              (GPIO_PIN7 + GPIO_PIN6));


      /* P8.5 Option Select SIMO, 8.4 select CLK */
      GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT8_R__
                      ,GPIO_PORT_P8, GPIO_PIN5 + GPIO_PIN4);

      /* 3-pin, master, MSB, High Polarity SPI */
     SPI_masterInit(__MSP430_BASEADDRESS_USCI_B1__,
                     SPI_CLOCKSOURCE_SMCLK,
                     UCS_getSMCLK(__MSP430_BASEADDRESS_UCS__),
                     2000000,
                     SPI_MSB_FIRST,
                     SPI_PHASE_DATA_CAPTURED_ONFIRST_CHANGED_ON_NEXT,
                     SPI_CLOCKPOLARITY_INACTIVITY_LOW);

      SPI_enable(__MSP430_BASEADDRESS_USCI_B1__);


// Initialize EXTCOMIN for LCD
#ifdef EXTCOMIN_TOGGLE
      GPIO_setAsPeripheralModuleFunctionOutputPin(__MSP430_BASEADDRESS_PORT3_R__
                      ,GPIO_PORT_P3, GPIO_PIN3);
      Timer_generatePWM(__MSP430_BASEADDRESS_T1A3__,TIMER_CLOCKSOURCE_ACLK,
                      TIMER_CLOCKSOURCE_DIVIDER_1, 65536-1,
                      TIMER_CAPTURECOMPARE_REGISTER_2,
                      TIMER_OUTPUTMODE_RESET_SET,
                      32786);

#endif

       GPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                               GPIO_PIN7);
       GPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT9_R__,GPIO_PORT_P9,
                               GPIO_PIN0);

       __delay_cycles(2000000);                 //250ms delay for BOOST start-up
}

//*****************************************************************************
//
//! Shuts down the display driver.
//!
//! This function shuts down the LCD driver and turn the LCD boost off.
//!
//! \return None.
//
//*****************************************************************************
void Sharp400x240_LCDShutdown(void)
{
        GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                                GPIO_PIN7);
        GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT9_R__,GPIO_PORT_P9,
                                GPIO_PIN0);
}


//*****************************************************************************
//
//! Draws a pixel on the screen.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX is the X coordinate of the pixel.
//! \param lY is the Y coordinate of the pixel.
//! \param ulValue is the color of the pixel.
//!
//! This function sets the given pixel to a particular color.  The coordinates
//! of the pixel are assumed to be within the extents of the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_PixelDraw(void *pvDisplayData, int lX, int lY,
                                   unsigned int ulValue)
{
	if(lX > LCD_HORIZONTAL_MAX)
		return;

	if(lY > LCD_VERTICAL_MAX)
			return;

	if( ClrBlack == ulValue)
		DisplayBuffer[lY][lX/8] &= ~(0x80 >> (lX-((lX/8)*8) ));
	else
		DisplayBuffer[lY][lX/8] |= (0x80 >> (lX-((lX/8)*8) ));

}
//*****************************************************************************
//
//! Draws a horizontal sequence of pixels on the screen.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX is the X coordinate of the first pixel.
//! \param lY is the Y coordinate of the first pixel.
//! \param lX0 is sub-pixel offset within the pixel data, which is valid for 1
//! or 4 bit per pixel formats.
//! \param lCount is the number of pixels to draw.
//! \param lBPP is the number of bits per pixel; must be 1, 4, or 8.
//! \param pucData is a pointer to the pixel data.  For 1 and 4 bit per pixel
//! formats, the most significant bit(s) represent the left-most pixel.
//! \param pucPalette is a pointer to the palette used to draw the pixels.
//!
//! This function draws a horizontal sequence of pixels on the screen, using
//! the supplied palette.  For 1 bit per pixel format, the palette contains
//! pre-translated colors; for 4 and 8 bit per pixel formats, the palette
//! contains 24-bit RGB values that must be translated before being written to
//! the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_DrawMultiple(void *pvDisplayData, int lX,
                                           int lY, int lX0, int lCount,
                                           int lBPP,
                                           const unsigned char *pucData,
                                           const unsigned int *pucPalette)
{  
	unsigned char *pData = &DisplayBuffer[0][0];
	unsigned int xj = 0;

	for(xj=0;xj<lCount/8;xj++)
		*(pData + (lY * (LCD_HORIZONTAL_MAX/8) + (xj) + lX/8)) = *pucData++;

}
//*****************************************************************************
//
//! Draws a horizontal line.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX1 is the X coordinate of the start of the line.
//! \param lX2 is the X coordinate of the end of the line.
//! \param lY is the Y coordinate of the line.
//! \param ulValue is the color of the line.
//!
//! This function draws a horizontal line on the display.  The coordinates of
//! the line are assumed to be within the extents of the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_LineDrawH(void *pvDisplayData, int lX1, int lX2,
                                   int lY, unsigned int ulValue)
{

	unsigned int xi = 0;

	if(ClrBlack == ulValue)
	{
		for(xi = 0; (xi + lX1)<= lX2; xi++)
			DisplayBuffer[lY][(lX1 + xi)/8] &= ~(0x80 >> ((lX1 + xi)-(((lX1 + xi)/8)*8) ));
	}
	else
	{
		for(xi = 0; (xi + lX1)<= lX2; xi++)
			DisplayBuffer[lY][(lX1 + xi)/8] |= (0x80 >> ((lX1 + xi)-(((lX1 + xi)/8)*8) ));
	}

}


//*****************************************************************************
//
//! Draws a vertical line.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param lX is the X coordinate of the line.
//! \param lY1 is the Y coordinate of the start of the line.
//! \param lY2 is the Y coordinate of the end of the line.
//! \param ulValue is the color of the line.
//!
//! This function draws a vertical line on the display.  The coordinates of the
//! line are assumed to be within the extents of the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_LineDrawV(void *pvDisplayData, int lX, int lY1,
                                   int lY2, unsigned int ulValue)
{
	unsigned int yi = 0;

	if(ClrBlack == ulValue)
	{
		for(yi = 0; (yi + lY1)<= lY2; yi++)
			DisplayBuffer[lY1 + yi][lX/8] &= ~(0x80 >> (lX-(((lX)/8)*8) ));
	}
	else
	{
		for(yi = 0; (yi + lY1)<= lY2; yi++)
			DisplayBuffer[lY1 + yi][lX/8] |= (0x80 >> (lX-(((lX)/8)*8) ));
	}

    
}

//*****************************************************************************
//
//! Fills a rectangle.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param pRect is a pointer to the structure describing the rectangle.
//! \param ulValue is the color of the rectangle.
//!
//! This function fills a rectangle on the display.  The coordinates of the
//! rectangle are assumed to be within the extents of the display, and the
//! rectangle specification is fully inclusive (in other words, both sXMin and
//! sXMax are drawn, along with sYMin and sYMax).
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_RectFill(void *pvDisplayData, const tRectangle *pRect,
                                  unsigned int ulValue)
{
	unsigned int xi =0 , yi = 0;

	if(ClrBlack == ulValue)
	{
		for (yi = pRect->sYMin; yi<= pRect->sYMax; yi++)
		for(xi = pRect->sXMin; xi<= pRect->sXMax; xi++)
			DisplayBuffer[yi][xi/8] &= ~(0x80 >> (xi-(((xi)/8)*8) ));	}
	else
	{
		for (yi = pRect->sYMin; yi<= pRect->sYMax; yi++)
		for(xi = pRect->sXMin; xi<= pRect->sXMax; xi++)
			DisplayBuffer[yi][xi/8] |= (0x80 >> (xi-(((xi)/8)*8) ));
	}

}

//*****************************************************************************
//
//! Translates a 24-bit RGB color to a display driver-specific color.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//! \param ulValue is the 24-bit RGB color.  The least-significant byte is the
//! blue channel, the next byte is the green channel, and the third byte is the
//! red channel.
//!
//! This function translates a 24-bit RGB color into a value that can be
//! written into the display's frame buffer in order to reproduce that color,
//! or the closest possible approximation of that color.
//!
//! \return Returns the display-driver specific color.
//
//*****************************************************************************
static unsigned int
Sharp400x240_ColorTranslate(void *pvDisplayData,
                                        unsigned long ulValue)
{
    //
    // Translate from a 24-bit RGB color to a 5-6-5 RGB color.
    //
    return(DPYCOLORTRANSLATE(ulValue));
}



//*****************************************************************************
//
//! Initialize DisplayBuffer.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//!
//!	\param ucValue is the backgroundcolor of the buffered data.
//!
//! This function initializes the display buffer and discards any cached data.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_InitializeDisplayBuffer(void *pvDisplayData, unsigned char ucValue)
{
	unsigned int i=0,j=0;
	for(i =0; i< LCD_VERTICAL_MAX; i++)
	for(j =0; j< (LCD_HORIZONTAL_MAX/8); j++)
		DisplayBuffer[i][j] = ucValue;
}

//*****************************************************************************
//
//! Flushes any cached drawing operations.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//!
//!	\param ucValue is the foregroundcolor of the buffered data.
//!
//! This functions flushes any cached drawing operations to the display.  This
//! is useful when a local frame buffer is used for drawing operations, and the
//! flush would copy the local frame buffer to the display.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_Flush (void *pvDisplayData)
{
	unsigned char *pucData = &DisplayBuffer[0][0];
	long xi =0;
	long xj = 0;
	//image update mode(1X000000b)
	unsigned char command = SHARP_LCD_CMD_WRITE_LINE;

	command |= BIT7;
	//COM inversion bit
	command = command^VCOMbit;
	// Set P1.2 High for CS
    GPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                                    GPIO_PIN6);

	WriteCommand(command);
	flagSendToggleVCOMCommand = SHARP_SKIP_TOGGLE_VCOM_COMMAND;
#ifdef LANDSCAPE
	for(xj=0; xj<LCD_VERTICAL_MAX; xj++)
		{
		  WriteCommand(reverse(xj + 1));

		  for(xi=0; xi<(LCD_HORIZONTAL_MAX/8); xi++)
		  {
			WriteCommand(*(pucData + (xj * (LCD_HORIZONTAL_MAX/8) + xi)));
		  }
		  WriteCommand(SHARP_LCD_TRAILER_BYTE);
		}
#else
#ifdef LANDSCAPE_FLIP
	for(xj=0; xj<LCD_VERTICAL_MAX; xj++)
	{
	  WriteCommand(reverse(LCD_VERTICAL_MAX - xj));

	  for(xi=(LCD_HORIZONTAL_MAX/8) - 1; xi >= 0; xi--)
	  {
		WriteCommand(reverse(*(pucData + (xj * (LCD_HORIZONTAL_MAX/8) + xi))));
	  }
	  WriteCommand(SHARP_LCD_TRAILER_BYTE);
	}
#endif
#endif

	WriteCommand(SHARP_LCD_TRAILER_BYTE);

	__delay_cycles(100);
	// Set P1.2 Low for CS*/
	GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
	                                GPIO_PIN6);
}

//*****************************************************************************
//
//! Send command to clear screen.
//!
//! \param pvDisplayData is a pointer to the driver-specific data for this
//! display driver.
//!
//! This function does a clear screen and the Display Buffer contents
//! are initialized to the current background color.
//!
//! \return None.
//
//*****************************************************************************
static void
Sharp400x240_ClearScreen (void *pvDisplayData, unsigned char ucValue)
{
  unsigned char command = SHARP_LCD_CMD_CLEAR_SCREEN;
  //clear screen mode(0X100000b)
  command = command^VCOMbit;
  //COM inversion bit
  GPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                                  GPIO_PIN6);

  WriteCommand(command);
  flagSendToggleVCOMCommand = SHARP_SKIP_TOGGLE_VCOM_COMMAND;
  WriteCommand(SHARP_LCD_TRAILER_BYTE);

  // Wait for last byte to be sent, then drop SCS
  __delay_cycles(100);
  GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
                                    GPIO_PIN6);

  Sharp400x240_InitializeDisplayBuffer(pvDisplayData, ucValue);
}

#ifndef EXTCOMIN_TOGGLE
void
Sharp400x240_SendToggleVCOMCommand()
{
	VCOMbit ^= SHARP_VCOM_TOGGLE_BIT;

	if(SHARP_SEND_TOGGLE_VCOM_COMMAND == flagSendToggleVCOMCommand)
	{
		unsigned char command = SHARP_LCD_CMD_CHANGE_VCOM;						    //clear screen mode(0X100000b)
		command = command^VCOMbit;					//COM inversion bit
	    GPIO_setOutputHighOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
	                                    GPIO_PIN6);

		WriteCommand(command);
		WriteCommand(SHARP_LCD_TRAILER_BYTE);

		// Wait for last byte to be sent, then drop SCS
		__delay_cycles(100);
	    GPIO_setOutputLowOnPin(__MSP430_BASEADDRESS_PORT8_R__,GPIO_PORT_P8,
	                                    GPIO_PIN6);
	}
	flagSendToggleVCOMCommand = SHARP_SEND_TOGGLE_VCOM_COMMAND;
}
#endif

//*****************************************************************************
//
//! The display structure that describes the driver for the 
//! sharpLCD panel 
//
//*****************************************************************************
const tDisplay g_sharp400x240LCD =
{
    sizeof(tDisplay),
    0,
    LCD_HORIZONTAL_MAX,
    LCD_VERTICAL_MAX,
    Sharp400x240_PixelDraw, //PixelDraw,
    Sharp400x240_DrawMultiple,
    Sharp400x240_LineDrawH,
    Sharp400x240_LineDrawV, //LineDrawV,
    Sharp400x240_RectFill, //RectFill,
    Sharp400x240_ColorTranslate,
    Sharp400x240_Flush, //Flush
    Sharp400x240_ClearScreen //Clear screen. Contents of display buffer unmodified
};


                                      
//*****************************************************************************
//
// Close the Doxygen group.
//! @}
//
//*****************************************************************************
