/*  
 * ======== main.c ========
 *
 * CDC to CDC Chat Demo:
 *
 * This USB demo example is to be used with two instances of a PC application
 * (e.g. HyperTerminal) The demo application demonstrates a simple chat program
 * between the two terminals; Characters that are transmitted from one terminal
 * are sent to the other.
 *
 * NOTE: In order to see the local echo in Hyper Terminal goto File -> Propoerties
 * -> Settings -> ASCII Setup and check the "Echo Typed Characters Locally"
 *
 * ----------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 * ---------------------------------------------------------------------------*/
#include <intrinsics.h>
#include <string.h>

#include "USB_config/descriptors.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB-specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"



#include "main.h"
#include "usbConstructs.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);
VOID ASCII (char*);

volatile BYTE bDataReceived_event0 = FALSE;
volatile BYTE bDataReceived_event1 = FALSE;

#define MAX_STR_LENGTH 60

char wholeString[MAX_STR_LENGTH] = "";
char pieceOfString[MAX_STR_LENGTH] = "";

/*  
 * ======== main ========
 */
VOID main (VOID)
{
    WDTCTL = WDTPW + WDTHOLD;                                   //Stop watchdog timer

    Init_Ports();                                               //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                                               //Init clocks
	
    USB_init();                 //Init USB

    //Enable various USB event handling routines

    USB_setEnabledEvents(
        kUSB_VbusOnEvent + kUSB_VbusOffEvent + kUSB_receiveCompletedEvent
        + kUSB_dataReceivedEvent + kUSB_UsbSuspendEvent + kUSB_UsbResumeEvent +
        kUSB_UsbResetEvent);


    //See if we're already attached physically to USB, and if so, connect to it
    //Normally applications don't invoke the event handlers, but this is an exception.

    if (USB_connectionInfo() & kUSB_vbusPresent){
        if (USB_enable() == kUSB_succeed){
            USB_reset();
            USB_connect();
        }
    }

    __enable_interrupt();                           //Enable interrupts globally
    while (1)
    {
        BYTE ReceiveError = 0, SendError = 0;
        //Check the USB state and loop accordingly
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
                __bis_SR_register(LPM3_bits + GIE);             //Enter LPM3 until USB is connected
                _NOP();
                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:
                __bis_SR_register(LPM0_bits + GIE);             //Enter LPM0 until awakened by an event handler

                __no_operation();                               //For debugger

                                                                //Exit LPM because of a data-receive event, and fetch the received
                                                                //data
                if (bDataReceived_event0){                      //CDC0
                    bDataReceived_event0 = FALSE;
                    memset(pieceOfString,0,MAX_STR_LENGTH);     //Clear pieceOfString
                    cdcReceiveDataInBuffer((BYTE*)pieceOfString,
                        MAX_STR_LENGTH,
                        CDC0_INTFNUM);
                    strncat(wholeString,pieceOfString,strlen(pieceOfString));
                    if (retInString(wholeString)){              //Wait for enter key to be pressed
                        strcat(wholeString,"\r\r\n");
                        if (cdcSendDataInBackground((BYTE*)wholeString,
                                strlen(wholeString),CDC1_INTFNUM,1)){      //Send message to other App
                            SendError = 0x01;                   //Something went wrong -- exit
                            break;
                        }
                        memset(wholeString,0,MAX_STR_LENGTH);   //Clear wholeString
                    }
                }
                if (bDataReceived_event1){                      //CDC1
                    bDataReceived_event1 = FALSE;               //Clear flag early -- just in case execution breaks below because of
                                                                //an error
                    memset(pieceOfString,0,MAX_STR_LENGTH);     //Clear pieceOfString
                    cdcReceiveDataInBuffer((BYTE*)pieceOfString,
                        MAX_STR_LENGTH,
                        CDC1_INTFNUM);
                    strncat(wholeString,pieceOfString,strlen(pieceOfString));
                    if (retInString(wholeString)){              //Wait for enter key to be pressed
                        strcat(wholeString,"\r\r\n");
                        if (cdcSendDataInBackground((BYTE*)wholeString,
                                strlen(wholeString),CDC0_INTFNUM,1)){      	//Send message to other App
                            SendError = 0x01;                   			//Something went wrong -- exit
                            break;
                        }
                        memset(wholeString,0,MAX_STR_LENGTH);   //Clear wholeString
                    }
                }
                break;

            case ST_ENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);             //Enter LPM3 until a resume or VBUS-off event
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }
        if (ReceiveError || SendError){
            //TO DO: User can place code here to handle error
        }
    }  //while(1)
}                                                               //main()

/*  
 * ======== ASCII ========
 */
//Some terminal apps (specifically Hyperterminal) aren't very careful about clearing the most significant bit of the ASCII byte.
//As a result, other apps capable of extended character sets may display Hyperterminal-originated characters incorrectly.  This
//function fixes this by blanking the most significant bit.
VOID ASCII (char* string)
{
    BYTE i = 0;
    BYTE len = strlen(string);

    while ((string[i] != 0x0A) && (string[i] != 0x0D) && (i < len))
    {
        string[i] = string[i] & 0x7F;
        i++;
    }
}

/*  
 * ======== Init_Clock ========
 */
VOID Init_Clock (VOID)
{
    //Initialization of clock module
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x0C;                                      //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif

        //use REFO for FLL and ACLK
        UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
        XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
    } 
	else {
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x10;                                      //enable XT1 pins
		#endif
        //Use the REFO oscillator to source the FLL and ACLK
        UCSCTL3 = SELREF__REFOCLK;
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

        XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0xFF;
	P2OUT = 0x00;
    P2DIR = 0xFF;
    P3OUT = 0x00;
    P3DIR = 0xFF;
    P4OUT = 0x00;
    P4DIR = 0xFF;
    P5OUT = 0x00;
    P5DIR = 0xFF;
    P6OUT = 0x00;
    P6DIR = 0xFF;
	P7OUT = 0x00;
    P7DIR = 0xFF;
    P8OUT = 0x00;
    P8DIR = 0xFF;
    #if defined (__MSP430F563x_F663x)
		P9OUT = 0x00;
		P9DIR = 0xFF;
    #endif
}

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
            SYSBERRIV = 0;                                      //clear bus error flag
            USB_disable();                                      //Disable
    }
}

/*  
 * ======== retInString ========
 */
//This function returns true if there's an 0x0D character in the string; and if so,
//it trims the 0x0D and anything that had followed it.
BYTE retInString (char* string)
{
    BYTE retPos = 0,i,len;
    char tempStr[MAX_STR_LENGTH] = "";

    strncpy(tempStr,string,strlen(string));                     //Make a copy of the string
    len = strlen(tempStr);
    while ((tempStr[retPos] != 0x0A) && (tempStr[retPos] != 0x0D) &&
           (retPos++ < len)) ;                                  //Find 0x0D; if not found, retPos ends up at len

    if ((retPos < len) && (tempStr[retPos] == 0x0D)){           //If 0x0D was actually found...
        for (i = 0; i < MAX_STR_LENGTH; i++){                   //Empty the buffer
            string[i] = 0x00;
        }
        strncpy(string,tempStr,retPos);                         //...trim the input string to just before 0x0D
        return ( TRUE) ;                                        //...and tell the calling function that we did so
    } else if ((retPos < len) && (tempStr[retPos] == 0x0A)){    //If 0x0D was actually found...
        for (i = 0; i < MAX_STR_LENGTH; i++){                   //Empty the buffer
            string[i] = 0x00;
        }
        strncpy(string,tempStr,retPos);                         //...trim the input string to just before 0x0D
        return ( TRUE) ;                                        //...and tell the calling function that we did so
    } else if (tempStr[retPos] == 0x0D){
        for (i = 0; i < MAX_STR_LENGTH; i++){                   //Empty the buffer
            string[i] = 0x00;
        }
        strncpy(string,tempStr,retPos);                         //...trim the input string to just before 0x0D
        return ( TRUE) ;                                        //...and tell the calling function that we did so
    } else if (retPos < len){
        for (i = 0; i < MAX_STR_LENGTH; i++){                   //Empty the buffer
            string[i] = 0x00;
        }
        strncpy(string,tempStr,retPos);                         //...trim the input string to just before 0x0D
        return ( TRUE) ;                                        //...and tell the calling function that we did so
    }

    return ( FALSE) ;                                           //Otherwise, it wasn't found
}

