/*  
 * ======== main.c ========
 * H2 Example
 *
 * Receive 1K bytes from Host Demo:
 *
 * This example implements a device whose only purpose is to receive a 1K chunk
 * of data from the host.
 *
 * For the demonstration of this example use the UsbApp.exe. The UsbApp.exe GUI
 * can send a maximum of 2028 bytes at a time. So after sending 1024 characters,
 * it returns "Thanks for the data"
 *
 * Copy the contents "data.txt" present in the product folder and paste it in the
 * send text input field.
 +----------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 *----------------------------------------------------------------------------*/
#include <intrinsics.h>
#include <string.h>

#include "USB_config/descriptors.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB-specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"

#include "USB_API/USB_HID_API/UsbHid.h"

#include "usbConstructs.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);

BYTE dataBuff[1024];

volatile BYTE bHIDDataReceived_event = FALSE;                                   //data received event
volatile BYTE bDataReceiveCompleted_event = FALSE;

volatile WORD USBEVIE = kUSB_VbusOnEvent + kUSB_VbusOffEvent +
                        kUSB_receiveCompletedEvent
                        + kUSB_dataReceivedEvent + kUSB_UsbSuspendEvent +
                        kUSB_UsbResumeEvent + kUSB_UsbResetEvent;
char outString[40] = "";
WORD i;
WORD x,y;
BYTE ret;

/*  
 * ======== main ========
 */
VOID main (VOID)
{
    WDTCTL = WDTPW + WDTHOLD;                                   //Stop watchdog timer
    Init_Ports();                                               //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                                               //Init clocks
	
    USB_init();
    USB_setEnabledEvents(USBEVIE);

    //Check if we're already physically attached to USB, and if so, connect to it
    //This is the same function that gets called automatically when VBUS gets attached.
    if (USB_connectionInfo() & kUSB_vbusPresent){
        USB_handleVbusOnEvent();
    }


    __enable_interrupt();                           //Enable interrupts globally
    while (1)
    {
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
                __bis_SR_register(LPM3_bits + GIE);                             //Enter LPM3 w/interrupt
                _NOP();
                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:
                __bis_SR_register(LPM0_bits + GIE);                             //Enter LPM0 until an event occurs.
                                                                                //This flag would have been set by the
                                                                                //handleDataReceived event; this event is only
                                                                                //enabled when waiting
                                                                                //for 'press any key'
                if (bHIDDataReceived_event){
                    bHIDDataReceived_event = FALSE;

                                                                                //Change the event flags, in preparation for
                                                                                //receiving 1K data
                    USBEVIE &= ~kUSB_dataReceivedEvent;                         //No more data-received.  We only used this for
                                                                                //'press any key'
                    USBEVIE |= kUSB_receiveCompletedEvent;                      //But enable receive-completed; we want to be
                                                                                //prompted when 1K data has been received
                    USB_setEnabledEvents(USBEVIE);
                    USBHID_rejectData(HID0_INTFNUM);                            //We don't care what char the key-press was

                                                                                //Prompt user for 1K data
                    strcpy(outString,"I'm ready to receive 1K of data.\r\n");   //Prepare the outgoing string
                    if (hidSendDataWaitTilDone((BYTE*)outString,
                            strlen(outString),HID0_INTFNUM,0)){                 //Send it over USB
                        USBHID_abortSend(&x,HID0_INTFNUM);                      //It failed for some reason; abort and leave the
                                                                                //main loop
                        break;
                    }

                                                                                //Set up rcv operation for 1K data
                    if (USBHID_receiveData(dataBuff,1024,
                        HID0_INTFNUM) == kUSBHID_busNotAvailable){
                        USBHID_abortReceive(&x,HID0_INTFNUM);                   //It failed because of surprise removal or suspended
                                                                                //by host;
                        break;                                                  //abort and leave the main loop
                    }
                }

                                                                                //This flag would have been set by the
                                                                                //handleReceiveCompleted event; this event is only
                                                                                //enabled
                                                                                //while receiving 1K data, and signals that all 1K
                                                                                //has been received
                if (bDataReceiveCompleted_event){
                    bDataReceiveCompleted_event = FALSE;
                    strcpy(outString,"Thanks for the data.\r\n");               //Prepare the outgoing string
                    if (hidSendDataInBackground((BYTE*)outString,
                            strlen(outString),HID0_INTFNUM,0)){                 //Send the response over USB
                        USBHID_abortSend(&x,HID0_INTFNUM);                      //It failed for some reason; abort and leave the
                                                                                //main loop
                        break;
                    }

                                                                                //Change the event flags, in preparation for 'press
                                                                                //any key'
                    USBEVIE &= ~kUSB_receiveCompletedEvent;                     //No more receive-completed.
                    USBEVIE |= kUSB_dataReceivedEvent;                          //This will tell us that data -- any data -- has
                                                                                //arrived (i.e., a key-press)
                    USB_setEnabledEvents(USBEVIE);
                }

                break;

            case ST_ENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);                             //Enter LPM3 w/interrupt
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }
    }  //while(1)
} //main()

/*  
 * ======== Init_Clock ========
 */
VOID Init_Clock (VOID)
{
    //Initialization of clock module
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x0C;                                      //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif

        //use REFO for FLL and ACLK
        UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
        XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
    } 
	else {
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x10;                                      //enable XT1 pins
		#endif
        //Use the REFO oscillator to source the FLL and ACLK
        UCSCTL3 = SELREF__REFOCLK;
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

        XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0xFF;
	P2OUT = 0x00;
    P2DIR = 0xFF;
    P3OUT = 0x00;
    P3DIR = 0xFF;
    P4OUT = 0x00;
    P4DIR = 0xFF;
    P5OUT = 0x00;
    P5DIR = 0xFF;
    P6OUT = 0x00;
    P6DIR = 0xFF;
	P7OUT = 0x00;
    P7DIR = 0xFF;
    P8OUT = 0x00;
    P8DIR = 0xFF;
    #if defined (__MSP430F563x_F663x)
		P9OUT = 0x00;
		P9DIR = 0xFF;
    #endif
}

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
                                                    //If bus error occured - the cleaning of flag and re-initializing of USB is
                                                    //required.
            SYSBERRIV = 0;                          //clear bus error flag
            USB_disable();                          //Disable
    }
}

