/*  
 * ======== main.c ========
 * H5 Example
 *
 * High-Bandwidth Sending Using sendData_waitTilDone Demo:
 *
 * The example shows how to implement high-bandwidth sending using
 * the post-call polling method, which doesnt allow execution to proceed
 * until all the data has been sent. It prompts for any key to be pressed,
 * and when this happens, the application sends a large amount of data
 * to the host
 *
 * Use the UsbApp.exe as the PC application.
 * -------------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 *----------------------------------------------------------------------------*/
#include <intrinsics.h>
#include <string.h>

#include "USB_config/descriptors.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB-specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"

#include "USB_API/USB_HID_API/UsbHid.h"

#include "usbConstructs.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);

volatile WORD rounds = 0;

volatile BYTE bHIDDataReceived_event = FALSE;   //Indicates data has been received without an open rcv operation

char outString[32];
WORD x;                                         //Dummy variable for use with abortSend()

#define BUF_SIZE 4096
BYTE dataBuf[BUF_SIZE];
BYTE y = 0x21;

/*  
 * ======== main ========
 */
VOID main (VOID)
{
    WDTCTL = WDTPW + WDTHOLD;                                   //Stop watchdog timer
    Init_Ports();                                               //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                                               //Init clocks
	
    USB_init();
	
    USB_setEnabledEvents(
        kUSB_VbusOnEvent + kUSB_VbusOffEvent + kUSB_UsbSuspendEvent +
        kUSB_UsbResumeEvent + kUSB_dataReceivedEvent);

    //Check if we're already physically attached to USB, and if so, connect to it
    //This is the same function that gets called automatically when VBUS gets attached.
    if (USB_connectionInfo() & kUSB_vbusPresent){
        USB_handleVbusOnEvent();
    }

    //Pre-fill the buffers with visible ASCII characters (0x21 to 0x7E)

    for (x = 0; x < BUF_SIZE; x++)
    {
        dataBuf[x] = y++;
        if (y > 0x7E){
            y = 0x21;
        }
    }

    __enable_interrupt();                           //Enable interrupts globally
    while (1)
    {
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
                __bis_SR_register(LPM3_bits + GIE);                     //Enter LPM3 w/interrupt
                _NOP();
                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:
                if (!bHIDDataReceived_event){                                   //Do this until a key is pressed
                    strcpy(outString,"Press any key.\r");                       //Prepare the outgoing string
                    if (hidSendDataWaitTilDone((BYTE*)outString,
                            strlen(outString),HID0_INTFNUM,0)){                 //Send it; no timeout
                        USBHID_abortSend(&x,HID0_INTFNUM);                      //Operation may still be open; cancel it
                        break;
                    }
                } else {
                    bHIDDataReceived_event = FALSE;
                    USBHID_rejectData(HID0_INTFNUM);
                    for (rounds = 0; rounds < 300; rounds++)
                    {
                        if (hidSendDataWaitTilDone(dataBuf,BUF_SIZE,HID0_INTFNUM,0)){	//Send all of RAM
                            USBHID_abortSend(&x,HID0_INTFNUM);                  		//Operation probably still open; cancel it
                            break;
                        }
                    }
                    strcpy(outString,"\r\n\r\n\r\nThe test is completed.\r\n"); //Prepare the outgoing string
                    if (hidSendDataWaitTilDone((BYTE*)outString,
                            strlen(outString),HID0_INTFNUM,0)){                 //Send it; no timeout
                        USBHID_abortSend(&x,HID0_INTFNUM);                      //Operation may still be open; cancel it
                        break;
                    }
                }
                break;

            case ST_ENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);                     //Enter LPM3 w/interrupt
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }
    }  //while(1)
} //main()

/*  
 * ======== Init_Clock ========
 */
VOID Init_Clock (VOID)
{
    //Initialization of clock module
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x0C;                                      //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif

        //use REFO for FLL and ACLK
        UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
        XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
    } 
	else {
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x10;                                      //enable XT1 pins
		#endif
        //Use the REFO oscillator to source the FLL and ACLK
        UCSCTL3 = SELREF__REFOCLK;
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

        XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0xFF;
	P2OUT = 0x00;
    P2DIR = 0xFF;
    P3OUT = 0x00;
    P3DIR = 0xFF;
    P4OUT = 0x00;
    P4DIR = 0xFF;
    P5OUT = 0x00;
    P5DIR = 0xFF;
    P6OUT = 0x00;
    P6DIR = 0xFF;
	P7OUT = 0x00;
    P7DIR = 0xFF;
    P8OUT = 0x00;
    P8DIR = 0xFF;
    #if defined (__MSP430F563x_F663x)
		P9OUT = 0x00;
		P9DIR = 0xFF;
    #endif
}

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
                                                    //If bus error occured - the cleaning of flag and re-initializing of USB is
                                                    //required.
            SYSBERRIV = 0;                          //clear bus error flag
            USB_disable();                          //Disable
    }
}

