/*  
 * ======== main.c ========
 * Mouse HID Demo:
 *
 * Sending/Receiving Reports:
 * This application creates a traditional HID device, as opposed to a datapipe
 * HID device.  It acts as a mouse on the host, moving the pointer in a circular
 * motion on the screen.  To re-gain control of the mouse, you can either remove
 * the USB cable or type the key sequence to exit debug mode (Ctrl+Shift+D in
 * IAR).
 *
 * A custom report descriptor is defined in descriptors.c.  It defines a top-level
 * application collection of "Generic Desktop", and a usage of "Mouse".  Windows
 * recognizes this configuration as a mouse, and begins servicing it directly.
 * In this way, Windows acts as the "application", rather than using a separate
 * one (like the HID Demo App).
 *
 * While connected to an active USB connection, this application spends most of
 * its time in LPM0.  Timer_A is configured to wake the CPU every 1/60 sec,
 * prompting the main loop to send another HID report.  er, this is unusual
 * and doesn't provide much value.
 *
 * The polling interval in the HID's IN endpoint descriptor is set to 16, meaning
 * the host will poll for a report every 16 USB frames (every 16ms).  Since this
 * is slightly less than 1/60 sec, occasionally the host will poll for a report
 * and not find one; this is OK.
 *
 *
 +----------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 *----------------------------------------------------------------------------*/

#include "USB_config/descriptors.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB-specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"

#include "USB_API/USB_HID_API/UsbHid.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);

typedef struct {
    BYTE buttons;
    BYTE dX;
    BYTE dY;
    BYTE dZ;
} MOUSE_REPORT;

MOUSE_REPORT mouseReport = { 0, 0, 0, 0 };          //HID report, to be sent to the PC.
const INT tableSinCosLookUp[93][2];                 //Lookup table for mouse data;
BYTE index = 1;                                     //Index for lookup table
BYTE sendNewMousePosition = FALSE;                  //Flag by which timer tells main
                                                    //loop to send a new report

/*  
 * ======== main ========
 */
VOID main (VOID)
{
    WDTCTL = WDTPW + WDTHOLD;                                   //Stop watchdog timer
    Init_Ports();                                               //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                                               //Init clocks

    USB_init();                                                 //Init USB

    //Configure an LED to blink when a report is sent
    P1DIR |= 0x01;
    P1OUT &= ~0x01;

    //Timer_A will repeatedly run to 547, using 32kHz clock, and then generate
    //an interrupt that wakes the main loop (every 1/60 sec)
    TA0CCTL0 = CCIE;                                //CCR0 interrupt enabled
    TA0CCR0 = 547;                                  //547/32768 = a period of 16.7ms
    TA0CTL = TASSEL_1 + TACLR;                      //ACLK, up mode


    USB_setEnabledEvents(
        kUSB_VbusOnEvent + kUSB_VbusOffEvent + kUSB_receiveCompletedEvent
        + kUSB_UsbSuspendEvent + kUSB_UsbResumeEvent);

    //In case USB is already attached (meaning no VBUS event will
    //occur), manually start the connection
    if (USB_connectionInfo() & kUSB_vbusPresent){
        USB_handleVbusOnEvent();
    }

    __enable_interrupt();                           //Enable interrupts globally
    while (1)
    {
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
                TA0CTL &= ~MC_1;
                __bis_SR_register(LPM3_bits + GIE); //Enter LPM3 w/interrupt
                _NOP();
                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:
                //Start timer
                TA0CTL |= MC_1;

                                                    //Enter LPM0, until the timer wakes the CPU
                __bis_SR_register(LPM0_bits + GIE);

                                                    //Timer has awakened the CPU.  Proceed with main loop...
                if (sendNewMousePosition){
                                                    //Build the report
                    mouseReport.dX =
                        (tableSinCosLookUp[index][0] -
                         tableSinCosLookUp[index - 1][0]) >> 1;
                    mouseReport.dY =
                        (tableSinCosLookUp[index][1] -
                         tableSinCosLookUp[index - 1][1]) >> 1;

                                                    //Send the report
                    USBHID_sendReport((void *)&mouseReport, HID0_INTFNUM);

                    P1OUT ^= 0x01;                  //Toggle LED on P1.0

                    if (index++ >= 90){
                        index = 1;
                    }
                }
                break;

            case ST_ENUM_SUSPENDED:
                TA0CTL &= ~MC_1;
                P1OUT &= ~BIT0;
                __bis_SR_register(LPM3_bits + GIE); //Enter LPM3 w/interrupt
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }
    }  //while(1)
} //main()

/*  
 * ======== Init_Clock ========
 */
VOID Init_Clock (VOID)
{
    //Initialization of clock module
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x0C;                                      //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif

        //use REFO for FLL and ACLK
        UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
        XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
    } 
	else {
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x10;                                      //enable XT1 pins
		#endif
        //Use the REFO oscillator to source the FLL and ACLK
        UCSCTL3 = SELREF__REFOCLK;
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

        XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0xFF;
	P2OUT = 0x00;
    P2DIR = 0xFF;
    P3OUT = 0x00;
    P3DIR = 0xFF;
    P4OUT = 0x00;
    P4DIR = 0xFF;
    P5OUT = 0x00;
    P5DIR = 0xFF;
    P6OUT = 0x00;
    P6DIR = 0xFF;
	P7OUT = 0x00;
    P7DIR = 0xFF;
    P8OUT = 0x00;
    P8DIR = 0xFF;
    #if defined (__MSP430F563x_F663x)
		P9OUT = 0x00;
		P9DIR = 0xFF;
    #endif
}

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
                                                    //If bus error occured - the cleaning of flag and re-initializing of USB is
                                                    //required.
            SYSBERRIV = 0;                          //clear bus error flag
            USB_disable();                          //Disable
    }
}

/*  
 * ======== TIMER0_A0_ISR ========
 */
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR (void)
{
    sendNewMousePosition = TRUE;                                //Set flag telling main loop to send a report
    __bic_SR_register_on_exit(LPM0_bits);                       //Keep CPU awake after returning;
                                                                //enables a run through the main loop
}

//Lookup table for mouse position values.  "const" indicates it will be stored
//in flash.
const INT tableSinCosLookUp[93][2] = {
    0,200,
    14,200,
    28,198,
    42,196,
    55,192,
    68,188,
    81,183,
    94,177,
    106,170,
    118,162,
    129,153,
    139,144,
    149,134,
    158,123,
    166,112,
    173,100,
    180,88,
    185,75,
    190,62,
    194,48,
    197,35,
    199,21,
    200,7,
    200,-7,
    199,-21,
    197,-35,
    194,-48,
    190,-62,
    185,-75,
    180,-88,
    173,-100,
    166,-112,
    158,-123,
    149,-134,
    139,-144,
    129,-153,
    118,-162,
    106,-170,
    94,-177,
    81,-183,
    68,-188,
    55,-192,
    42,-196,
    28,-198,
    14,-200,
    0,-200,
    -14,-200,
    -28,-198,
    -42,-196,
    -55,-192,
    -68,-188,
    -81,-183,
    -94,-177,
    -106,-170,
    -118,-162,
    -129,-153,
    -139,-144,
    -149,-134,
    -158,-123,
    -166,-112,
    -173,-100,
    -180,-88,
    -185,-75,
    -190,-62,
    -194,-48,
    -197,-35,
    -199,-21,
    -200,-7,
    -200,7,
    -199,21,
    -197,35,
    -194,48,
    -190,62,
    -185,75,
    -180,88,
    -173,100,
    -166,112,
    -158,123,
    -149,134,
    -139,144,
    -129,153,
    -118,162,
    -106,170,
    -94,177,
    -81,183,
    -68,188,
    -55,192,
    -42,196,
    -28,198,
    -14,200,
    0,200
};


