/*  
 * ======== main.c ========
 * Mass Storage with File System Emulation.
 *
 * Needs to run on MSP430F5529 Experimenters board.
 *
 +----------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 *----------------------------------------------------------------------------*/
#include <string.h>

#include "USB_config/descriptors.h"
#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"
#include "USB_API/USB_Common/usb.h"

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"
#include "F5xx_F6xx_Core_Lib/HAL_FLASH.h"

//MSC #includes
#include "USB_API/USB_MSC_API/UsbMscScsi.h"
#include "USB_API/USB_MSC_API/UsbMsc.h"
#include "USB_API/USB_MSC_API/UsbMscStateMachine.h"
#include "mscFseData.h"
#include "mscFse.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);


//The number of bytes per block.  In FAT, this is 512
const WORD BYTES_PER_BLOCK = 512;


//Data-exchange buffer between the API and the application.  The application allocates it,
//and then registers it with the API.  Later, the API will pass it back to the application when it
//needs the application to "process" it (exchange data with the media).
__no_init BYTE RW_dataBuf[512 * 2];


//The API allocates an instance of structure type USBMSC_RWbuf_Info to hold all information describing
//buffers to be processed.  The structure instance is a shared resource between the API and application.
//During initialization, we'll call USBMSC_fetchInfoStruct() to obtain the pointer from the API.
USBMSC_RWbuf_Info *RWbuf_info;


volatile BYTE fS1ButtonEvent = 0;               //Flag indicating button S1 was pressed (that is, button S1 on a F552x FET target
                                                //board)
WORD DataCnt = 0;                               //The index of the next available byte to be written within Data[]
BYTE logCnt = 0x30;                             //Holds an incrementing character to be written into each new data log entry

//The application must tell the API about the media.  This information is conveyed in a call to
//USBMSC_updateMediaInfo(), passing an instance of USBMSC_RWbuf_Info.  Since SD-cards are removable, this might
//need to occur at any time, so this is made global.
struct USBMSC_mediaInfoStr mediaInfo;

/*  
 * ======== main ========
 */
void main (void)
{
    WDTCTL = WDTPW + WDTHOLD;                   //Stop watchdog timer
    Init_Ports();                               //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                               //Init clocks

# if 0                                              //Use for FET boards
    //Configure P1.6 as a button input
    P1DIR &= ~BIT6;                                 //Input
    P1OUT |= BIT6;
    P1REN |= BIT6;                                  //Together with P1OUT, creates a pullup
    P1IFG &= ~BIT6;                                 //Clear the flag
    P1IE |= BIT6;                                   //Enable interrupts

# else  //Use for F5529 EXP board
    P1DIR &= ~BIT7;                                 //Input
    P1OUT |= BIT7;
    P1REN |= BIT7;                                  //Together with P1OUT, creates a pullup
    P1IFG &= ~BIT7;                                 //Clear the flag
    P1IE |= BIT7;                                   //Enable interrupts
# endif

    USB_init();                                 //Initialize the USB module


    //Enable all USB events
    USB_setEnabledEvents(kUSB_allUsbEvents);

    //The API maintains an instance of the USBMSC_RWbuf_Info structure.  If double-buffering were used, it would
    //maintain one for both the X and Y side.  (This version of the API only supports single-buffering,
    //so only one structure is maintained.)  This is a shared resource between the API and application; the
    //application must request the pointers.  This function returns the pointer for a given LUN and buffer side.
    RWbuf_info = USBMSC_fetchInfoStruct();


    //The application must tell the API about the media.  Since the media isn't removable, this is only called
    //once, at the beginning of execution.  If the media were removeable, the application must call this any time
    //the status of the media changes.
    mediaInfo.mediaPresent = 0x01;                                      //The medium is present, because internal flash is
                                                                        //non-removable.
    mediaInfo.mediaChanged = 0x00;                                      //It can't change, because it's in internal memory, which is
                                                                        //always present.
    mediaInfo.writeProtected = 0x00;                                    //It's not write-protected
    mediaInfo.lastBlockLba = 774;                                       //774 blocks in the volume. (This number is also found twice
                                                                        //in the volume itself; see mscFseData.c. They should
                                                                        //match.)
    mediaInfo.bytesPerBlock = BYTES_PER_BLOCK;                          //512 bytes per block. (This number is also found in the
                                                                        //volume itself; see mscFseData.c. They should match.)
    USBMSC_updateMediaInfo(0, &mediaInfo);

    //The data interchange buffer (used when handling SCSI READ/WRITE) is declared by the application, and
    //registered with the API using this function.  This allows it to be assigned dynamically, giving
    //the application more control over memory management.
    USBMSC_registerBufInfo(0, &RW_dataBuf[0], NULL, sizeof(RW_dataBuf));

    //If USB is already connected when the program starts up, then there won't be a USB_handleVbusOnEvent().
    //So we need to check for it, and manually connect if the host is already present.
    if (USB_connectionInfo() & kUSB_vbusPresent){
        if (USB_enable() == kUSB_succeed){
            USB_reset();
            USB_connect();
        }
    }

    __enable_interrupt();                                               //Enable general interrupts

    while (1)
    {
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
                __bis_SR_register(LPM3_bits + GIE);                     //Enter LPM3 until VBUS-on event

                                                                        //Check if the reason we woke was a button press; and if so,
                                                                        //log a new piece of data.
                if (fS1ButtonEvent){
                                                                        //Build string
                    char str[14] = "Data entry #0\n";
                    str[12] = logCnt++;                                 //Number the entries 0 through....?

                    memcpy(RW_dataBuf, Data559, BYTES_PER_BLOCK);       //Copy data block 559 from flash to RAM buffer
                    memcpy(&RW_dataBuf[DataCnt], str, sizeof(str));     //Write the new entry to the RAM buffer
                    flashWrite_LBA((PBYTE)Data559, RW_dataBuf);         //Copy it back to flash

                    DataCnt += sizeof(str);                             //Increment the index past the new entry
                    if ((DataCnt + sizeof(str) >= BYTES_PER_BLOCK)){    //Roll index back to 0, if no more room in the block
                        DataCnt = 0;
                    }

                    fS1ButtonEvent = 0;
                }

                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:

                                                                        //Call USBMSC_poll() to initiate handling of any received
                                                                        //SCSI commands.  Disable interrupts
                                                                        //during this function, to avoid conflicts arising from SCSI
                                                                        //commands being received from the host
                                                                        //AFTER decision to enter LPM is made, but BEFORE it's
                                                                        //actually entered (in other words, avoid
                                                                        //sleeping accidentally).
                __disable_interrupt();
                if (USBMSC_poll() == kUSBMSC_okToSleep){
                    __bis_SR_register(LPM0_bits + GIE);                 //Enable interrupts atomically with LPM0 entry
                }
                __enable_interrupt();


                                                                        //If the API needs the application to process a buffer, it
                                                                        //will keep the CPU awake by returning kUSBMSC_processBuffer
                                                                        //from USBMSC_poll().  The application should then check the
                                                                        //'operation' field of all defined USBMSC_RWbuf_Info
                                                                        //structure instances.  If any of them is non-null, then an
                                                                        //operation needs to be processed.  A value of
                                                                        //kUSBMSC_READ indicates the API is waiting for the
                                                                        //application to fetch data from the storage volume, in
                                                                        //response
                                                                        //to a SCSI READ command from the USB host.  After the
                                                                        //application does this, it must indicate whether the
                                                                        //operation succeeded, and then close the buffer operation
                                                                        //by calling USBMSC_bufferProcessed().
                while (RWbuf_info->operation == kUSBMSC_READ)
                {
                    RWbuf_info->returnCode = Read_LBA(RWbuf_info->lba,
                        RWbuf_info->bufferAddr,
                        RWbuf_info->lbCount);                                //Fetch a block from the medium, using file system emulation
                    USBMSC_bufferProcessed();                           //Close the buffer operation
                }

                                                                        //Same as above, except for WRITE.  If operation ==
                                                                        //kUSBMSC_WRITE, then the API is waiting for us to
                                                                        //process the buffer by writing the contents to the storage
                                                                        //volume.
                while (RWbuf_info->operation == kUSBMSC_WRITE)
                {
                    RWbuf_info->returnCode = Write_LBA(RWbuf_info->lba,
                        RWbuf_info->bufferAddr,
                        RWbuf_info->lbCount);                                //Write the block to the medium, using file system emulation
                    USBMSC_bufferProcessed();                           //Close the buffer operation
                }
                break;

            case ST_ENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);                     //Enter LPM3, until a resume or VBUS-off event
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }
    }  //while(1)
} //main()

/*  
 * ======== write_LUN1 ========
 * PORT1 interrupt service routine.  Check for button S1 (on a F552x FET target board) being pressed.
 */
#pragma vector=PORT1_VECTOR
__interrupt VOID Port1InterruptHandler (VOID)
{
    WORD P1IV_isr = P1IV;                                               //port1 interrupt

    fS1ButtonEvent = 1;                                                 //Set boolean flag
    P1IFG = 0;                                                          //Clear I/O interrupt flags
    __bic_SR_register_on_exit(LPM3_bits);                               //Keep CPU awake when this ISR exits; resume execution from
                                                                        //LPMx entry
}

/*  
 * ======== Init_Clock ========
 */
VOID Init_Clock (VOID)
{
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x0C;                                      //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif

        //use REFO for FLL and ACLK
        UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
        XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
    } 
	else {
		#if defined (__MSP430F552x) || defined (__MSP430F550x)
			P5SEL |= 0x10;                                      //enable XT1 pins
		#endif
        //Use the REFO oscillator to source the FLL and ACLK
        UCSCTL3 = SELREF__REFOCLK;
        UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

        //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
        Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

        XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0xFF;
	P2OUT = 0x00;
    P2DIR = 0xFF;
    P3OUT = 0x00;
    P3DIR = 0xFF;
    P4OUT = 0x00;
    P4DIR = 0xFF;
    P5OUT = 0x00;
    P5DIR = 0xFF;
    P6OUT = 0x00;
    P6DIR = 0xFF;
	P7OUT = 0x00;
    P7DIR = 0xFF;
    P8OUT = 0x00;
    P8DIR = 0xFF;
    #if defined (__MSP430F563x_F663x)
		P9OUT = 0x00;
		P9DIR = 0xFF;
    #endif
}

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
                                                        //If bus error occured - the cleaning of flag and re-initializing of USB is
                                                        //required.
            SYSBERRIV = 0;                              //clear bus error flag
            USB_disable();                              //Disable
    }
}

